//////////////////////////////////////////////////////////////////////////////
//
// ClockTest.c
//
//  This example tests the clock generator by setting the channels to
// various frequencies and then verifying that the proper frequency
// is reported back.
//
#if defined( WIN32 )

#include <windows.h>
#include <stdio.h>
#include <conio.h>

#elif defined( PLX_LINUX )

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#endif

#include "GscApi.h"

//
// Global Variables
//
int boardNumber = 1; // Board Number to Test
int bitRate[4] = {1000000,2000000,4000000,8000000};

//
// Forward declarations of local functions
//
int clockTest();
int setAndCheckClocks( int bitRate[]);


//-------------------------------------------------------- DisplayErrorMessage
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long.
//
void DisplayErrorMessage( int errorCode)
{
  // Buffer to hold the error message
  char errorMessage[80];

  // Call the API to interpret the message
  GscGetErrorString( errorCode, errorMessage);

  // Display the error
  printf("\n Error Code: %d, %s\n", errorCode, errorMessage);
}


//------------------------------------------------- DisplayErrorMessageAndWait
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long. The
// routine waits for the user to press a key before
// returning.
//
void DisplayErrorMessageAndWait( int errorCode)
{
  // Display the Error Message
  DisplayErrorMessage( errorCode);

  // Display the prompt
  printf("\n Press <ENTER> to continue\n");

  // Wait for <ENTER>
  getchar();
}


//----------------------------------------------------------------------- main
//
//  Main entry point to the example
//
int main(int argc, char * argv[])
{
  int status = 0x00;	 // Result from API calls
  char kbchar;

  // If an argument was entered, override the default boardNumber
  if (argc == 2) boardNumber = atoi(argv[1]);

  if (argc == 6)
	{
		bitRate[0] = atoi(argv[2]);
		bitRate[1] = atoi(argv[3]);
		bitRate[2] = atoi(argv[4]);
		bitRate[3] = atoi(argv[5]);
	}

  // Display the example header
  printf("\n");
  printf(" General Standards SIO4 Programmable Clock Test\n");
  printf(" The Clock Test will be performed on Board #%d\n", boardNumber);
  printf("\n");

  //
  // Initialize the board. This must be done before any channel operations
  // can be performed. The program is aborted if any errors occur.
  //
  status = GscOpen( boardNumber, GSC_API_VERSION);
  if (status)
  {
    DisplayErrorMessageAndWait(status);
    return 1;
  }

  // Loop through, performing the test, until the user aborts with an 'X'
  do
  {
    // Clear the status flag
    status = 0x00;

    // Perform the Clock Test on the desired board
    status = clockTest();

    // If a non-zero status was returned, display the error
    if (status)
		{
			DisplayErrorMessage(status);
		}

    // Prompt the user
    printf("\nType X to exit or any other key to repeat the test: ");
    kbchar = toupper( getchar());

  } while(kbchar != 'X');

  //
  // Shut down the board. This must be the last operation performed. It
  // releases the resources that are used by the GSC API and driver.
  //
  printf("\n");

  GscClose(boardNumber);

  return 0;
}


//
//  This routine performs a clock test on a SIO4 board. Each channel of the
// board will be set to a different bit rate and then the rates will be
// verified by reading back their current values.
//
int clockTest()
{
  int ch;
  //  int bitrate[4];
  int status;

  //
  //  Reset all 4 channels. This puts the channels into a known state.
  //
  for (ch=1; ch<=4; ch++)
  {
    // Reset the channel
    status = GscSio4ChannelReset( boardNumber, ch);
    if (status) return status;
  }

#if 0
  // Set all 4 channels to the MAX
  bitrate[0]=bitrate[1]=bitrate[2]=bitrate[3]=20000000;
  status = setAndCheckClocks( bitrate);
  if (status) return status;

  // Set all 4 channels to a minimum
  bitrate[0]=bitrate[1]=bitrate[2]=bitrate[3]= 1200;
  status = setAndCheckClocks( bitrate);
  if (status) return status;

  // Set all 4 channels to different values
  bitrate[0]=10000000;
  bitrate[1]=bitrate[0]/2;
  bitrate[2]=bitrate[1]/2;
  bitrate[3]=bitrate[2]/2;
  status = setAndCheckClocks( bitrate);
  if (status) return status;


  // Set all 4 channels to different values
  bitrate[3] = 10000000;
  bitrate[2]=bitrate[3]/2;
  bitrate[1]=bitrate[2]/2;
  bitrate[0]=bitrate[1]/2;
#endif

  status = setAndCheckClocks( bitRate);
  if (status) return status;

  return 0x00;
}

int setAndCheckClocks( int bitRate[])
{
  int status;
  int channel, result, diff;
  double ppm;

  // Program all channels
  for (channel=1; channel<=4; channel++)
  {
    status = GscSio4ChannelSetClock( boardNumber, channel, bitRate[channel-1]);
    if (status) return status;
  }

  // Read back and display the frequencies
  for (channel=1; channel<=4; channel++)
  {
    status = GscLocalRegisterWrite( boardNumber, 0xa8, (1 << channel) & 0x1e);
    if (status) return status;

    while(1)
		{
			// Get the current status
			status = GscLocalRegisterRead( boardNumber, 0xa8, &result);
			if (status) return status;

			// If the measurement is completed
			if (result & 0x04)
	    {
	      // Determine the frequency and break out of the loop
	      result /= 256;
	      result *= 10;
	      ppm = (double)bitRate[channel-1] - (double)result;
	      ppm = ppm/(double)bitRate[channel-1] * 1000000.0;
	      diff = abs((int)ppm);
	      break;
	    }
		}

		printf("Channel %d: Set to %8d, Read back %8d ", channel, bitRate[channel-1], result);
		printf("%s (%3d ppm)\n", (diff>100)?"***":"   ", diff);
	}

  printf("\n");
  return 0;
}
