//////////////////////////////////////////////////////////////////////////////
//
// DisplayBoards.c
//
//  This example displays information for each of the General Standards SIO4
// boards that are installed n a system.
//

#if defined(WIN32)
#include <windows.h>
#endif
#include <stdio.h>
#include "GscApi.h"


//
// Forward declarations of local functions
//
void ClearScreen(void);


//-------------------------------------------------------- DisplayErrorMessage
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long.
//
void DisplayErrorMessage( int errorCode)
{
	// Buffer to hold the error message
	char errorMessage[80];

	// Call the API to interpret the message
	GscGetErrorString( errorCode, errorMessage);

	// Display the error
	printf("\n Error Code: %d, %s\n", errorCode, errorMessage);
}


//------------------------------------------------- DisplayErrorMessageAndWait
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long. The
// routine waits for the user to press a key before
// returning.
//
void DisplayErrorMessageAndWait( int errorCode)
{
	// Display the Error Message
	DisplayErrorMessage( errorCode);

	// Display the prompt
	printf("\n Hit <ENTER> to continue\n");

	// Wait for <ENTER>
	getchar();
}


//----------------------------------------------------------------------- main
//
//  Main entry point to the example
//
int main(int argc, char * argv[])
{
  int numberOfBoards;			// Holds the number of boards in the system
  int status = 0x00;			// Result from API calls
  int i;
  int dllVersion, driverVersion, fpgaVersion, sizes, txSize, rxSize;

  //
  // Declare a structure to hold board information. This structure
  // will be filled in by the call to GscFindBoards.
  //
  GSC_DEVICES_STRUCT gscDevices[ MAXIMUM_NUMBER_OF_BOARDS];

  //
  // Display the program header
  ClearScreen();

  printf("\n\n");
  printf("                       General Standards SIO4\n");
	printf("                     Board Information Example\n");
  printf("\n\n");
  printf("        Bus/   Vendor/                DLL     Driver     FPGA      FIFOs\n");
  printf(" Board  Slot   Device    Serial #   Version   Version   Version    Tx Rx\n");
  printf(" -----  ----  ---------  ---------  --------  --------  --------  -------\n");

  //
  // Find all of the Sio4 Boards in the system. If an error occurs,
  // display the error message and exit.
  //

  status = GscFindBoards( &numberOfBoards, gscDevices);

  if (status)
  {
    DisplayErrorMessageAndWait( status);
    return 1;
  }

  //
  // Display a message if no boards were found.
  //

  if(numberOfBoards < 1)
  {
    printf(" No Boards found in the system\n");
  }

  //
  // Otherwise, display all of the information returned from the
  // GscFindBoards(...) call.
  //
  else
  {
    // Loop through all of the detected boards
    for (i=0; i<numberOfBoards; i++)
    {
      //
      // Print the info obtained from GscFindBoards(...)
      //
      printf("   %d    %d/%d  %04X/%04X  %s ",
				i+1,
				gscDevices[i].busNumber,
				gscDevices[i].slotNumber,
				gscDevices[i].vendorId,
				gscDevices[i].deviceId,
				gscDevices[i].serialNumber);

      //
      // Get the version and FIFO info from the boards
      //

      // Open the board
      GscOpen( i+1, GSC_API_VERSION);

      // Get the verison info
      GscGetVersions( i+1, &dllVersion, &driverVersion, &fpgaVersion);

      // Get the FIFO size for channel 1 (assuming all 4 channels are the same)
      GscSio4ChannelFifoSizes( i+1, 1, &sizes);

      // Close the board
      GscClose( i+1);

      // Display the versions
      printf("       %08X  %08X  %08X", dllVersion, driverVersion, fpgaVersion);

      // Calculate the Tx and Rx FIFO sizes. The Tx value is returned in the upper 16 bits
      // of sizes and the Rx value is in the lower 16 bits.
      txSize = (sizes>>16) & 0xffff;
      rxSize = sizes & 0xffff;

      // If the Tx value is > 1024, convert it to "k bytes" and display
      // Otherwise, display the value in bytes

      if (txSize >= 1024) printf("  %2dk", txSize/1024);
      else printf("  %3d",txSize);

      // If the Rx value is > 1024, convert it to "k bytes" and display
      // Otherwise, display the value in bytes

      if (rxSize >= 1024) printf(" %2dk\n", rxSize/1024);
      else printf(" %3d\n",rxSize);
    }
    printf("\n");
  }

  return 0;
}


//---------------------------------------------------------------- ClearScreen
//
//  This routine clears the console screen
//
void ClearScreen(void)
{
#if 0
  HANDLE MainConsole;
  COORD coordScreen = {0,0};
  DWORD cCharsWritten;
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  DWORD dwConSize;

  // Get a HANDLE to the Console Window
  MainConsole = GetStdHandle(STD_OUTPUT_HANDLE);

  // Get Console Information
  GetConsoleScreenBufferInfo(MainConsole, &csbi );

  // Figure Console Size
  dwConSize = csbi.dwSize.X * csbi.dwSize.Y;

  // Fill Console with spaces
  FillConsoleOutputCharacter(MainConsole, (TCHAR) ' ', dwConSize, coordScreen, &cCharsWritten );

  // Get Console Information
  GetConsoleScreenBufferInfo(MainConsole, &csbi );

  // Fill Console with Current Attributes
  FillConsoleOutputAttribute(MainConsole, csbi.wAttributes, dwConSize, coordScreen, &cCharsWritten );

  // Send the Cursor Home
  SetConsoleCursorPosition(MainConsole, coordScreen );

  return;
#endif
}

