//////////////////////////////////////////////////////////////////////////////
//
// InterruptTest.c
//
//  This example tests all of the possible Interrupt sources for an SIO4
// board.
//
#if defined( WIN32 )
#include <windows.h>
#include <stdio.h>
#elif defined ( PLX_LINUX )
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#endif
#include "GscApi.h"

#ifdef WIN32
#define usleep(val) Sleep(val / 1000 + 200);
#endif

enum
{
  RX_TRANSFER_COMPLETE = 0,
  TX_TRANSFER_COMPLETE,
  TX_FIFO_FULL,
  TX_FIFO_EMPTY,
  RX_FIFO_FULL,
  RX_FIFO_EMPTY,
  UNKNOWN_TYPE,
  NUMBER_OF_INTR
};

//
// Global Variables
//
int boardNumber = 1;
int channel = 1;
int intrCounts[ NUMBER_OF_INTR];

//
// Forward declarations of local functions
//
int  InterruptTest(int boardNumber, int channel);

void CALLBACK isrHandler( int board, int channel, int type)
{
  switch(type)
    {
    case GSC_INTR_RX_TRANSFER_COMPLETE: intrCounts[ RX_TRANSFER_COMPLETE]++; break;
    case GSC_INTR_TX_TRANSFER_COMPLETE: intrCounts[ TX_TRANSFER_COMPLETE]++; break;
    case GSC_INTR_TX_FIFO_EMPTY:        intrCounts[ TX_FIFO_EMPTY]++;        break;
    case GSC_INTR_TX_FIFO_FULL:         intrCounts[ TX_FIFO_FULL]++;         break;
    case GSC_INTR_RX_FIFO_EMPTY:        intrCounts[ RX_FIFO_EMPTY]++;        break;
    case GSC_INTR_RX_FIFO_FULL:         intrCounts[ RX_FIFO_FULL]++;         break;
    default:                            intrCounts[ UNKNOWN_TYPE]++;         break;
    }
}

void clearIntrCounts()
{
  memset(intrCounts, 0, sizeof(intrCounts));
}

int getIntrTotal()
{
	int i, total=0;
	for (i=0; i<NUMBER_OF_INTR; i++) total += intrCounts[i];
	return total;
}

void displayIntrInfo(int intr, char *result)
{
  char msg[100];
  switch(intr)
    {
    case RX_TRANSFER_COMPLETE: strcpy(msg, "Rx Transfer Complete"); break;
    case TX_TRANSFER_COMPLETE: strcpy(msg, "Tx Transfer Complete"); break;
    case TX_FIFO_EMPTY: strcpy(msg, "Tx Fifo Empty"); break;
    case TX_FIFO_FULL: strcpy(msg, "Tx Fifo Full"); break;
    case RX_FIFO_EMPTY: strcpy(msg, "Rx Fifo Empty"); break;
    case RX_FIFO_FULL: strcpy(msg, "Rx Fifo Full"); break;
    default: strcpy(msg, "Unknown Interrupt"); break;
    }

  if (result == NULL) printf("%s", msg);
  else strcpy(result,msg);
}

void displayAllActiveIntr()
{
  int i;
  for (i=0; i<NUMBER_OF_INTR; i++)
  {
    if (intrCounts[i] != 0)
    {
      displayIntrInfo(i, NULL);
      printf("(%d) ",intrCounts[i]);
    }
  }
}

int verifyIntrCounts( int intr, char *string)
{
  int i;

  // clear the string
  strcpy(string, "");

  // Check all of the counts
  for (i=0; i<NUMBER_OF_INTR; i++)
  {
    if (i==intr)
    {
      if (intrCounts[i] == 1)
      {
          strcpy(string, "Passed");
      }
      else
      {
          strcpy(string, "Failed");
      }
    }
    else
    {
      if (intrCounts[i] != 0)
      {
          strcpy(string, "Failed");
      }
    }
  }
  return 0;
}


//------------------------------------------------------------- testTxFifoFull
//
//  This routine will test to see if the Tx FIFO Full interrupt
// is functioning properly. It will perform the following steps:
//
//   1. Reset the channel but not enable the transmitter
//   2. Clear the interrupt counters
//
int testTxFifoFull()
{
  int fifoSize, status, index;

  // Reset the desired channel. This puts the channel into a known state.
  status = GscSio4ChannelReset( boardNumber, channel);
  if (status) return status;

  // Clear the interrupt counters
  clearIntrCounts();

  // Get the Tx Fifo size
  status = GscSio4ChannelFifoSizes( boardNumber, channel, &fifoSize);
  if (status) return status;
  fifoSize &= 0xffff;

  // Write to the Tx Fifo until it is one short of full
  for (index=0; index<(fifoSize-1); index++)
  {
    status = GscLocalRegisterWrite( boardNumber, 8+(channel<<4), 0xff);
    if (status) return status;
  }
  usleep(250000);

  if (intrCounts[ TX_FIFO_FULL]!=0)
  {
    printf("Intr too early - ");
    return -1;
  }
  else
  {
    status = GscLocalRegisterWrite( boardNumber, 8+(channel<<4), 0xff);
    if (status) return status;
    usleep(250);
    if (intrCounts[ TX_FIFO_FULL] != 1) return -1;
    else return 0;
  }

  return 0;
}


//------------------------------------------------------------ testTxFifoEmpty
//
//  This routine will test to see if the Tx FIFO Empty interrupt
// is functioning properly. It will perform the following steps:
//
//   1. Setup the channel with Tx enabled but no loopback
//   2. Clear the interrupt counters
//
int testTxFifoEmpty()
{
  int status;

  // Reset the desired channel. This puts the channel into a known state.
  status = GscSio4ChannelReset( boardNumber, channel);
  if (status) return status;

  // Set the channel mode so the data can be transmitted
  //status = GscSio4ChannelSetMode( boardNumber, channel, GSC_MODE_HDLC, 1000000);
  status = GscSio4ChannelSetMode( boardNumber, channel, GSC_MODE_MONOSYNC, 1000000);
  if (status) return status;

  // Clear the interrupt counters
  clearIntrCounts();

  // Write a byte to the Tx FIFO that will get transmitted
  status = GscLocalRegisterWrite( boardNumber, 8 + (channel<<4), 0xff);
  if (status) return status;
  usleep(250);

  if (intrCounts[ TX_FIFO_EMPTY] != 1) return -1;

  return 0;
}


//------------------------------------------------------------- testRxFifoFull
//
//  This routine will test to see if the Rx FIFO Full interrupt
// is functioning properly. It will perform the following steps:
//
//   1. Setup the channel with a loopback
//   2. Clear the interrupt counters
//
int testRxFifoFull()
{
  int fifoSize, status, index;
  GSC_ASYNC_CONFIG chCfg;

  // Reset the desired channel. This puts the channel into a known state.
  status = GscSio4ChannelReset( boardNumber, channel);
  if (status) return status;

  // Set the channel mode so the data can be transmitted
  if ((status = GscSio4AsyncGetDefaults( &chCfg )) ||
		(status = GscSio4AsyncSetConfig( boardNumber, channel, chCfg )))
    return status;

  // Set the loopback so that the Tx data will go into the Rx FIFO
  status = GscSio4SetOption( boardNumber, channel, GSC_SIO_LOOPBACK, GSC_LOOP_EXTERNAL );
  if (status) return status;

  // Clear the interrupt counters
  clearIntrCounts();

  // Get the Rx Fifo size
  status = GscSio4ChannelFifoSizes( boardNumber, channel, &fifoSize);
  if (status) return status;
  fifoSize = (fifoSize >>16) & 0xffff;

  // Write to the Tx Fifo until the Rx Fifo is one short of full
  for (index=0; index<(fifoSize-1); index++)
  {
    status = GscLocalRegisterWrite( boardNumber, 8+(channel<<4), 0xff);
    if (status) return status;
  }
  usleep(250000);

  if (intrCounts[ RX_FIFO_FULL]!=0)
  {
    printf("Failed, Intr too early\n");
    displayAllActiveIntr();
    GscSio4ChannelFifoCounts(boardNumber,channel, &fifoSize);
    printf("Fifo: %08X\n",fifoSize);
    return -1;
  }
  else
  {
    status = GscLocalRegisterWrite( boardNumber, 8+(channel<<4), 0xff);
    if (status) return status;
    usleep(250);
    if (intrCounts[ RX_FIFO_FULL] != 1) return -1;
    else return 0;
  }

  return 0;
}


//------------------------------------------------------------ testRxFifoEmpty
//
//  This routine will test to see if the Rx FIFO Empty interrupt
// is functioning properly. It will perform the following steps:
//
//   1. Setup the channel with a loopback
//   2. Clear the interrupt counters
//
int testRxFifoEmpty()
{
  int status, temp;
  GSC_ASYNC_CONFIG chCfg;

  // Reset the desired channel. This puts the channel into a known state.
  status = GscSio4ChannelReset( boardNumber, channel);
  if (status) return status;

  // Clear the interrupt counters
  clearIntrCounts();

  // Set the channel mode so the data can be transmitted
  if ((status = GscSio4AsyncGetDefaults( &chCfg )) ||
		(status = GscSio4AsyncSetConfig( boardNumber, channel, chCfg )))
    return status;

  // Set the loopback so that the Tx data will go into the Rx FIFO
  status = GscSio4SetOption( boardNumber, channel, GSC_SIO_LOOPBACK, GSC_LOOP_EXTERNAL );
  if (status) return status;

  // Write a byte to the Tx FIFO that will get to the Rx Fifo
  status = GscLocalRegisterWrite( boardNumber, 8 + (channel<<4), 0xff);
  if (status) return status;
  usleep(250);

  // Read the byte from the Rx Fifo so it will go empty
  status = GscLocalRegisterRead( boardNumber, 8 + (channel<<4), &temp);
  if (status) return status;
  usleep(250);

  if (intrCounts[ RX_FIFO_EMPTY] != 1) return -1;

  return 0;
}


//-------------------------------------------------------- DisplayErrorMessage
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long.
//
void DisplayErrorMessage( int errorCode)
{
	// Buffer to hold the error message
	char errorMessage[80];

	// Call the API to interpret the message
	GscGetErrorString( errorCode, errorMessage);

	// Display the error
	printf("\n Error Code: %d, %s\n", errorCode, errorMessage);
}


//------------------------------------------------- DisplayErrorMessageAndWait
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long. The
// routine waits for the user to press a key before
// returning.
//
void DisplayErrorMessageAndWait( int errorCode)
{
  // Display the Error Message
  DisplayErrorMessage( errorCode);

  // Display the prompt
  printf("\n Press <ENTER> to continue\n");

  // Wait for <ENTER>
  getchar();
}

//----------------------------------------------------------------------- main
//
//  Main entry point to the example
//
int main(int argc, char * argv[])
{
  int status = 0x00;			// Result from API calls
  char kbchar;

  // Override default board and channel if provided
  if (argc >= 2) boardNumber = atoi(argv[1]);
  if (argc >= 3) channel = atoi(argv[2]);

  // Display the header
  printf("\n");
  printf(" General Standards SIO4 Interrupt Test on Board #%d, Channel #%d\n", boardNumber, channel);
  printf("\n");

  //
  // Initialize the board. This must be done before any channel operations
  // can be performed. The program is aborted if any errors occur.
  //
  status = GscOpen( boardNumber, GSC_API_VERSION);
  if (status)
  {
    DisplayErrorMessageAndWait(status);
    return 1;
  }

	// Loop through, performing the test, until the user aborts with an 'X'
  do
  {
    // Clear the status flag
    status = 0x00;

    // Perform the loopback test on the desired channel
    status = InterruptTest(boardNumber, channel);

    // If a non-zero status was returned, display the error
    if (status)
		{
			DisplayErrorMessage(status);
		}

    // Prompt the user
    printf("\nType X to exit or any other key to repeat the test: ");
    kbchar = toupper(getchar());

  } while(kbchar!='X');

  //
  // Shut down the board. This must be the last operation performed. It
  // releases the resources that are used by the GSC API and driver.
  //
  GscClose(boardNumber);

  return 0;
}


//
// Register Tx and Rx Fifo interrupt callbacks and create Fifo conditions that generate those interrupts
//

int InterruptTest(int boardNumber, int channel)
{
  int status = 0x00;

  // Register all of the interrupts
	status  = GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_TRANSFER_COMPLETE,
					     GSC_RISING_EDGE, isrHandler);
  status += GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_TRANSFER_COMPLETE,
					     GSC_RISING_EDGE, isrHandler);
  status += GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_FIFO_FULL,
					     GSC_RISING_EDGE, isrHandler);
  status += GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_FIFO_EMPTY,
					     GSC_RISING_EDGE, isrHandler);
  status += GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_FIFO_EMPTY,
					     GSC_RISING_EDGE, isrHandler);
  status += GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_FIFO_FULL,
					     GSC_RISING_EDGE, isrHandler);
  if (status) return 0xfff;

  //
  // Test the Tx Fifo Full Interrupt
  //
  printf("   Testing Tx Fifo Full Interrupt : ");
  status = testTxFifoFull();
  if (status == 0) printf("Passed\n");
  else if (status == -1) printf("Failed\n");
  else return status;

  //
  // Test the Tx Fifo Empty Interrupt
  //
  printf("   Testing Tx Fifo Empty Interrupt: ");
  status = testTxFifoEmpty();
  if (status == 0) printf("Passed\n");
  else if (status == -1) printf("Failed\n");
  else return status;

  //
  // Test the Rx Fifo Full Interrupt
  //
  printf("   Testing Rx Fifo Full Interrupt : ");
  status = testRxFifoFull();
  if (status == 0) printf("Passed\n");
  else if (status == -1) printf("Failed\n");
  else return status;

  //
  // Test the Rx Fifo Empty Interrupt
  //
  printf("   Testing Rx Fifo Empty Interrupt: ");
  status = testRxFifoEmpty();
  if (status == 0) printf("Passed\n");
  else if (status == -1) printf("Failed\n");
  else return status;

  return 0;
}
