//////////////////////////////////////////////////////////////////////////////
//
// LoopBack.c
//
//  This example performs a single channel loopback. The SIO4 board, channel and all
// channel configuration options can be selected by using the GSC SIO4 API calls.
//
#if defined( WIN32 )
#include <windows.h>
#include <stdio.h>
#include <conio.h>
#elif defined( PLX_LINUX )
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#endif
#include "GscApi.h"

#ifdef WIN32
#define usleep(val) Sleep(val / 1000 + 200);
#endif

//
// Global Variables
//
#define TEST_BUFFER_SIZE 0x400	        // Size of test transfer
char RxBuffer[ TEST_BUFFER_SIZE];	// Buffer to hold data to transmit
char TxBuffer[ TEST_BUFFER_SIZE];	// Buffer to hold received data

int boardNumber = 1;
int channel = 1;
int rxch = 2;
int errorPasses = 0;
int totalPasses = 0;
int transferSize = TEST_BUFFER_SIZE;

//
// Forward declarations of local functions
//
int  LoopbackTest(int boardNumber, int channel);

//-------------------------------------------------------- DisplayErrorMessage
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long.
//
void DisplayErrorMessage( int errorCode)
{
	// Buffer to hold the error message
	char errorMessage[80];

	// Call the API to interpret the message
	GscGetErrorString( errorCode, errorMessage);

	// Display the error
	printf("\n Error Code: %d, %s\n", errorCode, errorMessage);
}


//------------------------------------------------- DisplayErrorMessageAndWait
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long. The
// routine waits for the user to press a key before
// returning.
//
void DisplayErrorMessageAndWait( int errorCode)
{
	// Display the Error Message
	DisplayErrorMessage( errorCode);

	// Display the prompt
	printf("\n Press <ENTER> to continue\n");

	// Wait for <ENTER>
	getchar();
}


//----------------------------------------------------------------------- main
//
//  Main entry point to the example
//
int main(int argc, char * argv[])
{
  int status = 0x00;			// Result from API calls
  char kbchar;
  GSC_HDLC_CONFIG cfg;

  // Override default board and channel if provided
  if (argc >= 2) boardNumber = atoi(argv[1]);
  if (argc >= 3) channel = atoi(argv[2]);
  if (argc >= 4)
  {
    if (strncmp(argv[3], "0x", 2) == 0)
			sscanf(argv[3], "0x%x", &transferSize);
    else
			transferSize = atoi(argv[3]);

      if (transferSize > TEST_BUFFER_SIZE) transferSize = TEST_BUFFER_SIZE;
	}

  // Display the example header
  printf("\n");
  printf(" General Standards SIO4 Single Channel Loopback Example\n");

  //
  // Initialize the board. This must be done before any channel operations
  // can be performed. The program is aborted if any errors occur.
  //
  status = GscOpen( boardNumber, GSC_API_VERSION);
  if (status)
    {
      DisplayErrorMessageAndWait(status);
      return 1;
    }

  // Reset the desired channel. This puts the channel into a known state.
  status = GscSio4ChannelReset( boardNumber, channel);
  if (status) return status;
  status = GscSio4ChannelReset( boardNumber, rxch);
  if (status) return status;

  //
  // Configure the loopback channel as desired for the test.
  //
  status = GscSio4HdlcGetDefaults( &cfg );
  cfg.packetFraming = GSC_DISABLED;
  cfg.encoding = GSC_ENCODING_NRZI_SPACE;
  cfg.loopbackMode = GSC_LOOP_EXTERNAL;
  status = GscSio4HdlcSetConfig( boardNumber, channel, cfg );

  printf(" The Loopback Test will be performed on Board #%d,"
	 " Channel #%d (%d - 0x%X bytes)\n", boardNumber, channel, transferSize, transferSize);

  kbchar = 'A';
  // Loop through, performing the test, until the user aborts with an 'X'
  do
  {
    // Clear the status flag
    status = 0x00;
    kbchar++;

    // Perform the loopback test on the desired channel
    status = LoopbackTest(boardNumber, channel);

    // If a non-zero status was returned, display the error
    if (status)
		{
			DisplayErrorMessage(status);
		}

	  // Prompt the user
	  printf("\nType X to exit or any other key to repeat the test: ");
	  kbchar = toupper(getchar());
	} while(kbchar!='X');

  //
  // Shut down the board. This must be the last operation performed. It
  // releases the resources that are used by the GSC API and driver.
  //
  GscClose(boardNumber);

  return 0;
}


//
//  This routine performs a single channel loopback (Tx and Rx are connected
// internally) on a channel of the selected board. One buffer worth of data
// is transmitted and received and then both the Tx and Rx buffers are
// compared and partially displayed.
//
int LoopbackTest(int boardNumber, int channel)
{
  int status = 0x00, index, count;

  // These variables are used to track the data transfers
  int txId, rxId;

  // Zero out the Rx buffer
  memset( RxBuffer, 0xa5, sizeof(RxBuffer));

  // Fill the Tx buffer with a pattern
  for (index=0; index<TEST_BUFFER_SIZE; index+=4)
  {
    TxBuffer[index]   = (char)0xaa;
    TxBuffer[index+1] = (char)0x55;
    TxBuffer[index+2] = (char)(index>>10);	// Upper half of index/4
    TxBuffer[index+3] = 19; //(char)(index>>2);	// Lower half of index/4
  }

  // Determine number of bytes to transfer
  count = transferSize;

  // Print the starting message
  printf(" Starting transfer...");

  // Start processing any received data
  if ((status = GscSio4ChannelReceiveData( boardNumber, channel, RxBuffer, count, &rxId)))
    return status;

  usleep(100000); // insure loopback demo success at high bit rates

  // Initiate the transfer of data
  if ((status = GscSio4ChannelTransmitData( boardNumber, channel, TxBuffer, count, &txId)))
    return status;

  // Print a blank line
  printf("\n");

  //
  // Wait up to 7 seconds for all of the data to arrive. The parameter "rxId"
  // informs the routine of the transfer to wait on. This variable was set during
  // the call to GscSio4ReceiveData(...).
  //
  status = GscSio4ChannelWaitForTransfer( boardNumber, channel, 7000, rxId, &index);
  if (status)
    {
      // Since we have an error, "index" will contain the number of bytes that we
      // actually transmitted.
      DisplayErrorMessage( status );
      printf("\n\t%04X bytes out of %04X transferred", index, count);
    }

  // Display the first 128 bytes of the Tx buffer
  printf("\n\tTx: ");
  for (index = 0x0; index < 0x80; index++)
    {
      // Print the next byte
      printf("%02X ", TxBuffer[ index] & 0xff);

      // New line after every 16 bytes
      if ((index % 0x10) == 0x0f) printf("\n\t    ");
    }

  // Display the first 128 bytes of the Rx buffer
  printf("\n\tRx: ");
  for (index = 0x0; index < 0x80; index++)
    {
      // Print the next byte
      printf("%02X ", RxBuffer[ index] & 0xff);

      // New line after every 16 bytes
      if ((index % 0x10) == 0x0f) printf("\n\t    ");
    }

  // Verify that the buffers match
  for (index=0; index<transferSize; index++)
    {
      if (TxBuffer[ index] != RxBuffer[ index])
	{
	  errorPasses++;
	  printf("\n\tData Mismatch at %04X (Tx=%02X, Rx=%02X)\n",
		 index,
		 TxBuffer[ index] & 0xff,
		 RxBuffer[ index] & 0xff);
	  break;
	}
    }

  GscSio4ChannelFifoCounts( boardNumber, channel, &status);
  printf("Fifo Counts: %08X\n", status);
  printf("Total Errors: %d out of %d passes\n", errorPasses, ++totalPasses);
  return 0x00;
}


