//-------------------------------------------------------------------------------------------------
//
// File Name:
//
//     GscAnaDSI.cpp
//
// Description:
//
//     This file contains the implementation of the GSC DSI analog board class. This class
//     implements the fundamental, underlying board-specific functionality used by the GscApi
//     analog board C library API.
//
// Programmer's notes:
//
//     06-06-2008:  Created file. Only  24dsi12 supported and tested. --mrg
//
//-------------------------------------------------------------------------------------------------

#include "math.h"
#include "time.h"
#include "GscAnaDSI.h"



//*-------------------------------------------------------------------GscAna24DSI12----------------
//*
//* Description:
//*
//*    Default constructor. Not used in GscApi library.
//*
//* Parameters:
//*
//*    Parameter                    Description
//*
//* Return Value:
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

GscAna24DSI12::GscAna24DSI12()
{
  mBoardNo = 0;
  mValidHw = false;
}



//*-------------------------------------------------------------------GscAna24DSI12---------------
//*
//* Description:
//*
//*    Constructor for DSI class.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     BoardNumber                   PlxApi board number of analog board.
//*
//* Return Value:
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

GscAna24DSI12::GscAna24DSI12(int BoardNumber)
{
  int status = 0;
  int cfg = 0;
  int i = 0;
  int bn = BoardNumber - 1;

  mValidHw = true;

  // Make sure the board type is as expected
  if (boardInformation[bn].boardType != GSC_BOARD_TYPE_ANALOG)
    mValidHw = false;

  // Make sure the board number is valid
  status = validateBoardNumber( bn );
  if (status)
    mValidHw = false;

  mBoardNo = BoardNumber;

  // determine the number of channels present
  status = fpgaRead( mBoardNo, REGOFFSET_FWR, &cfg );
  cfg &= 0x00030000;
  switch(cfg)
  {
  case 0x00010000: mNumChannelsPresent = 8; break;
  case 0x00020000: mNumChannelsPresent = 4; break;
  default:         mNumChannelsPresent = 12;
  }

  // determine whether legacy rate generators are present.
  mLegacyRateGen = false;
  status = fpgaRead( mBoardNo, REGOFFSET_FWR, &cfg );
  if ((cfg & 0x00008000) == 0)
    mLegacyRateGen = true;

  // GscApi config structure defaults
  mInputTypeMode    = GSC_ANA_NORMAL;
  mInputType        = GSC_ANA_SINGLE_ENDED;
  mInputRange       = GSC_ANA_BIP_10_0;
  mDataFormat       = GSC_ANA_OFFSET_BINARY;
  mExternalSync     = GSC_DISABLED;
  mDataWidth        = GSC_ANA_DATAWIDTH_16;
  mScanMarkingMode  = GSC_DISABLED;
  mScanMarkingValue = 0x00000000;
  mSampleClockSource = GSC_ANA_CLOCK_INTERNAL;
  mMasterClockHz    = 25600000;
  mSamplingHz       = 0;

  // Available DSI interrupts
  analogIntrInfo[0].intrType = GSC_INTR_ANA_AUTOCALCOMPLETED;
  analogIntrInfo[0].isrMask = 0x00000100; //auto-calibration completed.
  analogIntrInfo[1].intrType = GSC_INTR_ANA_CHANNELSREADY;
  analogIntrInfo[1].isrMask = 0x00000200; // channels ready.
  analogIntrInfo[2].intrType = GSC_INTR_ANA_THRESHOLDLOWTOHIGH;
  analogIntrInfo[2].isrMask = 0x00000300; // buffer threshold low ==> high transition
  analogIntrInfo[3].intrType = GSC_INTR_ANA_THRESHOLDHIGHTOLOW;
  analogIntrInfo[3].isrMask = 0x00000400; // buffer threshold high ==> low transition
  //analogIntrInfo[] =
  //{	// ISR Mask
  //  {GSC_INTR_ANA_AUTOCALCOMPLETED,     0x00000100},	// auto-calibration completed.
  //  {GSC_INTR_ANA_THRESHOLDLOWTOHIGH,   0x00000300},	// buffer threshold low ==> high transition
  //  {GSC_INTR_ANA_THRESHOLDHIGHTOLOW,   0x00000400}   // buffer threshold high ==> low transition
  //};

  // Initialize interrupt callbacks to NULL
  for (i=0; i < 4; i++)
    AnalogInterruptRoutines[mBoardNo][i] = NULL;
}



//*------------------------------------------------------------------~GscAna24DSI12----------------
//*
//* Description:
//*
//*    Destructor for DSI class.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*
//* Return Value:
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

GscAna24DSI12::~GscAna24DSI12()
{
  mBoardNo = 0;
}



//*-------------------------------------------------------------------GetConfigDefaults------------
//*
//* Description:
//*
//*     Retrieves DSI board defaults for the fields of an ANALOG_CONFIG struct.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     config                        Pointer to a GSC_ANALOG_CONFIG structure.
//*
//* Return Value:
//*     0 - success
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::GetConfigDefaults( PGSC_ANALOG_CONFIG config )
{
  int status = 0x00;

  // Load the defaults
  config->inputMode = GSC_ANA_NORMAL;
  config->inputType = GSC_ANA_SINGLE_ENDED;
  config->inputRange = GSC_ANA_BIP_10_0;
  config->dataFormat = GSC_ANA_OFFSET_BINARY;
  config->externalSync = GSC_DISABLED;
  config->dataWidth = GSC_ANA_DATAWIDTH_16;
  config->scanMarkingMode = GSC_DISABLED;
  config->scanMarkingValue = 0x00000000;
  config->sampleClockSource = GSC_ANA_CLOCK_INTERNAL;
  config->masterClockFrequency = 32768000;
  return status;
}



//*-------------------------------------------------------------------GetConfig--------------------
//*
//* Description:
//*
//*     Retrieves current config values for an DSI board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     config                        Pointer to a GSC_ANALOG_CONFIG structure.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::GetConfig(PGSC_ANALOG_CONFIG config)
{
  int status = 0x00;

  // Get the desired options
  GetOption( GSC_ANA_INPUT_MODE, &config->inputMode);
  GetOption( GSC_ANA_INPUT_TYPE, &config->inputType);
  GetOption( GSC_ANA_INPUT_RANGE, &config->inputRange);
  GetOption( GSC_ANA_DATA_FORMAT, &config->dataFormat);
  GetOption( GSC_ANA_EXTERNAL_SYNC, &config->externalSync);
  GetOption( GSC_ANA_DATA_WIDTH, &config->dataWidth);
  GetOption( GSC_ANA_SCAN_MARKING, &config->scanMarkingMode);
  GetOption( GSC_ANA_SCAN_MARKING_VALUE, &config->scanMarkingValue);
  GetOption( GSC_ANA_CLOCK_SOURCE, &config->sampleClockSource);
  GetOption( GSC_ANA_MASTER_CLOCK, &config->masterClockFrequency);

  return status;
}



//*-------------------------------------------------------------------SetConfig--------------------
//*
//* Description:
//*
//*     Sets the current config values for an DSI board to the values in the supplied
//*     GSC_ANALOG_CONFIG struct parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     config                        A GSC_ANALOG_CONFIG struct containing the desired config
//*                                   values.
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::SetConfig( GSC_ANALOG_CONFIG config )
{
  int status = 0x00;

  // Set the desired options
  SetOption( GSC_ANA_INPUT_MODE, config.inputMode);
  SetOption( GSC_ANA_INPUT_TYPE, config.inputType);
  SetOption( GSC_ANA_INPUT_RANGE, config.inputRange);
  SetOption( GSC_ANA_DATA_FORMAT, config.dataFormat);
  SetOption( GSC_ANA_EXTERNAL_SYNC, config.externalSync);
  SetOption( GSC_ANA_DATA_WIDTH, config.dataWidth);
  SetOption( GSC_ANA_SCAN_MARKING, config.scanMarkingMode);
  SetOption( GSC_ANA_SCAN_MARKING_VALUE, config.scanMarkingValue);
  SetOption( GSC_ANA_CLOCK_SOURCE, config.sampleClockSource);
  SetOption( GSC_ANA_MASTER_CLOCK, config.masterClockFrequency);
  return status;
}



//*-------------------------------------------------------------------GetOption--------------------
//*
//* Description:
//*
//*     Retrieves the value of the DSI board option specified by the optionName parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     optionName                    An enum GSC_OPTION_NAME value that specifies the board option
//*                                   to retrieve.
//*     optionValue[]                 An array of ints to hold the current option value. All
//*                                   analog board options are composed of a single int value, but
//*                                   this method supports options that require several int values
//*                                   for their representation.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::GetOption( enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status = 0x00;
  int oldValue  = 0;
  int newValue  = 0;
  int bn = 0;

  bn = mBoardNo - 1;

  // TODO:
  //    Potential new common options - list compiled.
  //

  switch ( optionName )
  {
    //
    // Get the Input Mode
    //
  case GSC_ANA_INPUT_MODE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000007;

    // Decode the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ANA_NORMAL; break;
    case 0x03: optionValue[0] = GSC_ANA_VREF;   break;
    case 0x02: optionValue[0] = GSC_ANA_ZERO;   break;
    default:   optionValue[0] = GSC_ANA_NORMAL; break;
    }
    break;


    //
    // Get the input range
    //
  case GSC_ANA_INPUT_RANGE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x0000000c;
    // Process the desired option
    switch (newValue)
    {
    case 0x00:
    case 0x01: optionValue[0] = GSC_ANA_BIP_2_5;  break;
    case 0x02: optionValue[0] = GSC_ANA_BIP_5_0; break;
    case 0x03: optionValue[0] = GSC_ANA_BIP_10_0;  break;
    default:   optionValue[0] = GSC_ANA_BIP_10_0; break;
    }

    break;


    //
    // Get the data format
    //
  case GSC_ANA_DATA_FORMAT:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000010;

    // Process the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ANA_TWOS_COMPLEMENT; break;
    case 0x10: optionValue[0] = GSC_ANA_OFFSET_BINARY;   break;
    default:   optionValue[0] = GSC_ANA_TWOS_COMPLEMENT; break;
    }
    break;


    //
    // Get the external sync
    //
  case GSC_ANA_EXTERNAL_SYNC:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000020;

    // Process the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ENABLED; break;
    case 0x20: optionValue[0] = GSC_DISABLED;  break;
    default:   optionValue[0] = GSC_DISABLED; break;
    }
    break;


    //
    // Get the data width
    //
  case GSC_ANA_DATA_WIDTH:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00300000;

    // Process the desired option
    switch (newValue)
    {
    case 0x00000000: optionValue[0] = GSC_ANA_DATAWIDTH_16;  break;	// 16 bits
    case 0x00100000: optionValue[0] = GSC_ANA_DATAWIDTH_18; break;	// 18 bits
    case 0x00200000: optionValue[0] = GSC_ANA_DATAWIDTH_20;  break;	// 20 bits
    case 0x00300000: optionValue[0] = GSC_ANA_DATAWIDTH_24;  break;	// 24 bits
    default:   optionValue[0] = GSC_ANA_DATAWIDTH_16;  break;
    }

    //
    // Get the clock source
    //
  case GSC_ANA_CLOCK_SOURCE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_RAR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x0000000f;

    // Process the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ANA_CLOCK_SECONDARY;  break;	  // Rate-a
    case 0x01: optionValue[0] = GSC_ANA_CLOCK_INTERNAL;   break;	  // Rate-b
    case 0x05: optionValue[0] = GSC_ANA_CLOCK_EXTERNAL;   break;	  // Ext sync
    default:   optionValue[0] = GSC_ANA_CLOCK_SECONDARY;  break;
    }
    break;

    //
    // Get the Master Clock Frequency
    //
  case GSC_ANA_MASTER_CLOCK:
    // Get the frequency
    optionValue[0] = mMasterClockHz;
    break;

    //
    // Get the number of active (being scanned) channels
    //
  case GSC_ANA_NUMACTIVECHANNELS:
    // Get the frequency
    optionValue[0] = mNumChannelsPresent;
    break;

    //
    // Get the number of channels present on the board
    //
  case GSC_ANA_MAXCHANNELS:
    // Get the frequency
    optionValue[0] = mNumChannelsPresent;
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------SetOption--------------------
//*
//* Description:
//*
//*     Retrieves the value of the DSI board option specified by the optionName parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     optionName                    An enum GSC_OPTION_NAME value that specifies the board option
//*                                   to set.
//*     optionValue                   The desired value for the specified option.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::SetOption( enum GSC_OPTION_NAME optionName, int optionValue)
{
  int status = 0x00;
  int oldValue = 0;
  int regValue = 0;
  int newValue  = 0;
  int newExtValue = 0;

  switch ( optionName )
  {
    //
    // Set the Input Mode
    //
  case GSC_ANA_INPUT_MODE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000007;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_NORMAL: newValue |= 0x00000000; break;
    case GSC_ANA_VREF:	 newValue |= 0x00000003; break;
    case GSC_ANA_ZERO:	 newValue |= 0x00000002; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mInputTypeMode = optionValue;

    break;


    //
    // Set the input range
    //
  case GSC_ANA_INPUT_RANGE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x0000000c;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_BIP_2_5:	 newValue |= 0x00000004; break;
    case GSC_ANA_BIP_5_0:	 newValue |= 0x00000008; break;
    case GSC_ANA_BIP_10_0: newValue |= 0x0000000c; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mInputRange = optionValue;

    break;


    //
    // Set the data format
    //
  case GSC_ANA_DATA_FORMAT:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000010;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_TWOS_COMPLEMENT: newValue |= 0x00000000; break;
    case GSC_ANA_OFFSET_BINARY: newValue |= 0x00000010;   break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mDataFormat = optionValue;

    break;


    //
    // Set the data width
    //
  case GSC_ANA_DATA_WIDTH:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_IBC, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00300000;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_DATAWIDTH_16: newValue |= 0x00000000;  break;	// 16 bits
    case GSC_ANA_DATAWIDTH_18: newValue |= 0x00000001; break;	// 18 bits
    case GSC_ANA_DATAWIDTH_20: newValue |= 0x00000002;  break;	// 20 bits
    case GSC_ANA_DATAWIDTH_24: newValue |= 0x00000003;  break;	// 24 bits
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_IBC, newValue);
    if (status) return status;

    mDataWidth = optionValue;

    break;

    //
    // Set the clock source
    //
  case GSC_ANA_CLOCK_SOURCE:
    // Get the current rate assignments value
    status = fpgaRead( mBoardNo, REGOFFSET_RAR, &oldValue);
    if (status) return status;
    // Get the current initiator bit value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regValue);
    if (status) return status;

    // Mask off clock assignment bits
    newValue = oldValue & ~0x0000000f;

    // Mask off initiator bit
    newExtValue = regValue & ~0x00000020;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_CLOCK_SECONDARY:
      newValue |= 0x00000000;  newExtValue |= 0x20;   break;	// Rate-a, both groups
    case GSC_ANA_CLOCK_INTERNAL:
      newValue |= 0x00000011;  newExtValue |= 0x20;   break;	// Rate-b, both groups
    case GSC_ANA_CLOCK_EXTERNAL:
      newValue |= 0x00000055;  break;	// Ext sync as target, both groups
    default: return GscApiInvalidParameter;
    }

    // Write the new rate assignment value
    status = fpgaWrite( mBoardNo, REGOFFSET_RAR, newValue);
    if (status) return status;

    // Write the new external sync bit value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newExtValue);
    if (status) return status;

    mSampleClockSource = optionValue;

    break;

    //
    // Set the Master Clock Frequency
    //
  case GSC_ANA_MASTER_CLOCK:
    // Save the frequency
    mMasterClockHz = optionValue;
    break;

    //
    // Set the sample rate
    //
  case GSC_ANA_SAMPLE_RATE:
    // Save the frequency
    status = SetSampleRate( optionValue );
    if (status == 0)
      mSamplingHz = optionValue;
    break;

    //
    // Enable/disable the rate generators
    //
  case GSC_ANA_RATEGEN:
    switch (optionValue)
    {
    case GSC_ENABLED:
      fpgaRead( mBoardNo, REGOFFSET_RAR, &oldValue );
      status = fpgaWrite( mBoardNo, REGOFFSET_RAR, oldValue & 0xFFFFFF00); // Enable both rate generators
      break;
    case GSC_DISABLED:
      fpgaRead( mBoardNo, REGOFFSET_RAR, &oldValue ); // Disable both rate generators
      status = fpgaWrite( mBoardNo, REGOFFSET_RAR, (oldValue & 0xFFFFFF00) | 0x77);
      break;
    default:
      return GscApiInvalidParameter;
    }
    break;

    //
    // Enable/disable the input buffer
    //
  case GSC_ANA_INPUTBUFFER:

    fpgaRead( mBoardNo, REGOFFSET_IBC, &oldValue );
    switch (optionValue)
    {
    case GSC_ENABLED:
      status = fpgaWrite( mBoardNo, REGOFFSET_IBC, oldValue & 0xFFF3FFFF);
      break;
    case GSC_DISABLED:
      status = fpgaWrite( mBoardNo, REGOFFSET_IBC, oldValue | 0x40000);
      break;
    default:
      return GscApiInvalidParameter;
    }
    break;

    //
    // Set the clear buffer on sync bit
    //
  case GSC_ANA_CLEARBUFFER_ON_SYNC:

    fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue );
    switch (optionValue)
    {
    case GSC_ENABLED:
      status = fpgaWrite( mBoardNo, REGOFFSET_BCR, oldValue | 0x20000);
      break;
    case GSC_DISABLED:
      status = fpgaWrite( mBoardNo, REGOFFSET_BCR, oldValue & 0xFFFDFFFF);
      break;
    default:
      return GscApiInvalidParameter;
    }
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------LocalRegisterRead------------
//*
//* Description:
//*
//*     Retrieves the value of an DSI board local register.
//*
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     reg                           An DSI board local register offset.
//*     result                        The current value contained in the specified local register.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*     The GscApi API defines "symbolic" rather than actual local register offsets for all the
//*     GSC ADC boards due to the significant variation in register offsets across the boards. The
//*     register offset supplied to this method when called by the GscApi analog board C functions
//*     is therefore such a "symbolic" value. This class method recognizes the "symbolic" register
//*     offsets and uses corresponding actual register offset constants defined as part of the
//*     class.
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::LocalRegisterRead( int reg, int *result )
{
  int status = 0;

  //read result
  *result = 0;
  switch (reg)
  {
  case GSC_ANALOG_BCR:
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, result );
    break;
  case GSC_ANALOG_RAG:
    status = fpgaRead( mBoardNo, REGOFFSET_RAC, result );
    break;
  case GSC_ANALOG_RBG:
    status = fpgaRead( mBoardNo, REGOFFSET_RBC, result );
    break;
  case GSC_ANALOG_RAR:
    status = fpgaRead( mBoardNo, REGOFFSET_RAR, result );
    break;
  case GSC_ANALOG_DIV:
    status = fpgaRead( mBoardNo, REGOFFSET_DIV, result );
    break;
  case GSC_ANALOG_BS:
    status = fpgaRead( mBoardNo, REGOFFSET_IBS, result );
    break;
  case GSC_ANALOG_IBC:
    status = fpgaRead( mBoardNo, REGOFFSET_IBC, result );
    break;
  case GSC_ANALOG_FWR:
    status = fpgaRead( mBoardNo, REGOFFSET_FWR, result );
    break;
  case GSC_ANALOG_AV:
    status = fpgaRead( mBoardNo, REGOFFSET_ACV, result );
    break;
  case GSC_ANALOG_IDB:
    status = fpgaRead( mBoardNo, REGOFFSET_IDB, result );
    break;
  default:
    if (!(reg & 0x8000)) // no constant defined in GscApi.h, go ahead and read from register at offset
      fpgaRead( mBoardNo, reg, result );
    else
      return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------LocalRegisterWrite-----------
//*
//* Description:
//*
//*     Sets the value of a DSI board local register.
//*
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     reg                           An DSI board local register offset.
//*     value                         The desired value for the specified local register.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*     See notes for LocalRegisterRead() method for explanation of "symbolic" register offsets.
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::LocalRegisterWrite( int reg, int value )
{
  int status = 0;

  // write value
  switch (reg)
  {
  case GSC_ANALOG_BCR:
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, value );
    break;
  case GSC_ANALOG_RAG:
    status = fpgaWrite( mBoardNo, REGOFFSET_RAC, value );
    break;
  case GSC_ANALOG_RBG:
    status = fpgaWrite( mBoardNo, REGOFFSET_RBC, value );
    break;
  case GSC_ANALOG_RAR:
    status = fpgaWrite( mBoardNo, REGOFFSET_RAR, value );
    break;
  case GSC_ANALOG_DIV:
    status = fpgaWrite( mBoardNo, REGOFFSET_DIV, value );
    break;
  case GSC_ANALOG_IBC:
    status = fpgaWrite( mBoardNo, REGOFFSET_IBC, value );
    break;
  default:
    if (!(reg & 0x8000)) // no constant defined in GscApi.h, go ahead and write register at offset
      fpgaWrite( mBoardNo, reg, value );
    else
      return GscApiInvalidParameter;
  }

  return status;
}

//*-------------------------------------------------------------------AutoCal----------------------
//*
//* Description:
//*
//*     Initiates auto-calibration of the DSI board and waits for it to complete by polling the
//*     AUTOCAL bit of the BCR register.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 GscApiAutoCalFailed, or a Plx error, or a GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::AutoCal( bool bWaitForCompletion )
{
  int status = 0x00;
  int tempValue;

  // Set the autocal bit
  status = fpgaRead( mBoardNo, REGOFFSET_BCR, &tempValue);
  if (status) return status;
  tempValue |= 0x00000080;
  status = fpgaWrite( mBoardNo, REGOFFSET_BCR, tempValue);
  if (status) return status;

  // Loop until bit goes clear
  do
  {
    Sleep(100);
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &tempValue);
    if (status) return status;
  } while (bWaitForCompletion && (tempValue & 0x00000080));

  // If autocal failed, return error
  if ((tempValue & 0x00001000) == 0) return GscApiAutoCalFailed;

  return status;
}



//*-------------------------------------------------------------------SetSampleRate----------------
//*
//* Description:
//*
//*     Sets the DSI board rate generators to sample data at the requested sample rate. At this
//*     time only rate generator A is used (rate A is assigned to both channel groups on the DSI
//*     board. The algorithm used is described in section 3.6 of the 24DSI12 reference manual:
//*     The requested sample rate is converted to prime factors and simplified into an initial
//*     Mvco/Nref ratio. A divisor is then found that adjusts the ratio as close as possible to 1.
//*     Finally the Nvco and Nref values are scaled to in-range values.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     sampleHz                      The destired sample rate in Hz.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::SetSampleRate(int sampleHz)
{
  int status = 0;
  int i = 0;
  int divA = 0;
  int nVco = 0;
  int nRef = 0;
  int tmpNum = 0;
  int tmpDen = 1;
  int exp = 0;
  int exp2 = 0, exp3 = 0, exp5 = 0;
  int prime = 0;
  double dRatio = 0.0;
  double dTmpRatio = 0.0;
  double dSampleDiff = 0.0;
  double dLastDiff = 0.0;
  double dMinDiff = (double) sampleHz;
  double dDiv = 0.0;
  double dTmpVco = 0.0;
  int Nrate = 0;
  int iRateCtrl = 0;
  int regval = 0;

  // validate requested sample rate
  if (sampleHz > MAX_SAMPLE_CLOCK) return GscApiInvalidParameter;
  if (sampleHz < MIN_SAMPLE_CLOCK) return GscApiInvalidParameter;

  // set up clock speed and divisor - working with equation 3-3 in the board manual.
  if (mLegacyRateGen == false)
  {
    // find prime factors <= 5 for (requested sample rate * 512) and cancel
    // common prime factors between this number and the board reference frequency
    tmpNum = (int)(sampleHz * 512);
    prime = 2;
    while (prime <= 5)
    {
      exp = 0;
      while((!(tmpNum % prime)) && (tmpNum != 1))
      {
        tmpNum /= prime;
        exp++;
      }
      // cancel common factors with master clock freq (2^18 * 5^5 = 32.768 Mhz)
      switch (prime)
      {
      case 2:
        exp2 = exp - 18;
        break;
      case 3:
        exp3 = exp;
      case 5:
        exp5 = exp - 3;
        break;
      default:
        break;
      }
      if (prime < 3) prime++;
      else prime += 2;
    }

    // compute numerator and denominator with cancelled prime factors
    if (exp2 > 0)
      tmpNum *= 1 << exp2;
    else
      tmpDen = 1 << abs(exp2);
    tmpNum *= (int) pow(3.0, exp3);
    if (exp5 > 0)
      tmpNum *= (int)pow(5.0, exp5);
    else
      tmpDen *= (int)pow(5.0, abs(exp5));


    // find rate-a divisor

    // Get the ratio within acceptable range
    dTmpRatio = dRatio = (double)tmpNum / tmpDen;
    divA = -1;
    while ((dTmpRatio < 0.78 || dTmpRatio > 1.5625) && divA <= 25)
    {
      if (++divA == 0)
        dTmpRatio = dRatio * 0.5;
      else
        dTmpRatio = dRatio * divA;
    }

    // Find divisor that yields ratio closest to unity
    dLastDiff = 1 - dTmpRatio;
    if (divA == -1)
    {
      // dRatio was in range without divisor, so assure refinement starts at
      // appropriate divisor, either 0 or 1.
      divA = 1;
      if (dRatio * 0.5 >= 0.78)
      {
        divA = 0;
        dTmpRatio = dRatio * 0.5;
        dLastDiff = 1 - dTmpRatio;
      }
    }
    while (fabs(1 - dTmpRatio) <= fabs(dLastDiff) && divA <= 25)
    {
      dLastDiff = 1 - dTmpRatio;
      if (++divA == 0)
        dTmpRatio = dRatio * 0.5;
      else
        dTmpRatio = dRatio * divA;
    }
    divA--;

    // find Nvco and Nref
    dTmpVco = (divA == 0) ? tmpNum / 2.0 : tmpNum * divA;
    nRef = tmpDen;
    i = 1;
    while (dTmpVco * i < 30 || nRef * i < 30)
      i++;
    // make sure we don't lose fractional values introduced by divA == 0
    if ((dTmpVco - ((int)dTmpVco) != 0.0) && ((i % 2) != 0)) i++;
    nVco = (int) (dTmpVco * i);
    nRef *= i;


    // set rate divisor register for both channel groups
    status = fpgaRead( mBoardNo, REGOFFSET_DIV, &regval );
    if (status) return status;
    status = fpgaWrite( mBoardNo, REGOFFSET_DIV, (regval & 0xffff0000) | (divA << 8) | divA);
    if (status) return status;

    // set rate control register
    status = fpgaRead( mBoardNo, REGOFFSET_RAC, &regval );
    if (status) return status;
    status = fpgaWrite( mBoardNo, REGOFFSET_RAC, (regval & 0xf800fc00) | (nRef << 16) | nVco);
    if (status) return status;

    // Make the Rate-A generator the source for both channel group 0 and 1
    status = SetOption( GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_SECONDARY);
    if (status) return status;

    // Give the clocks time to stablize
    Sleep(10);

  }
  else
  {
    // Legacy rate generators present

    // try all possible DIVISOR values
    for (i=25; i >= 0; i--)
    {
      //solve for Nrate given DIVISOR = i
      dDiv = i == 0 ? 0.5 : i;
      iRateCtrl = (int)floor(((((sampleHz * 512 * dDiv) / 25.6) - 1) * 100000) + 0.5);

      // keep track of smallest delta produced by Nrate, DIV pair.
      if ((dSampleDiff = fabs(sampleHz - (25.6 * (1 - Nrate / 100000) / (512 * dDiv)))) < dMinDiff)
      {
        dMinDiff = dSampleDiff;
        divA = i;
        Nrate = iRateCtrl;
      }
    }

    // set rate divisor register for both channel groups
    status = fpgaRead( mBoardNo, REGOFFSET_DIV, &regval );
    if (status) return status;
    status = fpgaWrite( mBoardNo, REGOFFSET_DIV, (regval & 0xffff0000) | (divA << 8) | divA);
    if (status) return status;

    // set rate control register
    status = fpgaRead( mBoardNo, REGOFFSET_RAC, &regval );
    if (status) return status;
    status = fpgaWrite( mBoardNo, REGOFFSET_RAC, (regval & 0xffff0000) | Nrate);
    if (status) return status;

    // Make the Rate-A generator the source for both channel group 0 and 1
    status = SetOption( GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_SECONDARY);
    if (status) return status;

    // Give the clocks time to stablize
    Sleep(10);

  }

  mSamplingHz = sampleHz;

  boardInformation[mBoardNo - 1].bytesPerMs[0] = (mSamplingHz * 4 * mNumChannelsPresent) / 1000;

  return 0;
}

//*-------------------------------------------------------------------ScanInputs-------------------
//*
//* Description:
//*
//*     Scans and stores ADC sample data for the requested channels.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     samples                       The number of samples to scan (for each channel).
//*     samplesPerSecond              The sample rate in Hz.
//*     firstChannel                  The first channel to scan, zero-based.
//*     lastChannel                   The last channel to scan, zero-based.
//*     buffer                        The buffer in which the sample data is stored.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*     This function ignores the firstChannel and lastChannel parameters.
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::ScanInputs( int samples,
                               double samplesPerSecond,
                               int firstChannel, int lastChannel, char *buffer)
{
  int status = 0;
  int numberOfBytes;
  int regval;
  int rxId, bytesAquired;
  //int ulBuffSize = 0;
  //int ulIbc = 0;
  //clock_t tnow;
  //int i = 0;

  // TODO:
  //   - maybe add timeout parameter
  //   - disable clock before returning? (rather than in rxdmahandler())


  // Make sure # of samples is > 0
  if (samples <= 0) return GscApiInvalidParameter;

  // Verify samples/second value
  if ((samples != 1) && (samplesPerSecond == 0)) return GscApiInvalidParameter;

  // Setup the "samples per millisecond" variable for the dma handler
  if (samples == 1) boardInformation[ mBoardNo -1].bytesPerMs[ 0] = mNumChannelsPresent;
  else boardInformation[ mBoardNo - 1].bytesPerMs[ 0] = mNumChannelsPresent * samplesPerSecond / 1000.0;

  // Determine the number of bytes to acquire.
  // (For this board, the channel sample values are always padded to 32 bits,
  //  regardless of data width in use, so the byte total is easily tallied)
  numberOfBytes = mNumChannelsPresent * samples * 4;

  // Make sure we are not acquiring data by disabling the input buffer
  status = fpgaRead( mBoardNo, REGOFFSET_IBC, &regval);
  if (status) return status;
  regval |= 0x00040000;
  status = fpgaWrite( mBoardNo, REGOFFSET_IBC, regval);
  if (status) return status;

  // set up clock speed and divisor - working with equation 3-3 in the board manual.
  if ((status = SetSampleRate( (int)samplesPerSecond )))
    return status;

  // Clear the fifo
  status = ClearInputBuffer();
  if (status) return status;
  //fpgaRead( mBoardNo, REGOFFSET_IBS, &ulBuffSize );

  // initiate auto calibration
  if ((status = GscAnaAutoCal( mBoardNo, true )))
    return status;

  // enable the fifo data input buffer
  status = fpgaRead( mBoardNo, REGOFFSET_IBC, &regval);
  if (status) return status;
  status = fpgaWrite( mBoardNo, REGOFFSET_IBC, regval & 0xFFFBFFFF);
  if (status) return status;

  /*
    tnow = clock();
    for (i=1; i <= 20; i++)
    {
    Sleep(50);
    fpgaRead( mBoardNo, REGOFFSET_IBS, &ulBuffSize );
    fpgaRead( mBoardNo, GSC_ANALOG_BCR, &ulIbc ); // Threshold
    //printf("buffer size after %f secs = %d.\n", ((double)(clock() - tnow)) / CLOCKS_PER_SEC,
    //       ulBuffSize);
    //printf("threshold flag after %f secs = 0x%05x.\n", i * 0.2, ulIbc);
    if (ulIbc & 0x4000) // threshold flag HIGH
      break;
    }
  */

  // Setup to receive the data
  status = gscAddDmaRequest( mBoardNo, 1, buffer, numberOfBytes, GSC_DATA_RECEIVE, &rxId);
  if (status) return status;

  // Wait for it to happen
  status = GscSio4ChannelWaitForTransfer( mBoardNo, 1, 2000, rxId, &bytesAquired);

  if (status) return status;

  // Clear the input buffer
  status = ClearInputBuffer();

  return status;
}



//*-------------------------------------------------------------------ReceiveScan------------------
//*
//* Description:
//*
//*     Performs a DMA operation to move the contents of the DSI input data buffer to the buffer
//*     parameter supplied to the method.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     char    *buffer               The buffer in which the sample data is stored.
//*     int     *count                The number of bytes stored in buffer.
//*     int     *transferStatus       Not currently used.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::ReceiveScan( char *buffer, int *count, int *transferStatus)
{
  int status = 0x00;
  int fifoCount;

  // Assume no data is available
  *count = 0;
  if (transferStatus != NULL) *transferStatus = 0;

  // Make sure the data is still in the Rx FIFO
  status = GetInputBufferSize( &fifoCount );
  if (status) return status;
  if (fifoCount < *count) return GscApiFifoOverrun;

  // Get the data using DMA
  if (drvMajorVersion >= 5)
    status = gscPerformDma( mBoardNo, 1, buffer, fifoCount, GSC_DATA_RECEIVE, 0);
  else
    status = gscPerformDma( mBoardNo, 1, buffer, fifoCount, GSC_DATA_RECEIVE, PrimaryPciChannel0);

  *count = fifoCount;

  return status;
}



//*-------------------------------------------------------------------RegisterInterrupt------------
//*
//* Description:
//*
//*     Registers a callback function for an interrupt. The function is invoked by the GscApi
//*     interrupt handler thread when the interrupt fires.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int               interrupt   The interrupt to be caught.
//*     GSC_CB_FUNCTION   *function   The callback invoked when the interrupt happens.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::RegisterInterrupt( int interrupt, GSC_CB_FUNCTION *function )
{
  int i, status = 0x00, offset = 0;
  int enableMask = 0x00;
  int disableMask = 0x00;

  // Set the callback for the interrupt that is being registered.
  for (i=0; i < 4; i++)
  {
    // Is the bit set
    if (interrupt == analogIntrInfo[ i].intrType)
    {
      // Save the callback routine
      AnalogInterruptRoutines[ mBoardNo-1][i] = function;

      // If the callback is null, update the disable mask
      if (function == NULL) disableMask |= analogIntrInfo[i].isrMask;
      else enableMask |= analogIntrInfo[i].isrMask;
    }
  }

  // if interrupt not supported, return appropriate error code.
  if (((enableMask == 0) && (function != NULL)) ||
      ((disableMask == 0) && (function == NULL)))
    return GscApiInvalidParameter;

  //  EnterCriticalSection(&PlxApiCalls);
  // Get the current interrupt mask
  status = fpgaRead( mBoardNo, REGOFFSET_BCR, &offset);
  if (status) return status;

  // Clear any bits that need disabling
  offset &= (~disableMask);

  // Set any bits that need enabling
  offset |= enableMask;
  //printf("RegisterInterrupt: new regval = 0x%x\n", offset);

  // Write the value back out
  status = fpgaWrite( mBoardNo, REGOFFSET_BCR, offset);
  // LeaveCriticalSection(&PlxApiCalls);


  return status;
}



//*-------------------------------------------------------------------ClearInterrupts--------------
//*
//* Description:
//*
//*     Clears fpga (local) interrupts on the DSI board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::ClearInterrupts()
{
  int status = 0;
  int regval = 0;

  // Clear all pending interrupts
  status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regval );
  status = fpgaWrite( mBoardNo, REGOFFSET_BCR, regval & 0xFFFFF7FF );

  return status;
}



//*-------------------------------------------------------------------InvokeInterruptCallback------
//*
//* Description:
//*
//*     Invokes the callback registered for an interrupt. The interrupt is cleared after the
//*     callback returns.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::InvokeInterruptCallback()
{
  int index = 0;
  int status = 0;
  int interruptMask = 0;

  // mask the interrupt
  status = fpgaRead( mBoardNo, REGOFFSET_BCR, &interruptMask );
  if (status) return status;

  if (((interruptMask & 0x00000700) == 0x00) || (interruptMask & 0x00000800) == 0) return 0;

  // handle interrupts for analog boards
  for(index=0; index < 4; index++)
  {
    // Break out of loop if no bits set
    if ((interruptMask & 0x00000700) == 0x00) break;

    // See if the interrupt is turned on
    if (interruptMask & analogIntrInfo[index].isrMask)
    {
      // make sure the interrupt request is for the masked interrupt
      if ((interruptMask & analogIntrInfo[index].isrMask) == analogIntrInfo[index].isrMask)
      {
        //printf("Interrupt mask = 0x%x\n", interruptMask);
        // If the routine is defined, call it
        if (AnalogInterruptRoutines[mBoardNo - 1][index] != NULL)
        {
          AnalogInterruptRoutines[mBoardNo - 1][index]( mBoardNo, 0, analogIntrInfo[index].intrType);
        }
        status = fpgaWrite( mBoardNo, REGOFFSET_BCR, interruptMask & 0xfffff7ff); // clear it
      }
    }
  }
  status = fpgaWrite( mBoardNo, REGOFFSET_BCR, interruptMask & 0xfffff7ff); // clear it
  return status;
}



//*-------------------------------------------------------------------GetInputBufferSize-----------
//*
//* Description:
//*
//*     Returns the number of bytes in the DSI board's input data buffer.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int    *FifoByteCount         The number of bytes in the input data buffer.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::GetInputBufferSize(int *FifoByteCount)
{
  int status = 0;
  int fifoCountADC = 0;

  // insure size returned is zero on error
  *FifoByteCount = 0;

  // read the buffer size register
  status = fpgaRead( mBoardNo, REGOFFSET_IBS, &fifoCountADC );
  if (status) return status;

  // convert ADC count to byte count based on data width in use
  *FifoByteCount = fifoCountADC * 4;

  return status;
}



//*-------------------------------------------------------------------ClearInputBuffer-------------
//*
//* Description:
//*
//*     Clears the contents of the input data buffer.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::ClearInputBuffer()
{
  int status = 0;
  int regval = 0;

  status = fpgaRead( mBoardNo, REGOFFSET_IBC, &regval);
  regval |= 0x00080000;
  status = fpgaWrite( mBoardNo, REGOFFSET_IBC, regval);

  return status;
}



//*-------------------------------------------------------------------GetNumChannelsPresent--------
//*
//* Description:
//*
//*     This method gets the number of input channels present in the DSI hardware.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     the number of channels present
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::GetNumChannelsPresent()
{
  return mNumChannelsPresent;
}



//*-------------------------------------------------------------------Initialize-------------------
//*
//* Description:
//*
//*     This method starts the initialization function on the DSI analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::Initialize()
{
  int         status = 0;
  int         regval = 0;
  clock_t     tnow = 0;

  // start initialization
  if ((status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regval )) != 0)
    return status;
  if ((status = fpgaWrite( mBoardNo, REGOFFSET_BCR, regval | 0x8000 )) != 0)
    return status;

  // wait for initialization to complete (may take up to 5 seconds on DSI board
  // this class method times out after 8 seconds.
  tnow = clock();
  do
  {
    Sleep(50);
    fpgaRead( mBoardNo, REGOFFSET_BCR, &regval);
  } while ((regval & 0x8000) && ((((double)(clock() - tnow)) / CLOCKS_PER_SEC) < 8.0));

  if (regval & 0x8000)
    return GscApiOperationTimeout;

  // clear interrupts after initialization completes
  status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regval );
  status = fpgaWrite( mBoardNo, REGOFFSET_BCR, regval & 0xfffff7ff);

  return status;
}



//*-------------------------------------------------------------------GetAutoCalStatus-------------
//*
//* Description:
//*
//*     This method gets the status of the autocalibration function on a DSI analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   *acStatus               The returned status, constructed by logically or'ing of the
//*                                   following values:
//*                                       GSC_ANA_AUTOCAL_INIT
//*                                       GSC_ANA_AUTOCAL_PASS
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::GetAutoCalStatus( int *acStatus )
{
  int status = 0;
  int regval = 0;

  if ((status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regval )) != 0)
    return status;

  if((regval & 0x80) != 0)
    *acStatus |= GSC_ANA_AUTOCAL_INIT;

  if(regval & 0x1000)
    *acStatus |= GSC_ANA_AUTOCAL_PASS;

  return 0;
}



//*----------------------------------------------------------------SetInputBufferThreshold---------
//*
//* Description:
//*
//*     This method sets the input buffer threshold on a DSI analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   threshold               The threshold value.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::SetInputBufferThreshold( int threshold )
{
  int status = 0;
  int regval = 0;

  if ((status = fpgaRead( mBoardNo, REGOFFSET_IBC, &regval )) != 0)
    return status;

  status = fpgaWrite( mBoardNo, REGOFFSET_IBC, (regval & 0xFFFC0000) | threshold );

  return status;
}



//*-------------------------------------------------------------------SoftwareSync-----------------
//*
//* Description:
//*
//*     This method initiates a software sync operation between a target board and an
//      initiator board in an externally-synced dual DSI board configuration.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna24DSI12::SoftwareSync()
{
  int regval = 0;

  // Sync ALL Boards, Software sync
  fpgaRead( mBoardNo, REGOFFSET_BCR, &regval);
  regval |= 0x40;
  fpgaWrite( mBoardNo, REGOFFSET_BCR, regval);

  return 0;
}
