//-------------------------------------------------------------------------------------------------
//
// File Name:
//
//     GscAnaSSA.cpp
//
// Description:
//
//     This file contains the implementation of the GSC SSA analog board class. This class
//     implements the fundamental, underlying board-specific functionality used by the GscApi
//     analog board C library API.
//
// Programmer's notes:
//
//     06-06-2008:  Created file. Only 6616AI64 supported and tested. --mrg
//
//-------------------------------------------------------------------------------------------------


#include "math.h"
#include "time.h"
#include "GscAnaSSA.h"


const double GscAna6616AI64SSA::MIN_SAMPLE_CLOCK = 0.008;


//*-------------------------------------------------------------------GscAna6616AI64SSA------------
//*
//* Description:
//*
//*    Default constructor. Not used in GscApi library.
//*
//* Parameters:
//*
//*    Parameter                    Description
//*
//* Return Value:
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

GscAna6616AI64SSA::GscAna6616AI64SSA()
{
  mBoardNo = 0;
  mValidHw = false;
}


//*-------------------------------------------------------------------GscAna6616AI64SSA------------
//*
//* Description:
//*
//*    Constructor for SSA class.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     BoardNumber                   PlxApi board number of analog board.
//*
//* Return Value:
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

GscAna6616AI64SSA::GscAna6616AI64SSA(int BoardNumber)
{
  int status = 0;
  int i = 0;
  int bn = BoardNumber - 1;
  int regval = 0;

  mValidHw = true;

  // Make sure the board type is as expected
  if (boardInformation[bn].boardType != GSC_BOARD_TYPE_ANALOG)
    mValidHw = false;

  // Make sure the board number is valid
  status = validateBoardNumber( bn );
  if (status)
    mValidHw = false;

  mBoardNo = BoardNumber;

  // GscApi config structure defaults
  mInputTypeMode    = GSC_ANA_NORMAL;
  mInputType        = GSC_ANA_SINGLE_ENDED;
  mInputRange       = GSC_ANA_BIP_10_0;
  mDataFormat       = GSC_ANA_OFFSET_BINARY;
  mExternalSync     = GSC_DISABLED;
  mDataWidth        = GSC_ANA_DATAWIDTH_16;
  mDataPacking      = GSC_DISABLED;
  mScanMarkingMode  = GSC_DISABLED;
  mScanMarkingValue = 0x00000000;
  mSampleClockSource = GSC_ANA_CLOCK_INTERNAL;
  mMasterClockHz    = 50000000;
  mSamplingHz       = 0;

  mNumberOfActiveChannels = 32; // SSA board default no. of channels scanned

  // determine the number of channels present on the board
  fpgaRead( BoardNumber, REGOFFSET_FWR, &regval );
  mNumChannelsPresent = (regval&0x10000) ? 32:64;

  // Available SSA interrupts
  analogIntrInfo[0].intrType = GSC_INTR_ANA_AUTOCALCOMPLETED;
  analogIntrInfo[0].isrMask = 0x00000001; //auto-calibration completed.
  analogIntrInfo[1].intrType = GSC_INTR_ANA_THRESHOLDLOWTOHIGH;
  analogIntrInfo[1].isrMask = 0x00000010; // buffer threshold low ==> high transition
  analogIntrInfo[2].intrType = GSC_INTR_ANA_THRESHOLDHIGHTOLOW;
  analogIntrInfo[2].isrMask = 0x00000020; // buffer threshold high ==> low transition
  analogIntrInfo[3].intrType = GSC_INTR_ANA_OVERUNDERFLOW;
  analogIntrInfo[3].isrMask = 0x00000030; // input buffer overflow or underflow

  // initialize interrupt callbacks to NULL
  for (i=0; i < 4; i++)
    AnalogInterruptRoutines[mBoardNo][i] = NULL;
}



//*------------------------------------------------------------------~GscAna6616AI64SSA------------
//*
//* Description:
//*
//*    Destructor for SSA class.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*
//* Return Value:
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

GscAna6616AI64SSA::~GscAna6616AI64SSA()
{
  mBoardNo = 0;
}



//*-------------------------------------------------------------------GetConfigDefaults------------
//*
//* Description:
//*
//*     Retrieves SSA board defaults for the fields of an ANALOG_CONFIG struct.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     config                        Pointer to a GSC_ANALOG_CONFIG structure.
//*
//* Return Value:
//*     0 - success
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::GetConfigDefaults( PGSC_ANALOG_CONFIG config )
{
  int status = 0x00;

  // Load the defaults
  config->inputMode = GSC_ANA_NORMAL;
  config->inputType = GSC_ANA_SINGLE_ENDED;
  config->inputRange = GSC_ANA_BIP_10_0;
  config->dataFormat = GSC_ANA_OFFSET_BINARY;
  config->externalSync = GSC_DISABLED;
  config->dataWidth = GSC_ANA_DATAWIDTH_16;
  config->dataPacking = GSC_DISABLED;
  config->scanMarkingMode = GSC_DISABLED;
  config->scanMarkingValue = 0x00000000;
  config->sampleClockSource = GSC_ANA_CLOCK_INTERNAL;
  config->masterClockFrequency = 50000000;
  return status;
}



//*-------------------------------------------------------------------GetConfig--------------------
//*
//* Description:
//*
//*     Retrieves current config values for an SSA board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     config                        Pointer to a GSC_ANALOG_CONFIG structure.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::GetConfig(PGSC_ANALOG_CONFIG config)
{
  int status = 0x00;

  // Get the desired options
  GetOption( GSC_ANA_INPUT_MODE, &config->inputMode);
  GetOption( GSC_ANA_INPUT_TYPE, &config->inputType);
  GetOption( GSC_ANA_INPUT_RANGE, &config->inputRange);
  GetOption( GSC_ANA_DATA_FORMAT, &config->dataFormat);
  GetOption( GSC_ANA_EXTERNAL_SYNC, &config->externalSync);
  GetOption( GSC_ANA_DATA_WIDTH, &config->dataWidth);
  GetOption( GSC_ANA_DATA_PACKING, &config->dataPacking);
  GetOption( GSC_ANA_SCAN_MARKING, &config->scanMarkingMode);
  GetOption( GSC_ANA_SCAN_MARKING_VALUE, &config->scanMarkingValue);
  GetOption( GSC_ANA_CLOCK_SOURCE, &config->sampleClockSource);
  GetOption( GSC_ANA_MASTER_CLOCK, &config->masterClockFrequency);

  return status;
}



//*-------------------------------------------------------------------SetConfig--------------------
//*
//* Description:
//*
//*     Sets the current config values for an SSA board to the values in the supplied
//*     GSC_ANALOG_CONFIG struct parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     config                        A GSC_ANALOG_CONFIG struct containing the desired config
//*                                   values.
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::SetConfig( GSC_ANALOG_CONFIG config )
{
  int status = 0x00;

  // Set the desired options
  SetOption( GSC_ANA_INPUT_MODE, config.inputMode);
  SetOption( GSC_ANA_INPUT_TYPE, config.inputType);
  SetOption( GSC_ANA_INPUT_RANGE, config.inputRange);
  SetOption( GSC_ANA_DATA_FORMAT, config.dataFormat);
  SetOption( GSC_ANA_EXTERNAL_SYNC, config.externalSync);
  SetOption( GSC_ANA_DATA_WIDTH, config.dataWidth);
  SetOption( GSC_ANA_DATA_PACKING, config.dataPacking);
  SetOption( GSC_ANA_SCAN_MARKING, config.scanMarkingMode);
  SetOption( GSC_ANA_SCAN_MARKING_VALUE, config.scanMarkingValue);
  SetOption( GSC_ANA_CLOCK_SOURCE, config.sampleClockSource);
  SetOption( GSC_ANA_MASTER_CLOCK, config.masterClockFrequency);
  return status;
}



//*-------------------------------------------------------------------GetOption--------------------
//*
//* Description:
//*
//*     Retrieves the value of the SSA board option specified by the optionName parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     optionName                    An enum GSC_OPTION_NAME value that specifies the board option
//*                                   to retrieve.
//*     optionValue[]                 An array of ints to hold the current option value. All
//*                                   analog board options are composed of a single int value, but
//*                                   this method supports options that require several int values
//*                                   for their representation.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::GetOption( enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status = 0x00;
  int oldValue  = 0;
  int newValue  = 0;
  int bn = 0;
  int snglChn = 0;

  bn = mBoardNo - 1;

  // TODO:
  //    Potential new common options:
  //        External Sync role (initiator/target) - not just enable/disable external sync.

  switch ( optionName )
  {
    //
    // Get the Input Mode
    //
  case GSC_ANA_INPUT_MODE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000007;

    // Decode the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ANA_NORMAL; break;
    case 0x03: optionValue[0] = GSC_ANA_VREF;   break;
    case 0x02: optionValue[0] = GSC_ANA_ZERO;   break;
    default:   optionValue[0] = GSC_ANA_NORMAL; break;
    }
    break;


    //
    // Get the input type
    //
  case GSC_ANA_INPUT_TYPE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000300;

    // Process the desired option
    switch (newValue)
    {
    case 0x0000: optionValue[0] = GSC_ANA_SINGLE_ENDED; break;
    case 0x0100: optionValue[0] = GSC_ANA_PSEUDO_DIFF;  break;
    case 0x0200: optionValue[0] = GSC_ANA_DIFFERENTIAL; break;
    default:     optionValue[0] = GSC_ANA_SINGLE_ENDED; break;
    }
    break;


    //
    // Get the input range
    //
  case GSC_ANA_INPUT_RANGE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000038;
    // Process the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ANA_BIP_2_5;  break;
    case 0x10: optionValue[0] = GSC_ANA_BIP_5_0;  break;
    case 0x20: optionValue[0] = GSC_ANA_BIP_10_0; break;
    case 0x08: optionValue[0] = GSC_ANA_UNI_5_0;  break;
    case 0x18: optionValue[0] = GSC_ANA_UNI_10_0; break;
    default:   optionValue[0] = GSC_ANA_BIP_10_0; break;
    }

    break;


    //
    // Get the data format
    //
  case GSC_ANA_DATA_FORMAT:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000040;

    // Process the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_ANA_TWOS_COMPLEMENT; break;
    case 0x40: optionValue[0] = GSC_ANA_OFFSET_BINARY;   break;
    default:   optionValue[0] = GSC_ANA_TWOS_COMPLEMENT; break;
    }
    break;


    //
    // Get the external sync
    //
  case GSC_ANA_EXTERNAL_SYNC:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000080;

    // Process the desired option
    switch (newValue)
    {
    case 0x00: optionValue[0] = GSC_DISABLED; break;
    case 0x80: optionValue[0] = GSC_ENABLED;  break;
    default:   optionValue[0] = GSC_DISABLED; break;
    }
    break;


    //
    // Get data packing option
    //
  case GSC_ANA_DATA_PACKING:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00040000;

    // Process the desired option
    switch (newValue)
    {
    case 0x00000: optionValue[0] = GSC_DISABLED; break;
    case 0x40000: optionValue[0] = GSC_ENABLED;  break;
    default:      optionValue[0] = GSC_DISABLED; break;
    }
    break;


    //
    // Get the data width
    //
  case GSC_ANA_DATA_WIDTH:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00040000;

    // Process the desired option
    switch (newValue)
    {
    case 0x00000: optionValue[0] = GSC_ANA_DATAWIDTH_32; break;
    case 0x40000: optionValue[0] = GSC_ANA_DATAWIDTH_16;  break;
    default:      optionValue[0] = GSC_ANA_DATAWIDTH_32; break;
    }
    break;

    //
    // Get the scan marking
    //
  case GSC_ANA_SCAN_MARKING:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000800;

    // Process the desired option
    switch (newValue)
    {
    case 0x0800: optionValue[0] = GSC_DISABLED; break;
    case 0x0000: optionValue[0] = GSC_ENABLED;  break;
    default:     optionValue[0] = GSC_DISABLED; break;
    }
    break;


    //
    // Get the scan marking value
    //
  case GSC_ANA_SCAN_MARKING_VALUE:
    // Read the first value
    status = fpgaRead( mBoardNo, REGOFFSET_SMF, &newValue);
    if (status) return status;

    // Read the second value
    status = fpgaRead( mBoardNo, REGOFFSET_SMS, &oldValue);
    if (status) return status;

    // Return the value
    optionValue[0] = ((oldValue<<16)&0xffff0000) | (newValue & 0xffff);
    break;


    //
    // Get the clock source
    //
  case GSC_ANA_CLOCK_SOURCE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_SSC, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & 0x00000418;

    // Process the desired option
    switch (newValue)
    {
    case 0x0408: optionValue[0] = GSC_ANA_CLOCK_INTERNAL;  break;	// Rate-b chained
    case 0x0000: optionValue[0] = GSC_ANA_CLOCK_SECONDARY; break;	// Rate-a
    case 0x0010: optionValue[0] = GSC_ANA_CLOCK_EXTERNAL;  break;	// Ext sync
    case 0x0018: optionValue[0] = GSC_ANA_CLOCK_MANUAL;    break;	// BCR bit
    default:     optionValue[0] = GSC_ANA_CLOCK_INTERNAL;  break;
    }
    break;

    //
    // Get the Master Clock Frequency
    //
  case GSC_ANA_MASTER_CLOCK:
    // Get the frequency
    optionValue[0] = mMasterClockHz;
    break;

    //
    // Get the number of active (being scanned) channels
    //
  case GSC_ANA_NUMACTIVECHANNELS:
    // Get the frequency
    optionValue[0] = mNumberOfActiveChannels;
    break;

    //
    // Get the number of channels present on the board
    //
  case GSC_ANA_MAXCHANNELS:
    // Get the frequency
    optionValue[0] = mNumChannelsPresent;
    break;

    //
    // Get the number of active (being scanned) channels
    //
  case GSC_ANA_SINGLEACTIVECHANNEL:
    // Get the frequency
    if ((status = fpgaRead( mBoardNo, REGOFFSET_SSC, &snglChn )) != 0)
      optionValue[0] = snglChn >> 12;
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------SetOption--------------------
//*
//* Description:
//*
//*     Retrieves the value of the SSA board option specified by the optionName parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     optionName                    An enum GSC_OPTION_NAME value that specifies the board option
//*                                   to set.
//*     optionValue                   The desired value for the specified option.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::SetOption( enum GSC_OPTION_NAME optionName, int optionValue)
{
  int status = 0x00;
  int oldValue = 0;
  int newValue  = 0;
  int snglchn = 0;
  int regValue = 0;

  switch ( optionName )
  {
    //
    // Set the Input Mode
    //
  case GSC_ANA_INPUT_MODE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000007;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_NORMAL: newValue |= 0x00000000; break;
    case GSC_ANA_VREF:	 newValue |= 0x00000003; break;
    case GSC_ANA_ZERO:	 newValue |= 0x00000002; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mInputTypeMode = optionValue;

    break;


    //
    // Set the input type
    //
  case GSC_ANA_INPUT_TYPE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000300;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_SINGLE_ENDED:	 newValue |= 0x00000000; break;
    case GSC_ANA_PSEUDO_DIFF:	 newValue |= 0x00000100; break;
    case GSC_ANA_DIFFERENTIAL:	 newValue |= 0x00000200; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mInputType = optionValue;

    break;


    //
    // Set the input range
    //
  case GSC_ANA_INPUT_RANGE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000038;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_BIP_2_5:	 newValue |= 0x00000000; break;
    case GSC_ANA_BIP_5_0:	 newValue |= 0x00000010; break;
    case GSC_ANA_BIP_10_0:	 newValue |= 0x00000020; break;
    case GSC_ANA_UNI_5_0:	 newValue |= 0x00000008; break;
    case GSC_ANA_UNI_10_0:	 newValue |= 0x00000018; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mInputRange = optionValue;

    break;


    //
    // Set the data format
    //
  case GSC_ANA_DATA_FORMAT:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000040;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_TWOS_COMPLEMENT:	newValue |= 0x00000000; break;
    case GSC_ANA_OFFSET_BINARY:		newValue |= 0x00000040; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mDataFormat = optionValue;

    break;


    //
    // Set the external sync
    //
  case GSC_ANA_EXTERNAL_SYNC:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000080;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_DISABLED:	newValue |= 0x00000000; break;
    case GSC_ENABLED:	newValue |= 0x00000080; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mExternalSync = optionValue;

    break;


    //
    // Set the data width
    //
  case GSC_ANA_DATA_WIDTH:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00040000;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_DATAWIDTH_16:	newValue |= 0x00040000; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mDataWidth = optionValue;
    break;

    //
    // Get data packing option
    //
  case GSC_ANA_DATA_PACKING:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00040000;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_DISABLED: newValue |= 0x00000; break;
    case GSC_ENABLED: newValue |= 0x40000;  break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mDataPacking = optionValue;

    break;

    //
    // Set the scan marking
    //
  case GSC_ANA_SCAN_MARKING:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000800;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_DISABLED:	newValue |= 0x00000800; break;
    case GSC_ENABLED:	newValue |= 0x00000000; break;
    default: return GscApiInvalidParameter;
    }

    // Write the new value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    if (status) return status;

    mScanMarkingMode = optionValue;

    break;


    //
    // Set the scan marking value
    //
  case GSC_ANA_SCAN_MARKING_VALUE:
    // Write the new first value
    status = fpgaWrite( mBoardNo, REGOFFSET_SMF, optionValue&0xffff);
    if (status) return status;

    // Write the new second value
    status = fpgaWrite( mBoardNo, REGOFFSET_SMS, (optionValue>>16)&0xffff);
    if (status) return status;

    mScanMarkingValue = optionValue;

    break;


    //
    // Set the clock source
    //
  case GSC_ANA_CLOCK_SOURCE:
    // Get the current value
    status = fpgaRead( mBoardNo, REGOFFSET_SSC, &oldValue);
    if (status) return status;
    // Get the current external sync bit value
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regValue);
    if (status) return status;

    // Mask off the bits of interest
    newValue = oldValue & ~0x00000418;

    // NOTE: Always enable external sync. This can be turned off with another option. It is kept enabled
    // for all clock source assignments so that the GSC_ANA_CLOCK_EXTERNAL setting always configures the
    // board as a target, and all other clock source settings configure the board as an initiator.
    regValue |= 0x00000080;

    // Process the desired option
    switch (optionValue)
    {
    case GSC_ANA_CLOCK_INTERNAL:	newValue |= 0x00000408; break;	// Rate-b chained
    case GSC_ANA_CLOCK_SECONDARY:	newValue |= 0x00000000; break;	// Rate-a
    case GSC_ANA_CLOCK_EXTERNAL:	newValue |= 0x00000010; break;	// Ext sync as target
    case GSC_ANA_CLOCK_MANUAL:		newValue |= 0x00000018; break;	// BCR bit
    default: return GscApiInvalidParameter;
    }

    // Write the new clock source value
    status = fpgaWrite( mBoardNo, REGOFFSET_SSC, newValue);
    if (status) return status;

    // Write the new external sync bit value
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, regValue);
    if (status) return status;

    mSampleClockSource = optionValue;

    break;

    //
    // Set the Master Clock Frequency
    //
  case GSC_ANA_MASTER_CLOCK:
    // Save the frequency
    mMasterClockHz = optionValue;
    break;

    //
    // Set the sample rate
    //
  case GSC_ANA_SAMPLE_RATE:
    // Save the frequency
    status = SetSampleRate( optionValue );
    if (status == 0)
      mSamplingHz = optionValue;
    break;

    //
    // Set the number of active input channels
    //
  case GSC_ANA_NUMACTIVECHANNELS:
    // write the SSC register
    status = fpgaRead( mBoardNo, REGOFFSET_SSC, &oldValue);
    if (status) return status;

    switch (optionValue & 0x07)
    {
    case 64:
      status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFFFFF8) | 6 );
      break;
    case 32:
      status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFFFFF8) | 5 );
      break;
    case 16:
      status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFFFFF8) | 4 );
      break;
    case 8:
      status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFFFFF8) | 3 );
      break;
    case 4:
      status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFFFFF8) | 2 );
      break;
    case 2:
      status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFFFFF8) | 1 );
      break;
    case 1:
      snglchn = (optionValue & 0xFFFF0000) >> 16;
      if (snglchn < 0 || snglchn >= mNumChannelsPresent)
        status = GscApiInvalidParameter;
      else
        status = fpgaWrite( mBoardNo, REGOFFSET_SSC, (oldValue & 0xFFFC0FF8) | (snglchn << 12));
      break;
    default:
      status = GscApiInvalidParameter;
    }

    // set the member variable
    if (status == 0)
      mNumberOfActiveChannels = optionValue;
    else
    {
      switch (mNumChannelsPresent)
      {
      case 64:
        fpgaWrite( mBoardNo, REGOFFSET_SSC, 6 );
        break;
      case 32:
        fpgaWrite( mBoardNo, REGOFFSET_SSC, 5 );
        break;
      default:
        status = GscApiInvalidParameter;
      }
    }
    break;

    //
    // Enable/disable the rate generators
    //
  case GSC_ANA_RATEGEN:
    switch (optionValue)
    {
    case GSC_ENABLED:
      fpgaRead( mBoardNo, REGOFFSET_RAG, &oldValue );
      status = fpgaWrite( mBoardNo, REGOFFSET_RAG, oldValue & 0xFFFEFFFF ); // Enable rate a generator
      break;
    case GSC_DISABLED:
      fpgaRead( mBoardNo, REGOFFSET_RAG, &oldValue ); // Disable rate a generator
      status = fpgaWrite( mBoardNo, REGOFFSET_RAG, (oldValue & 0xFFFFFF00) | 0x10000);
      break;
    default:
      return GscApiInvalidParameter;
    }
    break;

    //
    // Set DMA demand mode
    //
  case GSC_ANA_DMA_DEMANDMODE:
    fpgaRead( mBoardNo, REGOFFSET_BCR, &oldValue );

    // Mask off the bits of interest
    newValue = oldValue & ~0x00080000;

    switch (optionValue)
    {
    case GSC_ANA_DMA_AUTONOMOUS:  newValue |= 0x80000;  break;
    case GSC_ANA_DMA_THRESHOLD:   break;
    default:
      return GscApiInvalidParameter;
    }
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, newValue);
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------LocalRegisterRead------------
//*
//* Description:
//*
//*     Retrieves the value of an SSA board local register.
//*
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     reg                           An SSA board local register offset.
//*     result                        The current value contained in the specified local register.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*     The GscApi API defines "symbolic" rather than actual local register offsets for all the
//*     GSC ADC boards due to the significant variation in register offsets across the boards. The
//*     register offset supplied to this method when called by the GscApi analog board C functions
//*     is therefore such a "symbolic" value. This class method recognizes the "symbolic" register
//*     offsets and uses corresponding actual register offset constants defined as part of the
//*     class.
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::LocalRegisterRead( int reg, int *result )
{
  int status = 0;

  //read result
  switch (reg)
  {
  case GSC_ANALOG_BCR:
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, result );
    break;
  case GSC_ANALOG_ICR:
    status = fpgaRead( mBoardNo, REGOFFSET_ICR, result );
    break;
  case GSC_ANALOG_IDB:
    status = fpgaRead( mBoardNo, REGOFFSET_IDB, result );
    break;
  case GSC_ANALOG_IBC:
    status = fpgaRead( mBoardNo, REGOFFSET_IBC, result );
    break;
  case GSC_ANALOG_RAG:
    status = fpgaRead( mBoardNo, REGOFFSET_RAG, result );
    break;
  case GSC_ANALOG_RBG:
    status = fpgaRead( mBoardNo, REGOFFSET_RBG, result );
    break;
  case GSC_ANALOG_BS:
    status = fpgaRead( mBoardNo, REGOFFSET_IBS, result );
    break;
  case GSC_ANALOG_SSC:
    status = fpgaRead( mBoardNo, REGOFFSET_SSC, result );
    break;
  case GSC_ANALOG_ACA:
    status = fpgaRead( mBoardNo, REGOFFSET_ACA, result );
    break;
  case GSC_ANALOG_FWR:
    status = fpgaRead( mBoardNo, REGOFFSET_FWR, result );
    break;
  case GSC_ANALOG_AV:
    status = fpgaRead( mBoardNo, REGOFFSET_ACV, result );
    break;
  case GSC_ANALOG_SMFV:
    status = fpgaRead( mBoardNo, REGOFFSET_SMF, result );
    break;
  case GSC_ANALOG_SMSV:
    status = fpgaRead( mBoardNo, REGOFFSET_SMS, result );
    break;
  default:
    if (!(reg & 0x8000)) // no constant defined in GscApi.h, go ahead and read from register at offset
      fpgaRead( mBoardNo, reg, result );
    else
      return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------LocalRegisterWrite-----------
//*
//* Description:
//*
//*     Sets the value of an SSA board local register.
//*
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     reg                           An SSA board local register offset.
//*     value                         The desired value for the specified local register.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*     See notes for LocalRegisterRead() method for explanation of "symbolic" register offsets.
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::LocalRegisterWrite( int reg, int value )
{
  int status = 0;

  // write value
  switch (reg)
  {
  case GSC_ANALOG_BCR:
    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, value );
    break;
  case GSC_ANALOG_ICR:
    status = fpgaWrite( mBoardNo, REGOFFSET_ICR, value );
    break;
  case GSC_ANALOG_IBC:
    status = fpgaWrite( mBoardNo, REGOFFSET_IBC, value );
    break;
  case GSC_ANALOG_RAG:
    status = fpgaWrite( mBoardNo, REGOFFSET_RAG, value );
    break;
  case GSC_ANALOG_RBG:
    status = fpgaWrite( mBoardNo, REGOFFSET_RBG, value );
    break;
  case GSC_ANALOG_SSC:
    status = fpgaWrite( mBoardNo, REGOFFSET_SSC, value );
    break;
  case GSC_ANALOG_ACA:
    status = fpgaWrite( mBoardNo, REGOFFSET_ACA, value );
    break;
  case GSC_ANALOG_SMFV:
    status = fpgaWrite( mBoardNo, REGOFFSET_SMF, value );
    break;
  case GSC_ANALOG_SMSV:
    status = fpgaWrite( mBoardNo, REGOFFSET_SMS, value );
    break;
  default:
    if (!(reg & 0x8000)) // no constant defined in GscApi.h, go ahead and write register at offset
      fpgaWrite( mBoardNo, reg, value );
    else
      return GscApiInvalidParameter;
  }

  return status;
}



//*-------------------------------------------------------------------AutoCal----------------------
//*
//* Description:
//*
//*     Initiates auto-calibration of the SSA board and waits for it to complete by polling the
//*     AUTOCAL bit of the BCR register.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//*     bWaitForCompletion            Flag indicating whether method will poll for autocal
//*                                   completion.
//*
//* Return Value:
//*     0 - success, > 0 GscApiAutoCalFailed, or a Plx error, or a GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::AutoCal( bool bWaitForCompletion )
{
  int status = 0x00;
  int nAttempts = 0x00;
  int tempValue;

  // Set the autocal bit
  status = fpgaRead( mBoardNo, REGOFFSET_BCR, &tempValue);
  if (status) return status;
  tempValue |= 0x00002000;
  status = fpgaWrite( mBoardNo, REGOFFSET_BCR, tempValue);
  if (status) return status;

  // Loop until bit goes clear
  nAttempts = 0;
  do
  {
    Sleep(100);
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &tempValue);
    if (status) return status;
  } while ((bWaitForCompletion && (tempValue & 0x00002000)) && ++nAttempts < 10);

  // If autocal failed, return error
  if ((tempValue & 0x00004000) == 0) return GscApiAutoCalFailed;

  return status;
}



//*-------------------------------------------------------------------SetSampleRate----------------
//*
//* Description:
//*
//*     Sets the SSA board rate generators to sample data at the requested sample rate.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     sampleHz                      The destired sample rate in Hz.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::SetSampleRate( int sampleHz )
{
  int status = 0x00;
  int divA = 0, divB;

  // Determine the Rate-B divisor
  if (sampleHz >= 465) divB = 1;
  else if (sampleHz >= 58) divB = 8;
  else if (sampleHz >= 7.5) divB = 64;
  else if (sampleHz >= 1.0) divB = 512;
  else if (sampleHz >= 0.12) divB = 4096;
  else if (sampleHz >= 0.02) divB = 32768;
  else divB = 65000;

  // Determine the Rate-A divisor
  divA = (int)(mMasterClockHz/(double)sampleHz/(double)divB);
  //divA = (int)(boardInformation[ bn].masterClock/samplesPerSecond);

  // Make sure divisors are chained
  status = SetOption( GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_INTERNAL);
  //status = GscAnaSetOption( boardNumber, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_SECONDARY);
  if (status) return status;

  // Set the two freqencies
  status = fpgaWrite( mBoardNo, REGOFFSET_RAG, (divA & 0xffff));
  if (status) return status;

  status = fpgaWrite( mBoardNo, REGOFFSET_RBG, divB);
  if (status) return status;

  // Give the clocks time to stablize
  Sleep(10);

  mSamplingHz = sampleHz;

  boardInformation[mBoardNo - 1].bytesPerMs[0] = (mSamplingHz * 4 * mNumberOfActiveChannels) / 1000;

  return 0;
}



//*-------------------------------------------------------------------ScanInputs-------------------
//*
//* Description:
//*
//*     Scans and stores ADC sample data for the requested channels.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     samples                       The number of samples to scan (for each channel).
//*     samplesPerSecond              The sample rate in Hz.
//*     firstChannel                  The first channel to scan, zero-based.
//*     lastChannel                   The last channel to scan, zero-based.
//*     buffer                        The buffer in which the sample data is stored.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::ScanInputs( int samples,
                                   double samplesPerSecond,
                                   int firstChannel, int lastChannel, char *buffer)
{
  int status = 0x00;
  int numberOfBytes;
  int tempValue;
  int rxId, bytesAcquired;
  int dataPacking, dataMarking;
  //int ulBuffSize = 0;
  //int ulIbc = 0;
  //int i = 0;
  //clock_t tnow;

  // TODO:
  //   - add a timeout parameter
  //   - disable clock before returning? (rather than in rxdmahandler())


  // Make sure # of samples is > 0
  if (samples <= 0) return GscApiInvalidParameter;

  // Verify samples/second value
  if ((samples != 1) && (samplesPerSecond == 0)) return GscApiInvalidParameter;
  if (samplesPerSecond > MAX_SAMPLE_CLOCK) return GscApiInvalidParameter;
  if (samplesPerSecond < MIN_SAMPLE_CLOCK) return GscApiInvalidParameter;

  // Make sure last channel is >= to the first
  if (lastChannel < firstChannel) return GscApiInvalidParameter;

  // Make sure channels are within range
  if ((firstChannel < 0) || (lastChannel > 63)) return GscApiInvalidParameter;

  // Determine the number of active channels
  mNumberOfActiveChannels = lastChannel - firstChannel + 1;

  // Setup the "samples per millisecond" variable for the dma handler
  if (samples == 1) boardInformation[ mBoardNo -1].bytesPerMs[ 0] = mNumberOfActiveChannels;
  else boardInformation[ mBoardNo - 1].bytesPerMs[ 0] = mNumberOfActiveChannels * 4 *samplesPerSecond/1000.0;

  // Write the channel numbers
  status = fpgaWrite ( mBoardNo, REGOFFSET_ACA, firstChannel | (lastChannel << 8));
  if (status) return status;

  // Setup to use the channels
  status = fpgaRead( mBoardNo, REGOFFSET_SSC, &tempValue);
  if (status) return status;

  //tempValue |= 0x00000007;
  tempValue |= 0x00000005;

  status = fpgaWrite( mBoardNo, REGOFFSET_SSC, tempValue);
  if (status) return status;

  //
  // Determine the number of bytes to acquire. If not performing data packing,
  //  the number is just 4 bytes per channel times the number of samples. If
  //  data packing is enabled, there are 2 bytes per channel times the number
  //  of samples. If data packing and data scan marking are enabled there are 4
  //  extra bytes for the markers on each sample.
  //
  status = GetOption( GSC_ANA_DATA_PACKING, &dataPacking);
  status = GetOption( GSC_ANA_SCAN_MARKING, &dataMarking);
  if (dataPacking == GSC_DISABLED)
  {
    numberOfBytes = mNumberOfActiveChannels * 4 * samples;
  }
  else
  {
    numberOfBytes = mNumberOfActiveChannels * 2 * samples;
    if (dataMarking == GSC_ENABLED) numberOfBytes += (4 * samples);
  }

  // Make sure we are not acquiring data by setting sample clock to manual
  status = SetOption( GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_MANUAL);
  if (status) return status;

  // If more than 1 sample is requested, setup the sample clock
  if (samples > 1)
  {
    // Clear the fifo
    status = ClearInputBuffer();
    if (status) return status;

    // configure sample rate clocks
    if ((status = SetSampleRate( (int)samplesPerSecond )))
      return status;
  }
  else   // Otherwise, perform a manual trigger
  {
    // Trigger a manual acquisition
    status = fpgaRead( mBoardNo, REGOFFSET_BCR, &tempValue);
    if (status) return status;

    tempValue |=0x1000;

    status = fpgaWrite( mBoardNo, REGOFFSET_BCR, tempValue);
    if (status) return status;
  }

  // initiate auto cal
  if ((status = GscAnaAutoCal( mBoardNo, true )))
    return status;

  // Enable Clock
  status = fpgaRead( mBoardNo, REGOFFSET_RAG, &tempValue);
  if (status) return status;
  status = fpgaWrite( mBoardNo, REGOFFSET_RAG, tempValue & 0xFFFEFFFF );
  if (status) return status;

/*
  tnow = clock();
  for (i=1; i <= 20; i++)
  {
    Sleep(100);
    GscLocalRegisterRead( mBoardNo, REGOFFSET_IBS, &ulBuffSize );
    GscLocalRegisterRead( mBoardNo, GSC_ANALOG_IBC, &ulIbc ); // Threshold
    printf("buffer size after %f secs = %d.\n", ((double)(clock() - tnow)) / CLOCKS_PER_SEC, ulBuffSize);
    printf("theshold flag after %f secs = 0x%05x.\n", i * 0.2, ulIbc);
    if (ulIbc & 0x80000) // threshold flag HIGH
      break;
  }
  */

  // Setup to receive the data
  status = gscAddDmaRequest( mBoardNo, 1, buffer, numberOfBytes, GSC_DATA_RECEIVE, &rxId);
  //printf("Adding DMA request for %d bytes to be scanned into the analog board data buffer.\n", numberOfBytes);
  if (status) return status;

  // Wait for it to happen
  //printf("time after dma request submitted is  %f secs.\n", ((double)(clock() - tnow)) / CLOCKS_PER_SEC );
  status = GscSio4ChannelWaitForTransfer( mBoardNo, 1, 5000, rxId, &bytesAcquired);
  //printf("time after waitfortransfer is  %f secs.\n", ((double)(clock() - tnow)) / CLOCKS_PER_SEC );
  //printf("After waitfortransfer call, bytes acquired = %d\n", bytesAcquired);

  if (status) return status;

  // disable the rate generator
  status = fpgaRead( mBoardNo, REGOFFSET_RAG, &tempValue );
  if (status) return status;
  status = fpgaWrite( mBoardNo, REGOFFSET_RAG, 0x10000 | tempValue);
  if (status) return status;

  // Clear the fifo
  status = ClearInputBuffer();
  if (status) return status;

  // Disable rate generator
  status = GscAnaSetOption( mBoardNo, GSC_ANA_CLOCK_SOURCE, GSC_ANA_CLOCK_MANUAL);

  return status;
}



//*-------------------------------------------------------------------ReceiveScan------------------
//*
//* Description:
//*
//*     Performs a DMA operation to move the contents of the SSA input data buffer to the buffer
//*     parameter supplied to the method.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     char    *buffer               The buffer in which the sample data is stored.
//*     int     *count                The number of bytes stored in buffer.
//*     int     *transferStatus       Not currently used.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::ReceiveScan( char *buffer, int *count, int *transferStatus)
{
  int status = 0x00;
  int fifoCount;
  //int tempCount, isOdd;

  // Assume no data is available
  *count = 0;
  if (transferStatus != NULL) *transferStatus = 0;

  // Make sure the data is still in the Rx FIFO
  status = GetInputBufferSize( &fifoCount );
  if (status) return status;
  if (fifoCount < *count) return GscApiFifoOverrun;

  // If the count is odd, make it even
/* do i need this for the analog boards?
  isOdd = 0x00;
  tempCount = *count;
  if ( tempCount & 0x01)
  {
    isOdd = 0xff;
    tempCount &= ~0x01;
  }
*/

  // Get the data using DMA
  if (drvMajorVersion >= 5)
    status = gscPerformDma( mBoardNo, 1, buffer, fifoCount, GSC_DATA_RECEIVE, 0);
  else
    status = gscPerformDma( mBoardNo, 1, buffer, fifoCount, GSC_DATA_RECEIVE, PrimaryPciChannel0);

  // If the count was odd
/*
do i need this for the analog boards?
  if (isOdd) {
    // Read the last byte
    fpgaRead( boardNumber, (channel << 4) | 0x08, &isOdd);
    *(buffer + tempCount) = isOdd;
  }
*/
  *count = fifoCount;

  return status;
}



//*-------------------------------------------------------------------RegisterInterrupt------------
//*
//* Description:
//*
//*     Registers a callback function for an interrupt. The callback function is invoked by the
//*     GscApi interrupt handler thread when the interrupt fires.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int               interrupt   The interrupt to be caught.
//*     GSC_CB_FUNCTION   *function   The callback invoked when the interrupt happens.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::RegisterInterrupt( int interrupt, GSC_CB_FUNCTION *function )
{
  int i, status = 0x00, offset = 0;
  int enableMask = 0x00;
  int disableMask = 0x00;

  // Set the callback for the interrupt that is being registered.
  for (i=0; i < 4; i++)
  {
    // Is the bit set
    if ( interrupt & analogIntrInfo[ i].intrType)
    {
      // Save the callback routine
      AnalogInterruptRoutines[ mBoardNo-1][i] = function;

      // If the callback is null, update the disable mask
      if (function == NULL) disableMask |= analogIntrInfo[i].isrMask;
      else enableMask |= analogIntrInfo[i].isrMask;
    }
  }

  // if interrupt not supported, return appropriate error code.
  if (((enableMask == 0) && (function != NULL)) ||
      ((disableMask == 0) && (function == NULL)))
    return GscApiInvalidParameter;

  // Get the current interrupt mask
  status = fpgaRead( mBoardNo, REGOFFSET_ICR, &offset);
  if (status) return status;

  // Clear any bits that need disabling
  offset &= (~disableMask);

  // Set any bits that need enabling
  offset |= enableMask;

  // Write the value back out
  status = fpgaWrite( mBoardNo, REGOFFSET_ICR, offset);


  return status;
}



//*-------------------------------------------------------------------ClearInterrupts--------------
//*
//* Description:
//*
//*     Clears fpga (local) interrupts on the SSA board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::ClearInterrupts()
{
  int status = 0;
  int regval = 0;

  // Clear all pending interrupts
  status = fpgaRead( mBoardNo, REGOFFSET_ICR, &regval );
  status = fpgaWrite( mBoardNo, REGOFFSET_ICR, regval & 0xFFFFFF77 );

  return status;
}



//*-------------------------------------------------------------------InvokeInterruptCallback------
//*
//* Description:
//*
//*     Invokes the callback registered for an interrupt. The interrupt is cleared after the
//*     callback returns.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::InvokeInterruptCallback()
{
  int index = 0;
  int status = 0;
  int interruptMask = 0;

  // mask the interrupt
  status = fpgaRead( mBoardNo, REGOFFSET_ICR, &interruptMask );
  if (status) return status;

  // handle interrupts for analog boards
  for(index=0; index < 4; index++)
  {
    // Break out of loop if no bits set
    if (interruptMask == 0x00) break;

    // See if the interrupt is turned on
    if (interruptMask & analogIntrInfo[index].isrMask)
    {
      // make sure the interrupt request is for the masked interrupt
      if ((interruptMask & 0x80 && analogIntrInfo[index].isrMask > 0x0f) ||
        (interruptMask & 0x08 && analogIntrInfo[index].isrMask < 0x10))
      {
        //printf("Interrupt mask = 0x%x\n", interruptMask);
        // If the routine is defined, call it
        if (AnalogInterruptRoutines[mBoardNo - 1][index] != NULL)
        {
          AnalogInterruptRoutines[mBoardNo - 1][index]( mBoardNo, 0,
                                                        analogIntrInfo[index].intrType);
        }
        if (analogIntrInfo[index].isrMask & 0x70) // IRQ1 occurred
          status = fpgaWrite( mBoardNo, REGOFFSET_ICR, interruptMask & 0x7F); // clear it
        else
          status = fpgaWrite( mBoardNo, REGOFFSET_ICR, interruptMask & 0xF7); // clear it
      }
    }
  }
  return status;
}



//*-------------------------------------------------------------------GetInputBufferSize-----------
//*
//* Description:
//*
//*     Returns the number of bytes in the SSA board's input data buffer.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int    *FifoByteCount         The number of bytes in the input data buffer.
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::GetInputBufferSize(int *FifoByteCount)
{
  int status = 0;
  int fifoCountADC = 0;
  int dataPacking = 0;
  int dataMarking = 0;

  // insure size returned is zero on error
  *FifoByteCount = 0;

  // read the buffer size register
  status = fpgaRead( mBoardNo, REGOFFSET_IBS, &fifoCountADC );
  if (status) return status;
  fifoCountADC &= 0x0003ffff;

  // convert ADC count to byte count based on data width in use
  status = GetOption( GSC_ANA_DATA_PACKING, &dataPacking);
  if (status) return status;
  status = GetOption( GSC_ANA_SCAN_MARKING, &dataMarking);
  if (status) return status;

  if (dataPacking == GSC_DISABLED)
    *FifoByteCount = fifoCountADC * 4;
  else
  {
    *FifoByteCount = fifoCountADC * 2;
    if (dataMarking == GSC_ENABLED) *FifoByteCount += (4 * (fifoCountADC / mNumberOfActiveChannels));
  }

  return status;
}

//*-------------------------------------------------------------------ClearInputBuffer-------------
//*
//* Description:
//*
//*     Clears the contents of the input data buffer.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 Plx or GscApi error.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::ClearInputBuffer()
{
  int status = 0;
  int regval = 0;

  status = fpgaRead( mBoardNo, REGOFFSET_IBC, &regval);
  regval |= 0x00040000;
  status = fpgaWrite( mBoardNo, REGOFFSET_IBC, regval);

  return status;
}



//*-------------------------------------------------------------------GetNumChannelsPresent--------
//*
//* Description:
//*
//*     This routine gets the number of input channels present in the SSA hardware.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     the number of channels present
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::GetNumChannelsPresent()
{
  return mNumChannelsPresent;
}



//*-------------------------------------------------------------------Initialize-------------------
//*
//* Description:
//*
//*     This routine starts the initialization function on the SSA analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::Initialize()
{
  int         status = 0;
  int         regval = 0;

  // start initialization
  if ((status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regval )) != 0)
    return status;
  if ((status = fpgaWrite( mBoardNo, REGOFFSET_BCR, regval | 0x8000 )) != 0)
    return status;

  // Initialization requires 3 milliseconds on the SSA board. Give the board ample time to
  // complete the initialization process, then verify completion by checking the initialize
  // bit in the BCR.
  //
  Sleep(10);

  if (regval & 0x8000)
    return GscApiOperationTimeout;

  return 0;
}



//*-------------------------------------------------------------------GetAutoCalStatus-------------
//*
//* Description:
//*
//*     This gets the status of the autocalibration function on an SSA analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   *acStatus               The returned status, constructed by logically or'ing of the
//*                                   following values:
//*                                       GSC_ANA_AUTOCAL_INIT
//*                                       GSC_ANA_AUTOCAL_PASS
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::GetAutoCalStatus( int *acStatus )
{
  int status = 0;
  int regval = 0;

  if ((status = fpgaRead( mBoardNo, REGOFFSET_BCR, &regval )) != 0)
    return status;

  if((regval & 0x2000) != 0)
    *acStatus |= GSC_ANA_AUTOCAL_INIT;

  if(regval & 0x4000)
    *acStatus |= GSC_ANA_AUTOCAL_PASS;

  return 0;
}



//*----------------------------------------------------------------SetInputBufferThreshold---------
//*
//* Description:
//*
//*     This routine sets the input buffer threshold on an SSA analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   threshold               The threshold value.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAna6616AI64SSA::SetInputBufferThreshold( int threshold )
{
  int status = 0;
  int regval = 0;

  if ((status = fpgaRead( mBoardNo, REGOFFSET_IBC, &regval )) != 0)
    return status;

  status = fpgaWrite( mBoardNo, REGOFFSET_IBC, (regval & 0xFFFC0000) | threshold );

  return status;
}
