//-------------------------------------------------------------------------------------------------
//
// File Name:
//
//    GscApiAnalog.cpp
//
// Description:
//
//    This module contains the C <--> C++ "bridge" functions for the analog board portion of the
//    GscApi interface. It contains analog board functions externed with C linkage so they can be
//    called from the body of C code in the GscApi library at large. The implementation of the
//    analog board API lies in the AnalogBoard-derived classes. Each function in this file simply
//    calls the appropriate class method to perform the work required by the function.
//
// Programmer's notes:
//
//     06-06-2008:  Created file.  --mrg
//
//-------------------------------------------------------------------------------------------------

#include "GscApiLocal.h"
#include "time.h"
#include "GscAnalogBoard.h"
#include "GscAnaSSA.h"
#include "GscAnaDSI.h"

// Use C linkage and function name decoration for exported analog board functions
extern "C" GSCAPI_API int GscAnaGetDefaults( int boardNumber, PGSC_ANALOG_CONFIG config );
extern "C" GSCAPI_API int GscAnaGetConfig(int boardNumber, PGSC_ANALOG_CONFIG config);
extern "C" GSCAPI_API int GscAnaSetConfig(int boardNumber, GSC_ANALOG_CONFIG config);
extern "C" GSCAPI_API int GscAnaGetOption( int boardNumber, enum GSC_OPTION_NAME optionName, int optionValue[]);
extern "C" GSCAPI_API int GscAnaSetOption( int boardNumber, enum GSC_OPTION_NAME optionName, int optionValue);
extern "C" GSCAPI_API int GscAnaLocalRegisterRead( int BoardNo, int reg, int *result);
extern "C" GSCAPI_API int GscAnaLocalRegisterWrite(int boardNumber, int reg, int value);
extern "C" GSCAPI_API int GscAnaScanInputs( int boardNumber, int samples, double samplesPerSecond,
                                            int firstChannel, int lastChannel, char *buffer);
extern "C" GSCAPI_API int GscAnaAutoCal( int boardNumber, int WaitForCompletion);
extern "C" GSCAPI_API int GscAnaRegisterInterrupt( int boardNumber, int interrupt, GSC_CB_FUNCTION *function);
extern "C" GSCAPI_API int GscAnaReceiveScan( int boardNumber, char *buffer, int *count, int *transferStatus);
extern "C" GSCAPI_API int GscAnaInitializeBoard( int BoardNo );
extern "C" GSCAPI_API int GscAnaGetAutoCalStatus( int BoardNo, int *sts );
extern "C" GSCAPI_API int GscAnaClearInputBuffer( int BoardNo );
extern "C" GSCAPI_API int GscAnaGetInputBufferCount( int BoardNo, int *BuffSize );
extern "C" GSCAPI_API int GscAnaSetInputBufferThreshold( int BoardNo, int threshold );
extern "C" GSCAPI_API int GscAnaSoftwareSync( int BoardNo );

extern "C" int gscCreateBoardObj( int BoardNo );
extern "C" int gscAnaInvokeInterruptCallback( int BoardNo );
extern "C" int gscAnaClearInterrupts( int BoardNo );

// C-callable wrapper functions to access reg offset static memebers in AnalogBoard-derived classes
extern "C" int gscAnaGetIDBRegOffset( int BoardNo );

extern "C" GSC_CB_FUNCTION *AnalogInterruptRoutines[ MAXIMUM_NUMBER_OF_BOARDS][4];

// global array to hold AnalogBoard-derived objects

GscAnalogBoard *Bds[10];



//*-------------------------------------------------------------------gscCreateBoardObj------------
//*
//* Description:
//*
//*     Creates a new GscAnalogBoard-derived object and adds it to the global array of analog
//*     board objects. The class constructor called is dictated by the type of the board
//*     designated by the BoardNo parameter. The board type is currently maintained by the GscApi
//*     library in the global boardInformation[] array.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board to be represented by the object.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int gscCreateBoardObj( int BoardNo )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  // instantiate correct analogboard object based on subid determined during open
  if (!strcmp(boardInformation[BoardNo-1].subId, "-16AI64SSA/C"))
    Bds[BoardNo - 1] = new GscAna6616AI64SSA( BoardNo );
  else if (!strcmp(boardInformation[BoardNo-1].subId, "-24SDI12"))
    Bds[BoardNo - 1] = new GscAna24DSI12( BoardNo );

  return status;
}



//*-------------------------------------------------------------------GscAnaLocalRegisterRead------
//*
//* Description:
//*
//*     This routine is used to read a local register (FPGA) on the GSC analog boards.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*     int   reg                     The offset of the local register to be read.
//*     int   *result                 The contents of the register.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaLocalRegisterRead( int BoardNo, int reg, int *result)
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  return Bds[BoardNo - 1]->LocalRegisterRead( reg, result );
}



//*-------------------------------------------------------------------GscAnaLocalRegisterWrite-----
//*
//* Description:
//*
//*     This routine is used to write to a local register (FPGA) on the GSC analog boards.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*     int   reg                     The offset of the local register to write.
//*     int   value                   The value to write to the register.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaLocalRegisterWrite(int BoardNo, int reg, int value)
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  return Bds[BoardNo - 1]->LocalRegisterWrite( reg, value );
}



//*-------------------------------------------------------------------GscAnaClearInputBuffer-------
//*
//* Description:
//*
//*     This routine resets (empties) the input data buffer on a GSC analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaClearInputBuffer( int BoardNo )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->ClearInputBuffer();

  return status;
}



//*----------------------------------------------------------------GscAnaSetInputBufferThreshold---
//*
//* Description:
//*
//*     This routine sets the input buffer threshold.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*     int   threshold               The threshold value.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaSetInputBufferThreshold( int BoardNo, int threshold )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->SetInputBufferThreshold( threshold );
  return status;
}



//*-------------------------------------------------------------------GscAnaGetInputBufferCount----
//*
//* Description:
//*
//*     This routine retrieves the number of bytes of sample data in the input data buffer on a
//*     GSC analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*     int   *BuffSize               The size of the input data buffer in bytes.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaGetInputBufferCount( int BoardNo, int *BuffSize )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->GetInputBufferSize( BuffSize );
  return status;
}



//*-------------------------------------------------------------------gscAnaClearInterrupts--------
//*
//* Description:
//*
//*     This routine clears interrupts for the analog board identified by the BoardNo parameter.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int gscAnaClearInterrupts( int BoardNo )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->ClearInterrupts( );
  return status;
}



//*-------------------------------------------------------------------gscAnaInvokeInterruptCallback
//*
//* Description:
//*
//*     This routine calls the InvokeInterruptCallback() method of the analog board class, which
//*     in turn invokes the callback function for an interrupt. This function is called by the
//*     GscApi library interrupt handler thread when a Plx interrupt occurs for an analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int gscAnaInvokeInterruptCallback( int BoardNo )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->InvokeInterruptCallback( );
  return status;
}



//*-------------------------------------------------------------------GscAnaGetDefaults------------
//*
//* Description:
//*
//*     This routine fills in the provided ANALOG Configuration structure with default values.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int                 BoardNo   The Plx number of the board.
//*     PGSC_ANALOG_CONFIG  BoardNo   A pointer to the configuration structure to fill with
//*                                   default values.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaGetDefaults( int BoardNo, PGSC_ANALOG_CONFIG pConfig )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->GetConfigDefaults( pConfig );
  return status;
}



//*-------------------------------------------------------------------GscAnaGetConfig--------------
//*
//* Description:
//*
//*     This routine returns a configuration structure populated with the currently set values
//*     for the analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int                 BoardNo   The Plx number of the board.
//*     PGSC_ANALOG_CONFIG  BoardNo   A pointer to the configuration structure to fill with
//*                                   the current values.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaGetConfig(int BoardNo, PGSC_ANALOG_CONFIG config)
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->GetConfig(config);
  return status;
}



//*-------------------------------------------------------------------GscAnaSetConfig--------------
//*
//* Description:
//*
//*     This routine uses the input configuration structure to setup an analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int                 BoardNo   The Plx number of the board.
//*     GSC_ANALOG_CONFIG   config    The configuration structure containing the values to set.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaSetConfig(int BoardNo, GSC_ANALOG_CONFIG config)
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->SetConfig( config );
  return status;
}



//*-------------------------------------------------------------------GscAnaGetOption--------------
//*
//* Description:
//*
//*     This routine gets the value of the identified option for an analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int             BoardNo       The Plx number of the board.
//*     GSC_OPTION_NAME optionName    The option whose value is to be retrieved
//*     int             optionValue[] The array to be populated with the current option value.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaGetOption( int BoardNo, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->GetOption( optionName, optionValue );
  return status;
}



//*-------------------------------------------------------------------GscAnaSetOption--------------
//*
//* Description:
//*
//*     This routine sets the identified analog board option to the supplied value.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int             BoardNo       The Plx number of the board.
//*     GSC_OPTION_NAME optionName    The option whose value is to be retrieved
//*     int             optionValue   The new value to which the option will be set.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaSetOption( int BoardNo,
                     enum GSC_OPTION_NAME optionName,
                     int optionValue)
{
  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->SetOption( optionName, optionValue );
  return status;
}



//*-------------------------------------------------------------------GscAnaScanInputs-------------
//*
//* Description:
//*
//*     This routine scans a set of channels.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int     BoardNo               The Plx number of the board.
//*     int     samples               The number of samples to scan.
//*     double  samplesPerSecond      The sampling rate in Hz.
//*     int     firstChannel          The first channel to scan.
//*     int     lastChannel           The last channel to scan.
//*     char    *buffer               The buffer that will hold the scanned values.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaScanInputs( int BoardNo,
                      int samples,
                      double samplesPerSecond,
                      int firstChannel,
                      int lastChannel,
                      char *buffer)
{

  int status = 0;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->ScanInputs( samples, samplesPerSecond, firstChannel, lastChannel, buffer );
  return status;
}



//*-------------------------------------------------------------------GscAnaAutoCal----------------
//*
//* Description:
//*
//*     This routine executes the auto calibration function on the analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int     BoardNo               The Plx number of the board.
//*     int     waitForCompletion     Boolean value indicating whether to poll for autocal
//*                                   completion
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaAutoCal( int BoardNo, int waitForCompletion)
{
  int status = 0x00;
  bool bWaitOrNot = true;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  if (waitForCompletion == 0)
    bWaitOrNot = false;

  status = Bds[BoardNo - 1]->AutoCal( bWaitOrNot );
  return status;
}



//*-------------------------------------------------------------------GscAnaReceiveScan------------
//*
//* Description:
//*
//*     This routine checks to see if a sample of size equal to count has been scanned into the
//*     analog board data buffer. If the data has been scanned, a DMA transfer is performed to
//*     return the data to the supplied buffer. If not, the routine returns immediately.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int             BoardNo       The Plx number of the board.
//*     char    *buffer               The buffer that will hold the scanned values.
//*     char    *count                The buffer number of bytes written to the buffer.
//*     char    *transferStatus       Currently not used.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaReceiveScan(
                  int     BoardNo,
                  char    *buffer,
                  int     *count,
                  int     *transferStatus)
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->ReceiveScan( buffer, count, transferStatus );
  return status;
}



//*-------------------------------------------------------------------GscAnaRegisterInterrupt------
//*
//* Description:
//*
//*     This routine is used to register a callback function for a specific analog board interrupt.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int             BoardNo       The Plx number of the board.
//*     int             interrupt     The interrupt for which the callback will be registered.
//*     GSC_CB_FUNCTION *function     The callback function.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaRegisterInterrupt( int BoardNo,
                             int interrupt,
                             GSC_CB_FUNCTION *function)
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  status = Bds[BoardNo - 1]->RegisterInterrupt( interrupt, function );
  return status;
}



//*-------------------------------------------------------------------gscAnaGetIDBRegOffset-------
//*
//* Description:
//*
//*     This routine is used to retrieve the input data buffer register offset for an analog
//*     board. This is used by the GscApi dma thread when transfers are being done.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int     BoardNo               The Plx number of the board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int gscAnaGetIDBRegOffset( int BoardNo )
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  return Bds[BoardNo - 1]->GetIDBRegOffset();
}



//*-------------------------------------------------------------------gscAnaGetNumChannelsPresent--
//*
//* Description:
//*
//*     This routine gets the number of input channels present in the hardware.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int gscAnaGetNumChannelsPresent( int BoardNo )
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  return Bds[BoardNo - 1]->GetNumChannelsPresent();
}



//*-------------------------------------------------------------------GscAnaInitializeBoard--------
//*
//* Description:
//*
//*     This routine starts the initialization function on an analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaInitializeBoard( int BoardNo )
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  // Verify that the board has been opened.
  if ((drvMajorVersion == 6 && drvMinorVersion >=1 &&sio4BoardHandles[BoardNo - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      ((drvMajorVersion == 5 || (drvMajorVersion == 6 && drvMinorVersion == 0)) && sio4BoardHandles6[BoardNo - 1].hDevice == (PLX_DRIVER_HANDLE)0) ||
      (drvMajorVersion < 5 && boardInformation[BoardNo - 1].handle == (HANDLE)0))
  {
    status = GscOpen( BoardNo, GSC_API_VERSION);
    if (status) return status;
  }

  return Bds[BoardNo - 1]->Initialize();
}



//*-------------------------------------------------------------------GscAnaGetAutoCalStatus-------
//*
//* Description:
//*
//*     This function gets the status of the autocalibration function on an analog board.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the board.
//*     int   *sts                    The returned status, constructed by logically or'ing of the
//*                                   following values:
//*                                       GSC_ANA_AUTOCAL_INIT
//*                                       GSC_ANA_AUTOCAL_PASS
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaGetAutoCalStatus( int BoardNo, int *sts )
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  return Bds[BoardNo - 1]->GetAutoCalStatus( sts );
}



//*-------------------------------------------------------------------GscAnaSoftwareSync-----------
//*
//* Description:
//*
//*     This routine initiates a software sync operation between a target board and an
//      initiator board in an externally-synced dual board configuration.
//*
//* Parameters:
//*
//*     Parameter                     Description
//*------------------------------------------------------------------------------------------------
//*     int   BoardNo                 The Plx number of the initiator board.
//*
//* Return Value:
//*     0 - success, > 0 GscApi error
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscAnaSoftwareSync( int BoardNo )
{
  int status = 0x00;

  // Make sure the board number is valid
  status = validateBoardNumber( BoardNo );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[BoardNo-1].boardType != GSC_BOARD_TYPE_ANALOG)
    return GscApiInvalidBoardType;

  return Bds[BoardNo - 1]->SoftwareSync();
}
