//////////////////////////////////////////////////////////////////////////////
//
// GscApiAsync.c
//
//  This module contains the routines that handle the configuration of -ASYNC
// mode of operation.
//
#include "GscApiLocal.h"

//---------------------------------------------- GscSio4ChannelGetAsyncDefaults
//
//  This routine fills in the provided ASYNC Configuration
// structure with default values.
//
int GscSio4AsyncGetDefaults( PGSC_ASYNC_CONFIG config )
{
  int status = 0x00;

  // Setup the Channel defaults
  config->bitRate = 1000000;
  config->encoding = GSC_ENCODING_NRZ;
  config->protocol = GSC_PROTOCOL_RS422_RS485;
  config->termination = GSC_TERMINATION_ENABLED;
  config->parity = GSC_PARITY_NONE;
  config->stopBits = GSC_STOP_BITS_1;

  // Setup the Transmitter defaults
  config->txStatus = GSC_ENABLED;
  config->txCharacterLength = 8;
  config->txClockSource = GSC_CLOCK_INTERNAL;

  // Setup the Receiver defaults
  config->rxStatus = GSC_ENABLED;
  config->rxCharacterLength = 8;
  config->rxClockSource = GSC_CLOCK_INTERNAL;

  // Setup the Pin Configuration Variables
  config->interfaceMode = GSC_PIN_DTE;
  config->txDataPinMode = GSC_PIN_AUTO;
  config->rxDataPinMode = GSC_PIN_AUTO;
  config->txClockPinMode = GSC_PIN_GPIO;
  config->rxClockPinMode = GSC_PIN_GPIO;
  config->rtsPinMode = GSC_PIN_GPIO;
  config->ctsPinMode = GSC_PIN_GPIO;
  config->loopbackMode = GSC_LOOP_NONE;

  // Setup FIFO feature configuration fields
  config->fifoTimetagByte = GSC_DISABLED;
  config->fifoParityErrorByte = GSC_DISABLED;

  return status;
}


//------------------------------------------------ GscSio4ChannelAsyncGetConfig
//
//  This routine returns the current configuration
// structure for a single channel that is set to ASYNC
// mode.
//
int GscSio4AsyncGetConfig(int boardNumber, int channel, PGSC_ASYNC_CONFIG config)
{
  int status = 0;
  int value = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
    return GscApiInvalidBoardType;

  // If the channel is not configured, get the defaults
  if (boardInformation[ boardNumber-1].channelMode[ channel-1] == GSC_MODE_UNINITIALIZED)
  {
    // Load the default values
    status = GscSio4AsyncGetDefaults( config);
    return status;
  }

  // Get the data character lengths
  status = zilogGetDataLength( boardNumber, channel, &config->txCharacterLength, &config->rxCharacterLength);
  if (status) return status;

  // Determine the parity mode
  status = gscSio4ChannelGetParity( boardNumber, channel, &config->parity);
  if (status) return status;

  // Get the Stop Bit value
  config->stopBits = boardInformation[ boardNumber-1].channelStopBits[ channel-1];

  // Determine the encoding and bit rate
  config->encoding = boardInformation[ boardNumber-1 ].channelEncoding[ channel-1 ];
  config->bitRate = boardInformation[ boardNumber-1 ].channelBitRate[ channel-1 ];

  // Determine the protocol and termination settings
  status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol,
                                      &config->termination);
  if (status) return( status );

  // Determine the various pin modes
  config->interfaceMode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
                                     &config->txDataPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
                                     &config->rxDataPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
                                     &config->txClockPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
                                     &config->rxClockPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
                                     &config->ctsPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
                                     &config->rtsPinMode);
  if (status) return( status );

  // Determine the loopback mode
  config->loopbackMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

  // Determine the clock sources
  status = gscsio4ChannelGetClockSource( boardNumber, channel, &config->txClockSource,
                                         &config->rxClockSource);
  if (status) return( status );

  // Determine the receiver status
  status = zilogGetReceiverControl( boardNumber, channel, &config->rxStatus);
  if (status) return status;

  // Determine the transmitter status
  status = zilogGetTransmitterControl( boardNumber, channel, &config->txStatus);
  if (status) return status;

  // Determine the status of the FIFO features
  config->fifoTimetagByte = GSC_DISABLED;
  config->fifoParityErrorByte = GSC_DISABLED;

  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);
  if (status) return status;

  if (value & 0x04)
	config->fifoTimetagByte = GSC_ENABLED;
  if (value & 0x08)
	config->fifoParityErrorByte = GSC_ENABLED;

  return status;
}


//------------------------------------------------ GscSio4ChannelAsyncSetConfig
//
//  This routine sets the current configuration
// structure for a single channel that is set to ASYNC
// mode.
//
int GscSio4AsyncSetConfig(int boardNumber, int channel, GSC_ASYNC_CONFIG config)
{
  int status = 0x00;
  int value = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel );
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
    return GscApiInvalidBoardType;

  // Save some global info
  boardInformation[ boardNumber-1].channelMode[ channel-1] = GSC_MODE_ASYNC;
  boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)config.bitRate/1000/8;

  // If the board is a BXR, change the channel to standard mode
  if (!strcmp(boardInformation[boardNumber-1].subId, "-SIO4BXR"))
  {
    int chMode = 0;

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    if (status) return status;

    chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
    chMode |= (0x01 << ((channel - 1) * 8)); // set channel type to standard

    status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
    if (status) return status;
  }

  // Set to Async Mode, 16x clock, one stop bit
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, 0x0000);
  if (status) return status;

  // Set the data character lengths
  status = zilogSetDataLength( boardNumber, channel, config.txCharacterLength, config.rxCharacterLength);
  if (status) return status;

  // Setup the parity mode
  status = quickSetParity( boardNumber, channel, config.parity);
  if (status) return status;

  // Setup stop bits
  status = gscSio4ChannelSetStopBits( boardNumber, channel, config.stopBits );
  if (status) return status;

  // Setup the encoding and bit rate
  zilogSetEncoding( boardNumber, channel, config.encoding, GSC_MODE_ASYNC, config.bitRate);
  if (status) return status;

  // Setup the protocol and termination
  status = quickSetProtocol( boardNumber, channel, config.protocol, config.termination);
  if (status) return status;

  // Setup the Pin Modes
  status = quickSetDteDce(boardNumber, channel, config.interfaceMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, config.txDataPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, config.rxDataPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, config.txClockPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, config.rxClockPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, config.ctsPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, config.rtsPinMode);
  if (status) return status;

  // Setup the loopback mode
  status = quickSetLoopBack( boardNumber, channel, config.loopbackMode);
  if (status) return status;

  // Setup the clock sources
  status = quickNormalSetClockSource( boardNumber, channel, config.txClockSource, config.rxClockSource);
  if (status) return status;

  // Enable/Disable the receiver
  status = zilogReceiverControl( boardNumber, channel, config.rxStatus);
  if (status) return status;

  // Enable/Disable the Transmitter (no auto enables)
  status = zilogTransmitterControl( boardNumber, channel, config.txStatus);
  if (status) return status;

  // Enable/Disable the FIFO features
  status = gscSio4ChannelSetFifoTimetagByte( boardNumber, channel, config.fifoTimetagByte );
  if (status) return status;
  status = gscSio4ChannelSetFifoParityErrorByte( boardNumber, channel, config.fifoParityErrorByte );

  //----
  // Make sure packet framing is disabled.
  //----

  boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = GSC_DISABLED;

  // Get the current Channel Control Register and make sure packet framing is off
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
  if (status) return status;

  // Clear the TCB type bits
  value &= ~0xc000;

	// Write the new Channel Control Register value
	status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, value);
	if (status) return status;

  return status;
}

//----------------------------------------------------- GscSio4AsyncGetOption
//
//  This routine gets the value of the identified option.
//
int gscSio4AsyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status = 0x00;

  switch ( optionName )
  {
  case GSC_SIO_DATASIZE:
    {
      int *dataSize = &optionValue[0];
      status = zilogGetDataLength( boardNumber, channel, dataSize, dataSize);
    }
    break;

  case GSC_SIO_STOPBITS:
    optionValue[0] = boardInformation[ boardNumber-1].channelStopBits[ channel-1];
    break;


  case GSC_SIO_TXDATAPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RXDATAPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
                                       &optionValue[0]);
    break;

  case GSC_SIO_TXCLOCKPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RXCLOCKPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
                                       &optionValue[0]);
    break;

  case GSC_SIO_CTSPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RTSPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RECEIVER:
    {
      int *rxMode = &optionValue[0];

      status = zilogGetReceiverControl( boardNumber, channel, rxMode);
    }
    break;

  case GSC_SIO_TRANSMITTER:
    {
      int *txMode  = &optionValue[0];

      status = zilogGetTransmitterControl( boardNumber, channel, txMode);
    }
    break;

  case GSC_SIO_FIFO_TIMETAG_BYTE:
    {
      int value = 0;

      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);

      optionValue[0] = GSC_DISABLED;
      if (value & 0x04)
	      optionValue[0] = GSC_ENABLED;
    }
    break;

  case GSC_SIO_FIFO_PARITYERROR_BYTE:
    {
      int value = 0;

      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);

      optionValue[0] = GSC_DISABLED;
      if (value & 0x08)
	      optionValue[0] = GSC_ENABLED;
    }
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

//----------------------------------------------------- GscSio4AsyncSetOption
//
//  This routine sets the identified option to the supplied value.
//
int gscSio4AsyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
  int status = 0;
  int oldValue = 0;
  int newValue  = 0;
  int newValue2 = 0;

  switch ( optionName )
  {
  case GSC_SIO_DATASIZE:
    {
      int dataSize = optionValue;
      status = zilogSetDataLength( boardNumber, channel, dataSize, dataSize);
      if (status) return status;
    }
    break;

  case GSC_SIO_ENCODING:
    {
      int encoding = optionValue;

      // Convert board and channel into indexes
      int bn = boardNumber - 1;
      int ch = channel - 1;

      return zilogSetEncoding( boardNumber,
                               channel,
                               encoding,
                               boardInformation[ bn].channelMode[ ch],
                               boardInformation[ bn].channelBitRate[ ch]);
    }

    break;

  case GSC_SIO_STOPBITS:
    // Setup stop bits
    {
      int stopBits = optionValue;

      // Handle the proper stop bits setting
      switch( stopBits)
      {
      case GSC_STOP_BITS_0:	newValue = 0x8000; newValue2 = 0x0000; break;
      case GSC_STOP_BITS_1:	newValue = 0x0000; newValue2 = 0x0000; break;
      case GSC_STOP_BITS_1_5:	newValue = 0xc000; newValue2 = 0x0700; break;
      case GSC_STOP_BITS_2:	newValue = 0x4000; newValue2 = 0x0000; break;
      default: return GscApiInvalidParameter;
      }

      // Save the Stop Bit value
      boardInformation[ boardNumber-1].channelStopBits[ channel-1] = stopBits;

      // Get current Channel Mode Register Value
      status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &oldValue);
      if (status) return status;

      // Update the value and write it back out
      newValue = (oldValue & ~0xc000) | newValue;
      status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, newValue);
      if (status) return status;

      // Get current Channel Control Register Value
      status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &oldValue);
      if (status) return status;

      // Update the value and write it back out
      newValue = (oldValue & ~0x0f00) | newValue2;
      status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, newValue);
    }

    break;

  case GSC_SIO_TXDATAPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, optionValue);
    break;

  case GSC_SIO_RXDATAPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, optionValue);
    break;

  case GSC_SIO_TXCLOCKPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, optionValue);
    break;

  case GSC_SIO_RXCLOCKPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, optionValue);
    break;

  case GSC_SIO_CTSPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, optionValue);
    break;

  case GSC_SIO_RTSPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, optionValue);
    break;

  case GSC_SIO_RECEIVER:
    {
      int rxMode = optionValue;

      status = zilogReceiverControl( boardNumber, channel, rxMode);
    }
    break;

  case GSC_SIO_TRANSMITTER:
    {
      int txMode  = optionValue;

      status = zilogTransmitterControl( boardNumber, channel, txMode);
    }

    break;

  case GSC_SIO_FIFO_TIMETAG_BYTE:
    {
      status = gscSio4ChannelSetFifoTimetagByte( boardNumber, channel, optionValue );
    }
    break;

  case GSC_SIO_FIFO_PARITYERROR_BYTE:
    {
      status = gscSio4ChannelSetFifoParityErrorByte( boardNumber, channel, optionValue );
    }
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

