//////////////////////////////////////////////////////////////////////////////
//
// GscApiBiSync.c
//
//  This module contains all of the routines that handle the BiSync mode
// operation.
//
#include "GscApiLocal.h"


//------------------------------------------------- GscSio4BiSyncGetDefaults
//
//  This routine fills in the provided BiSync Configuration
// structure with default values.
//
int GscSio4BiSyncGetDefaults( PGSC_BISYNC_CONFIG config )
{
	int status = 0;

	// Setup the Channel defaults
	config->bitRate = 1000000;
	config->encoding = GSC_ENCODING_NRZ;
	config->protocol = GSC_PROTOCOL_RS422_RS485;
	config->termination = GSC_TERMINATION_ENABLED;
	config->parity = GSC_PARITY_NONE;
	config->crcMode = GSC_CRC_NONE;
	config->crcInitialValue = GSC_CRC_INIT_0;

	// Setup the Transmitter defaults
	config->txStatus = GSC_ENABLED;
	config->txCharacterLength = 8;
	config->txClockSource = GSC_CLOCK_INTERNAL;
	config->txIdleCondition = GSC_IDLE_ALL_0;
	config->txSyncWord = 0x0000;
	config->txUnderRun = GSC_SYN1;
	config->txPreamble = GSC_ENABLED;
	config->txPreambleLength = GSC_PREAMBLE_8BITS;
	config->txPreamblePattern = GSC_PREAMBLE_ALL_0;
	config->txShortSync = GSC_DISABLED;

	// Setup the Receiver defaults
	config->rxStatus = GSC_ENABLED;
	config->rxCharacterLength = 8;
	config->rxClockSource = GSC_CLOCK_INTERNAL;
	config->rxSyncWord = 0x0000;
	config->rxSyncStrip = GSC_ENABLED;
	config->rxShortSync = GSC_DISABLED;

	// Setup the Pin Configuration Variables
	config->interfaceMode = GSC_PIN_DTE;
	config->txDataPinMode = GSC_PIN_AUTO;
	config->rxDataPinMode = GSC_PIN_AUTO;
	config->txClockPinMode = GSC_PIN_AUTO;
	config->rxClockPinMode = GSC_PIN_AUTO;
	config->ctsPinMode = GSC_PIN_GPIO;
	config->rtsPinMode = GSC_PIN_GPIO;
	config->loopbackMode = GSC_LOOP_NONE;

	config->packetFraming = GSC_DISABLED;

	return status;
}

//--------------------------------------------------- gscSio4ChannelBiSyncInit
//
//  This routine initializes a single channel to BiSync
// mode using the default parameters.
//
int gscSio4BiSyncInit(int boardNumber, int channel)
{
	int status = 0x00;
	GSC_BISYNC_CONFIG config;

	// Setup the defaults
	status = GscSio4BiSyncGetDefaults( &config );
	if (status) return status;

	// Program the channel
	status = GscSio4BiSyncSetConfig( boardNumber, channel, config);
	if (status) return status;

	return status;
}


//------------------------------------------------ GscSio4BiSyncGetConfig
//
//  This routine returns the current configuration
// structure for a single channel that is set to BiSync
// mode.
//
int GscSio4BiSyncGetConfig( int boardNumber, int channel, PGSC_BISYNC_CONFIG config )
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected
	if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
	  return GscApiInvalidBoardType;

  // If the channel is not configured, get the defaults
	if (boardInformation[ boardNumber-1].channelMode[ channel-1] == GSC_MODE_UNINITIALIZED)
	{
		// Load the default values
		status = GscSio4BiSyncGetDefaults( config );
		return status;
	}

	// Get the data character lengths
	status = zilogGetDataLength( boardNumber, channel, &config->txCharacterLength, &config->rxCharacterLength);
	if (status) return status;

	// Determine the encoding and bit rate
	config->encoding = boardInformation[ boardNumber-1 ].channelEncoding[ channel-1 ];
	config->bitRate = boardInformation[ boardNumber-1 ].channelBitRate[ channel-1 ];

	// Determine the protocol and termination settings
	status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol,
		&config->termination);
	if (status) return( status );

	// Determine the parity mode
	status = gscSio4ChannelGetParity( boardNumber, channel, &config->parity);
	if (status) return status;

	// Determine the CRC mode and initial value
	status = zilogGetCrcMode( boardNumber, channel, &config->crcMode, &config->crcInitialValue);
	if (status) return status;

	// Determine the idle line condition
	status = zilogGetIdleLineCondition( boardNumber, channel, &config->txIdleCondition);
	if (status) return status;

	// Get the Sync Words
	status = gscSio4ChannelGetBiSyncPattern( boardNumber, channel, &config->txSyncWord, &config->rxSyncWord);
    if (status) return status;

    // Get the Tx Under Run condition
    status = gscSio4ChannelGetBiSyncTxUnderrun( boardNumber, channel, &config->txUnderRun);
    if (status) return status;

    // Get the Tx Preamble Enable
    status = gscSio4ChannelGetBiSyncTxPreamble( boardNumber, channel, &config->txPreamble);
    if (status) return status;

    // Get the Tx Preamble Length
    status = gscSio4ChannelGetBiSyncTxPreambleLength( boardNumber, channel, &config->txPreambleLength);
    if (status) return status;

    // Get the Tx Preamble Pattern
    status = gscSio4ChannelGetBiSyncTxPreamblePattern( boardNumber, channel, &config->txPreamblePattern);
    if (status) return status;

    // Get the Tx Short Sync mode
    status = gscSio4ChannelGetBiSyncTxShortSync( boardNumber, channel, &config->txShortSync);
    if (status) return status;

    // Get the Rx Sync Strip mode
    status = gscSio4ChannelGetBiSyncRxSyncStrip( boardNumber, channel, &config->rxSyncStrip);
    if (status) return status;

    // Get the Rx Short Sync mode
    status = gscSio4ChannelGetBiSyncRxShortSync( boardNumber, channel, &config->rxShortSync);
    if (status) return status;

	// Determine the various pin modes
	config->interfaceMode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
		&config->txDataPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
		&config->rxDataPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
		&config->txClockPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
		&config->rxClockPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
		&config->ctsPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
		&config->rtsPinMode);
	if (status) return( status );

	// Determine the loopback mode
	config->loopbackMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

	// Determine the clock sources
	status = gscsio4ChannelGetClockSource( boardNumber, channel, &config->txClockSource,
		&config->rxClockSource);
	if (status) return( status );

	// Determine the receiver mode
	status = zilogGetReceiverControl( boardNumber, channel, &config->rxStatus);
	if (status) return status;

	// Determine the transmitter mode
	status = zilogGetTransmitterControl( boardNumber, channel, &config->txStatus);
	if (status) return status;

	// Get the packet framing mode
	config->packetFraming = boardInformation[ boardNumber-1].channelPacketFraming[ channel-1];
	return status;
}


//------------------------------------------------ GscSio4ChannelBiSyncSetConfig
//
//  This routine sets the current configuration
// structure for a single channel that is set to BiSync
// mode.
//
int GscSio4BiSyncSetConfig(int boardNumber, int channel, GSC_BISYNC_CONFIG config)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected
	if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
	  return GscApiInvalidBoardType;

	// Save some global info
	boardInformation[ boardNumber-1].channelMode[ channel-1] = GSC_MODE_BISYNC;
	boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)config.bitRate/1000/8;

  // If the board is a BXR, change the channel to standard mode
  if (!strcmp(boardInformation[boardNumber-1].subId, "-SIO4BXR"))
  {
    int chMode = 0;

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    if (status) return status;

    chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
    chMode |= (0x01 << ((channel - 1) * 8)); // set channel type to standard

    status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
    if (status) return status;
  }

  // Set the data character lengths
	status = zilogSetDataLength( boardNumber, channel, config.txCharacterLength, config.rxCharacterLength);
	if (status) return status;

	// Setup the encoding and bit rate
	zilogSetEncoding( boardNumber, channel, config.encoding, GSC_MODE_HDLC, config.bitRate);
	if (status) return status;

	// Setup the protocol and termination
	status = quickSetProtocol( boardNumber, channel, config.protocol, config.termination);
	if (status) return status;

	// Setup the parity mode
	status = quickSetParity( boardNumber, channel, config.parity);
	if (status) return status;

	// Setup the CRC mode
	status = zilogSetCrcMode( boardNumber, channel, config.crcMode, config.crcInitialValue);
	if (status) return status;

	// Set the Sync Words
	status = gscSio4ChannelSetBiSyncPattern( boardNumber, channel, config.txSyncWord, config.rxSyncWord);
	if (status) return status;

	//
	// Program the Channel Mode Register
	//

	// Set both Tx and Rx to BiSync
	value = 0x0505;

	// Set the Tx Underrun Condition
	switch (config.txUnderRun)
	{
	case GSC_SYN1:							 break;
	case GSC_SYN0_SYN1:		value |= 0x4000; break;
	case GSC_CRC_SYN1:		value |= 0x8000; break;
	case GSC_CRC_SYN0_SYN1:	value |= 0xc000; break;
	}

	// Set the Tx Preamble
	if (config.txPreamble == GSC_ENABLED) value |= 0x2000;

	// Set the Tx Short Sync
	if (config.txShortSync == GSC_ENABLED) value |= 0x1000;

	// Set the Rx Sync Strip
	if (config.rxSyncStrip == GSC_ENABLED) value |= 0x0020;

	// Set the Rx Short Sync
	if (config.rxShortSync == GSC_ENABLED) value |= 0x0010;

	// Write the new Mode Register value
	status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, value);
	if (status) return status;

//
// Program the Channel Control Register
//
	// Start with nothing set
	value = 0x00;

	// Set the Tx Preamble Length bits
	switch (config.txPreambleLength)
	{
	case GSC_PREAMBLE_8BITS:					 break;
	case GSC_PREAMBLE_16BITS:	value |= 0x0400; break;
	case GSC_PREAMBLE_32BITS:	value |= 0x0800; break;
	case GSC_PREAMBLE_64BITS:	value |= 0x0c00; break;
	}

	// Set the Tx Preamble Pattern bits
	switch (config.txPreamblePattern)
	{
	case GSC_PREAMBLE_ALL_0:					 break;
	case GSC_PREAMBLE_ALL_1:	value |= 0x0100; break;
	case GSC_PREAMBLE_ALL_1_0:	value |= 0x0200; break;
	case GSC_PREAMBLE_ALL_0_1:	value |= 0x0300; break;
	}

	// Write the new Channel Control Register value
	status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, value);
	if (status) return status;

	// Setup the idle line condition
	status = zilogSetIdleLineCondition( boardNumber, channel, config.txIdleCondition);
	if (status) return status;

	// Setup the Pin Modes
	status = quickSetDteDce(boardNumber, channel, config.interfaceMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, config.txDataPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, config.rxDataPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, config.txClockPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, config.rxClockPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, config.ctsPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, config.rtsPinMode);
	if (status) return status;

	// Setup the loopback mode
	status = quickSetLoopBack( boardNumber, channel, config.loopbackMode);
	if (status) return status;

	// Setup the clock source
	status = quickNormalSetClockSource( boardNumber, channel, config.txClockSource, config.rxClockSource);
	if (status) return status;

	// Enable/Disable the receiver
	status = zilogReceiverControl( boardNumber, channel, config.rxStatus);
	if (status) return status;

	// Enable/Disable the Transmitter (no auto enables)
	status = zilogTransmitterControl( boardNumber, channel, config.txStatus);
	if (status) return status;

//
// Program the Packet Framing mode
//
	// Get the current Channel Control Register
	status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
	if (status) return status;

	// Clear the TCB type bits
	value &= ~0xc000;

	// If packet framing is enabled
	if (config.packetFraming == GSC_ENABLED)
	{
		// Set the channel to use 32 bit TCBs
		value |=  0x8000;

		// Write the new Channel Control Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, value);
		if (status) return status;

		// Trigger the DMA
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCAR, 0x7000);
		if (status) return status;

		// Load the Receive Count Limit Register with -1
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_RCLR, 0xffff);
		if (status) return status;

		// Enable Zilog Rx Status IRQs and set Master Interrupt Enable
		status = GscSio4ChannelRegisterWrite(boardNumber, channel, USC_ICR, 0x80E0);
		if (status) return status;

		// Arm the Zilog Rx EOM Interrupt
		status = GscSio4ChannelRegisterWrite(boardNumber, channel, USC_RICR, 0x10);
		if (status) return status;

		// Register the interrupt service routine for the USC channel
		GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_USC, GSC_RISING_EDGE, uscInterruptServiceRoutine);
	}

	// No packet framing, just write the new Control Register value out
	else
	{
		// Write the new Channel Control Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, value);
		if (status) return status;
	}

	// Update the channel information
	boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = config.packetFraming;

	return status;
}

//----------------------------------------------------- GscSio4BiSyncGetOption
//
//  This routine gets the value of the identified option.
//
int gscSio4BiSyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
	int status = 0x00;
	int value;

	switch ( optionName )
	{
	case GSC_SIO_DATASIZE:
		{
		int *dataSize = &optionValue[0];
		status = zilogGetDataLength( boardNumber, channel, dataSize, dataSize);
		}
		break;

	case GSC_SIO_CRCMODE:
		{
		int *crcMode      = &optionValue[0];
		int *initialValue = &optionValue[1];

		// Determine the CRC mode and initial value
		status = zilogGetCrcMode( boardNumber, channel, crcMode, initialValue);
		if (status) return status;
		}

		break;

	case GSC_SIO_SYNCWORD: // get the sync pattern
		{
		int *txSync = &optionValue[0];
		int *rxSync = &optionValue[1];

		// Get the Tx Sync Value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_TSR, txSync);
		if (status) return status;

		// Get the Rx Sync Value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_RSR, rxSync);
		}

		break;

	case GSC_SIO_TXUNDERRUN: // Get the Tx Underrun condition mode
		{
		int *txUnderrun = &optionValue[0];
		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
		if (status) return status;

		// Determine the Tx Underrun Condition
		switch (value&0xc000)
		{
		case 0x0000: *txUnderrun = GSC_SYN1;		  break;
		case 0x4000: *txUnderrun = GSC_SYN0_SYN1;	  break;
		case 0x8000: *txUnderrun = GSC_CRC_SYN1;	  break;
		case 0xc000: *txUnderrun = GSC_CRC_SYN0_SYN1; break;
		}
		}
		break;

	case GSC_SIO_TXPREAMBLE: // Get the Tx Preamble value
		{
		int *txPreamble = &optionValue[0];

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
		if (status) return status;

		// Determine the Tx Preamble
		if (value & 0x2000) *txPreamble = GSC_ENABLED;
		else *txPreamble = GSC_DISABLED;
		}

		break;

	case GSC_SIO_TXSHORTSYNC: // Get the Tx Sync length (short or 8-bit)
		{
		int *txShortSync = &optionValue[0];

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
		if (status) return status;

		// Determine the Tx Short Sync
		if (value & 0x1000) *txShortSync = GSC_ENABLED;
		else *txShortSync = GSC_DISABLED;
		}

		break;

	case GSC_SIO_RXSYNCSTRIP: // Get the Rx Sync Strip mode
		{
		int *rxSyncStrip = &optionValue[0];

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
		if (status) return status;

		// Determine the Rx Sync Strip
		if (value & 0x0020) *rxSyncStrip = GSC_ENABLED;
		else *rxSyncStrip = GSC_DISABLED;
		}

		break;

	case GSC_SIO_RXSHORTSYNC: // Get the Rx Sync Length (short or 8-bit)
		{
		int *rxShortSync = &optionValue[0];

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
		if (status) return status;

		// Determine the Rx Short Sync
		if (value & 0x0010) *rxShortSync = GSC_ENABLED;
		else *rxShortSync = GSC_DISABLED;
		}

		break;

	case GSC_SIO_TXPREAMBLELEN: // Get the Tx Preamble Length
		{
			int *txPreambleLength = &optionValue[0];
			// Read the current Channel Control Register value
			status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
			if (status) return status;

			// Get the Tx Preamble Length bits
			switch (value&0x0c00)
			{
			case 0x0000: *txPreambleLength = GSC_PREAMBLE_8BITS;  break;
			case 0x0400: *txPreambleLength = GSC_PREAMBLE_16BITS; break;
			case 0x0800: *txPreambleLength = GSC_PREAMBLE_32BITS; break;
			case 0x0c00: *txPreambleLength = GSC_PREAMBLE_64BITS; break;
			}
		}

		break;

	case GSC_SIO_TXPREAMBLEPATTERN: // Get the Tx Preamble pattern
		{
			int *txPreamblePattern = &optionValue[0];
			// Read the current Channel Control Register value
			status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
			if (status) return status;

			// Get the Tx Preamble Pattern bits
			switch (value&0x0300)
			{
			case 0x0000: *txPreamblePattern = GSC_PREAMBLE_ALL_0;	break;
			case 0x0100: *txPreamblePattern = GSC_PREAMBLE_ALL_1;	break;
			case 0x0200: *txPreamblePattern = GSC_PREAMBLE_ALL_1_0; break;
			case 0x0300: *txPreamblePattern = GSC_PREAMBLE_ALL_0_1; break;
			}
		}

		break;

	case GSC_SIO_TXIDLECONDITION:
		// Get Shared Zero mode
		status = zilogGetIdleLineCondition( boardNumber, channel, &optionValue[0]);
		break;

	case GSC_SIO_TXDATAPINMODE:
		// Setup the pin mode
		status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
			&optionValue[0]);
		break;

	case GSC_SIO_RXDATAPINMODE:
		// Setup the pin mode
		status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
			&optionValue[0]);
		break;

	case GSC_SIO_TXCLOCKPINMODE:
		// Setup the pin mode
		status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
			&optionValue[0]);
		break;

	case GSC_SIO_RXCLOCKPINMODE:
		// Setup the pin mode
		status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
			&optionValue[0]);
		break;

	case GSC_SIO_CTSPINMODE:
		// Setup the pin mode
		status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
			&optionValue[0]);
		break;

	case GSC_SIO_RTSPINMODE:
		// Setup the pin mode
		status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
			&optionValue[0]);
		break;

	case GSC_SIO_RECEIVER:
		{
		int *rxMode = &optionValue[0];

		status = zilogGetReceiverControl( boardNumber, channel, rxMode);
		}
		break;

	case GSC_SIO_TRANSMITTER:
		{
		int *txMode  = &optionValue[0];

		status = zilogGetTransmitterControl( boardNumber, channel, txMode);
		}
		break;

	default:
		return GscApiInvalidParameter;
	}

	return status;
}

//----------------------------------------------------- GscSio4BiSyncSetOption
//
//  This routine sets the identified option to the supplied value.
//
int gscSio4BiSyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
	int status = 0;
	int tempValue = 0;

	switch ( optionName )
	{
	case GSC_SIO_DATASIZE:
		{
		int dataSize = optionValue;
		status = zilogSetDataLength( boardNumber, channel, dataSize, dataSize);
		if (status) return status;
		}
		break;

	case GSC_SIO_ENCODING:
		{
		int encoding = optionValue;

		// Convert board and channel into indexes
		int bn = boardNumber - 1;
		int ch = channel - 1;

		return zilogSetEncoding( boardNumber,
								 channel,
								 encoding,
								 boardInformation[ bn].channelMode[ ch],
								 boardInformation[ bn].channelBitRate[ ch]);
		}

		break;

	case GSC_SIO_CRCMODE:
		{
		int crcMode      = (optionValue & 0xffff0000) >> 16;
		int initialValue = optionValue & 0x0000ffff;

		// Make the changes
		status = zilogSetCrcMode( boardNumber, channel, crcMode, initialValue);
		}

		break;

	case GSC_SIO_SYNCWORD:
		{
		int txSync = (optionValue & 0xffff0000) >> 16;
		int rxSync = optionValue & 0x0000ffff;

		// Write the Tx Sync Value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_TSR, txSync);
		if (status) return status;

		// Get the Rx Sync Value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_RSR, rxSync);
		}

		break;

	case GSC_SIO_TXUNDERRUN:
		{
		int txUnderrun = optionValue;

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &tempValue);
		if (status) return status;

		// Clear the bits
		tempValue &= ~0xc000;

		// Determine the Tx Underrun Condition
		switch (txUnderrun)
		{
			case GSC_SYN1:          tempValue |= 0x0000;	break;
			case GSC_SYN0_SYN1:     tempValue |= 0x4000;	break;
			case GSC_CRC_SYN1:      tempValue |= 0x8000;	break;
			case GSC_CRC_SYN0_SYN1: tempValue |= 0xc000;    break;
		}

		// Write the new Channel Mode Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, tempValue);
		}

		break;

	case GSC_SIO_TXPREAMBLE:
		{
		int txPreamble = optionValue;

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &tempValue);
		if (status) return status;

		// Enable/disable the Tx Preamble
		tempValue &= ~0x2000;
		if (txPreamble == GSC_ENABLED) tempValue |= 0x2000;

		// Write the new Channel Mode Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, tempValue);
		}

		break;

	case GSC_SIO_TXSHORTSYNC:
		{
		int txShortSync = optionValue;

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &tempValue);
		if (status) return status;

		// Enable/disable the Tx Shortsync
		tempValue &= ~0x1000;
		if (txShortSync == GSC_ENABLED) tempValue |= 0x1000;

		// Write the new Channel Mode Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, tempValue);
		}

		break;

	case GSC_SIO_RXSYNCSTRIP:
		{
		int rxSyncStrip = optionValue;

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &tempValue);
		if (status) return status;

		// Enable/disable Rx Sync stripping
		tempValue &= ~0x0020;
		if (rxSyncStrip == GSC_ENABLED) tempValue |= 0x0020;

		// Write the new Channel Mode Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, tempValue);
		}

		break;

	case GSC_SIO_RXSHORTSYNC:
		{
		int rxShortSync = optionValue;

		// Read the current Channel Mode Register value
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &tempValue);
		if (status) return status;

		// Enable/disable Rx short sync
		tempValue &= ~0x0010;
		if (rxShortSync == GSC_ENABLED) tempValue |= 0x0010;

		// Write the new Channel Mode Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMR, tempValue);
		}

		break;

	case GSC_SIO_TXPREAMBLELEN:
		{
		int txPreambleLength = optionValue;

		// Read the current Channel Control Register
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &tempValue);
		if (status) return status;

		// Set the Tx Preamble Length bits
		tempValue &= ~0x0c00;
		switch (txPreambleLength)
		{
		case GSC_PREAMBLE_8BITS:						 break;
		case GSC_PREAMBLE_16BITS:	tempValue |= 0x0400; break;
		case GSC_PREAMBLE_32BITS:	tempValue |= 0x0800; break;
		case GSC_PREAMBLE_64BITS:	tempValue |= 0x0c00; break;
		}

		// Write the new Channel Control Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, tempValue);
		if (status) return status;
		}

		break;

	case GSC_SIO_TXPREAMBLEPATTERN:
		{
		int txPreamblePattern = optionValue;

		// Read the current Channel Control Register
		status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &tempValue);
		if (status) return status;

		// Set the Tx Preamble Pattern bits
		tempValue &= ~0x0300;
		switch (txPreamblePattern)
		{
		case GSC_PREAMBLE_ALL_0:					 break;
		case GSC_PREAMBLE_ALL_1:	tempValue |= 0x0100; break;
		case GSC_PREAMBLE_ALL_1_0:	tempValue |= 0x0200; break;
		case GSC_PREAMBLE_ALL_0_1:	tempValue |= 0x0300; break;
		}

		// Write the new Channel Control Register value
		status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CCR, tempValue);
		if (status) return status;
		}

		break;

	case GSC_SIO_TXIDLECONDITION:
		// Set Shared Zero mode
		status = zilogSetIdleLineCondition( boardNumber, channel, optionValue);
		break;

	case GSC_SIO_TXDATAPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, optionValue);
		break;

	case GSC_SIO_RXDATAPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, optionValue);
		break;

	case GSC_SIO_TXCLOCKPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, optionValue);
		break;

	case GSC_SIO_RXCLOCKPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, optionValue);
		break;

	case GSC_SIO_CTSPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, optionValue);
		break;

	case GSC_SIO_RTSPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, optionValue);
		break;

	case GSC_SIO_RECEIVER:
		{
		int rxMode = optionValue;

		status = zilogReceiverControl( boardNumber, channel, rxMode);
		}
		break;

	case GSC_SIO_TRANSMITTER:
		{
		int txMode  = optionValue;

		status = zilogTransmitterControl( boardNumber, channel, txMode);
		}

	default:
		return GscApiInvalidParameter;
	}

	return status;
}

//--------------------------------------------- gscSio4ChannelGetBiSyncPattern
//--------------------------------------------- gscSio4ChannelSetBiSyncPattern
//
//  These routines are used to get/set the sync pattern for BiSync mode.
//
int gscSio4ChannelGetBiSyncPattern( int boardNumber, int channel, int *TxSync, int *RxSync)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get the Tx Sync Value
	status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_TSR, TxSync);
	if (status) return status;

	// Get the Rx Sync Value
	status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_RSR, RxSync);
	if (status) return status;

	return status;
}

int gscSio4ChannelSetBiSyncPattern( int boardNumber, int channel, int TxSync, int RxSync)
{
	int status = 0x00;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Write the Tx Sync Value
	status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_TSR, TxSync);
	if (status) return status;

	// Get the Rx Sync Value
	status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_RSR, RxSync);
	if (status) return status;

	return status;
}

//------------------------------------------ gscSio4ChannelGetBiSyncTxUnderrun
//
//  This routine is used to get the Tx Underrun condition mode.
//
int gscSio4ChannelGetBiSyncTxUnderrun( int boardNumber, int channel, int *txUnderrun)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Tx Underrun Condition
    switch (value&0xc000)
    {
    case 0x0000: *txUnderrun = GSC_SYN1;		  break;
    case 0x4000: *txUnderrun = GSC_SYN0_SYN1;	  break;
    case 0x8000: *txUnderrun = GSC_CRC_SYN1;	  break;
    case 0xc000: *txUnderrun = GSC_CRC_SYN0_SYN1; break;
    }

    return status;
}

//------------------------------------------ gscSio4ChannelGetBiSyncTxPreamble
//
//  This routine is used to get the Tx Preamble value.
//
int gscSio4ChannelGetBiSyncTxPreamble( int boardNumber, int channel, int *txPreamble)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Tx Preamble
    if (value & 0x2000) *txPreamble = GSC_ENABLED;
    else *txPreamble = GSC_DISABLED;

    return status;
}

//----------------------------------------- gscSio4ChannelGetBiSyncTxShortSync
//
//  This routine is used to get the Tx Sync Length (short or 8 bit).
//
int gscSio4ChannelGetBiSyncTxShortSync( int boardNumber, int channel, int *txShortSync)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Tx Short Sync
    if (value & 0x1000) *txShortSync = GSC_ENABLED;
    else *txShortSync = GSC_DISABLED;

    return status;
}

//----------------------------------------- gscSio4ChannelGetBiSyncRxSyncStrip
//
//  This routine is used to get the Rx Sync Strip mode.
//
int gscSio4ChannelGetBiSyncRxSyncStrip( int boardNumber, int channel, int *rxSyncStrip)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Rx Sync Strip
    if (value & 0x0020) *rxSyncStrip = GSC_ENABLED;
    else *rxSyncStrip = GSC_DISABLED;

    return status;
}

//----------------------------------------- gscSio4ChannelGetBiSyncRxShortSync
//
//  This routine is used to get the Rx Sync Length (short or 8 bit).
//
int gscSio4ChannelGetBiSyncRxShortSync( int boardNumber, int channel, int *rxShortSync)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Rx Short Sync
    if (value & 0x0010) *rxShortSync = GSC_ENABLED;
    else *rxShortSync = GSC_DISABLED;

    return status;
}

//------------------------------------ gscSio4ChannelGetBiSyncTxPreambleLength
//
//  This routine is used to get the Tx Preamble Length.
//
int gscSio4ChannelGetBiSyncTxPreambleLength( int boardNumber, int channel, int *txPreambleLength)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Control Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
    if (status) return status;

    // Get the Tx Preamble Length bits
    switch (value&0x0c00)
    {
    case 0x0000: *txPreambleLength = GSC_PREAMBLE_8BITS;  break;
    case 0x0400: *txPreambleLength = GSC_PREAMBLE_16BITS; break;
    case 0x0800: *txPreambleLength = GSC_PREAMBLE_32BITS; break;
    case 0x0c00: *txPreambleLength = GSC_PREAMBLE_64BITS; break;
    }

    return status;
}

//----------------------------------- gscSio4ChannelGetBiSyncTxPreamblePattern
//
//  This routine is used to get the Tx Preamble pattern.
//
int gscSio4ChannelGetBiSyncTxPreamblePattern( int boardNumber, int channel, int *txPreamblePattern)
{
    int status = 0x00;
    int value;

    // Make sure the board and channel number are valid
    status = validateBoardAndChannelNumber( boardNumber, channel);
    if (status) return status;

    // Read the current Channel Control Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
    if (status) return status;

    // Get the Tx Preamble Pattern bits
    switch (value&0x0300)
    {
    case 0x0000: *txPreamblePattern = GSC_PREAMBLE_ALL_0;	break;
    case 0x0100: *txPreamblePattern = GSC_PREAMBLE_ALL_1;	break;
    case 0x0200: *txPreamblePattern = GSC_PREAMBLE_ALL_1_0; break;
    case 0x0300: *txPreamblePattern = GSC_PREAMBLE_ALL_0_1; break;
    }

    return status;
}

