//////////////////////////////////////////////////////////////////////////////
//
// GscApiBiSync16.c
//
//  This module contains all of the routines that handle the BiSync16 mode
// operation. This mode of operation is only available on the -BiSync16 version
// of the hardware.
//
#include "GscApiLocal.h"


//------------------------------------------------- GscSio4BiSync16GetDefaults
//
//  This routine fills in the provided BiSync16 Configuration
// structure with their default values.
//
int GscSio4BiSync16GetDefaults( PGSC_BISYNC16_CONFIG config )
{
	int status = 0x00;

	// Setup the Channel defaults
	config->bitRate = 1000000;
	config->encoding = GSC_ENCODING_NRZ;
	config->protocol = GSC_PROTOCOL_RS422_RS485;
	config->termination = GSC_TERMINATION_ENABLED;

	// Setup the Transmitter defaults
	config->txStatus = GSC_ENABLED;
	config->txSyncWord = 0xffff;
	config->txIdleCondition = 0x0000;
	config->txBitOrder = GSC_LSB_FIRST;
	config->txByteOrder = GSC_LSB_FIRST;

	// Setup the Receiver defaults
	config->rxStatus = GSC_ENABLED;
	config->rxSyncWord = 0xffff;
	config->maxRxCount = 0xffff;

	// Setup the Pin Configuration Variables
	config->interfaceMode = GSC_PIN_DTE;
	config->txDataPinMode = GSC_PIN_AUTO;
	config->rxDataPinMode = GSC_PIN_AUTO;
	config->txClockPinMode = GSC_PIN_AUTO;
	config->rxClockPinMode = GSC_PIN_AUTO;
	config->ctsPinMode = GSC_PIN_GPIO;
	config->rtsPinMode = GSC_PIN_GPIO;
	config->loopbackMode = GSC_LOOP_NONE;

	return status;
}


//-------------------------------------------------------- gscSio4BiSync16Init
//
//  This routine initializes a single channel to BiSync16
// mode using the default parameters.
//
int gscSio4BiSync16Init(int boardNumber, int channel)
{
	int status = 0x00;
	GSC_BISYNC16_CONFIG config;

	// Load the default values
	status = GscSio4BiSync16GetDefaults( &config);

	// Program the channel
	status = GscSio4BiSync16SetConfig( boardNumber, channel, config);
	if (status) return status;

	return status;
}


//----------------------------------------------------- GscSio4GetBiSync16Config
//
//  This routine returns the current configuration
// structure for a single channel that is set to BiSync16
// mode.
//
int GscSio4BiSync16GetConfig(int boardNumber, int channel, PGSC_BISYNC16_CONFIG config)
{
	int status = 0;
	int value  = 0;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected
	if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_BISYNC16)
	  return GscApiInvalidBoardType;

	// Determine the encoding and bit rate
	config->encoding = boardInformation[ boardNumber-1 ].channelEncoding[ channel-1 ];
	config->bitRate = boardInformation[ boardNumber-1 ].channelBitRate[ channel-1 ];

	// Determine the protocol and termination settings
	status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol,
		&config->termination);
	if (status) return( status );

	// Get the Sync and Idle Words
	status = fpgaRead( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Set the values in the config struct
	config->txSyncWord = config->rxSyncWord = value & 0x0000ffff;
	config->txIdleCondition = (value >> 16) & 0x0000ffff;

	// Get the byte and bit ordering
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Set the values in the config struct
	if (value & 0x04000000) config->txByteOrder = GSC_MSB_FIRST;
	else config->txByteOrder = GSC_LSB_FIRST;

	if (value & 0x08000000) config->txBitOrder = GSC_LSB_FIRST;
	else config->txBitOrder = GSC_MSB_FIRST;

	// Get the Max Rx Count
	status = fpgaRead( boardNumber, MAX_RX_COUNT_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Set the value in the config struct
	config->maxRxCount = value & 0xffff;

	// Determine the various pin modes
	config->interfaceMode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
		&config->txDataPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
		&config->rxDataPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
		&config->txClockPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
		&config->rxClockPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
		&config->ctsPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
		&config->rtsPinMode);
	if (status) return( status );

	// Determine the loopback mode
	config->loopbackMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

	// Determine whether the receiver is enabled
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
	if (status) return status;

	// Determine the Rx mode
	if (value & 0x01000000)
		config->rxStatus = GSC_ENABLED;
	else
		config->rxStatus = GSC_DISABLED;

	// Determine whether the transmitter is enabled
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
	if (status) return status;

	// Determine the Tx mode
	if  (value & 0x0200000)
		config->txStatus = GSC_ENABLED;
	else
		config->rxStatus = GSC_DISABLED;

	return status;
}


//----------------------------------------------------- GscSio4BiSync16SetConfig
//
//  This routine sets the current configuration
// structure for a single channel that is set to BiSync16
// mode.
//
int GscSio4BiSync16SetConfig(int boardNumber, int channel, GSC_BISYNC16_CONFIG config)
{
	int status = 0;
	int value  = 0;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected
	if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_BISYNC16)
	  return GscApiInvalidBoardType;

	// Save some global info
	boardInformation[ boardNumber-1].channelMode[ channel-1] = GSC_MODE_BISYNC16;
	boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)config.bitRate/1000/8;

	// Setup the encoding and bitRate
	fpgaSetEncoding( boardNumber, channel, config.encoding, config.bitRate);
	if (status) return status;

	// Setup the protocol and termination
	status = quickSetProtocol( boardNumber, channel, config.protocol, config.termination);
	if (status) return status;

	// Set the Sync and Idle Words - idle word in higher-order 16 bits, syncword in lower
	status = fpgaWrite( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4, (config.txIdleCondition << 16) & config.txSyncWord );
	if (status) return status;

	// Set the byte and bit ordering

	// Get value current channel control status register contents
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Clear bits of interest
	value &= ~0x0c000000;

	// Set byte and bit order bits accordingly
	if (config.txByteOrder == GSC_MSB_FIRST) value |= 0x04000000;
	if (config.txBitOrder == GSC_LSB_FIRST) value |= 0x08000000;
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value);
	if (status) return status;

	// Set the Max Rx Count
	status = fpgaWrite( boardNumber, MAX_RX_COUNT_BASE_REG + (channel-1)*4, config.maxRxCount);
	if (status) return status;

	// Setup the Pin Modes
	status = quickSetDteDce(boardNumber, channel, config.interfaceMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, config.txDataPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, config.rxDataPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, config.txClockPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, config.rxClockPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, config.ctsPinMode);
	if (status) return status;
	status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, config.rtsPinMode);
	if (status) return status;

	// Setup the loopback mode
	status = quickSetLoopBack( boardNumber, channel, config.loopbackMode);
	if (status) return status;

    // Enable/Disable the receiver and transmitter

	// Get channel control status register contents
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Set receiver bit
	if (config.rxStatus == GSC_ENABLED) value |= 0x01000000;
	else value &= ~0x01000000;

	// Set transmitter bit
	if (config.txStatus == GSC_ENABLED) value |= 0x02000000;
	else value &= ~0x02000000;

	// Write new value to channel control status register
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value);

	return status;
}


//----------------------------------------------------- GscSio4BiSync16GetOption
//
//  This routine gets the value of the identified option.
//
int gscSio4BiSync16GetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
	int status = 0;
	int value  = 0;
	int offset = 0;

	switch ( optionName )
	{
	case GSC_SIO_SYNCWORD:
		{
		int *txSyncWord = &optionValue[0];
		int *rxSyncWord = &optionValue[1];

		// Get sync detect register contents
		status = fpgaRead( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4, &value);
		if (status) return status;

		// Get sync word value
		*txSyncWord = *rxSyncWord = value & 0x0000ffff;
		}

		break;

	case GSC_SIO_TXIDLECONDITION:
		{
		int *txIdleCondition = &optionValue[0];

		// Get sync detect register contents
		status = fpgaRead( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4, &value);
		if (status) return status;

		// Get idle word value
		*txIdleCondition = (value >> 16) & 0x0000ffff;
		}

		break;

	case GSC_SIO_ORDERING:
		{
		int *byteOrder = &optionValue[0];
		int *bitOrder  = &optionValue[1];

		// Get value
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
		if (status) return status;

		// Return values in the proper variables
		if (value & 0x04000000) *byteOrder = GSC_MSB_FIRST;
		else *byteOrder = GSC_LSB_FIRST;

		if (value & 0x08000000) *bitOrder = GSC_LSB_FIRST;
		else *bitOrder = GSC_MSB_FIRST;
		}

		break;

	case GSC_SIO_MAXRXCOUNT:
		{
		int *maxRxCount = &optionValue[0];

		// Get value
		status = fpgaRead( boardNumber, MAX_RX_COUNT_BASE_REG + (channel-1)*4, &value);
		if (status) return status;

		// Return values in the proper variables
		*maxRxCount = value & 0xffff;
		}

		break;

	case GSC_SIO_TXDATAPINMODE:
	case GSC_SIO_RXDATAPINMODE:
	case GSC_SIO_TXCLOCKPINMODE:
	case GSC_SIO_RXCLOCKPINMODE:
	case GSC_SIO_CTSPINMODE:
	case GSC_SIO_RTSPINMODE:
		// Validate the Pin name
		status = validatePinName( optionName, &offset, NULL, NULL);
		if (status == 0)
			optionValue[0] = boardInformation[ boardNumber-1].channelPinMode[ channel-1][offset];
		break;

	case GSC_SIO_RECEIVER:
		{
		int *mode = &optionValue[0];

		// Get value
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
		if (status) return status;

		// Return values in the proper variables
		if (value & 0x01000000) *mode = GSC_ENABLED;
		else *mode = GSC_DISABLED;
		}

		break;

	case GSC_SIO_TRANSMITTER:
		{
		int *mode = &optionValue[0];

		// Get value
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
		if (status) return status;

		// Return values in the proper variables
		if (value & 0x02000000) *mode = GSC_ENABLED;
		else *mode = GSC_DISABLED;
		}

		break;

	default:
		return GscApiInvalidParameter;
	}

	return status;
}

//----------------------------------------------------- GscSio4BiSync16SetOption
//
//  This routine sets the identified option to the supplied value.
//
int gscSio4BiSync16SetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
	int status = 0;
	int tempValue = 0;

	switch ( optionName )
	{
	case GSC_SIO_ENCODING:
		{
		int encoding = optionValue;

		// Convert board and channel into indexes
		int bn = boardNumber - 1;
		int ch = channel - 1;

		return fpgaSetEncoding( boardNumber,
								channel,
								encoding,
								boardInformation[ bn].channelBitRate[ ch]);
		}

		break;

	case GSC_SIO_SYNCWORD:
		// Write new syncword value
		status = fpgaWrite( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4,
			optionValue);
		break;

	case GSC_SIO_TXIDLECONDITION:
		// Write new idleword value - idle word in higher-order 16-bits
		status = fpgaWrite( boardNumber, SYNC_CHARACTER_BASE_REG + (channel-1)*4,
			optionValue << 16 );
		break;

	case GSC_SIO_ORDERING:
		{
		int byteOrder = (optionValue & 0xffff0000) >> 16;
		int bitOrder  = optionValue & 0x0000ffff;

		// Get value
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &tempValue);
		if (status) return status;

		// Clear bits of interest
		tempValue &= ~0x0c000000;

		// Update value
		if (byteOrder == GSC_MSB_FIRST) tempValue |= 0x04000000;
		if (bitOrder == GSC_LSB_FIRST) tempValue |= 0x08000000;
		status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, tempValue);
		}

		break;

	case GSC_SIO_MAXRXCOUNT:
		{
		int maxRxCount = optionValue;

		// Set value
		status = fpgaWrite( boardNumber, MAX_RX_COUNT_BASE_REG + (channel-1)*4, maxRxCount);
		}

		break;

	case GSC_SIO_TXDATAPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, optionValue);
		break;

	case GSC_SIO_RXDATAPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, optionValue);
		break;

	case GSC_SIO_TXCLOCKPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, optionValue);
		break;

	case GSC_SIO_RXCLOCKPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, optionValue);
		break;

	case GSC_SIO_CTSPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_ENV, optionValue);
		break;

	case GSC_SIO_RTSPINMODE:
		// Setup the pin mode
		status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_ENV, optionValue);
		break;

	case GSC_SIO_RECEIVER:
		{
		int mode = optionValue;

		// Get value
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &tempValue);
		if (status) return status;

		// Update values
		if (mode == GSC_ENABLED) tempValue |= 0x01000000;
		else tempValue &= ~0x01000000;

		// Write them back out
		status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, tempValue);
		}

		break;

	case GSC_SIO_TRANSMITTER:
		{
		int mode = optionValue;

		// Get value
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &tempValue);
		if (status) return status;

		// Update values
		if (mode == GSC_ENABLED) tempValue |= 0x02000000;
		else tempValue &= ~0x02000000;

		// Write them back out
		status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, tempValue);
		}

		break;
	default:
		return GscApiInvalidParameter;
	}

	return status;

}

int GscSio4BiSync16EnterHuntMode( int boardNumber, int channel)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Write the bit high
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value | 0x80000000);
	if (status) return status;

	// Write the bit back low
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value & ~0x80000000);
	if (status) return status;

	return status;
}


int GscSio4BiSync16AbortTx( int boardNumber, int channel)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Write the bit high
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value | 0x40000000);
	if (status) return status;

	// Write the bit back low
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value & ~0x40000000);
	if (status) return status;

	return status;
}

int GscSio4BiSync16Pause( int boardNumber, int channel)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Write the bit high
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value | 0x40000000);
	if (status) return status;

	return status;
}

int GscSio4BiSync16Resume( int boardNumber, int channel)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, &value);
	if (status) return status;

	// Write the bit low
	status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG + (channel-1)*16, value & ~0x40000000);
	if (status) return status;

	return status;
}


int GscSio4BiSync16GetTxCounts( int boardNumber, int channel, int *remaining, int *initial)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, TX_COUNT_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Return values in the proper variables
	*remaining = (value>>16) & 0xffff;
	*initial = value & 0xffff;

	return status;
}


int GscSio4BiSync16GetRxCounts( int boardNumber, int channel, int *remaining, int *initial)
{
	int status = 0x00;
	int value;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Get value
	status = fpgaRead( boardNumber, RX_COUNT_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Return values in the proper variables
	*remaining = (value>>16) & 0xffff;
	*initial = value & 0xffff;

	return status;
}

