//-------------------------------------------------------------------------------------------------
//
// File Name:
//
//    GscApiCTC.c
//
// Description:
//
//    This module contains all of the routines that handle the CTC mode of operation.
//
// Programmer's notes:
//
//     04-30-2014:  Created file.  --mrg
//
//-------------------------------------------------------------------------------------------------

#include "GscApiLocal.h"

#define MAX_FRAMES_CHANNEL 80

const int CTC_CTRL_TX_ENABLE = 0x02;
const int CTC_CTRL_RX_ENABLE = 0x200;
const int CTC_CTRL_START_TRANSMIT = 0x04;

const int GSC_CTC_TIMER_RESET = 0x01;
const int CTC_MAX_FRAME_COUNT = 4095;

static int txActiveMajorFrame[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];

typedef struct _BUFFERED_FRAME
{
  int startMinorFrame;
  int endMinorFrame;
} BUFFERED_FRAME;

BUFFERED_FRAME bufferedFrames[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAX_FRAMES_CHANNEL];


//*----------------------------------------------------------GscSio4ChannelGetCTCDefaults----------
//*
//* Description:
//*
//*     This routine fills in the provided CTC Configuration structure with their default values.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     PGSC_CTC_CONFIG     config        Pointer to the board config structure to be populated
//*                                       with default values.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCGetDefaults( PGSC_CTC_CONFIG config)
{
  int status = 0x00;

  // Setup the Channel defaults
  config->bitRate = 1000000;
  config->protocol = GSC_PROTOCOL_RS422_RS485;
  config->termination = GSC_TERMINATION_ENABLED;
  config->majorFrameSize = 10;
  config->minorFrameSize = 3840;
  config->timerMode = GSC_CTC_TIMER_BIN;
  config->timerValue = 0;

  // Setup the Transmitter defaults
  config->txStatus = GSC_ENABLED;
  config->txFrameCount = GSC_ENABLED;
  config->txFrameTimer = GSC_ENABLED;

  // Setup the Receiver defaults
  config->rxStatus = GSC_ENABLED;

  // Setup the Pin Configuration Variables
  config->interfaceMode = GSC_PIN_DTE;
  config->loopbackMode = GSC_LOOP_NONE;

  // need max frame size
  return status;
}


//*----------------------------------------------------------GscSio4ChannelCTCGetConfig------------
//*
//* Description:
//*
//*     This routine returns the current configuration structure for a single channel that is set
//*     to CTC mode.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     PGSC_CTC_CONFIG     config        Pointer to the board config structure to be populated
//*                                       with default values.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCGetConfig(int boardNumber, int channel, PGSC_CTC_CONFIG config)
{
	int status = 0;
	int value = 0;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected (include BXR board, which can also do SYNC mode)
	if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_CTC)
	  return GscApiInvalidBoardType;

	// If the channel is not configured, get the defaults
	if (boardInformation[ boardNumber-1].channelMode[ channel-1] == GSC_MODE_UNINITIALIZED)
	{
		// Load the default values
		status = GscSio4CTCGetDefaults( config);
		return status;
	}

	// Get the data character lengths
	status = fpgaRead( boardNumber, TX_COUNT_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	// Determine the bit rate
	config->bitRate = boardInformation[ boardNumber-1 ].channelBitRate[ channel-1 ];

	// Determine the protocol and termination settings
	status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol,
		&config->termination);
	if (status) return( status );

	// Determine the various pin modes
	config->interfaceMode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];

	// Determine the loopback mode
	config->loopbackMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

	// Determine whether the receiver is enabled
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
	if (status) return status;

	// Determine the Rx mode
	if (value & 0x01000000)
		config->rxStatus = GSC_ENABLED;
	else
		config->rxStatus = GSC_DISABLED;

	// Determine whether the transmitter is enabled
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
	if (status) return status;

	// Determine the Tx mode
	if  (value & 0x02000000)
		config->txStatus = GSC_ENABLED;
	else
		config->rxStatus = GSC_DISABLED;

	return status;
}


//*----------------------------------------------------------GscSio4ChannelCTCSetConfig------------
//*
//* Description:
//*
//*     This routine sets the current configuration structure for a single channel.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     GSC_CTC_CONFIG      config        The board config structure containing the board
//*                                       parameters to configure.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCSetConfig(int boardNumber, int channel, PGSC_CTC_CONFIG config)
{
  int status = 0x00;
  int value = 0;
  int ctrlVal = 0;
  int clockMultiplier = 2;
  int i = 0;
  int j = 0;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Initialize active major frame array
  for (i=0; i < MAXIMUM_NUMBER_OF_BOARDS; i++)
    for (j=0; j < NUMBER_OF_CHANNELS; j++)
      txActiveMajorFrame[i][j] = -1;

  // Make sure the board type is as expected (include BXR board, which can also do SYNC mode)

  // *** TODO: Showing up as -ASYNC right now, so ignore - consult with Steve.
  //if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_CTC)
	//  return GscApiInvalidBoardType;

  // Save some global info
  boardInformation[ boardNumber-1].channelMode[ channel-1] = GSC_MODE_CTC;
  boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)config->bitRate/1000/8;

  // Read the current Control/Status register
  status = fpgaRead( boardNumber, CTC_CONTROL_REG | (channel << 12), &ctrlVal);
  if (status) return status;

  // Do a channel reset here for now. Will need to modify GscApi call for CTC board
  status = fpgaWrite(boardNumber, CTC_CONTROL_REG | (channel << 12), ctrlVal | 0x01);

  // Setup the bitRate
  status = GscSio4ChannelSetClock( boardNumber, channel, config->bitRate * clockMultiplier);
  if (status) return status;

  // Setup the major (max frame count) and minor frame size
  status = fpgaWrite(boardNumber, CTC_FRAME_SIZE_REG | (channel << 12), config->minorFrameSize);
  if (status) return status;
  status = fpgaWrite(boardNumber, CTC_TX_MAX_FRAMESIZE_REG | (channel << 12), config->majorFrameSize);
  if (status) return status;

  // Setup the timer mode and value
  //status = gscSio4ChannelSetMsbLsbOrder( boardNumber, channel, config->txMsbLsb, config->rxMsbLsb );
  if (status) return status;

  // Setup the Pin Modes
  status = quickSetDteDce(boardNumber, channel, config->interfaceMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, GSC_PIN_AUTO);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, GSC_PIN_AUTO);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, GSC_PIN_AUTO);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, GSC_PIN_AUTO);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, GSC_PIN_GPIO);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, GSC_PIN_GPIO);
  if (status) return status;

  // Setup the loopback mode
  if (config->loopbackMode == GSC_LOOP_EXTERNAL)
  {
    status = fpgaRead( boardNumber, CTC_TEST_REG, &value);
    value |= 0x01 << (channel - 1);
    status = fpgaWrite( boardNumber, CTC_TEST_REG, value);
  }

  // Setup the clock sources
  status = gscSio4ChannelSetClockSource( boardNumber, channel, GSC_CLOCK_EXTERNAL, GSC_CLOCK_EXTERNAL);
  if (status) return status;

  // Enable/Disable the receiver and transmitter

  // Process the desired Rx mode
  switch (config->rxStatus)
  {
  case GSC_ENABLED:	ctrlVal |=  0x0200; break;
  case GSC_DISABLED:	ctrlVal &= ~0x0200; break;
  default:			return GscApiInvalidParameter;
  }

  // Process the desired Tx mode
  switch (config->txStatus)
  {
  case GSC_ENABLED:	ctrlVal |=  0x02; break;
  case GSC_DISABLED:	ctrlVal &= ~0x02; break;
  default:			return GscApiInvalidParameter;
  }

  // Write the new value back out
  //status = fpgaWrite(boardNumber, CTC_CONTROL_REG | (channel << 12), ctrlVal);
  //if (status) return status;

  // reset the FIFOs -- how to??
  //status = resetRxFifo( boardNumber, channel );
  //if (status) return status;

  //status = GscSio4ChannelResetTxFifo( boardNumber, channel );
  //if (status) return( status );

  status = fpgaRead( boardNumber, CTC_CONTROL_REG | (channel << 12), &value);
  return status;
}



//*----------------------------------------------------------GscSio4CTCAddMinorFrame---------------
//*
//* Description:
//*
//*     This routine adds a CTC minor frame to Tx SRAM for the specified channel.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     CTC_MINOR_FRAME     *frame        The minor frame to write to TX SRAM.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCAddMinorFrame( int boardNumber,
                              int channel,
                              CTC_MINOR_FRAME *fr
                           )
{
  int status = 0;
  int count = 0;
  unsigned int regval = 0x00;
  int bytesTransferred = 0;
  int i = 0;
  int frameSizeBytes = 0;

  frameSizeBytes = fr->size * 3 / 2;

  BeginCriticalSection(qmutex);

  //status = fpgaRead( boardNumber, CTC_CONTROL_REG | (channel << 12), &regval);

  // set the frame size register (12-bit words)
  status = fpgaWrite(boardNumber, CTC_FRAME_SIZE_REG | (channel << 12), fr->size);

  // set the frame header offset register to write the frame
  status = fpgaWrite( boardNumber, CTC_TX_HEADER_OFFSET_REG | (channel << 12), fr->frameNumber );

  // write the frame header to the data register
  switch(fr->timerSelect)
  {
    case GSC_CTC_FRAME_TIMER:    regval |= (0 << 16); break;
    case GSC_CTC_FRAME_TIMER_HI: regval |= (1 << 16); break;
    case GSC_CTC_FRAME_TIMER_LO: regval |= (2 << 16); break;
    case GSC_CTC_FRAME_TIMER_US: regval |= (3 << 16); break;
    default: regval |= (0 << 16);
  }

  regval |= fr->lastFrame == 1 ? (1 << 15) : 0;
  regval |= fr->framingErrEnable == 1 ? (1 << 14) : 0;
  regval |= fr->timerInsertEnable == 1 ? (1 << 13) : 0;
  regval |= fr->frameCountInsertEnable == 1 ? (1 << 12) : 0;
  regval |= fr->nextFrame & 0x0fff;

  status = fpgaWrite( boardNumber, CTC_TX_HEADER_DATA_REG | (channel << 12), regval );

  // set the frame buffer offset register
  status = fpgaWrite( boardNumber, CTC_TX_BUFFER_OFFSET_REG | (channel << 12), (fr->frameNumber) * frameSizeBytes );

  //status = fpgaRead( boardNumber, CTC_CONTROL_REG | (channel << 12), &regval);

  // write the frame buffer to the data register (DMA eventually, commented out below for now)
  count = fr->size * 3 / 2;
  //status = GscSio4ChannelTransmitDataAndWait( boardNumber, channel, fr->data, count, 2000, &bytesTransferred );
  if (drvMajorVersion >= 5)
    status = gscPerformDma( boardNumber, channel, fr->data, count, GSC_DATA_TRANSMIT, 0);
  else
    status = gscPerformDma( boardNumber, channel, fr->data, count, GSC_DATA_TRANSMIT, PrimaryPciChannel1);

  //for (i=0; i < (fr->size * 3 / 2) / 4; i++)
  //  status = fpgaWrite( boardNumber, CTC_TX_BUFFER_DATA_REG | (channel << 12), *((int*)&fr->data[i*4]) );
  EndCriticalSection(qmutex);

  // Read back tx sram to make sure frame was correctly written
  //status = fpgaWrite( boardNumber, CTC_TX_HEADER_OFFSET_REG | (channel << 12), fr->frameNumber );
  //status = fpgaWrite( boardNumber, CTC_TX_BUFFER_OFFSET_REG | (channel << 12), (fr->frameNumber) * frameSizeBytes );
  //for (i=0; i < count; i++)
  //  status = fpgaRead16( boardNumber, CTC_TX_BUFFER_DATA_REG | (channel << 12), &sram[i]);
  //for (i=0; i < (fr->size * 3 / 2) / 4; i++)
  //  status = fpgaWrite( boardNumber, CTC_TX_BUFFER_DATA_REG | (channel << 12), *((int*)&fr->data[i*4]) );

  return status;
}



//*----------------------------------------------------------GscSio4CTCAddMajorFrame---------------
//*
//* Description:
//*
//*     This routine adds a CTC major frame to Tx SRAM for the specified channel.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     CTC_MAJOR_FRAME     *frame        The major frame to write to TX SRAM.
//*     int                 frameSize     The major frame size expressed as the number of minor
//*                                       frames.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCAddMajorFrame( int boardNumber,
                              int channel,
                              CTC_MAJOR_FRAME *fr,
                              int frameSize
                           )
{
  int status = 0;
  int i = 0;

  if (fr == NULL)
    return GscApiInvalidParameter;

  if (fr->majorFrameNumber > MAX_FRAMES_CHANNEL)
    return GscApiInvalidParameter;

  // keep track of starting and ending minor frame for each added major frame

  for (i=0; i < frameSize; i++)
    if ((status = GscSio4CTCAddMinorFrame(boardNumber, channel, &fr->frames[i])) != 0)
      return status;

  // Save major frame information
  bufferedFrames[boardNumber - 1][channel - 1][fr->majorFrameNumber].startMinorFrame = fr->frames[0].frameNumber;
  bufferedFrames[boardNumber - 1][channel - 1][fr->majorFrameNumber].endMinorFrame = fr->frames[frameSize - 1].frameNumber;

  return status;
}



//*----------------------------------------------------------GscSio4CTCSwitchMajorFrame------------
//*
//* Description:
//*
//*     This routine adds a CTC major frame to Tx SRAM for the specified channel and hands off
//*     active CTC transmission to the added major frame and causing the previously transmitting
//*     major frame to go dormant.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     CTC_MAJOR_FRAME     *frame        The new major frame to start transmittingAM.
//*     int                 frameSize     The major frame size expressed as the number of minor
//*                                       frames.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCSwitchMajorFrame( int boardNumber,
                                int channel,
                                CTC_MAJOR_FRAME *fr,
                                int frameSize
                              )
{
  int status = 0;
  int i = 0;
  int regval = 0;
  int majfrnum = 0;

  // Assume only major frame numbers 0 and 1 are valid (double buffering)

  // Make sure new major frame is not the major frame currently being transmitted.

  // Retrieve active major frame number for board and channel
  majfrnum = txActiveMajorFrame[boardNumber - 1][channel - 1];
  // Write the new frame
  status = GscSio4CTCAddMajorFrame(boardNumber, channel, fr, frameSize);

  // Hand off transmission to new frame
  status = fpgaWrite( boardNumber, CTC_TX_HEADER_OFFSET_REG | (channel << 12),
    bufferedFrames[boardNumber - 1][channel - 1][majfrnum].endMinorFrame );
  status = fpgaRead( boardNumber, CTC_TX_HEADER_DATA_REG | (channel << 12), &regval );

  regval &= 0xfffff000;
  regval |= fr->frames[0].frameNumber;

  status = fpgaWrite( boardNumber, CTC_TX_HEADER_OFFSET_REG | (channel << 12),
    bufferedFrames[boardNumber - 1][channel - 1][majfrnum].endMinorFrame );
  status = fpgaWrite( boardNumber, CTC_TX_HEADER_DATA_REG | (channel << 12), regval );

  // Reset active major frame
  txActiveMajorFrame[boardNumber - 1][channel - 1] = fr->majorFrameNumber;
  return status;
}



//*----------------------------------------------------------GscSio4CTCGetActiveMajorFrame---------
//*
//* Description:
//*
//*     This routine returns the frame number of the major frame actively being transmitted.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     int                 *frameNumber  The major frame size expressed as the number of minor
//*                                       frames.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCGetActiveMajorFrame( int boardNumber,
                                   int channel,
                                   int *frameNumber
                                 )
{
  int status = 0;
  int i = 0;
  int regval = 0;
  int minorFrNum = 0;

  if (frameNumber == (int *)NULL)
    return GscApiInvalidParameter;

  // Read contents of tx frame header current register

  status = fpgaRead( boardNumber, CTC_TX_HEADER_CURRENT_REG | (channel << 12), &regval );
  minorFrNum = (regval & 0x03ff0000) >> 16;

  // Locate minor frame number within major frame
  i = 0;
  while (i < MAX_FRAMES_CHANNEL)
  {
    if ((bufferedFrames[boardNumber - 1][channel - 1][i].startMinorFrame <= minorFrNum) &&
        (bufferedFrames[boardNumber - 1][channel - 1][i].endMinorFrame >= minorFrNum))
    {
      *frameNumber = i;
      break;
    }
    i++;
  }

  if (i == MAX_FRAMES_CHANNEL)
    return GscApiCTCNoBufferedFrames;
  else
    return status;
}



//*----------------------------------------------------------GscSio4CTCTransmitFrames--------------
//*
//* Description:
//*
//*     This routine transmits a CTC minor frame on the specified channel.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     int                 frameNumber   The starting frame.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCTransmitFrames( int boardNumber,
                              int channel,
                              int frameNumber
                           )
{
  int status = 0;
  unsigned int regval = 0x00;
  int i = 0;
  int majfrno = 0;


  // restore offset registers to frame being transmitted.
  status = fpgaWrite( boardNumber, CTC_TX_HEADER_OFFSET_REG | (channel << 12), frameNumber );
  status = fpgaWrite( boardNumber, CTC_TX_BUFFER_OFFSET_REG | (channel << 12), 0 );

  // write frame start register
  status = fpgaWrite( boardNumber, CTC_TX_FRAME_START_REG | (channel << 12), frameNumber);

  // write control register to start transmit
	status = fpgaRead( boardNumber, CTC_CONTROL_REG | (channel << 12), &regval);
  status = fpgaWrite( boardNumber, CTC_CONTROL_REG | (channel << 12), regval | CTC_CTRL_START_TRANSMIT | CTC_CTRL_TX_ENABLE | CTC_CTRL_RX_ENABLE);

  status = GscSio4CTCGetActiveMajorFrame(boardNumber, channel, &majfrno);
  txActiveMajorFrame[boardNumber - 1][channel - 1] = majfrno;

  return status;
}


//*----------------------------------------------------------GscSio4CTCReceiveFrames---------------
//*
//* Description:
//*
//*     This routine retrieves CTC minor frames from the specified channel, with a limit on the
//*     number for frames returned defined by the nFrames parameter.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     int                 channel       The serial IO channel.
//*     int                 *nFrames      On input, the maximum number of frames to retrieve. On
//*                                       output, the actual number of frames retrieved.
//*     char                *buffer         The minor frame to transmit.
//*     int                 *nErrors      Count of errors associated with frames in the fifo.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCReceiveFrames(int boardNumber, int channel, int *nFrames, char *buffer, int *nErrors)
{
  int i = 0;
  int status = 0;
  int count = 0;
  int fifoCount = 0;
  int minorFrameSize = 0;
  int regval = 0;
  int minorFrameSizeBytes = 0;
  int iFifoData[3] = {0,0,0};
  char *pbuff = (char *) NULL;
  int regCount = 0;
  int bytesTransferred = 0;
  int framesAvailable = 0;

  /* for testing - force walk through fifo for next beginning of frame */
  //static int nCalls[4] = {0,1,1,1};

  // TODO:
  // 1. may need modified internal routine to get fifo counts.
  // 2. empty frame count register?

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Compute number of bytes to read from fifo
  status = fpgaRead(boardNumber, CTC_FRAME_SIZE_REG | (channel << 12), &minorFrameSize);
  if (status) return status;

  minorFrameSizeBytes = minorFrameSize * 3 / 2;
  count = *nFrames * minorFrameSizeBytes;

  BeginCriticalSection( qmutex);

  // Make sure the data is still in the Rx FIFO
  //      status = GscSio4ChannelFifoCounts( boardNumber, channel, &fifoCounts);
  //status = getRxFifoCount( boardNumber, channel, &fifoCount);
  status = fpgaRead(boardNumber, CTC_RX_HEADER_FIFOCOUNT_REG | (channel << 12), &regval);
  status += fpgaRead(boardNumber, CTC_RX_FRAME_FIFOCOUNT_REG | (channel << 12), &fifoCount);
  if (status)
  {
    *nFrames = 0;
    EndCriticalSection(qmutex);
    return status;
  }

  // Retrieve error count and clear register
  status = fpgaRead(boardNumber, CTC_RX_FRAME_ERRORCOUNT_REG | (channel << 12), nErrors);
  *nErrors = *nErrors & 0xffff;
  status = fpgaWrite(boardNumber, CTC_RX_FRAME_ERRORCOUNT_REG | (channel << 12), *nErrors | 0x80000000);

  //fifoCounts = (fifoCounts >> 16) & 0xffff;
  if (fifoCount < count)
  {
    // If zero complete frames are available then return, otherwise read as many frames as possible.
    framesAvailable = fifoCount / minorFrameSizeBytes;
    if (framesAvailable == 0)
    {
      EndCriticalSection(qmutex);
      *nFrames = 0;
      return GscApiFifoOverrun;
    }
    else if (framesAvailable < *nFrames)
    {
      *nFrames = framesAvailable;
      count = framesAvailable * minorFrameSizeBytes;
    }
  }

  //
  // When fetching frames, check for beginning of frame bit. If missing, search
  // fifo, quiting search when only partial frame left in fifo.
  //

  while (fifoCount >= count)
  {
    /*
    if (nCalls[channel - 1] % 2 == 0)
    {
    status = fpgaRead( boardNumber, (channel << 4) | 0x08, &iFifoData[0]);
    nCalls[channel - 1] = 1;
    }
    */

    status = fpgaRead( boardNumber, (channel << 4) | 0x08, &iFifoData[0]);
    if (status)
    {
      *nFrames = 0;
      EndCriticalSection(qmutex);
      return status;
    }
    if (iFifoData[0] & 0x0100)
    {
      // Found beginning of frame, finish consuming first 32-bit word from the fifo.
      buffer[0] = iFifoData[0];
      for(i=1; i <= 3; i++)
      {
        status = fpgaRead( boardNumber, (channel << 4) | 0x08, &iFifoData[0]);
        buffer[i] = iFifoData[0] & 0xff;
      }
      count -= 4;
      pbuff = &buffer[4];
      break;
    }
    fifoCount--;
  }

  //nCalls[channel - 1]++;

  if (fifoCount < count)
  {
    // we did not find a beginning of frame bit.
    *nFrames = 0;
    EndCriticalSection(qmutex);
    return GscApiFifoOverrun;
  }

  // Get the data using the second DMA channel
  if (drvMajorVersion >= 5)
    //status = gscPerformPlxPhysDma(boardNumber, channel, buffer, count, GSC_DATA_RECEIVE, 1);
    status = gscPerformDma( boardNumber, channel, pbuff, count, GSC_DATA_RECEIVE, 0);
  else
    status = gscPerformDma( boardNumber, channel, pbuff, count, GSC_DATA_RECEIVE, PrimaryPciChannel1);

  //status = GscSio4ChannelReceiveDataAndWait( boardNumber, channel, pbuff, count, 2000, &bytesTransferred );

  // Do the requested number of reads
  /*
  regCount = count;
  while (regCount > 0)
  {
    status = fpgaRead8( boardNumber, (channel << 4) | 0x08, pbuff);
    if (status) return status;
    pbuff = pbuff + 1;
    regCount--;
  }
  */
  EndCriticalSection(qmutex);
  return status;
}


//*----------------------------------------------------------GscSio4CTCResetTimer------------------
//*
//* Description:
//*
//*     This routine resets the CTC timer value.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCResetTimer(int boardNumber)
{
  int status = 0;

  return status = fpgaWrite( boardNumber, CTC_TIMER_CONTROL_REG, GSC_CTC_TIMER_RESET );
}


//*----------------------------------------------------------GscSio4CTCSetTimer--------------------
//*
//* Description:
//*
//*     This routine sets the CTC timer BCD or binary value.
//*
//* Parameters:
//*
//*     Parameter                         Description
//*------------------------------------------------------------------------------------------------
//*     int                 boardNumber   The board number as determined by the FindBoards()
//*                                       function.
//*     unsigned short      usecs
//*     unsigned int        hilo
//*     int                 timerMode
//*
//* Return Value:
//*     0 - success; > 0, a GscApi error code.
//*
//* Programmer's notes:
//*
//*------------------------------------------------------------------------------------------------

int GscSio4CTCSetTimer(int boardNumber, unsigned short usecs,  unsigned int hilo, int timerMode)
{
  int status = 0;

  status = fpgaWrite( boardNumber, CTC_TIMER_USEC_REG, usecs );
  status = fpgaWrite( boardNumber, CTC_TIMER_HILO_REG, hilo );

  return status;
}
