//////////////////////////////////////////////////////////////////////////////
//
// GscApiClock.c
//
//  This module contains all of the routines needed to program the Cypress
// Programmable Oscillator.
//
#include "GscApiLocal.h"
#include <math.h>

#define POST_DIVIDER_BITS    11         // Number of bits in the post divider
unsigned char	block_1[20];			// Bytes to be written to 08->1B
unsigned char	block_2[24];			// Bytes to be written to 40->57
unsigned char	second_block_1[20];		// Bytes to be written to 08->1B (2nd chip)
unsigned char	second_block_2[24];		// Bytes to be written to 40->57 (2nd chip)

enum {SRC_REF=0,SRC_PLL1,SRC_PLL2,SRC_PLL3,SRC_PLL4, SRC_UNKNOWN};

// This array hold the current Frequency for the 4 clocks
PLL_PARAMS params[4];

// This array hold the current Frequency for the REF and up to 4 PLLs
PLL_USAGE pllUsage[5];

//---------------------------------------------------------------- ProgramChip
//
//  This routine actually programs the clock chip with the data
// in the block_1 and block_2 arrays.
//
void ProgramChip( int boardNumber, int postDivider)
{
  int i, value;

  // Write the post divider info
  fpgaWrite( boardNumber, POSC_CONTROL_STATUS_REG, postDivider);

  // Write the first 18 bytes from block 1
  for (i=0; i<18; i++)
    {
      // Write the address
      fpgaWrite( boardNumber, POSC_RAM_ADDRESS_REG, i+8);

      // Write the Data
      fpgaWrite( boardNumber, POSC_RAM_DATA_REG, block_1[i]);

	  // Write the 2nd block Data
      fpgaWrite( boardNumber, POSC_RAM2_DATA_REG, second_block_1[i]);
    }

  // Write the next 24 bytes from block 2
  for (i=0; i<24; i++)
    {
      // Write the address
      fpgaWrite( boardNumber, POSC_RAM_ADDRESS_REG, i+0x40);

      // Write the Data
      fpgaWrite( boardNumber, POSC_RAM_DATA_REG, block_2[i]);

	  // Write the 2nd block Data
      fpgaWrite( boardNumber, POSC_RAM2_DATA_REG, second_block_2[i]);
    }

  // Write the Go command and allow it to start
  fpgaWrite( boardNumber, POSC_CONTROL_STATUS_REG, 0x0001);

  // Wait for it to complete
  do
    {
      // Give it some time
      Sleep(1);

      // Read status
      fpgaRead( boardNumber, POSC_CONTROL_STATUS_REG, &value);
    } while ((value & 0x01) == 0);
}

//---------------------------------------------------- getLfValue
//
//  This routine is used to calculate the compensation
// value for a given set of P and PO values.
//
unsigned char getLfValue( int P, int PO)
{
  int pt = 2*P+6+PO;
  if (pt < 232) return 0x00;
  if (pt < 627) return 0x08;
  if (pt < 835) return 0x10;
  if (pt <1044) return 0x18;
  return 0x20;
}

//----------- FillInPLL1
//
void FillInPLL1( PLL_PARAMS info)
{
  int i;

  // Fill in first block
  block_2[0] = info.Q;
  block_2[1] = info.P & 0x00ff;
  block_2[2] = 0x40 | (info.PO<<2) | (info.P>>8) | getLfValue( info.P, info.PO);

  // Copy first block to other 7
  for (i=1;i<8;i++)
    {
      block_2[3*i] = block_2[0];
      block_2[3*i+1] = block_2[1];
      block_2[3*i+2] = block_2[2];
    }
}

//----------- FillInPLL2
//
void FillInPLL2( PLL_PARAMS info)
{
  // Fill in block
  block_1[9] = info.Q;
  block_1[10] = info.P & 0x00ff;
  block_1[11] = 0x40 | (info.PO<<2) | (info.P>>8) | getLfValue(info.P, info.PO);
}

//----------- FillInPLL3
//
void FillInPLL3( PLL_PARAMS info)
{
  // Fill in block
  block_1[12] = info.Q;
  block_1[13] = info.P & 0x00ff;
  block_1[14] = 0x40 | (info.PO<<2) | (info.P>>8) | getLfValue(info.P, info.PO);
}

void FillInPLL4( PLL_PARAMS info)
{
  // Fill in block
  second_block_1[12] = info.Q;
  second_block_1[13] = info.P & 0x00ff;
  second_block_1[14] = 0x40 | (info.PO<<2) | (info.P>>8) | getLfValue(info.P, info.PO);
}

//-------------------------------------------------------- DetermineParameters
//
//  This routine determines the optimum values for the PLL
// paramters given the desired frequency.
//
void DetermineParameters( int iChan)
{
  int Div;
  int Q, PO;
  float volatile firstStep, secondStep, pFloat, pError;
  int desiredFreq = params[iChan].Freq;
  int diff;
  int minDiv,maxDiv;
  int minTotalDiv,maxTotalDiv;
  int refDiv;
  int postDiv;
  int i;
  int loopDone = 0x00;
  int currentPostDivider, currentPD;

  // Initialize results, assume reference driver
  params[iChan].Div = 0;
  params[iChan].Q = 0;
  params[iChan].P = 0;
  params[iChan].PO = 0;
  params[iChan].Error = 100;
  params[iChan].PLL = SRC_REF;
  params[iChan].PostDivider = 0;
  params[iChan].pError = 1;

  // Do nothing for 0 freq.
  if (desiredFreq == 0) return;

  // Check to see if a previous PLL setting can be used
  for (i=0;i<5;i++)
  {
    if (desiredFreq <= pllUsage[i].Freq)
    {
      // Get a possible divider
      refDiv = pllUsage[i].Freq / desiredFreq;
      diff = abs( (refDiv * desiredFreq) - pllUsage[i].Freq);
      if (diff <= (pllUsage[i].Freq/10000))
      {
        // See if we need a post divider
        postDiv = 0;
        while (refDiv > 127)
        {
          if ((refDiv % 2) != 0) break;
          refDiv /=2;
          postDiv++;
        }

        if ((refDiv < 128) && (postDiv <= POST_DIVIDER_BITS))
        {
          params[iChan].Error = diff;
          params[iChan].pError = 0;
          params[iChan].Div = refDiv;
          params[iChan].Q = pllUsage[i].Q;
          params[iChan].P = pllUsage[i].P;
          params[iChan].PO = pllUsage[i].PO;
          params[iChan].PLL = i;
          params[iChan].PostDivider = postDiv;
          return;
        }
      }
    }
  }

  // Since we are not using the reference, change to unknown
  params[iChan].PLL = SRC_UNKNOWN;

  // The maximum and minimum PLL frequencies are 400MHz and 100MHz, so the range of
  // dividers can be determined.
  maxTotalDiv = (400000000 / desiredFreq) + 1;
  minTotalDiv = maxTotalDiv/4;

  // Calculate the first step of the equation
  firstStep = (float)desiredFreq/(float)(2*REFERENCE_FREQ);

  // Loop through all of the possible dividers
  currentPostDivider = 0;
  do
  {
    currentPD = (int)pow(2,currentPostDivider);
    minDiv = minTotalDiv/currentPD;
    if (minDiv>127) currentPostDivider++;
    if (currentPostDivider >= POST_DIVIDER_BITS) break;
  } while (minDiv > 127);

  while (loopDone == 0x00)
  {
    currentPD = (int)pow(2,currentPostDivider);
    minDiv = minTotalDiv/currentPD;
    if ((minDiv * currentPD * desiredFreq) < 100000000) minDiv++;
    if (minDiv < 1) minDiv = 1;
    maxDiv = maxTotalDiv/currentPD;
    if ((maxDiv * currentPD * desiredFreq) > 400000000) maxDiv--;
    if (maxDiv > 127) maxDiv = 127;

    // Loop through all of the divisors
    for (Div=minDiv;Div<=maxDiv;Div++)
    {
      secondStep = firstStep * (float)(Div * currentPD);

      for (Q=0;Q<256;Q++)
      {
        for (PO=0;PO<2;PO++)
        {
          // Calculate the P value and its error
          pFloat = secondStep * (float)(Q+2) - (float)(PO/2) - (float)3.0;
          pError = pFloat - (int)pFloat;

          // If this is a new minimum error, save the info
          if ((pError < params[iChan].pError) && (pFloat < 1024.0))

          {
            params[iChan].PostDivider = currentPostDivider;
            params[iChan].Div = Div;
            params[iChan].Q = Q;
            params[iChan].P = (int)pFloat;
            params[iChan].PO = PO;
            params[iChan].pError = pError;

            // If the error is 0, we are done
            if (pError <= 0.0001) return;
          }
        }
      }
    }

    if (minDiv == maxDiv) loopDone = 0xff;
    else currentPostDivider++;
    if (currentPostDivider > POST_DIVIDER_BITS) loopDone = 0xff;
  }
}


//-------------------------------------------------- ProgramOscillatorFromFreq
void ProgOscFromFreq( int boardNumber)
{

  int iChan,pll=SRC_PLL1;
  int postDivider;

  // Initialize the PLL frequency array
  pllUsage[0].Freq = REFERENCE_FREQ;	// Reference
  pllUsage[1].Freq = 0;				// PLL 1
  pllUsage[2].Freq = 0;				// PLL 2
  pllUsage[3].Freq = 0;				// PLL 3
  pllUsage[4].Freq = 0;				// PLL 4

  // Get the parameters and determine the PLL arangement
  for (iChan=0;iChan<4;iChan++)
    {
      // Get the PLL setup for this channel
      DetermineParameters(iChan);

      // If a previous PLL was not used
      if (params[iChan].PLL == SRC_UNKNOWN)
	{
	  if (pll < 5)
	    {
	      pllUsage[pll].Freq = params[iChan].Div*params[iChan].Freq*(int)pow(2,params[iChan].PostDivider);
	      pllUsage[pll].Q = params[iChan].Q;
	      pllUsage[pll].P = params[iChan].P;
	      pllUsage[pll].PO = params[iChan].PO;
	      params[iChan].PLL = pll++;
	    }
	  else
	    {
	      params[iChan].Div = 1;
	      params[iChan].PLL = 0;
	    }
	}
    }

  // Fill the block arrays
  memset( block_1, 0, sizeof(block_1));
  memset( block_2, 0, sizeof(block_2));

  block_1[0] = params[0].Div;		// 8
  block_1[1] = block_1[0];		// 9
  block_1[2] = params[1].Div;		// a
  block_1[3] = block_1[2];		// b
  block_1[4] = params[2].Div;		// c
  block_1[5] = params[3].Div;		// d
  block_1[7] = 0x50;			// f
  block_1[8] = 0x55;			// 10
//	block_1[18] = 0xea;				// 1a
//	block_1[19] = 0x08;				// 1b

  // Fill in the proper PLL info for each clockout
  for (iChan=0;iChan<4;iChan++)
    {
      // Make room for next bits
      block_1[6] >>= 2;

      switch (params[iChan].PLL)
	{
	case SRC_PLL1: FillInPLL1( params[iChan]); block_1[6] |= 0x40; break;
	case SRC_PLL2: FillInPLL2( params[iChan]); block_1[6] |= 0x80; break;
	case SRC_PLL3: FillInPLL3( params[iChan]); block_1[6] |= 0xc0; break;
	}
    }

  // Copy the first chip data into the second
  memcpy( second_block_1, block_1, sizeof(block_1));
  memcpy( second_block_2, block_2, sizeof(block_2));

  // See if PLL 4 was used for channel 4
  if (params[3].PLL == SRC_PLL4)
  {
	  FillInPLL4( params[3]);
	  second_block_1[6] |= 0xc0;
  }

  // Generate the post Divider info
  postDivider = ((params[3].PostDivider & 0x0f) << 20) +
    ((params[2].PostDivider & 0x0f) << 16) +
    ((params[1].PostDivider & 0x0f) << 12) +
    ((params[0].PostDivider & 0x0f) <<  8) +
    0x80;

  // Program the chip
  ProgramChip( boardNumber, postDivider);


//temp
#if 0
  for(iChan=0;iChan<4;iChan++)
    {
      printf("Ch %d: Freq:=%10d, PLL=%d, PllFreq=%10d, P=%3d, Q=%3d, PO=%d, Div=%3d, PostDiv=%2d, Error=%6.4f\n",
	     iChan+1,
	     params[iChan].Freq,
	     params[iChan].PLL,
	     pllUsage[ params[iChan].PLL].Freq,
	     params[iChan].P,
	     params[iChan].Q,
	     params[iChan].PO,
	     params[iChan].Div,
	     params[iChan].PostDivider,
	     params[iChan].pError
	     );
    }
  printf("\n");
#endif
//temp
}



//----------------------------------------------------- GscSio4ChannelSetClock
int GscSio4ChannelSetClock( int boardNumber, int channel, int frequency)
{

  int i,status=0x00;
  int bn=boardNumber-1;
  int ch=channel-1;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // If the new frequency is the same as the old, just return
  //if (frequency == boardInformation[bn].channelFrequency[ ch])
  //  return status;

  // Setup the new frequency
  boardInformation[ bn].channelFrequency[ch] = frequency;
  for (i=0;i<4;i++) params[i].Freq = boardInformation[ bn].channelFrequency[i];

  ProgOscFromFreq(boardNumber);

  return status;
}

//----------------------------------------------- GscSio4ChannelGetClockSource
int gscsio4ChannelGetClockSource( int boardNumber, int channel, int *txSource, int *rxSource)
{
	int status=0x00;
	int pinSource;

	// Read the current pin source register
	status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG + (channel-1)*4, &pinSource);
	if (status) return status;

	// Handle the -Sync board separately
	if ((boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
      (gscSio4ChannelIsBxrSync(boardNumber, channel)))
	{
		// GSC_CLOCK_EXT_RX_AUX is not supported by SYNC boards

		// Process the Tx Source
		switch(pinSource & 0x0007)
		{
		case 0x0002:	*txSource = GSC_CLOCK_EXTERNAL; break;
		default:		*txSource = GSC_CLOCK_INTERNAL; break;
		}

		// Process the Rx Source. GSC_CLOCK_INTERNAL is not permitted for -sync
		// board.
		*rxSource = GSC_CLOCK_EXTERNAL;

	}

	// Otherwise, handle the standard board
	else
	{
		// Process the Tx Source.
		switch(pinSource & 0x0007)
		{
		case 0x0004:	*txSource = GSC_CLOCK_EXTERNAL;		break;
		case 0x0005:	*txSource = GSC_CLOCK_EXT_RX_AUX;	break;
		default:		*txSource = GSC_CLOCK_INTERNAL;		break;
		}

		// Process the Rx Source
		switch(pinSource & 0x0038)
		{
		case 0x0020:	*rxSource = GSC_CLOCK_EXTERNAL;		break;
		case 0x0028:	*rxSource = GSC_CLOCK_EXT_RX_AUX;	break;
		default:		*rxSource = GSC_CLOCK_INTERNAL;		break;
		}
	}

	return status;
}

//----------------------------------------------- quickNormalSetClockSource
int quickNormalSetClockSource( int boardNumber, int channel, int txSource, int rxSource)
{

  int status=0x00;
  int pinSource;

  // Read the current pin source register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG + (channel-1)*4, &pinSource);
  if (status) return status;

  // Process the Tx Source
  switch(txSource)
    {
    case GSC_CLOCK_INTERNAL:	pinSource &= ~0x0007; break;
    case GSC_CLOCK_EXTERNAL:	pinSource &= ~0x0007; pinSource |= 0x0004; break;
    case GSC_CLOCK_EXT_RX_AUX:	pinSource &= ~0x0007; pinSource |= 0x0005; break;
    default:					return GscApiInvalidParameter;
    }

  // Process the Rx Source
  switch(rxSource)
    {
    case GSC_CLOCK_INTERNAL:	pinSource &= ~0x0038; break;
    case GSC_CLOCK_EXTERNAL:	pinSource &= ~0x0038; pinSource |= 0x0020; break;
    case GSC_CLOCK_EXT_RX_AUX:	pinSource &= ~0x0038; pinSource |= 0x0028; break;
    default:					return GscApiInvalidParameter;
    }

  // Write the new pin source register
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG + (channel-1)*4, pinSource);

  return status;
}
