//////////////////////////////////////////////////////////////////////////////
//
// GscApiCommon.c
//
//  This module contains all of the routines that are common to multiple modes
// of operation. Since these routines are not available directly by the user,
// they assume that the board and channel numbers have been validated by earlier
// routines.
//
#include <stdio.h>
#include "GscApiLocal.h"
#include "GscApiPlxLegacy.h"
#include "GscApiPlxdload.h"


//------------------------------------------------------- zilogRead/zilogWrite
//
//  These routines read/write to the Zilog USC
//
int zilogRead(int boardNumber, int channel, int reg, int *value)
{
  int status = 0x00;
  U32 loByte, hiByte;

  // Calculate channel address
  channel = ((channel << 8) | reg);

  // Read the 16 bit value with two 8 bit reads
  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceRead_Rtdl( &sio4BoardHandles[ boardNumber-1], 2, channel, &loByte, 0x01, BitSize8, FALSE );
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceRead6_Rtdl( &sio4BoardHandles6[ boardNumber-1], 2, channel, &loByte, 0x01, BitSize8, FALSE );
  else
    status = PlxBusIopRead_Rtdl( boardInformation[boardNumber-1].handle, IopSpace0, channel, FALSE, &loByte, 0x01, BitSize8);
  if(status == ApiSuccess)
  {
    channel++;
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_PciBarSpaceRead_Rtdl( &sio4BoardHandles[ boardNumber-1], 2, channel, &hiByte, 0x01, BitSize8, FALSE );
    else if (drvMajorVersion >= 5)
      status = PlxPci_PciBarSpaceRead6_Rtdl( &sio4BoardHandles6[ boardNumber-1], 2, channel, &hiByte, 0x01, BitSize8, FALSE );
    else
      status = PlxBusIopRead_Rtdl( boardInformation[boardNumber-1].handle, IopSpace0, channel, FALSE, &hiByte, 0x01, BitSize8);
  }
  EndCriticalSection(PlxApiCalls);

  // Return if an error occurred
  if(status != ApiSuccess) return status;

  // Return the value
  *value = ((hiByte&0xff)<<8)|(loByte&0xff);

  return 0x00;
}

int zilogWrite(int boardNumber, int channel, int reg, int value)
{
  int status = 0x00;
  int data;

  // Get lower byte
  data = value & 0xff;

  // Calculate proper channel address
  channel = ((channel << 8)| reg);

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceWrite_Rtdl( &sio4BoardHandles[boardNumber-1], 2, channel, &data, 0x01, BitSize8, FALSE );
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceWrite6_Rtdl( &sio4BoardHandles6[boardNumber-1], 2, channel, &data, 0x01, BitSize8, FALSE );
  else
    status = PlxBusIopWrite_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, channel, FALSE, &data, 0x01, BitSize8);

  // If that byte succeeded
  if(status == ApiSuccess){
    // Get the upper byte
    data = (value>>8) & 0xff;
    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_PciBarSpaceWrite_Rtdl( &sio4BoardHandles[boardNumber-1], 2, channel+1, &data, 0x01, BitSize8, FALSE );
    else if (drvMajorVersion >= 5)
      status = PlxPci_PciBarSpaceWrite6_Rtdl( &sio4BoardHandles6[boardNumber-1], 2, channel+1, &data, 0x01, BitSize8, FALSE );
    else
      status = PlxBusIopWrite_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, channel+1, FALSE, &data, 0x01, BitSize8);
  }
  EndCriticalSection(PlxApiCalls);

  // Return error code if not successful
  if(status != ApiSuccess) return status;

  return 0x00;
}


//--------------------------------------------------------- zilogGetDataLength
//
//  These routines set/return the currently programmed Tx and Rx
// data lengths in the Zilog USC.
//
int zilogGetDataLength(int boardNumber, int channel, int *txDataSize, int *rxDataSize)
{
  int status = 0x00;
  int newValue;

  // Get current Transmit Mode Register Value
  status = zilogRead( boardNumber, channel, USC_TMR, &newValue);
  if (status) return status;

  // Return the Tx Data Size
  newValue = (newValue >> 2) & 0x07;
  if (newValue == 0) *txDataSize = 8;
  else *txDataSize = newValue;

  // Get current Receiver Mode Register Value
  status = zilogRead( boardNumber, channel, USC_RMR, &newValue);
  if (status) return status;

  // Return the Rx Data Size
  newValue = (newValue >> 2) & 0x07;
  if (newValue == 0) *rxDataSize = 8;
  else *rxDataSize = newValue;

  return status;
}
int zilogSetDataLength(int boardNumber, int channel, int txDataSize, int rxDataSize)
{
  int status = 0x00;
  int oldValue, newValue;

  // Range check the data size
  if((txDataSize < 1) || (txDataSize > 8)) return GscApiInvalidParameter;
  if((rxDataSize < 1) || (rxDataSize > 8)) return GscApiInvalidParameter;

  // Make 8 bits => 0 and shift the bits up 2 places
  txDataSize = (txDataSize & 0x07) << 2;
  rxDataSize = (rxDataSize & 0x07) << 2;

  // Get current Transmit Mode Register Value
  status = zilogRead( boardNumber, channel, USC_TMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0x001c) | txDataSize;
  status = zilogWrite( boardNumber, channel, USC_TMR, newValue);
  if (status) return status;

  // Get current Receiver Mode Register Value
  status = zilogRead( boardNumber, channel, USC_RMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0x001c) | rxDataSize;
  status = zilogWrite( boardNumber, channel, USC_RMR, newValue);
  if (status) return status;

  return status;
}

int zilogSetTxUnderRun( int boardNumber, int channel, int mode)
{
  int status = 0x00;
  int temp;

  // Read the channel mode register and clear the Tx UnderRun bits
  status = zilogRead( boardNumber, channel, USC_CMR, &temp);
  if (status) return status;
  temp &= ~0xc000;

  // Set the Tx Underrun Condition
  switch (mode)
  {
  case GSC_ABORT:				 		     break;
  case GSC_EXTENDED_ABORT: temp |= 0x4000; break;
  case GSC_FLAG:			 temp |= 0x8000; break;
  case GSC_CRC_FLAG:		 temp |= 0xc000; break;
  default:				 return GscApiInvalidParameter;
  }

  // Write the cahnnel mode register back out
  status = zilogWrite( boardNumber, channel, USC_CMR, temp);
  return status;
}


int zilogGetTxPreamble( int boardNumber, int channel, int *mode, int *pattern)
{
  int status = 0x00;
  int curCMR, curCCR;

  // Read the Channel mode register and clear the Preamble bit
  status = zilogRead(boardNumber, channel, USC_CMR, &curCMR);
  if (status) return status;

  // Read the Channel control register and clear the Preamble bits
  status = zilogRead(boardNumber, channel, USC_CCR, &curCCR);
  if (status) return status;

  // Determine the mode
  if (curCMR & 0x2000)
  {
    switch (curCCR & 0x0c00)
    {
    case 0x0000:		*mode = GSC_PREAMBLE_8BITS;		break;
    case 0x0400:		*mode = GSC_PREAMBLE_16BITS;	break;
    case 0x0800:		*mode = GSC_PREAMBLE_32BITS;	break;
    case 0x0c00:		*mode = GSC_PREAMBLE_64BITS;	break;
    default:			break;
    }
  }
  else
    *mode = GSC_DISABLED;

  // Determine the pattern
  switch( curCCR & 0x0300 )
  {
  case 0x0000:		*pattern = GSC_PREAMBLE_ALL_0;		break;
  case 0x0100:		*pattern = GSC_PREAMBLE_ALL_1;		break;
  case 0x0200:		*pattern = GSC_PREAMBLE_ALL_1_0;	break;
  case 0x0300:		*pattern = GSC_PREAMBLE_ALL_0_1;	break;
  default:			break;
  }

  return status;
}

int zilogSetTxPreamble( int boardNumber, int channel, int mode, int pattern)
{
  int status = 0x00;
  int newCMR, newCCR;

  // Read the Channel mode register and clear the Preamble bit
  status = zilogRead(boardNumber, channel, USC_CMR, &newCMR);
  if (status) return status;
  newCMR &= ~0x2000;

  // Read the Channel control register and clear the Preamble bits
  status = zilogRead(boardNumber, channel, USC_CCR, &newCCR);
  if (status) return status;
  newCCR &= ~0x0f00;

  // Process the mode
  switch (mode)
  {
  case GSC_DISABLED:												break;
  case GSC_PREAMBLE_8BITS:	newCMR |= 0x2000; newCCR |= 0x0000;	break;
  case GSC_PREAMBLE_16BITS:	newCMR |= 0x2000; newCCR |= 0x0400;	break;
  case GSC_PREAMBLE_32BITS:	newCMR |= 0x2000; newCCR |= 0x0800;	break;
  case GSC_PREAMBLE_64BITS:	newCMR |= 0x2000; newCCR |= 0x0c00;	break;
  default:					return GscApiInvalidParameter;
  }

  // Process the pattern
  switch( pattern)
  {
  case GSC_PREAMBLE_ALL_0:	newCCR |= 0x0000;	break;
  case GSC_PREAMBLE_ALL_1:	newCCR |= 0x0100;	break;
  case GSC_PREAMBLE_ALL_1_0:	newCCR |= 0x0200;	break;
  case GSC_PREAMBLE_ALL_0_1:	newCCR |= 0x0300;	break;
  default:					return GscApiInvalidParameter;
  }

  // Update the Channel Mode Register
  status = zilogWrite( boardNumber, channel, USC_CMR, newCMR);
  if (status) return status;

  // Update the Channel Control Register
  status = zilogWrite( boardNumber, channel, USC_CCR, newCCR);

  return status;
}

int zilogSetTxSharedZero( int boardNumber, int channel, int mode)
{
  int status = 0x00;
  int newCMR;

  // Read the Channel mode register and clear the Shared Zero bit
  status = zilogRead(boardNumber, channel, USC_CMR, &newCMR);
  if (status) return status;
  newCMR &= ~0x1000;

  // Process the mode
  switch (mode)
  {
  case GSC_DISABLED:						break;
  case GSC_ENABLED:	newCMR |= 0x1000;	break;
  default:			return GscApiInvalidParameter;
  }

  // Update the Channel Mode Register
  status = zilogWrite( boardNumber, channel, USC_CMR, newCMR);

  return status;
}

int zilogGetRxAddrSearch( int boardNumber, int channel, int *mode, int *address)
{
  int status = 0x00;
  int curCMR, curRSR;

  // Read the Channel mode register and clear the Rx Address Search Bits
  status = zilogRead(boardNumber, channel, USC_CMR, &curCMR);
  if (status) return status;

  // Determine the mode
  switch (curCMR & 0x00f0)
  {
  case 0x0010:
  case 0x0050:
  case 0x0090:
  case 0x00d0:		*mode = GSC_ADDR_ONLY;				break;
  case 0x0020:
  case 0x00a0:		*mode = GSC_ADDR_CONTROL1;			break;
  case 0x0060:		*mode = GSC_ADDR_CONTROL2;			break;
  case 0x0030:		*mode = GSC_EXT_ADDR_CONTROL1;		break;
  case 0x0070:		*mode = GSC_EXT_ADDR_CONTROL2;		break;
  case 0x00b0:		*mode = GSC_EXT_ADDR_EXT_CONTROL1;	break;
  case 0x00f0:		*mode = GSC_EXT_ADDR_EXT_CONTROL2;	break;
  default:			*mode = GSC_DISABLED;				break;
  }

  // Update the address register
  status = zilogRead(boardNumber, channel, USC_RSR, &curRSR);
  if (status) return status;
  *address = curRSR & 0x00ff;

  return status;
}

int zilogSetRxAddrSearch( int boardNumber, int channel, int mode, int address)
{
  int status = 0x00;
  int newCMR, newRSR;

  // Read the Channel mode register and clear the Rx Address Search Bits
  status = zilogRead(boardNumber, channel, USC_CMR, &newCMR);
  if (status) return status;
  newCMR &= ~0x00f0;

  // Process the mode
  switch (mode)
  {
  case GSC_DISABLED:						 		  break;
  case GSC_ADDR_ONLY:				newCMR |= 0x0010; break;
  case GSC_ADDR_CONTROL1:			newCMR |= 0x0020; break;
  case GSC_ADDR_CONTROL2:			newCMR |= 0x0060; break;
  case GSC_EXT_ADDR_CONTROL1:		newCMR |= 0x0030; break;
  case GSC_EXT_ADDR_CONTROL2:		newCMR |= 0x0070; break;
  case GSC_EXT_ADDR_EXT_CONTROL1:	newCMR |= 0x00b0; break;
  case GSC_EXT_ADDR_EXT_CONTROL2:	newCMR |= 0x00f0; break;
  default:						return GscApiInvalidParameter;
  }

  // Update the Channel Mode Register
  status = zilogWrite( boardNumber, channel, USC_CMR, newCMR);
  if (status) return status;

  // Update the address register
  status = zilogRead(boardNumber, channel, USC_RSR, &newRSR);
  if (status) return status;
  newRSR &= ~0x00ff;
  newRSR |= (address & 0xff);
  status = zilogWrite( boardNumber, channel, USC_RSR, newRSR);

  return status;
}

int zilogGetCrcMode( int boardNumber, int channel, int *crcMode, int *initialValue)
{
  int status = 0x00;
  int value = 0;

  // Get current Receiver Mode Register Value
  status = zilogRead( boardNumber, channel, USC_RMR, &value);
  if (status) return status;

  // Determine the current CRC mode
  switch( value & 0x1a00 )
  {
  case 0x0000:			*crcMode = GSC_CRC_NONE; break;
  case 0x0a00:			*crcMode = GSC_CRC_16; break;
  case 0x1200:			*crcMode = GSC_CRC_32; break;
  case 0x0200:			*crcMode = GSC_CRC_CCITT; break;
  default:				return GscApiInvalidParameter;
  }

  // Determine the initial value
  if ( value & 0x0400 )
    *initialValue = GSC_CRC_INIT_1;
  else
    *initialValue = GSC_CRC_INIT_0;

  return status;
}

int zilogSetCrcMode( int boardNumber, int channel, int crcMode, int initialValue)
{
  int status = 0x00;
  int newValue, oldValue;

  // Handle the proper CRC mode
  switch( crcMode)
  {
  case GSC_CRC_NONE:		newValue = 0x0000; break;
  case GSC_CRC_16:		newValue = 0x0a00; break;
  case GSC_CRC_32:		newValue = 0x1200; break;
  case GSC_CRC_CCITT:		newValue = 0x0200; break;
  default:				return GscApiInvalidParameter;
  }

  // Handle the initial value
  if (initialValue == GSC_CRC_INIT_1) newValue |= 0x0400;
  else if (initialValue != GSC_CRC_INIT_0) return GscApiInvalidParameter;

  // Save the CRC mode and initial value
  boardInformation[ boardNumber-1].channelCrcMode[ channel-1] = crcMode;
  boardInformation[ boardNumber-1].channelCrcValue[ channel-1] = initialValue;

  // Get current Receiver Mode Register Value
  status = zilogRead( boardNumber, channel, USC_RMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  oldValue = (oldValue & ~0x1e00) | newValue;
  status = zilogWrite( boardNumber, channel, USC_RMR, oldValue);
  if (status) return status;

  // Enable the EOM/EOF CRC for all CRC types
  if (crcMode != GSC_CRC_NONE) newValue |= 0x0100;

  // Get current Transmit Mode Register Value
  status = zilogRead( boardNumber, channel, USC_TMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  oldValue = (oldValue & ~0x1f00) | newValue;
  status = zilogWrite( boardNumber, channel, USC_TMR, oldValue);
  if (status) return status;

  return status;
}

int zilogGetReceiverControl( int boardNumber, int channel, int *rxMode)
{
  int status = 0x00, data = 0x00;

  // Read the current Rx mode register
  status = zilogRead( boardNumber, channel, USC_RMR, &data);
  if (status) return status;

  // Determine whether the receiver is enabled.
  *rxMode = GSC_DISABLED;
  if (data & 0x0002)
    *rxMode = GSC_ENABLED;

  return status;
}

int zilogReceiverControl( int boardNumber, int channel, int rxMode)
{
  int status = 0x00, oldData;

  // Read the current Rx mode register
  status = zilogRead( boardNumber, channel, USC_RMR, &oldData);
  if (status) return status;

  // Process the desired Rx mode
  switch (rxMode)
  {
  case GSC_ENABLED:	oldData |=  0x0002; break;
  case GSC_DISABLED:	oldData &= ~0x0002; break;
  default:			return GscApiInvalidParameter;
  }

  // Write the new value back out
  status = zilogWrite(boardNumber, channel, USC_RMR, oldData);
  return status;
}

int zilogGetTransmitterControl( int boardNumber, int channel, int *txMode)
{
  int status = 0x00, data = 0x00;

  // Read the current Tx mode register
  status = zilogRead( boardNumber, channel, USC_TMR, &data);
  if (status) return status;

  // Process the desired Tx mode
  *txMode = GSC_DISABLED;
  if (data & 0x0002)
    *txMode = GSC_ENABLED;

  return status;
}

int zilogTransmitterControl( int boardNumber, int channel, int txMode)
{
  int status = 0x00, oldData;

  // Read the current Tx mode register
  status = zilogRead( boardNumber, channel, USC_TMR, &oldData);
  if (status) return status;

  // Process the desired Tx mode
  switch (txMode)
  {
  case GSC_ENABLED:	oldData |=  0x0002; break;
  case GSC_DISABLED:	oldData &= ~0x0002; break;
  default:			return GscApiInvalidParameter;
  }

  // Write the new value back out
  status = zilogWrite(boardNumber, channel, USC_TMR, oldData);
  return status;
}

//-------------------------------------------------- zilogSetEncoding
//
int zilogSetEncoding(int boardNumber, int channel, int encoding, int mode, int bitRate)
{
  int status = 0x00;
  int newValue, oldValue;
  int newCMCR = -1, newHCR = -1;
  int clockMultiplier;

  // Convert board and channel into indexes
  int bn = boardNumber - 1;
  int ch = channel - 1;

  // Determine the maximum BiPhase sampling rate (just in case)
  if      ( bitRate <= MAXIMUM_USC_CLOCK/32)	{ clockMultiplier = 32; newHCR = 0x0200;}
  else if ( bitRate <= MAXIMUM_USC_CLOCK/16) 	{ clockMultiplier = 16;	newHCR = 0x4600;}
  else if ( bitRate <= MAXIMUM_USC_CLOCK/8 ) 	{ clockMultiplier = 8; 	newHCR = 0x8a00;}
  else clockMultiplier = 0; // Flags a bad combination of clock speed and biphase

  // Handle the proper encoding
  switch( encoding)
  {
  case GSC_ENCODING_NRZ:
    newValue = 0x0000;
    newCMCR = (mode == GSC_MODE_ASYNC)?0x0012:0x0011;
    newHCR = 0x0000;
    if (mode == GSC_MODE_ASYNC) clockMultiplier = 16;
    else clockMultiplier = 1;
    break;

  case GSC_ENCODING_NRZB:
    newValue = 0x2000;
    newCMCR = (mode == GSC_MODE_ASYNC)?0x0012:0x0011;
    newHCR = 0x0000;
    if (mode == GSC_MODE_ASYNC) clockMultiplier = 16;
    else clockMultiplier = 1;
    break;

  case GSC_ENCODING_NRZI_MARK:
    newValue = 0x4000;
    newCMCR = 0x0011;
    newHCR = 0x0000;
    clockMultiplier = 1;
    break;

  case GSC_ENCODING_NRZI_SPACE:
    newValue = 0x6000;
    newCMCR = 0x0011;
    newHCR = 0x0000;
    clockMultiplier = 1;
    break;

  case GSC_ENCODING_BIPHASE_MARK:
    if (clockMultiplier == 0) return GscApiInvalidCombination;
    newValue = 0x8000;
    newCMCR = 0x30f3;
    break;

  case GSC_ENCODING_BIPHASE_SPACE:
    if (clockMultiplier == 0) return GscApiInvalidCombination;
    newValue = 0xa000;
    newCMCR = 0x30f3;
    break;

  case GSC_ENCODING_BIPHASE_LEVEL:
    if (clockMultiplier == 0) return GscApiInvalidCombination;
    newValue = 0xc000;
    newCMCR = 0x30f3;
    newHCR |= 0x0100;
    break;

  case GSC_ENCODING_DIFF_BIPHASE_LEVEL:
    if (clockMultiplier == 0) return GscApiInvalidCombination;
    newValue = 0xe000;
    newCMCR = 0x30f3;
    newHCR |= 0x0100;
    break;

  default:
    return GscApiInvalidParameter;
  }

  // Save the new info
  boardInformation[ bn].channelEncoding[ ch] = encoding;
  boardInformation[ bn].channelBitRate[ch] = bitRate;

  // Get current Transmit Mode Register Value
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_TMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & 0x1fff) | newValue;
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_TMR, newValue);
  if (status) return status;

  // Get current Receiver Mode Register Value
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_RMR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & 0x1fff) | newValue;
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_RMR, newValue);
  if (status) return status;

  // Update the Clock Mode Control Register
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_CMCR, newCMCR);
  if (status) return status;

  // Get current Hardware Configuration Register Value
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_HCR, &oldValue);
  if (status) return status;

  // Update the value and write it back out
  newValue = (oldValue & ~0xcf00) | newHCR;
  status = GscSio4ChannelRegisterWrite( boardNumber, channel, USC_HCR, newValue);
  if (status) return status;

  // Setup the baud rate (bit rate)
  status = GscSio4ChannelSetClock( boardNumber, channel, bitRate*clockMultiplier);
  if (status) return status;

  return status;
}

int zilogGetIdleLineCondition(int boardNumber, int channel, int *mode)
{
  int status = 0x00;
  int value = 0x00;

  // Read the current Transmit Control Status Register and mask out the idle line bits
  status = zilogRead( boardNumber, channel, USC_TCSR, &value);
  if (status) return status;

  // Determine the idle line condition
  switch(value & 0x0700)
  {
  case 0x0000:		*mode = GSC_IDLE_FLAGS;						break;
  case 0x0100:		*mode = GSC_IDLE_ALTERNATING_01;			break;
  case 0x0200:		*mode = GSC_IDLE_ALL_0;						break;
  case 0x0300:		*mode = GSC_IDLE_ALL_1;						break;
  case 0x0500:		*mode = GSC_IDLE_ALTERNATING_MARK_SPACE;	break;
  case 0x0600:		*mode = GSC_IDLE_ALL_SPACE;					break;
  case 0x0700:		*mode = GSC_IDLE_ALL_MARK;					break;
  default:			break;
  }

  return status;
}

int zilogSetIdleLineCondition(int boardNumber, int channel, int mode)
{
  int status = 0x00;
  int newValue;

  // Read the current Transmit Control Status Register and mask out the idle line bits
  status = zilogRead( boardNumber, channel, USC_TCSR, &newValue);
  if (status) return status;
  newValue &= ~0x0700;

  // Handle the proper idle line condition
  switch( mode)
  {
  case GSC_IDLE_FLAGS:					newValue |= 0x0000;	break;
  case GSC_IDLE_ALTERNATING_01:			newValue |= 0x0100;	break;
  case GSC_IDLE_ALL_0:					newValue |= 0x0200;	break;
  case GSC_IDLE_ALL_1:					newValue |= 0x0300;	break;
  case GSC_IDLE_ALTERNATING_MARK_SPACE:	newValue |= 0x0500;	break;
  case GSC_IDLE_ALL_SPACE:				newValue |= 0x0600;	break;
  case GSC_IDLE_ALL_MARK:					newValue |= 0x0700;	break;
  default:								return GscApiInvalidParameter;
  }

  // Write the new Transmit Control Status Register
  status = zilogWrite( boardNumber, channel, USC_TCSR, newValue);
  return status;
}


//--------------------------------------------------------- fpgaRead/fpgaWrite
//
//  These routines are used to read/write the local registers (FPGA)
// on the SIO4 board.
//
int fpgaRead( int boardNumber, int reg, int *result)
{
  int status = 0x00;

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceRead_Rtdl(&sio4BoardHandles[ boardNumber-1], 2, reg, result, 0x04, BitSize32, FALSE);
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceRead6_Rtdl(&sio4BoardHandles6[ boardNumber-1], 2, reg, result, 0x04, BitSize32, FALSE);
  else
    status = PlxBusIopRead_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, reg, FALSE, result, 0x04, BitSize32);
  EndCriticalSection(PlxApiCalls);

  // Return error code if not successful
  if(status != ApiSuccess) return status;
  return 0x00;
}

int fpgaWrite( int boardNumber, int reg, int value)
{
  int status = 0x00;

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceWrite_Rtdl( &sio4BoardHandles[boardNumber-1], 2, reg, &value, 0x04, BitSize32, FALSE );
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceWrite6_Rtdl( &sio4BoardHandles6[boardNumber-1], 2, reg, &value, 0x04, BitSize32, FALSE );
  else
    status = PlxBusIopWrite_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, reg, FALSE, &value, 0x04, BitSize32);
  EndCriticalSection(PlxApiCalls);

  // Return error code if not successful
  if(status != ApiSuccess) return status;
  return 0x00;
}


int fpgaRead16( int boardNumber, int reg, unsigned short *result)
{
  int status = 0x00;

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceRead_Rtdl(&sio4BoardHandles[ boardNumber-1], 2, reg, result, 0x02, BitSize16, FALSE);
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceRead6_Rtdl(&sio4BoardHandles6[ boardNumber-1], 2, reg, result, 0x02, BitSize16, FALSE);
  else
    status = PlxBusIopRead_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, reg, FALSE, result, 0x02, BitSize16);
  EndCriticalSection(PlxApiCalls);

  // Return error code if not successful
  if(status != ApiSuccess) return status;
  return 0x00;
}


int fpgaRead8( int boardNumber, int reg, unsigned char *result)
{
  int status = 0x00;

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceRead_Rtdl(&sio4BoardHandles[ boardNumber-1], 2, reg, result, 0x01, BitSize8, FALSE);
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceRead6_Rtdl(&sio4BoardHandles6[ boardNumber-1], 2, reg, result, 0x01, BitSize8, FALSE);
  else
    status = PlxBusIopRead_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, reg, FALSE, result, 0x02, BitSize16);
  EndCriticalSection(PlxApiCalls);

  // Return error code if not successful
  if(status != ApiSuccess) return status;
  return 0x00;
}

int fpgaWrite16( int boardNumber, int reg, unsigned short value)
{
  int status = 0x00;

  BeginCriticalSection(PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_PciBarSpaceWrite_Rtdl( &sio4BoardHandles[boardNumber-1], 2, reg, &value, 0x02, BitSize16, FALSE );
  else if (drvMajorVersion >= 5)
    status = PlxPci_PciBarSpaceWrite6_Rtdl( &sio4BoardHandles6[boardNumber-1], 2, reg, &value, 0x02, BitSize16, FALSE );
  else
    status = PlxBusIopWrite_Rtdl(boardInformation[boardNumber-1].handle, IopSpace0, reg, FALSE, &value, 0x02, BitSize16);
  EndCriticalSection(PlxApiCalls);

  // Return error code if not successful
  if(status != ApiSuccess) return status;
  return 0x00;
}


int fpgaSetEncoding(int boardNumber, int channel, int encoding, int bitRate)
{
  int status = 0x00;
  int oldValue;
  int clockMultiplier;

  // Convert board and channel into indexes
  int bn = boardNumber - 1;
  int ch = channel - 1;

  // Get the current Pin Source Register Value
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+4*ch, &oldValue);
  if (status) return status;

  // Handle the proper encoding
  switch( encoding)
  {
  case GSC_ENCODING_NRZ:
    oldValue &= ~0x000301c0;
    clockMultiplier = 2;
    break;

  case GSC_ENCODING_NRZB:
    oldValue |= 0x00010040;
    clockMultiplier = 2;
    break;

  default:
    strcpy( boardInformation[ bn].lastError[ ch], "Encoding Value Invalid");
    return GscApiInvalidParameter;
  }

  // Write it back out
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+4*ch, oldValue);
  if (status) return status;

  // Save the new info
  boardInformation[ bn].channelEncoding[ ch] = encoding;

  // Setup the baud rate (bit rate)
  status = GscSio4ChannelSetClock( boardNumber, channel, bitRate*clockMultiplier);
  if (status) return status;

  return status;
}


//---------------------------------------------------------- processSerialCard
//
//  This routine decodes the firmware register on the serial
// cards and fills in the board information structure.
//
int processSerialCard( int boardNumber)
{
  int status = 0x00;
  int bn = boardNumber-1;
  int ch;
  int fwRegister;
  int busSpeed, busWidth, formFactor, subId;
  int boardRev, boardVersion;
  char busSpeedText[5];

  // Read the ID register
  status = fpgaRead( boardNumber, FW_REVISION_REG, &fwRegister);
  if (status) return status;

  // Check for valid SIO4/SIO8
  if ((fwRegister & 0xc0000000) == 0xc0000000)
  {
    // Save the FPGA version information
    boardInformation[ bn].fpgaVersion = fwRegister;
    sprintf(boardInformation[ bn].fpgaVersionText, "%08X", fwRegister);

    // Get the various fields from the register
    busSpeed = (fwRegister >> 29) & 0x0001;
    busWidth = (fwRegister >> 28) & 0x0001;
    formFactor = (fwRegister >> 24) & 0x000f;
    subId = (fwRegister >> 20) & 0x000f;
    boardRev = (fwRegister >> 16) & 0x000f;
    boardVersion = (fwRegister >> 8) & 0x00ff;

    // Get the bus speed string
    if (busSpeed == 1) strcpy(busSpeedText, "66");
    else strcpy(busSpeedText, "");	// Don't display 33, only 66

    // Get the bus width string
    if (busWidth == 1) strcpy(boardInformation[bn].busWidth, "64");
    else strcpy(boardInformation[bn].busWidth, "32");

    // Get the form factor
    switch (formFactor)
    {
      // PCI
    case 0x01:
      strcpy(boardInformation[bn].formFactor, "PCI");
      switch (subId)
      {
      case 0x00: strcpy(boardInformation[bn].subId, "-SIO4B"); break;
      case 0x01: strcpy(boardInformation[bn].subId, "-SIO4BX"); break;
      case 0x02: strcpy(boardInformation[bn].subId, "-SIO8BX"); break;
      default:   strcpy(boardInformation[bn].subId, " Card"); break;
      }
      break;

      // PMC
    case 0x02:
      strcpy(boardInformation[bn].formFactor, "PMC");
      switch (subId)
      {
      case 0x00: strcpy(boardInformation[bn].subId, "-SIO4AR"); break;
      case 0x01: strcpy(boardInformation[bn].subId, "-SIO4BX"); break;
      case 0x02: strcpy(boardInformation[bn].subId, "-SIO4BXR"); break;
      default:   strcpy(boardInformation[bn].subId, " Card"); break;
      }
      break;

      // cPCI
    case 0x03:
      strcpy(boardInformation[bn].formFactor, "cPCI");
      switch (subId)
      {
      case 0x00: strcpy(boardInformation[bn].subId, "-SIO4A"); break;
      default:   strcpy(boardInformation[bn].subId, " Card"); break;
      }
      break;

      // PC104P
    case 0x04:
      strcpy(boardInformation[bn].formFactor, "PC104P");
      switch (subId)
      {
      case 0x00: strcpy(boardInformation[bn].subId, "-SIO4B"); break;
      case 0x01: strcpy(boardInformation[bn].subId, "-SIO4BX"); break;
      default:   strcpy(boardInformation[bn].subId, " Card"); break;
      }
      break;

    // PCIe
    case 0x05:
      strcpy(boardInformation[bn].formFactor, "PCIe");
      strcpy(busSpeedText, "");
      //strcpy(boardInformation[bn].subId, "-SIO8BX");
      strcpy(boardInformation[bn].subId, "");
      strcpy(boardInformation[bn].version, "");
      break;

    // XMC
    case 0x06:
      strcpy(boardInformation[bn].formFactor, "XMC");
      strcpy(busSpeedText, "");
      //strcpy(boardInformation[bn].subId, "-SIO8BX");
      strcpy(boardInformation[bn].subId, "");
      strcpy(boardInformation[bn].version, "");
      break;

      // Unknown
    default:
      strcpy(boardInformation[bn].formFactor, "Unknown");
      break;
    }

    // Get the hardware rev level
    if (boardRev == 0) strcpy(boardInformation[bn].rev, "NR");
    else sprintf(boardInformation[bn].rev, "%c", boardRev+'A'-1);

    // Get the board version
    switch (boardVersion)
    {
    case 0x02:
    case 0x03:
    case 0x04:
    case 0x05:
      strcpy(boardInformation[bn].version, "-SYNC");
      boardInformation[ bn].boardType = GSC_BOARD_TYPE_SYNC;
      break;
    case 0x06:
      strcpy(boardInformation[bn].version, "-BISYNC16");
      boardInformation[ bn].boardType = GSC_BOARD_TYPE_BISYNC16;
      break;
    case 0x07:
      strcpy(boardInformation[bn].version, "-RASYNC");
      boardInformation[ bn].boardType = GSC_BOARD_TYPE_RASYNC;
      break;
    case 0x08:
    case 0x0e:
      strcpy(boardInformation[bn].version, "-ASYNC");
      boardInformation[ bn].boardType = GSC_BOARD_TYPE_ASYNC;
      break;
    case 0x0c:
      strcpy(boardInformation[bn].version, "-QPSK");
      boardInformation[ bn].boardType = GSC_BOARD_TYPE_QPSK;
      break;
    default:
      strcpy(boardInformation[bn].version, "");
      boardInformation[ bn].boardType = GSC_BOARD_TYPE_NORMAL;
      boardInformation[ bn].features |= GSC_FEATURE_HAS_ZILOG_CHIP;
      break;
    }
  }

  // Update the board description if we have an identified board
  if (boardInformation[bn].formFactor[0] != '?')
  {
    // Build the board description, starting with teh form factor
    strcpy(boardInformation[ bn].boardDescription, boardInformation[bn].formFactor);

    // Append bus speed
    strcat(boardInformation[ bn].boardDescription, busSpeedText);

    // Append the sub type
    strcat(boardInformation[ bn].boardDescription, boardInformation[bn].subId);

    // Append the board version
    strcat(boardInformation[ bn].boardDescription, boardInformation[bn].version);
  }

  // Set the number of channels (currently all have 4)
  boardInformation[ bn].numberOfChannels = 4;

  // Get the Fifo Sizes
  for (ch=0; ch<boardInformation[ bn].numberOfChannels; ch++)
  {
    // Read the register
    status = fpgaRead( boardNumber, FIFO_SIZE_BASE_REG+ch*4, &fwRegister);
    if (status) return status;

    // Save the values
    boardInformation[ bn].txFifoSizes[ ch] = fwRegister & 0xffff;
    boardInformation[ bn].rxFifoSizes[ ch] = (fwRegister>>16) & 0xffff;
  }

   // Read the Feature register
  status = fpgaRead( boardNumber, FEATURES_REG, &fwRegister);
  if (status) return status;
  {
      if (fwRegister & 0x04000000)
          boardInformation[bn].features |= GSC_FEATURE_HAS_RSB_FIFO;
  }

  return status;
}

//---------------------------------------------------------- processAnalogCard
//
//  This routine decodes the firmware register on the serial
// cards and fills in the board information structure.
//
int processAnalogCard( int boardNumber, int subId)
{
  int status = 0x00;
  int bn = boardNumber-1;
  int tempRegister;

  // Set the board type
  boardInformation[ bn].boardType = GSC_BOARD_TYPE_ANALOG;

  // Setup the board and register information
  switch (subId)
  {
  case 0x3100:	// PMC-24SDI12
    // Read the Board Configuration Register
    status = fpgaRead( boardNumber, 0, &tempRegister);
    if (status) return status;

    // Save the FPGA version information
    boardInformation[ bn].fpgaVersion = tempRegister;
    sprintf(boardInformation[ bn].fpgaVersionText, "%08X", tempRegister);

    // Load the Board info
    strcpy(boardInformation[bn].formFactor, "PMC");
    strcpy(boardInformation[bn].subId, "-24SDI12");
    strcpy(boardInformation[bn].version, "");

    // Get the channel count
    if (tempRegister & 0x10000) boardInformation[ bn].numberOfChannels = 8;
    else if (tempRegister & 0x20000) boardInformation[ bn].numberOfChannels = 4;
    else boardInformation[ bn].numberOfChannels = 12;

    // Load the board description
    sprintf(boardInformation[bn].boardDescription, "PMC-24SDI12-%d", boardInformation[ bn].numberOfChannels);

    // Get the Rx Fifo size (set to 8k - only use is tied to SIO4 function called from
    // rxdmahandler to get the fifo size, from which a maximum delay for the rx dma thread
    // is derived). The size must be a 16-bit value.
    boardInformation[ bn].rxFifoSizes[0] = 0x2000;

    // Tx Fifo is always 0
    boardInformation[ bn].txFifoSizes[0] = 0;
    break;

  case 0x3101:	// PMC66-16AI64SSA/C
    // Read the Board Configuration Register
    status = fpgaRead( boardNumber, 0, &tempRegister);
    if (status) return status;

    // Save the FPGA version information
    boardInformation[ bn].fpgaVersion = tempRegister;
    sprintf(boardInformation[ bn].fpgaVersionText, "%08X", tempRegister);

    // Load the Board info
    strcpy(boardInformation[bn].formFactor, "PMC");
    strcpy(boardInformation[bn].subId, "-16AI64SSA/C");
    strcpy(boardInformation[bn].version, "");

    // Load the board description
    strcpy(boardInformation[bn].boardDescription, "PMC66-16AI64SSA/C");

    // Get the channel count
    if (tempRegister & 0x10000) boardInformation[ bn].numberOfChannels = 32;
    else boardInformation[ bn].numberOfChannels = 64;

    // Get the Rx Fifo size (set to 8k - only use is tied to SIO4 function called from
    // rxdmahandler to get the fifo size, from which a maximum delay for the rx dma thread
    // is derived). The size must be a 16-bit value.
    boardInformation[ bn].rxFifoSizes[0] = 0x2000;

    // Tx Fifo is always 0
    boardInformation[ bn].txFifoSizes[0] = 0;
    break;

  default:
    break;
  }

  return status;
}

//--------------------------------------------------------- determineBoardType
//
//  This routine is used to gather information about the specified
// board.
//
int determineBoardType(int boardNumber)
{
  int status = 0;
  int bn = boardNumber-1;
  int subSystem;
  RETURN_CODE rc;

  // Assume there is no valid information
  //	strcpy(boardInformation[bn].busSpeed, "??");
  strcpy(boardInformation[bn].busWidth, "??");
  strcpy(boardInformation[bn].formFactor, "??");
  strcpy(boardInformation[bn].subId, "??");
  strcpy(boardInformation[bn].rev, "??");
  strcpy(boardInformation[bn].version, "??");
  boardInformation[bn].features = 0x0000;
  strcpy(boardInformation[ bn].fpgaVersionText, "00000000");
  boardInformation[bn].fpgaVersion = 0x0000;
  strcpy(boardInformation[ bn].boardDescription, "Unknown");

  // Read the Vendor Subsystem ID from the board
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    subSystem = PlxPci_PciRegisterReadFast_Rtdl( &sio4BoardHandles[ bn], CFG_SUB_VENDOR_ID, &rc);
  else if (drvMajorVersion >= 5)
    subSystem = PlxPci_PciRegisterReadFast6_Rtdl( &sio4BoardHandles6[ bn], CFG_SUB_VENDOR_ID, &rc);
  else if (drvMajorVersion == 4)
    subSystem = PlxPciRegisterReadFast_Rtdl( boardInformation[ bn].handle, CFG_SUB_VENDOR_ID, &rc);
  else
    subSystem = 0x2401 << 16;  // hardcode to SIO4 for Plx 3.4
  subSystem = (subSystem>>16) & 0xffff;

  // Process the subSystem ID
  switch (subSystem)
  {
    // SIO4 type cards
  case 0x2401:	// 9080 version
  case 0x3198:	// 9056 version
    status = processSerialCard( boardNumber);
    break;

    // Analog Cards
  case 0x3100:	// PMC-24DSI12
  case 0x3101:	// PMC66-16AI64SSA/C
    status = processAnalogCard( boardNumber, subSystem);
    break;

  }

  return status;
}



//
// Register Access functions that assume the board and channel have been validated already
//
int getRxFifoCount( int boardNumber, int channel, int *rxFifoCount)
{
  int status =0x00;
  int bn = boardNumber-1;
  int ch = channel-1;
  int counts;

  // Handle the Analog boards Separately
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // Read the counts
    status = GscAnaGetInputBufferCount( boardNumber, &counts);
    *rxFifoCount = counts;
  }

  // Must be an SIOx board
  else
  {
    // Read the counts
    status = fpgaRead( boardNumber, FIFO_COUNT_BASE_REG+ch*4, &counts);
    *rxFifoCount = (counts >> 16) & 0xffff;
  }

  // Return the status
  return status;
}

int resetRxFifo( int boardNumber, int channel)
{
  int status =0x00;
  int bn = boardNumber-1;
  int ch = channel-1;
  int tempValue;
  int offset;

  // Handle the Analog boards Separately
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    status = GscAnaClearInputBuffer( boardNumber );
    Sleep(5);
  }

  // Must be an SIOx board
  else
  {
    // Calculate the offset to the proper register
    offset = ch<<4;

    // Read the control/status register
    status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + offset, &tempValue);
    if (status) return status;

    // Keep only the transceiver enable info for SIO4B, and bits 30-24 for -SYNC boards
    tempValue &= 0x7f00003c;

    // Reset the FIFO to force the new value to program
    status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG + offset, tempValue | 0x0002);
    Sleep(5);
  }

  // Return the status
  return status;
}

int resetBoard( int boardNumber)
{
  int status =0x00;
  int bn = boardNumber-1;

  // Handle the Analog boards Separately
  if (boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
  {
    // Reset the board by writing to the Board Control Register
    status = fpgaWrite( boardNumber, 0, 0x00008000);
    if (status) return status;
  }

  // Must be an SIOx board
  else
  {
    // Reset the board by writing to the Board Control Register
    status = fpgaWrite( boardNumber, BOARD_CONTROL_REG, 0x80000000);
    if (status) return status;
  }

  // Give the reset time to happen
  Sleep(10);

  // Return the status
  return status;
}
