//////////////////////////////////////////////////////////////////////////////
//
// GscApiHdlc.c
//
//  This module contains all of the routines that handle the HDLC mode
// of operation. This mode is only valid on boards that contain a Zilog
// USC chip.
//
#include "GscApiLocal.h"

//---------------------------------------------- GscSio4ChannelGetHdlcDefaults
//
//  This routine fills in the provided HDLC Configuration
// structure with their default values.
//
int GscSio4HdlcGetDefaults( PGSC_HDLC_CONFIG config)
{
  int status = 0x00;

  // Setup the Channel defaults
  config->bitRate = 1000000;
  config->encoding = GSC_ENCODING_NRZ;
  config->protocol = GSC_PROTOCOL_RS422_RS485;
  config->termination = GSC_TERMINATION_ENABLED;
  config->parity = GSC_PARITY_NONE;
  config->crcMode = GSC_CRC_NONE;
  config->crcInitialValue = GSC_CRC_INIT_0;

  // Setup the Transmitter defaults
  config->txStatus = GSC_ENABLED;
  config->txCharacterLength = 8;
  config->txUnderRun = GSC_ABORT;
  config->txPreamble = GSC_DISABLED;
  config->txPreamblePattern = GSC_PREAMBLE_ALL_1;
  config->txSharedZero = GSC_DISABLED;
  config->txClockSource = GSC_CLOCK_INTERNAL;
  config->txIdleCondition = GSC_IDLE_FLAGS;
  config->txSinglePacketEnable = GSC_DISABLED;

  // Setup the Receiver defaults
  config->rxStatus = GSC_ENABLED;
  config->rxCharacterLength = 8;
  config->rxAddrSearchMode = GSC_DISABLED;
  config->rxAddress = 0xff;
  config->rxClockSource = GSC_CLOCK_INTERNAL;
  config->rxReceiveStatusBlocks = GSC_DISABLED;

  // Setup the Pin Configuration Variables
  config->interfaceMode = GSC_PIN_DTE;
  config->txDataPinMode = GSC_PIN_AUTO;
  config->rxDataPinMode = GSC_PIN_AUTO;
  config->txClockPinMode = GSC_PIN_AUTO;
  config->rxClockPinMode = GSC_PIN_AUTO;
  config->ctsPinMode = GSC_PIN_GPIO;
  config->rtsPinMode = GSC_PIN_GPIO;
  config->txUSCClockPinMode = GSC_PIN_AUTO;
  config->loopbackMode = GSC_LOOP_NONE;

  // Setup the Misc defaults
  config->packetFraming = GSC_ENABLED;

  // Setup FIFO feature configuration fields
  config->fifoTimetagByte = GSC_DISABLED;
  config->fifoParityErrorByte = GSC_DISABLED;

  return status;
}

////----------------------------------------------------- gscSio4ChannelHdlcInit
////
////  This routine initializes a single channel to HDLC
//// mode using the default parameters.
////
//int gscSio4HdlcInit(int boardNumber, int channel)
//{
//      int status = 0x00;
//      GSC_HDLC_CONFIG config;
//
//      // Load the default values
//      status = GscSio4HdlcGetDefaults( &config);
//
//      // Program the channel
//      status = GscSio4HdlcSetConfig( boardNumber, channel, config);
//      if (status) return status;
//
//      return status;
//}


//------------------------------------------------ GscSio4ChannelHdlcGetConfig
//
//  This routine returns the current configuration
// structure for a single channel that is set to HDLC
// mode.
//
int GscSio4HdlcGetConfig(int boardNumber, int channel, PGSC_HDLC_CONFIG config)
{
  int status = 0x00;
  int value;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
    return GscApiInvalidBoardType;

  // If the channel is not configured, get the defaults
  if (boardInformation[ boardNumber-1].channelMode[ channel-1] == GSC_MODE_UNINITIALIZED)
  {
    // Load the default values
    status = GscSio4HdlcGetDefaults( config);
    return status;
  }

  // Get the data character lengths
  status = zilogGetDataLength( boardNumber, channel, &config->txCharacterLength, &config->rxCharacterLength);
  if (status) return status;

  // Determine the encoding and bit rate
  config->encoding = boardInformation[ boardNumber-1 ].channelEncoding[ channel-1 ];
  config->bitRate = boardInformation[ boardNumber-1 ].channelBitRate[ channel-1 ];

  // Determine the protocol and termination settings
  status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol,
                                      &config->termination);
  if (status) return( status );

  // Determine the parity mode
  status = gscSio4ChannelGetParity( boardNumber, channel, &config->parity);
  if (status) return status;

  // Determine the CRC mode and initial value
  status = zilogGetCrcMode( boardNumber, channel, &config->crcMode, &config->crcInitialValue);
  if (status) return status;

  // Read the current Mode Register value
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
  if (status) return status;

  // Determine the Tx Underrun Condition
  switch (value&0xc000)
  {
  case 0x0000: config->txUnderRun = GSC_ABORT;            break;
  case 0x4000: config->txUnderRun = GSC_EXTENDED_ABORT; break;
  case 0x8000: config->txUnderRun = GSC_FLAG;                     break;
  case 0xc000: config->txUnderRun = GSC_CRC_FLAG;                 break;
  }

  // Determine the Tx Preamble
  status = zilogGetTxPreamble( boardNumber, channel, &config->txPreamble,
                               &config->txPreamblePattern);
  if (status) return status;

  // Determine the Tx Shared Zero Flags
  if (value & 0x1000) config->txSharedZero = GSC_ENABLED;
  else config->txSharedZero = GSC_DISABLED;

  // Determine the Rx Address Search Mode
  status = zilogGetRxAddrSearch( boardNumber, channel, &config->rxAddrSearchMode,
                                 &config->rxAddress);
  if (status) return status;

  // Determine the idle line condition
  status = zilogGetIdleLineCondition( boardNumber, channel, &config->txIdleCondition);
  if (status) return status;

  // Determine the various pin modes
  config->interfaceMode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
                                     &config->txDataPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
                                     &config->rxDataPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
                                     &config->txClockPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
                                     &config->rxClockPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
                                     &config->ctsPinMode);
  if (status) return( status );
  status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
                                     &config->rtsPinMode);
  if (status) return( status );

  // Determine the loopback mode
  config->loopbackMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

  // Determine the clock sources
  status = gscsio4ChannelGetClockSource( boardNumber, channel, &config->txClockSource,
                                         &config->rxClockSource);
  if (status) return( status );

  // Determine the receiver mode
  status = zilogGetReceiverControl( boardNumber, channel, &config->rxStatus);
  if (status) return status;

  // Determine the transmitter mode
  status = zilogGetTransmitterControl( boardNumber, channel, &config->txStatus);
  if (status) return status;

  // Get the packet framing mode
  config->packetFraming = boardInformation[ boardNumber-1].channelPacketFraming[ channel-1];

  // Get the RSB enabled/disabled status
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
  if (status) return status;

  // if either 16 bit (D7/D6 = 01) or 32 bit (D7/D6 = 10), RSB enabled
  if (value & 0xC0)
    config->rxReceiveStatusBlocks = GSC_ENABLED;
  else
        config->rxReceiveStatusBlocks = GSC_DISABLED;

  // Determine the status of the FIFO features
  config->fifoTimetagByte = GSC_DISABLED;
  config->fifoParityErrorByte = GSC_DISABLED;

  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);
  if (status) return status;

  if (value & 0x04)
    config->fifoTimetagByte = GSC_ENABLED;
  if (value & 0x08)
    config->fifoParityErrorByte = GSC_ENABLED;

  return status;
}


//------------------------------------------------ GscSio4ChannelHdlcSetConfig
//
//  This routine sets the current configuration
// structure for a single channel that is set to HDLC
// mode.
//
int GscSio4HdlcSetConfig(int boardNumber, int channel, GSC_HDLC_CONFIG config)
{
  int status = 0x00;
  int value, val2;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make sure the board type is as expected
  if (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_NORMAL)
    return GscApiInvalidBoardType;

  // Save some global info
  boardInformation[ boardNumber-1].channelMode[ channel-1] = GSC_MODE_HDLC;
  boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)config.bitRate/1000/8;

  // If the board is a BXR, ensure the channel is set to standard mode
  if (!strcmp(boardInformation[boardNumber-1].subId, "-SIO4BXR"))
  {
    int chMode = 0;

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    if (status) return status;

    chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
    chMode |= (0x01 << ((channel - 1) * 8)); // set channel type to standard

    status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
    if (status) return status;
  }

  // Set mode to HDLC
  status = zilogWrite( boardNumber, channel, USC_CMR, 0x0606);
  if (status) return status;

  // Set the data character lengths
  status = zilogSetDataLength( boardNumber, channel, config.txCharacterLength, config.rxCharacterLength);
  if (status) return status;

  // Setup the encoding and bit rate
  zilogSetEncoding( boardNumber, channel, config.encoding, GSC_MODE_HDLC, config.bitRate);
  if (status) return status;

  // Setup the protocol and termination
  status = quickSetProtocol( boardNumber, channel, config.protocol, config.termination);
  if (status) return status;

  // Setup the parity mode
  status = quickSetParity( boardNumber, channel, config.parity);
  if (status) return status;

  // Setup the CRC mode
  status = zilogSetCrcMode( boardNumber, channel, config.crcMode, config.crcInitialValue);
  if (status) return status;

  // Set the Tx Underrun Mode
  status = zilogSetTxUnderRun( boardNumber, channel, config.txUnderRun);
  if (status) return status;

  // Set the Tx Preamble Mode
  status = zilogSetTxPreamble( boardNumber, channel, config.txPreamble, config.txPreamblePattern);
  if (status) return status;

  // Set Shared Zero mode
  status = zilogSetTxSharedZero( boardNumber, channel, config.txSharedZero);
  if (status) return status;

  // Set the Rx Address Search Mode
  status = zilogSetRxAddrSearch( boardNumber, channel, config.rxAddrSearchMode, config.rxAddress);
  if (status) return status;

  // Setup the idle line condition
  status = zilogSetIdleLineCondition( boardNumber, channel, config.txIdleCondition);
  if (status) return status;

  // Setup the Pin Modes
  status = quickSetDteDce(boardNumber, channel, config.interfaceMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, config.txDataPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, config.rxDataPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, config.txClockPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, config.rxClockPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, config.ctsPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, config.rtsPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_USCCLOCK, config.txUSCClockPinMode);
  if (status) return status;

  // Setup the loopback mode
  status = quickSetLoopBack( boardNumber, channel, config.loopbackMode);
  if (status) return status;

  // Setup the clock source
  status = quickNormalSetClockSource( boardNumber, channel, config.txClockSource, config.rxClockSource);
  if (status) return status;

  // Enable/Disable the receiver
  status = zilogReceiverControl( boardNumber, channel, config.rxStatus);
  if (status) return status;

  // Enable/Disable the Transmitter (no auto enables)
  status = zilogTransmitterControl( boardNumber, channel, config.txStatus);
  if (status) return status;

  // Enable/disable use of Almost Empty value to trigger transfer from FIFO to Zilog
  status = fpgaRead(boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);
  if (config.txSinglePacketEnable == GSC_ENABLED)
  {
    value |= 0x4000000;
    value &= ~0x2000000;
  }
  else
    value &= ~0x4000000;
  status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), value);

  // Enable/Disable the FIFO features
  status = gscSio4ChannelSetFifoTimetagByte( boardNumber, channel, config.fifoTimetagByte );
  if (status) return status;
  status = gscSio4ChannelSetFifoParityErrorByte( boardNumber, channel, config.fifoParityErrorByte );
  if (status) return status;

  //
  // Program the Packet Framing mode
  //
  // Get the current Channel Control Register
  status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CCR, &value);
  if (status) return status;

  // Clear the TCB type bits
  value &= ~0xc000;

  // Update the board info structures copy of the framing mode
  boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = config.packetFraming;

  // If packet framing is enabled
  if (config.packetFraming == GSC_ENABLED)
  {
    /* turn on rx packet info fifo feature if available */
    if (boardInformation[boardNumber - 1].features & GSC_FEATURE_HAS_RSB_FIFO)
    {
      status = fpgaRead(boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &val2);
      if (status) return status;
      // enable new feature - D27
      status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), val2 | 0x08000000);
      if (status) return status;
          // RSB FIFO needs RSB enabled
          config.rxReceiveStatusBlocks = GSC_ENABLED;
    }

    // Set the channel to use 32 bit TCBs
    value |=  0x8000;

    // Activate RSBs if enabled
    if (config.rxReceiveStatusBlocks == GSC_ENABLED)
      if (boardInformation[boardNumber - 1].features & GSC_FEATURE_HAS_RSB_FIFO)
        value |= 0x80; /* 32-bit */
      else
        value |= 0x40; /* 16-bit */

    // Write the new Channel Control Register value
    status = zilogWrite( boardNumber, channel, USC_CCR, value);
    if (status) return status;

    // Trigger the DMA
    status = zilogWrite( boardNumber, channel, USC_CCAR, 0x7000);
    if (status) return status;

    // Load the Receive Count Limit Register with -1
    status = zilogWrite( boardNumber, channel, USC_RCLR, 0xffff);
    if (status) return status;

    // Enable Zilog Rx Status IRQs and set Master Interrupt Enable
    status = zilogWrite(boardNumber, channel, USC_ICR, 0x80E0);
    if (status) return status;

    // Arm the Zilog Rx EOM Interrupt
    status = zilogWrite(boardNumber, channel, USC_RICR, 0x10);
    if (status) return status;

    // Enable the USC interrupt for this channel
    status = fpgaRead( boardNumber, INTERRUPT_CONTROL_REG, &value);
    if (status) return status;
    value |= 8<<((channel-1)*4);
    status = fpgaWrite( boardNumber, INTERRUPT_CONTROL_REG, value);

    BeginCriticalSection( qmutex);
    rxCompleteChannels[boardNumber - 1][channel - 1] = TRUE;
    EndCriticalSection( qmutex);
  }

  // No packet framing, just write the new Control Register value out
  else
  {
    rxCompleteChannels[boardNumber - 1][channel - 1] = FALSE;

    // Write the new Channel Control Register value
    status = zilogWrite( boardNumber, channel, USC_CCR, value);
    if (status) return status;
  }

  return status;
}


//----------------------------------------------------- GscSio4HdlcGetOption
//
//  This routine gets the value of the identified option.
//
int gscSio4HdlcGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status = 0x00;
  int value = 0;

  switch ( optionName )
  {
  case GSC_SIO_DATASIZE:
    {
      int *dataSize = &optionValue[0];
      status = zilogGetDataLength( boardNumber, channel, dataSize, dataSize);
    }
    break;

  case GSC_SIO_CRCMODE:
    {
      int *crcMode      = &optionValue[0];
      int *initialValue = &optionValue[1];

      // Determine the CRC mode and initial value
      status = zilogGetCrcMode( boardNumber, channel, crcMode, initialValue);
      if (status) return status;
    }

    break;

  case GSC_SIO_TXUNDERRUN:
    // Read the current Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Tx Underrun Condition
    switch (value&0xc000)
    {
    case 0x0000: optionValue[0] = GSC_ABORT;              break;
    case 0x4000: optionValue[0] = GSC_EXTENDED_ABORT; break;
    case 0x8000: optionValue[0] = GSC_FLAG;                       break;
    case 0xc000: optionValue[0] = GSC_CRC_FLAG;           break;
    }
    break;

  case GSC_SIO_TXPREAMBLE:
    // Get the Tx Preamble Mode and pattern
    status = zilogGetTxPreamble( boardNumber, channel, &optionValue[0],
                                 &optionValue[1] );
    break;

  case GSC_SIO_TXSHAREDZERO:
    // Read the current Mode Register value
    status = GscSio4ChannelRegisterRead( boardNumber, channel, USC_CMR, &value);
    if (status) return status;

    // Determine the Tx Shared Zero Flags
    if (value & 0x1000) optionValue[0] = GSC_ENABLED;
    else optionValue[0] = GSC_DISABLED;

    break;

  case GSC_SIO_RXADDRSEARCH:
    // Determine the Rx Address Search Mode
    status = zilogGetRxAddrSearch( boardNumber, channel, &optionValue[0],
                                   &optionValue[1] );
    break;

  case GSC_SIO_TXIDLECONDITION:
    // Get Shared Zero mode
    status = zilogGetIdleLineCondition( boardNumber, channel, &optionValue[0]);
    break;

  case GSC_SIO_TXDATAPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RXDATAPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
                                       &optionValue[0]);
    break;

  case GSC_SIO_TXCLOCKPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RXCLOCKPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
                                       &optionValue[0]);
    break;

  case GSC_SIO_CTSPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_CTS,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RTSPINMODE:
    // Setup the pin mode
    status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RTS,
                                       &optionValue[0]);
    break;

  case GSC_SIO_RECEIVER:
    {
      int *rxMode = &optionValue[0];

      status = zilogGetReceiverControl( boardNumber, channel, rxMode);
    }
    break;

  case GSC_SIO_TRANSMITTER:
    {
      int *txMode  = &optionValue[0];

      status = zilogGetTransmitterControl( boardNumber, channel, txMode);
    }
    break;

  case GSC_SIO_FIFO_TIMETAG_BYTE:
    {
      int value = 0;

      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);

      optionValue[0] = GSC_DISABLED;
      if (value & 0x04)
        optionValue[0] = GSC_ENABLED;
    }
    break;

  case GSC_SIO_FIFO_PARITYERROR_BYTE:
    {
      int value = 0;

      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG + ((channel - 1) << 4), &value);

      optionValue[0] = GSC_DISABLED;
      if (value & 0x08)
        optionValue[0] = GSC_ENABLED;
    }
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

//----------------------------------------------------- GscSio4HdlcSetOption
//
//  This routine sets the identified option to the supplied value.
//
int gscSio4HdlcSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
  int status = 0;

  switch ( optionName )
  {
  case GSC_SIO_DATASIZE:
    {
      int dataSize = optionValue;
      status = zilogSetDataLength( boardNumber, channel, dataSize, dataSize);
      if (status) return status;
    }
    break;

  case GSC_SIO_ENCODING:
    {
      int encoding = optionValue;

      // Convert board and channel into indexes
      int bn = boardNumber - 1;
      int ch = channel - 1;

      return zilogSetEncoding( boardNumber,
                               channel,
                               encoding,
                               boardInformation[ bn].channelMode[ ch],
                               boardInformation[ bn].channelBitRate[ ch]);
    }

    break;

  case GSC_SIO_CRCMODE:
    {
      int crcMode      = (optionValue & 0xffff0000) >> 16;
      int initialValue = optionValue & 0x0000ffff;

      // Make the changes
      status = zilogSetCrcMode( boardNumber, channel, crcMode, initialValue);
    }

    break;

  case GSC_SIO_TXUNDERRUN:
    // Set the Tx Underrun Mode
    status = zilogSetTxUnderRun( boardNumber, channel, optionValue);
    break;

  case GSC_SIO_TXPREAMBLE:
    // Set the Tx Preamble Mode
    status = zilogSetTxPreamble( boardNumber, channel, (optionValue & 0xffff0000) >> 16,
                                 optionValue & 0x0000ffff );
    break;

  case GSC_SIO_TXSHAREDZERO:
    // Setup the idle line condition
    status = zilogSetTxSharedZero( boardNumber, channel, optionValue);
    break;

  case GSC_SIO_RXADDRSEARCH:
    // Set the Rx Address Search Mode
    status = zilogSetRxAddrSearch( boardNumber, channel, (optionValue & 0xffff0000) >> 16,
                                   optionValue & 0x0000ffff );
    break;

  case GSC_SIO_TXIDLECONDITION:
    // Set Shared Zero mode
    status = zilogSetIdleLineCondition( boardNumber, channel, optionValue);
    break;

  case GSC_SIO_TXDATAPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, optionValue);
    break;

  case GSC_SIO_RXDATAPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, optionValue);
    break;

  case GSC_SIO_TXCLOCKPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, optionValue);
    break;

  case GSC_SIO_RXCLOCKPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, optionValue);
    break;

  case GSC_SIO_CTSPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_CTS, optionValue);
    break;

  case GSC_SIO_RTSPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RTS, optionValue);
    break;

  case GSC_SIO_RECEIVER:
    {
      int rxMode = optionValue;

      status = zilogReceiverControl( boardNumber, channel, rxMode);
    }
    break;

  case GSC_SIO_TRANSMITTER:
    {
      int txMode  = optionValue;

      status = zilogTransmitterControl( boardNumber, channel, txMode);
    }

    break;

  case GSC_SIO_FIFO_TIMETAG_BYTE:
    {
      status = gscSio4ChannelSetFifoTimetagByte( boardNumber, channel, optionValue );
    }
    break;

  case GSC_SIO_FIFO_PARITYERROR_BYTE:
    {
      status = gscSio4ChannelSetFifoParityErrorByte( boardNumber, channel, optionValue );
    }
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

