//////////////////////////////////////////////////////////////////////////////
//
//  GscApiInterrupt.c
//
//  This module contains all of the interrupt handling routines that are
// used by the GSC Sio4 API.
//
#include <stdio.h>
#ifdef PLX_LINUX
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#endif
#include "GscApiLocal.h"
#include "GscApiPlxLegacy.h"
#include "GscApiPlxdload.h"


// Array to hold all of the callback functions
GSC_CB_FUNCTION	*interruptRoutines[ MAXIMUM_NUMBER_OF_BOARDS][32];
GSC_CB_FUNCTION	*transferCompleteRoutines[ MAXIMUM_NUMBER_OF_BOARDS][8];
GSC_CB_FUNCTION	*rxTransferCompleteRoutines[ MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
GSC_CB_FUNCTION	*USCInterruptRoutines[ MAXIMUM_NUMBER_OF_BOARDS];
GSC_CB_FUNCTION	*AnalogInterruptRoutines[ MAXIMUM_NUMBER_OF_BOARDS][4];

// Array to hold the pending completion routines
int completionPending[ MAXIMUM_NUMBER_OF_BOARDS];
#ifdef WIN32
CRITICAL_SECTION completionPendingLock[ MAXIMUM_NUMBER_OF_BOARDS];
#endif
int rxCompletionPending[ MAXIMUM_NUMBER_OF_BOARDS][MAXIMUM_RX_PACKETS];
int rxCompletionTo[MAXIMUM_NUMBER_OF_BOARDS];
int rxCompletionFrom[MAXIMUM_NUMBER_OF_BOARDS];

INTERRUPT_INFO interruptInfo[32] =
{ // ISR Mask
  {GSC_INTR_SYNC_DETECT,          0x00000001},
  {GSC_INTR_TX_FIFO_ALMOST_EMPTY, 0x00000002},
  {GSC_INTR_RX_FIFO_ALMOST_FULL,  0x00000004},
  {GSC_INTR_USC,		  0x00000008},
  {GSC_INTR_SYNC_DETECT,	  0x00000010},
  {GSC_INTR_TX_FIFO_ALMOST_EMPTY, 0x00000020},
  {GSC_INTR_RX_FIFO_ALMOST_FULL,  0x00000040},
  {GSC_INTR_USC,		  0x00000080},
  {GSC_INTR_SYNC_DETECT,	  0x00000100},
  {GSC_INTR_TX_FIFO_ALMOST_EMPTY, 0x00000200},
  {GSC_INTR_RX_FIFO_ALMOST_FULL,  0x00000400},
  {GSC_INTR_USC,		  0x00000800},
  {GSC_INTR_SYNC_DETECT,	  0x00001000},
  {GSC_INTR_TX_FIFO_ALMOST_EMPTY, 0x00002000},
  {GSC_INTR_RX_FIFO_ALMOST_FULL,  0x00004000},
  {GSC_INTR_USC,		  0x00008000},
  {GSC_INTR_TX_FIFO_EMPTY,	  0x00010000},
  {GSC_INTR_TX_FIFO_FULL,	  0x00020000},
  {GSC_INTR_RX_FIFO_EMPTY,	  0x00040000},
  {GSC_INTR_RX_FIFO_FULL,	  0x00080000},
  {GSC_INTR_TX_FIFO_EMPTY,	  0x00100000},
  {GSC_INTR_TX_FIFO_FULL,	  0x00200000},
  {GSC_INTR_RX_FIFO_EMPTY,	  0x00400000},
  {GSC_INTR_RX_FIFO_FULL,	  0x00800000},
  {GSC_INTR_TX_FIFO_EMPTY,	  0x01000000},
  {GSC_INTR_TX_FIFO_FULL,	  0x02000000},
  {GSC_INTR_RX_FIFO_EMPTY,	  0x04000000},
  {GSC_INTR_RX_FIFO_FULL,	  0x08000000},
  {GSC_INTR_TX_FIFO_EMPTY,	  0x10000000},
  {GSC_INTR_TX_FIFO_FULL,	  0x20000000},
  {GSC_INTR_RX_FIFO_EMPTY,	  0x40000000},
  {GSC_INTR_RX_FIFO_FULL,	  0x80000000},
};

INTERRUPT_INFO completeInfo[8] =
{	// ISR Mask
  {GSC_INTR_TX_TRANSFER_COMPLETE, 0x00000001},	// Channel 1
  {GSC_INTR_RX_TRANSFER_COMPLETE, 0x00000002},
  {GSC_INTR_TX_TRANSFER_COMPLETE, 0x00000004},	// Channel 2
  {GSC_INTR_RX_TRANSFER_COMPLETE, 0x00000008},
  {GSC_INTR_TX_TRANSFER_COMPLETE, 0x00000010},	// Channel 3
  {GSC_INTR_RX_TRANSFER_COMPLETE, 0x00000020},
  {GSC_INTR_TX_TRANSFER_COMPLETE, 0x00000040},	// Channel 4
  {GSC_INTR_RX_TRANSFER_COMPLETE, 0x00000080}
};

INTERRUPT_INFO analogIntrInfo[4] =
{	// ISR Mask
  {GSC_INTR_ANA_AUTOCALCOMPLETED,     0x00000001},	// auto-calibration completed.
  {GSC_INTR_ANA_THRESHOLDLOWTOHIGH,   0x00000010},	// buffer threshold low ==> high transition
  {GSC_INTR_ANA_THRESHOLDHIGHTOLOW,   0x00000020},  // buffer threshold high ==> low transition
  {GSC_INTR_ANA_OVERUNDERFLOW,        0x00000040}	  // input buffer overflow or underflow
};



//------------------------------ UscInterruptServiceRoutine
//
void CALLBACK uscInterruptServiceRoutine( int boardNumber, int channel, int interrupt)
{
  int packetReceived = 0x00;
  int mask, count;
  int	ccsr;
  int	rpif;
  int bn = boardNumber - 1;

  // check to see if this is a channel being used by this process (all active processes using
  // the GscApi shared library will receive notification of each occurring usc interrupt, so this
  // routine needs to check the channel number to see if the current process is using the channel)
  if (rxCompleteChannels[bn][channel - 1] == FALSE)
    return;

  // Read the DCCR to see if an Rx Status interrupt is pending
  zilogRead(boardNumber, channel, USC_DCCR, &mask);

  // Only process Rx Status interrupts
  if (mask & 0x0020)
  {
    // Clear the Rx Status IP bit and set the IUS bit
    zilogWrite(boardNumber, channel, USC_DCCR, 0xe0a0);

    // Read the Command/Status register to see which interrupt occurred
    zilogRead(boardNumber, channel, USC_RCSR, &mask);

    // Only looking at the EOF interrupt
    if (mask & 0x10)
    {
      // Clear out the bits that we just processed
      zilogWrite(boardNumber, channel, USC_RCSR, mask & 0x0010);

      BeginCriticalSection( qmutex);

      // Get all of the entries from the Receiver Count Register FIFO
      while(1)
      {
        if (boardInformation[bn].features & GSC_FEATURE_HAS_RSB_FIFO)
        {
		  // D31 = Valid &  D30 = FIFO full (overflow)
          // If no entries are available in the FIFO, break out of the loop
          fpgaRead(boardNumber, RX_PACKETINFO_FIFO_BASE_REG + (4 * (channel-1)), &rpif);
	  if (!(rpif & 0x80000000)) break;

          // Store the packet length and status
          rxPacketLengths[bn][channel-1][rxPacketTo[bn][channel-1]] = rpif & 0xffff;
          rxPacketStatus[bn][channel-1][rxPacketTo[bn][channel-1]] = (rpif & 0x3fff0000) >> 16;
        }
        else
        {
          // Read the status register
          zilogRead( boardNumber, channel, USC_CCSR, &ccsr);

          // If no entries are available in the FIFO, break out of the loop
          if ((ccsr & 0x4000) == 0x0000) break;

          // Get the Receiver Count register and status
          zilogRead(boardNumber, channel, USC_RCCR, &count);
          rxPacketLengths[ bn][ channel-1][ rxPacketTo[ bn][ channel-1]] = ~count & 0xffff;
          rxPacketStatus[ bn][ channel-1][ rxPacketTo[ bn][ channel-1]] = mask & 0x012e;
        }

        if (++rxPacketTo[ bn][ channel-1] >= MAXIMUM_RX_PACKETS) rxPacketTo[ bn][ channel-1] = 0;

        // Record the callback if one was specified
        insertRxCompletionPending( boardNumber, channel);
        packetReceived++;
      }

      // If anything was received, signal its thread
      if (packetReceived != 0x00) setRxCompletionPending( boardNumber);

      EndCriticalSection( qmutex);
    }

    // Clear all of the Rx Status Interrupt IA bits and then set the RxBound bit back
    zilogRead(boardNumber, channel, USC_RICR, &count);
    count &= 0xf6;
    zilogWrite(boardNumber, channel, USC_RICR, 0x00);
    zilogWrite(boardNumber, channel, USC_RICR, count);

    // Reset the Rx Status IUS bit
    zilogWrite(boardNumber, channel, USC_DCCR, 0xa000);
  }
}


//------------------------------------------------- completionInterruptHandler
//
//  This routine runs as a separate thread and processes any
// completion Interrupts associated with a single board.
//
#ifdef PLX_LINUX
void *completionInterruptHandler(void *pParam)
{
  int bn = (*(int *)pParam)-1;
  int currentInterrupts, index;
  int messageId = GSC_THREAD_COMPLETION_TYPE;
  THREAD_MESSAGE threadMessage;

  // Loop forever
  while (1)
  {
    // Wait for something to happen
    if (msgrcv(boardInformation[bn].intrQueueId, (void*)&threadMessage, sizeof(int), messageId,0) == -1)
    {
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR));
    }

    // Check for an abort message
    if (threadMessage.command == GSC_THREAD_ABORT_CMD)
    {
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_INTR));
    }

    // Read the interrupt status word and clear those bits
    currentInterrupts = getCompletionPending( bn+1);

    // Loop through all of the bits and call the appropriate routines
    for (index=0; index<8; index++)
    {
      // Break out of loop if no bits set
      if (currentInterrupts == 0x00) break;

      // See if the LSB is set
      if (currentInterrupts & 0x01)
      {
        // If the routine is defined, call it
        if (transferCompleteRoutines[ bn][ index] != NULL)
        {
          transferCompleteRoutines[ bn][ index]( bn+1, (index>>1)+1, completeInfo[ index].intrType);
        }
      }

      // Shift the interrupt mask
      currentInterrupts >>= 1;
    }
  }
}
#endif

//---------------------------------------------------- rxCompletionInterruptHandler
//
//  This routine runs as a separate thread and processes any
// Rx Completion Interrupts associated with a single board.
//
#ifdef PLX_LINUX

void *rxCompletionInterruptHandler(void *pParam)
{
  int  bn = (*(int *)pParam)-1;
  int  channel;
  int messageId = GSC_THREAD_RX_COMPLETION_TYPE;  // Message type for Rx Completion interrupts
  THREAD_MESSAGE threadMessage;

  // Loop forever
  while (1)
  {
    // Wait forever something to happen
    if (msgrcv(boardInformation[bn].intrQueueId, (void*)&threadMessage, sizeof(int), messageId, 0) == -1)
    {
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR_RX));
    }

    // Check for an abort message
    if (threadMessage.command == GSC_THREAD_ABORT_CMD)
    {
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_INTR_RX));
    }

    // Otherwise, it must be an Rx Transfer Complete Interrupt
    //  Process all of the pending interrupts while we are here

    while(1)
    {
      // Get the channel of the Pending Completion Interrupt
      BeginCriticalSection( qmutex);

      channel = getRxCompletionPending( bn+1);

      EndCriticalSection( qmutex);

      // Exit the loop if no interrupt is pending
      if (channel == 0) break;

      // If the routine is defined, call it
      if (rxTransferCompleteRoutines[ bn][ channel-1] != NULL)
      {
        rxTransferCompleteRoutines[ bn][ channel-1]( bn+1, channel, GSC_INTR_RX_TRANSFER_COMPLETE);
      }
    }
  }
}

#else

DWORD WINAPI rxCompletionInterruptHandler(LPVOID pParam)
{
  int			bn;
  HANDLE		intrHandles[2];
  DWORD		eventStatus;
  int			channel;

  // Get the proper board number
  bn = (*(int *)pParam) - 1;

  // Setup the abort event
  intrHandles[0] = CreateEvent( NULL, TRUE, FALSE, threadAbort[ bn]);
  ResetEvent( intrHandles[0] );

  // Setup the transfer complete event
  intrHandles[1] = CreateEvent( NULL, FALSE, FALSE, rxTransferCompleteEvent[ bn]);
  ResetEvent( intrHandles[1] );

  // Loop forever
  while (1)
  {
    // Wait forever for an event
    eventStatus = WaitForMultipleObjects(2, intrHandles, FALSE, INFINITE);

    // If abort event set, exit
    if (eventStatus == WAIT_OBJECT_0) return THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_INTR_RX);

    // Otherwise, it must be an Rx Transfer Complete Interrupt
    //  Process all of the pending interrupts while we are here
    while(1)
    {
      // Get the channel of the Pending Completion Interrupt
      channel = getRxCompletionPending( bn+1);

      // Exit the loop if no interrupt is pending
      if (channel == 0) break;

      // If the routine is defined, call it
      if (rxTransferCompleteRoutines[ bn][ channel-1] != NULL)
      {
        rxTransferCompleteRoutines[ bn][ channel-1]( bn+1, channel, GSC_INTR_RX_TRANSFER_COMPLETE);
      }
    }
  }
}

#endif

//----------------------------------------------------- fpgaInterruptHandler
//
//  This routine runs as a separate thread and processes
// any interrupts generated by the FPGA through the PLX
// chip for a single board.
//
#ifdef PLX_LINUX

void *fpgaInterruptHandler( void *pParam)

#else

DWORD WINAPI gscSio4InterruptHandler(LPVOID pParam)

#endif
{
  int bn = (*(int*)pParam) - 1;
  int index = 0, status = 0, ch = 0;
  PLX_INTR intrMask;
  PLX_INTR3 intrMask3;
  PLX_INTERRUPT6   intSources6;
  PLX_INTERRUPT    intSources;
#ifdef WIN32
  PLX_NOTIFY_OBJECT notifyHnd;
  HANDLE intrHandles[3];
  DWORD	eventStatus = 0;
#endif
  int currentInterrupts, interruptMask;
#ifdef PLX_LINUX
  int messageId = GSC_THREAD_FPGA_INTR_TYPE;
  THREAD_MESSAGE threadMessage;
  PLX_NOTIFY_OBJECT fpgaNotifyHnd;
  HANDLE fpgaIntrHandle;
  char temp[100];
#endif

#ifdef WIN32
  // Setup the abort event
  intrHandles[0] = CreateEvent( NULL, TRUE, FALSE, threadAbort[ bn]);
  ResetEvent( intrHandles[0] );

  // Setup the transfer complete event
  intrHandles[1] = CreateEvent( NULL, FALSE, FALSE, transferCompleteEvent[ bn]);
  ResetEvent( intrHandles[1] );
#endif

  // Attach to the drivers interrupt handler
  memset(&intrMask, 0, sizeof(PLX_INTR));
  intrMask.IopToPciInt = 1;
  memset(&intrMask3, 0, sizeof(PLX_INTR3));
  intrMask3.IopToPciInt = 1;
  memset(&intSources6, 0, sizeof(PLX_INTERRUPT6));
  intSources6.LocalToPci_1 = 1;
  //intSources6.LocalToPci_2 = 1;
  intSources6.PciMain = 1;
  memset(&intSources, 0, sizeof(PLX_INTERRUPT));
  intSources.LocalToPci = 1;
  //intSources.LocalToPci = 2;
  intSources.PciMain = 1;

  BeginCriticalSection( PlxApiCalls);
#ifdef PLX_LINUX

  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_NotificationRegisterFor_Rtdl(&sio4BoardHandles[ bn], &intSources, &fpgaNotifyHnd);
  else if (drvMajorVersion >= 5)
    status = PlxPci_NotificationRegisterFor6_Rtdl(&sio4BoardHandles6[ bn], &intSources6, &fpgaNotifyHnd);
  else if ((drvMajorVersion == 4) && (drvMinorVersion == 4))
    status = PlxNotificationRegisterFor_Rtdl(boardInformation[ bn].handle, &intrMask, &fpgaNotifyHnd);
  else
    status = PlxIntrAttach_Rtdl(boardInformation[ bn].handle, intrMask3, &fpgaIntrHandle);

#else

  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    status = PlxPci_NotificationRegisterFor_Rtdl(&sio4BoardHandles[ bn], &intSources, &notifyHnd);
  else if (drvMajorVersion >= 5)
    status = PlxPci_NotificationRegisterFor6_Rtdl(&sio4BoardHandles6[ bn], &intSources6, &notifyHnd);
  else if ((drvMajorVersion == 4) && (drvMinorVersion == 4))
    status = PlxNotificationRegisterFor_Rtdl(boardInformation[bn].handle, &intrMask, &notifyHnd);
  else
    status = PlxIntrAttach_Rtdl(boardInformation[bn].handle, intrMask3, &intrHandles[2]);

#endif
  EndCriticalSection( PlxApiCalls);
  if (status != ApiSuccess)
  {
    // Release the critical section object
#ifdef PLX_LINUX
    pthread_exit((void*)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR));
#else
    // DeleteCriticalSection(&completionPendingLock[ bn]);
    return THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR);
#endif
  }

  // Enable the FPGA interrupt (IopToPci) as well as the PCI main interrupt
  memset(&intrMask, 0, sizeof(PLX_INTR));
  intrMask.IopToPciInt = 1;
  intrMask.PciMainInt = 1;
  memset(&intrMask3, 0, sizeof(PLX_INTR3));
  intrMask3.IopToPciInt = 1;
  intrMask3.PciMainInt = 1;
  memset(&intSources6, 0, sizeof(PLX_INTERRUPT6));
  intSources6.LocalToPci_1 = 1;
  //intSources6.LocalToPci_2 = 1;
  intSources6.PciMain = 1;
  memset(&intSources, 0, sizeof(PLX_INTERRUPT));
  intSources.LocalToPci = 1;
  //intSources.LocalToPci = 2;
  intSources.PciMain = 1;

  BeginCriticalSection( PlxApiCalls);
  if (drvMajorVersion == 6 && drvMinorVersion >= 1)
  {
    status = PlxPci_InterruptDisable_Rtdl( &sio4BoardHandles[ bn], (PLX_INTERRUPT *)&intSources);
    status = PlxPci_InterruptEnable_Rtdl( &sio4BoardHandles[ bn], (PLX_INTERRUPT *)&intSources );
  }
  else if (drvMajorVersion >= 5)
  {
    status = PlxPci_InterruptDisable6_Rtdl( &sio4BoardHandles6[ bn], (PLX_INTERRUPT6 *)&intSources6);
    status = PlxPci_InterruptEnable6_Rtdl( &sio4BoardHandles6[ bn], (PLX_INTERRUPT6 *)&intSources6 );
  }
  else if (drvMajorVersion == 4)
  {
    status = PlxIntrDisable_Rtdl( boardInformation[bn].handle, &intrMask );
    status = PlxIntrEnable_Rtdl( boardInformation[bn].handle, &intrMask );
  }
  else
  {
    status = PlxIntrDisable3_Rtdl( boardInformation[bn].handle, &intrMask3 );
    status = PlxIntrEnable3_Rtdl( boardInformation[bn].handle, &intrMask3 );
  }
  EndCriticalSection( PlxApiCalls);

  // Loop forever
  while (1)
  {
    // Wait for something to happen
#ifdef PLX_LINUX

    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
      status = PlxPci_NotificationWait_Rtdl( &sio4BoardHandles[bn], &fpgaNotifyHnd, 100);
    else if (drvMajorVersion >= 5)
      status = PlxPci_NotificationWait6_Rtdl( &sio4BoardHandles6[bn], &fpgaNotifyHnd, 100);
    else if ((drvMajorVersion == 4) && (drvMinorVersion == 4))
      status = PlxNotificationWait_Rtdl( boardInformation[bn].handle, &fpgaNotifyHnd, 100);
    else
      status = PlxIntrWait_Rtdl( boardInformation[bn].handle, fpgaIntrHandle, 100);

#else

    if (drvMajorVersion == 6 && drvMinorVersion >= 1)
    {
      eventStatus = WaitForMultipleObjects(2, intrHandles, FALSE, 0);
      status = PlxPci_NotificationWait_Rtdl( &sio4BoardHandles[bn], &notifyHnd, 100);
    }
    else if (drvMajorVersion >= 5)
    {
      eventStatus = WaitForMultipleObjects(2, intrHandles, FALSE, 0);
      status = PlxPci_NotificationWait6_Rtdl( &sio4BoardHandles6[bn], &notifyHnd, 100);
    }
    else if ((drvMajorVersion == 4) && (drvMinorVersion == 4))
    {
      eventStatus = WaitForMultipleObjects(2, intrHandles, FALSE, 0);
      status = PlxNotificationWait_Rtdl( boardInformation[bn].handle, &notifyHnd, 100);
    }
    else
      eventStatus = WaitForMultipleObjects(3, intrHandles, FALSE, INFINITE);

#endif

#ifdef PLX_LINUX // refine this merge later, once Linux behavior understood better
    // See if we timed out
    if ((status == ApiWaitTimeout) || (status == ApiWaitCanceled))
    {
      // Check for an abort message
      if (msgrcv(boardInformation[bn].intrQueueId, (void*)&threadMessage, sizeof(int), messageId, IPC_NOWAIT) != -1)
      {
        // Release the critical section object
        // linux DeleteCriticalSection(&completionPendingLock[ bn]);
        pthread_exit((void*)THREAD_STATUS(GSC_THREAD_NORMAL_EXIT, 0, 0, GSC_THREAD_INTR));
      }
      // Re-arm the PLX interrupt
      BeginCriticalSection( PlxApiCalls );
      if ((drvMajorVersion < 4) || ((drvMajorVersion == 4) && (drvMinorVersion < 4)))
        status = PlxIntrAttach_Rtdl(boardInformation[bn].handle, intrMask3, &fpgaIntrHandle);
      if (drvMajorVersion == 6 && drvMinorVersion >= 1)
        status = PlxPci_InterruptEnable_Rtdl( &sio4BoardHandles[bn], &intSources );
      else if (drvMajorVersion >= 5)
        status = PlxPci_InterruptEnable6_Rtdl( &sio4BoardHandles6[bn], &intSources6 );
      else if (drvMajorVersion == 4)
        status = PlxIntrEnable_Rtdl( boardInformation[bn].handle, &intrMask );
      else
        status = PlxIntrEnable3_Rtdl( boardInformation[bn].handle, &intrMask3 );
      EndCriticalSection( PlxApiCalls );
      if (status != ApiSuccess)
      {
        // Release the critical section object
        // linux DeleteCriticalSection(&completionPendingLock[ bn]);
        pthread_exit((void *)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR));
      }
    }

    // See if we got our interrupt
    else if (status == ApiSuccess)
    {
      // Read the interrupt status word
      fpgaRead( bn+1, INTERRUPT_STATUS_REG, &currentInterrupts);

      // Clear all of those bits, except the USC interrupts
      if ( boardInformation[ bn].boardType != GSC_BOARD_TYPE_NORMAL)
      {
        // All interrupts are handled the same
        interruptMask = currentInterrupts;
      }
      else
      {
        // Process all but the USC normally
        interruptMask = currentInterrupts & ~0x00008888;
        currentInterrupts &= 0x00008888;
      }
      fpgaWrite( bn+1, INTERRUPT_STATUS_REG, interruptMask);

      // Handle the Possible USC Interrupts
      for (index=3; index<16; index+=4)
      {
        if ( currentInterrupts & interruptInfo[ index].isrMask)
        {
          // Get channel number
          ch = (index>>2) & 0x03;

          // Call the USC handler if Packet framing is enabled
          if (boardInformation[ bn].channelPacketFraming[ ch] == GSC_ENABLED)
            uscInterruptServiceRoutine( bn+1, ch+1, GSC_INTR_USC);

          // Call any defined user USC ISR
          if (interruptRoutines[ bn][ index] != NULL)
            interruptRoutines[ bn][ index]( bn+1, ch+1, GSC_INTR_USC);

          // Clear the bit
          fpgaWrite( bn+1, INTERRUPT_STATUS_REG, interruptInfo[ index].isrMask);
        }
      }

      // handle interrupts for analog boards
      if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
        status = gscAnaInvokeInterruptCallback( bn + 1 );

      // Loop through all of the bits and call the appropriate routines
      for (index=0; index<32; index++)
      {
        // Break out of loop if no bits set
        if (interruptMask == 0x00) break;

        // See if the LSB is set
        if (interruptMask & 0x01)
        {
          // If the routine is defined, call it
          if (interruptRoutines[ bn][ index] != NULL)
            interruptRoutines[ bn][ index]( bn+1, ((index>>2)&0x03)+1, interruptInfo[ index].intrType);
          //		// Clear the interrupt bit if it is a USC interrupt
          //		if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_NORMAL)
          //		{
          //			if (interruptInfo[ index].intrType == GSC_INTR_USC)
          //			{
          //				fpgaWrite( bn+1, INTERRUPT_STATUS_REG, interruptInfo[ index].isrMask);
          //			}
          //		}
        }

        // Shift the interrupt mask
        interruptMask >>= 1;
      }

      // Re-arm the PLX interrupt
      BeginCriticalSection( PlxApiCalls );
      if ((drvMajorVersion < 4) || ((drvMajorVersion == 4) && (drvMinorVersion < 4)))
        status = PlxIntrAttach_Rtdl(boardInformation[bn].handle, intrMask3, &fpgaIntrHandle);
      if (drvMajorVersion == 6 && drvMinorVersion >= 1)
        status = PlxPci_InterruptEnable_Rtdl( &sio4BoardHandles[bn], &intSources );
      else if (drvMajorVersion >= 5)
        status = PlxPci_InterruptEnable6_Rtdl( &sio4BoardHandles6[bn], &intSources6 );
      else if (drvMajorVersion == 4)
        status = PlxIntrEnable_Rtdl( boardInformation[bn].handle, &intrMask );
      else
        status = PlxIntrEnable3_Rtdl( boardInformation[bn].handle, &intrMask3 );
      EndCriticalSection( PlxApiCalls );
      if (status != ApiSuccess)
      {
        // Release the critical section object
        // linux DeleteCriticalSection(&completionPendingLock[ bn]);
        pthread_exit((void *)THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR));
      }

    }

    // Otherwise, we got an error
    else
    {
      GscGetErrorString(status, temp);
      printf("FPGA: Error, exitting thread(%d - %s)\n", status, temp);
      pthread_exit((void*)THREAD_STATUS(GSC_THREAD_ERROR_EXIT, 0, 0, GSC_THREAD_INTR));
    }

#elif defined(WIN32)

    // If abort event set, exit
    if (eventStatus == WAIT_OBJECT_0)
    {
      // Release the critical section object
      // DeleteCriticalSection(&completionPendingLock[ bn]);

      return THREAD_STATUS(GSC_THREAD_NORMAL_EXIT,0,0,GSC_THREAD_INTR);
    }

    // Check for a Transfer Complete Interrupt
    if (eventStatus == WAIT_OBJECT_0+1)
    {
      // Read the interrupt status word and clear those bits
      currentInterrupts = getCompletionPending( bn+1);

      // Loop through all of the bits and call the appropriate routines
      for (index=0; index<8; index++)
      {
        // Break out of loop if no bits set
        if (currentInterrupts == 0x00) break;

        // See if the LSB is set
        if (currentInterrupts & 0x01)
        {
          // If the routine is defined, call it
          if (transferCompleteRoutines[ bn][ index] != NULL)
          {
            transferCompleteRoutines[ bn][ index]( bn+1, (index>>1)+1, completeInfo[ index].intrType);
          }
        }

        // Shift the interrupt mask
        currentInterrupts >>= 1;
      }
    }

    // Otherwise, it must be a regular interrupt
    if (status != ApiWaitTimeout)
    {
      // Read the interrupt status word
      fpgaRead( bn+1, INTERRUPT_STATUS_REG, &currentInterrupts);

      // Clear all of those bits, except the USC interrupts
      if ( boardInformation[ bn].boardType != GSC_BOARD_TYPE_NORMAL)
      {
        // All interrupts are handled the same
        interruptMask = currentInterrupts;
      }
      else
      {
        // Process all but the USC normally
        interruptMask = currentInterrupts & ~0x00008888;
        currentInterrupts &= 0x00008888;
      }
      fpgaWrite( bn+1, INTERRUPT_STATUS_REG, interruptMask);

      // Handle the Possible USC Interrupts
      for (index=3; index<16; index+=4)
      {
        if ( currentInterrupts & interruptInfo[ index].isrMask)
        {
          // Get channel number
          ch = (index>>2) & 0x03;

          // Call the USC handler if Packet framing is enabled
          if (boardInformation[ bn].channelPacketFraming[ ch] == GSC_ENABLED)
            uscInterruptServiceRoutine( bn+1, ch+1, GSC_INTR_USC);

          // Call any defined user USC ISR
          if (interruptRoutines[ bn][ index] != NULL)
            interruptRoutines[ bn][ index]( bn+1, ch+1, GSC_INTR_USC);

          // Clear the bit
          fpgaWrite( bn+1, INTERRUPT_STATUS_REG, interruptInfo[ index].isrMask);
        }
      }

      // handle interrupts for analog boards
      if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_ANALOG)
        status = gscAnaInvokeInterruptCallback( bn + 1 );

      // Loop through all of the bits and call the appropriate routines
      for (index=0; index<32; index++)
      {
        // Break out of loop if no bits set
        if (interruptMask == 0x00) break;

        // See if the LSB is set
        if (interruptMask & 0x01)
        {
          // If the routine is defined, call it
          if (interruptRoutines[ bn][ index] != NULL)
          {
            interruptRoutines[ bn][ index]( bn+1, ((index>>2)&0x03)+1, interruptInfo[ index].intrType);
          }

          //			    // Clear the interrupt bit if it is a USC interrupt
          //			    if ( boardInformation[ bn].boardType == GSC_BOARD_TYPE_NORMAL)
          //			    {
          //				    if (interruptInfo[ index].intrType == GSC_INTR_USC)
          //				    {
          //					    fpgaWrite( bn+1, INTERRUPT_STATUS_REG, interruptInfo[ index].isrMask);
          //				    }
          //			    }
        }

        // Shift the interrupt mask
        interruptMask >>= 1;
      }

      // Re-arm the PLX interrupt
      BeginCriticalSection( PlxApiCalls );
      if ((drvMajorVersion < 4) || ((drvMajorVersion == 4) && (drvMinorVersion < 4)))
        status = PlxIntrAttach_Rtdl(boardInformation[bn].handle, intrMask3, &intrHandles[2]);
      if (drvMajorVersion == 6 && drvMinorVersion >= 1)
        status = PlxPci_InterruptEnable_Rtdl( &sio4BoardHandles[bn], &intSources );
      else if (drvMajorVersion >= 5)
        status = PlxPci_InterruptEnable6_Rtdl( &sio4BoardHandles6[bn], &intSources6 );
      else if (drvMajorVersion == 4)
        status = PlxIntrEnable_Rtdl( boardInformation[bn].handle, &intrMask );
      else
        status = PlxIntrEnable3_Rtdl( boardInformation[bn].handle, &intrMask3 );
      EndCriticalSection( PlxApiCalls );
      if (status != ApiSuccess)
      {
        // Release the critical section object
        // DeleteCriticalSection(&completionPendingLock[ bn]);

        return THREAD_STATUS(GSC_THREAD_ERROR_EXIT,0,0,GSC_THREAD_INTR);
      }

    }

#endif
    }
}

//------------------------------------------------ gscSio4InitializeInterrupts
//
//  This routine initializes the interrupt structures for one
// board and starts its interrupt processor thread.
//
int gscSio4InitializeInterrupts( int boardNumber)
{
#ifdef WIN32
  HANDLE hThread;
#endif
  int channel, status = 0;
  int bn = boardNumber - 1;
#ifdef PLX_LINUX
  pthread_t txIntrThread;
  pthread_attr_t intrThreadAttribute;
#endif

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;


  // Setup the interrupts for this board
  if (bdRefCount[bn] == 0)
  {
    // Clear the to/from pointers for packet tracking
    for (channel=1; channel<=NUMBER_OF_CHANNELS; channel++)
    {
      rxPacketTo[ bn][ channel-1] = 0;
      rxPacketFrom[ bn][ channel-1] = 0;
      rxCompletionTo[ bn] = 0;
      rxCompletionFrom[ bn] = 0;
    }

    // Clear all pending completion interrupts
    completionPending[bn] = 0;

    for (channel=1; channel<=NUMBER_OF_CHANNELS; channel++)
    {

      // Clear all interrupts except USC
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_SYNC_DETECT,          GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_USC,                  GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_FIFO_EMPTY,        GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_FIFO_FULL,         GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_FIFO_ALMOST_EMPTY, GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_FIFO_EMPTY,        GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_FIFO_FULL,         GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_FIFO_ALMOST_FULL,  GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_TX_TRANSFER_COMPLETE, GSC_RISING_EDGE, NULL);
      GscSio4ChannelRegisterInterrupt( boardNumber, channel, GSC_INTR_RX_TRANSFER_COMPLETE, GSC_RISING_EDGE, NULL);
    }

    // Initialize the critical section object
#ifndef PLX_LINUX
    // InitializeCriticalSection(&completionPendingLock[ bn]);
#endif
  }

#ifdef PLX_LINUX
  // Initialize the thread attribute structure
  status = pthread_attr_init(&intrThreadAttribute);
  if (status) return ApiFailed;

  // Set the thread to detached
  status = pthread_attr_setdetachstate(&intrThreadAttribute, PTHREAD_CREATE_DETACHED);
  if (status) return ApiFailed;

  // Start the Interrupt Handler Thread
  status = pthread_create(intrThread[boardNumber - 1], &intrThreadAttribute, fpgaInterruptHandler, (void *)&boardNumber);
  if (status) return ApiFailed;

  // Start the completion Interrupt handler
  //status = pthread_create(&txIntrThread, &intrThreadAttribute, completionInterruptHandler, (void *)&boardNumber);
  //if (status) return ApiFailed;

  // Start the Rx Completion Interrupt thread
  status = pthread_create(rxIntrThread[boardNumber - 1], &intrThreadAttribute, rxCompletionInterruptHandler, (void *)&boardNumber);
  if (status) return ApiFailed;

  // Destroy the attribute object
  (void)pthread_attr_destroy(&intrThreadAttribute);
#else

  // Start the Interrupt thread and increase its priority
  hThread = CreateThread(0,0,gscSio4InterruptHandler, (LPVOID)&boardNumber, 0, NULL);
  if (hThread != NULL) SetThreadPriority( hThread, THREAD_PRIORITY_HIGHEST);

  // Start the Rx Completion Interrupt thread
  CreateThread(0,0,rxCompletionInterruptHandler, (LPVOID)&boardNumber, 0, NULL);
#endif

  Sleep(50); // don't let address of boardNumber var get reused until after threads are initialized.

  return status;
}


//-------------------------------------------- GscSio4ChannelRegisterInterrupt
//
//  This routine is used to register an interrupt.
//
int GscSio4ChannelRegisterInterrupt( int boardNumber, int channel, int interrupt, int type, GSC_CB_FUNCTION *function)
{
  int i, status = 0x00, offset;
  int enableMask=0x00000000;
  int disableMask=0x00000000;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Check the lower 4 possible interrupts
  offset = 4 * (channel -1);
  for (i=offset; i<offset+4;i++)
  {
    // Is the bit set
    if ( interrupt & interruptInfo[ i].intrType)
    {
      // Save the callback routine
      interruptRoutines[ boardNumber-1][ i] = function;

      // If the callback is null, update the disable mask
      if (function == NULL) disableMask |= interruptInfo[ i].isrMask;
      else enableMask |= interruptInfo[ i].isrMask;
    }
  }

  // Check the upper 4 possible interrupts
  offset += 16;
  for (i=offset; i<offset+4;i++)
  {
    // Is the bit set
    if ( interrupt & interruptInfo[ i].intrType)
    {
      // Save the callback routine
      interruptRoutines[ boardNumber-1][ i] = function;

      // If the callback is null, update the disable mask
      if (function == NULL) disableMask |= interruptInfo[ i].isrMask;
      else enableMask |= interruptInfo[ i].isrMask;
    }
  }

  // If packet framing is enabled, don't allow USC interrupts to be disabled
  if (boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] == GSC_ENABLED)
  {
    disableMask &= ~0x00008888;
  }

  // Get the current interrupt mask
  status = fpgaRead( boardNumber, INTERRUPT_CONTROL_REG, &offset);
  if (status) return status;

  // Clear any bits that need disabling
  offset &= (~disableMask);

  // Set any bits that need enabling
  offset |= enableMask;

  // Write the value back out
  status = fpgaWrite( boardNumber, INTERRUPT_CONTROL_REG, offset);

  // Read in the Interrupt Hi/Lo register
  status = fpgaRead( boardNumber, INTERRUPT_HI_LO_REG, &offset);
  if (status) return status;

  // If the interrupt type is rising
  if ((type == GSC_RISING_EDGE) || (type == GSC_HIGH_TRUE))
  {
    // Set the bits for the new interrupts
    offset |= enableMask;
  }
  else
  {
    // Clear the bits for the new interrupts
    offset &= ~enableMask;
  }

  // Write the value back out
  status = fpgaWrite( boardNumber, INTERRUPT_HI_LO_REG, offset);

  // Determine offset into the ISR array
  offset = (channel-1) * 2;

  // Check for Tx Transfer complete interrupt
  if (interrupt & GSC_INTR_TX_TRANSFER_COMPLETE)
  {
    // Put the function (or NULL) into the array
    transferCompleteRoutines[boardNumber-1][offset] = function;
  }

  // Check for Rx Transfer complete interrupt
  if (interrupt & GSC_INTR_RX_TRANSFER_COMPLETE)
  {
    // If framing mode is enabled, put function into rxCompletion array
    if (boardInformation[boardNumber-1].channelPacketFraming[channel-1] == GSC_ENABLED)
      rxTransferCompleteRoutines[boardNumber-1][channel-1] = function;

    // Otherwise, put the function into the standard array
    else
      transferCompleteRoutines[boardNumber-1][offset+1] = function;
  }

  return status;
}


//------------------------------------------------ gscAnaInitializeInterrupts
//
//  This routine initializes the interrupt structures for one
// board and starts its interrupt processor thread.
//
int gscAnaInitializeInterrupts( int boardNumber)
{
#ifdef WIN32
  HANDLE hThread;
  int bn = boardNumber - 1;
#endif
  int status = 0;
#ifdef PLX_LINUX
  pthread_t intrThread;
  pthread_attr_t intrThreadAttribute;
#endif

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Initialize the critical section object
#ifndef PLX_LINUX
  // InitializeCriticalSection(&completionPendingLock[ bn]);
#endif

#ifdef PLX_LINUX
  // Initialize the thread attribute structure
  status = pthread_attr_init(&intrThreadAttribute);
  if (status) return ApiFailed;

  // Set the thread to detached
  status = pthread_attr_setdetachstate(&intrThreadAttribute, PTHREAD_CREATE_DETACHED);
  if (status) return ApiFailed;

  // Start the Interrupt Handler Thread
  status = pthread_create(&intrThread, &intrThreadAttribute, fpgaInterruptHandler, (void *)&boardNumber);
  if (status) return ApiFailed;

  // Destroy the attribute object
  (void)pthread_attr_destroy(&intrThreadAttribute);
#else

  // Start the Interrupt thread and increase its priority
  hThread = CreateThread(0,0,gscSio4InterruptHandler, (LPVOID)&boardNumber, 0, NULL);
  if (hThread != NULL) SetThreadPriority( hThread, THREAD_PRIORITY_HIGHEST);

#endif

  Sleep(50); // don't let address of boardNumber var get reused until after threads are initialized.

  return status;
}


int getCompletionPending( int boardNumber)
{
  int pending;

  // Lock the variable
  //	EnterCriticalSection( &completionPendingLock[ boardNumber-1]);

  // Get the value
  pending = completionPending[ boardNumber-1];

  // Clear those bits
  completionPending[ boardNumber-1] &= ~pending;

  // Unlock the variable
  //	LeaveCriticalSection( &completionPendingLock[ boardNumber-1]);

  // return the pening interupts
  return pending;
}

void setCompletionPending( int boardNumber, int channel, int txRx)
{
#ifdef WIN32
  HANDLE hEvent;
#endif
  int pending = 1 << (2*(channel-1) + txRx);
#ifdef PLX_LINUX
  int messageId = GSC_THREAD_COMPLETION_TYPE;
  THREAD_MESSAGE threadMessage;
#endif

  // Lock the variable
  //	EnterCriticalSection( &completionPendingLock[ boardNumber-1]);

  // Mask on the new bit
  completionPending[ boardNumber-1] |= pending;

  // Unlock the variable
  //	LeaveCriticalSection( &completionPendingLock[ boardNumber-1]);

#ifdef PLX_LINUX
  // Flag the interrupt handler
  threadMessage.type = messageId;
  threadMessage.command = GSC_THREAD_START_CMD;
  if (msgsnd(boardInformation[boardNumber-1].intrQueueId, (void*)&threadMessage, sizeof(int), 0) == -1)
  {
    //      return ApiFailed;
  }
#else
  // Signal the interrupt thread that a change has occurred
  hEvent = CreateEvent( NULL, FALSE, FALSE, transferCompleteEvent[ boardNumber-1]);
  SetEvent ( hEvent );
  CloseHandle ( hEvent );
#endif
}

void insertRxCompletionPending( int boardNumber, int channel)
{
  int bn = boardNumber - 1;

  // Put channel into queue
  rxCompletionPending[ bn][ rxCompletionTo[ bn]] = channel;
  if (++rxCompletionTo[ bn] >= MAXIMUM_RX_PACKETS) rxCompletionTo[ bn] = 0;
}

void setRxCompletionPending( int boardNumber)
{
#ifdef PLX_LINUX
  int messageId = GSC_THREAD_RX_COMPLETION_TYPE;
  THREAD_MESSAGE threadMessage;
#else
  HANDLE hEvent;
#endif

#ifdef PLX_LINUX
  // Signal the interrupt handler
  threadMessage.type = messageId;
  threadMessage.command = GSC_THREAD_START_CMD;
  if (msgsnd(boardInformation[boardNumber-1].intrQueueId, (void*)&threadMessage, sizeof(int), 0) == -1)
  {
    //      return ApiFailed;
  }
#else
  // Signal the interrupt thread that a change has occurred
  hEvent = CreateEvent( NULL, FALSE, FALSE, rxTransferCompleteEvent[ boardNumber-1]);
  SetEvent ( hEvent );
  CloseHandle ( hEvent );
#endif
}

int getRxCompletionPending( int boardNumber)
{
  int channel;
  int bn = boardNumber - 1;

  // If nothing is in the queue, return 0
  if (rxCompletionTo[ bn] == rxCompletionFrom[ bn]) return 0;

  // Get channel from queue
  channel = rxCompletionPending[ bn][ rxCompletionFrom[ bn]];
  if (++rxCompletionFrom[ bn] >= MAXIMUM_RX_PACKETS) rxCompletionFrom[ bn] = 0;
  return channel;
}

#ifdef __linux__
int GscSetIntrSchedRR(int boardNumber, int priority)
{
  int status = 0;
  struct sched_param param;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Validate thread priority
  if (priority < 1 || priority > 99)
    return GscApiInvalidParameter;

  param.sched_priority = priority;

  status = pthread_setschedparam(*intrThread[boardNumber - 1], SCHED_RR, &param);

  return status;
}

int GscSetIntrSchedFifo(int boardNumber, int priority)
{
  int status = 0;
  struct sched_param param;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Validate thread priority
  if (priority < 1 || priority > 99)
    return GscApiInvalidParameter;

  param.sched_priority = priority;

  status = pthread_setschedparam(*intrThread[boardNumber - 1], SCHED_FIFO, &param);

  return status;
}

int GscSetRxIntrSchedRR(int boardNumber, int priority)
{
  int status = 0;
  struct sched_param param;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Validate thread priority
  if (priority < 1 || priority > 99)
    return GscApiInvalidParameter;

  param.sched_priority = priority;

  status = pthread_setschedparam(*rxIntrThread[boardNumber - 1], SCHED_RR, &param);

  return status;
}

int GscSetRxIntrSchedFifo(int boardNumber, int priority)
{
  int status = 0;
  struct sched_param param;

  // Make sure the boardNumber is valid
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Validate thread priority
  if (priority < 1 || priority > 99)
    return GscApiInvalidParameter;

  param.sched_priority = priority;

  status = pthread_setschedparam(*rxIntrThread[boardNumber - 1], SCHED_FIFO, &param);

  return status;
}
#endif
