//////////////////////////////////////////////////////////////////////////////
//
//  GscApiLocal.h
//
//  This is the local header file for the General Standards Corp API
// implementation. It is not distributed with the API library.
//

#ifndef GSCAPILOCAL_H
#define GSCAPILOCAL_H

// Includes required when compiling for Linux
#ifdef PLX_LINUX
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#endif

#define API_RETURN_CODE_STARTS PLX_STATUS_START

#include "PlxApi.h"
#include "PciTypes.h"
#include "PlxTypes.h"
#include "Plx.h"
#include "PciRegs.h"
#include "GscApiPlxLegacy.h"

#ifdef __cplusplus
extern "C" {
#endif

//
// Force the definitions in GscApi.h to be exported to a LIB file on WIN32
// platform
//
#ifdef PLX_LINUX
#define GSCAPI_API
#else
#define GSCAPI_API __declspec(dllexport)
#endif

#include "GscApi.h"
#include "GscApiPrivate.h"

//
// Macro to change WIN32 Sleep into Linux usleep
//
#ifdef PLX_LINUX
#define Sleep(x) usleep(1000*x)
#endif

//
// Macro to enable data logging. Uncomment to log data
//
//#define DATA_LOGGING

//
// Maximum number of channels to allocate space for
//
#define MAX_NUM_OF_CHANNELS     4

//
// Maximum number of drivers supported by the Plx Api
//
#define MAX_NUM_OF_DRIVERS      9

//
// This Macro defines the reference frequency in Hz to the clock chip
//
#define REFERENCE_FREQ  20000000


//
// Macro used to generate a return value for the threads
//
#define THREAD_STATUS(a,b,c,d)  ((a<<24)+(b<<16)+(c<<8)+(d))

//
// Extra error codes defined by the API
//
enum GSC_ERROR_CODES
{
  GscApiInvalidBoardNumber = API_RETURN_CODE_STARTS + 0x100,
  GscApiInvalidChannel,
  GscApiInvalidDmaNumWords,
  GscApiDmaBufferTooSmall,
  GscApiInvalidDmaBufferAddr,
  GscApiDmaChannelBusy,
  GscApiInvalidFWRev,
  GscApiInvalidParameter,
  GscApiOperationTimeout,
  GscApiInvalidCombination,
  GscApiDllHeaderMismatch,
  GscApiDataPacketTooLong,
  GscApiFifoOverrun,
  GscApiInvalidBoardType,
  GscApiAutoCalFailed,
  GscApiMisalignedBuffer,
  GscApiCTCNoBufferedFrames,
  GscApiLastError
};


//
// Tokens for thread information
//
enum GSC_THREAD_INFO
{
  GSC_THREAD_TX_DMA             = 1,
  GSC_THREAD_RX_DMA             = 2,
  GSC_THREAD_INTR                       = 3,
  GSC_THREAD_INTR_RX            = 4,
  GSC_THREAD_ABORT_CMD,
  GSC_THREAD_START_CMD,
  GSC_THREAD_XFER_COMPLETE,
  GSC_THREAD_NORMAL_EXIT              = 0x00,
  GSC_THREAD_ERROR_EXIT       = 0xff,
  GSC_THREAD_TX_DMA_TYPE        = 0x0000,
  GSC_THREAD_RX_DMA_TYPE        = 0x0100,
  GSC_THREAD_COMPLETION_TYPE    = 0x0200,
  GSC_THREAD_RX_COMPLETION_TYPE = 0x0300,
  GSC_THREAD_FPGA_INTR_TYPE     = 0x0400,
};

//
// Tokens for the board types
//
enum GSC_BOARD_TYPES
{
  GSC_BOARD_TYPE_NORMAL,
  GSC_BOARD_TYPE_SYNC,
  GSC_BOARD_TYPE_BISYNC16,
  GSC_BOARD_TYPE_RASYNC,
  GSC_BOARD_TYPE_ASYNC,
  GSC_BOARD_TYPE_QPSK,
  GSC_BOARD_TYPE_ANALOG,
  GSC_BOARD_TYPE_CTC
};

enum GSC_BOARD_FEATURES
{
  GSC_FEATURE_HAS_ZILOG_CHIP    = 0x0001,       // Flags the presense of a Zilog Serial Chip
  GSC_FEATURE_HAS_RSB_FIFO      = 0x0002,       // Flags the presense of Rx packet info FIFO
};

//
// Structure that holds the PLL parameters while the values
// are being calculated
//
typedef struct {
  int   Div, P, Q, PO;
  int   Error, Freq, PLL;
  int   PostDivider;
  float pError;
} PLL_PARAMS;

//
// Structure that holds the current usage information for each PLL
//
typedef struct {
  int           Freq;
  int           Q;
  int           P;
  int           PO;
} PLL_USAGE;


//
// Structure to hold the board/channel information
//
typedef struct _BOARD_INFORMATION
{
  HANDLE        handle;                                         // Plx Handle to the board
  int           boardType;                                      // Type of board (Sync, Normal, etc.)
  int           features;                                       // Bits that list board features and characteristics
  int           masterClock;                                    // Master clock frequency in Hz
  int           numberOfChannels;                               // Number of individual channels
  int           txFifoSizes[ MAX_NUM_OF_CHANNELS];              // List of Fifo Sizes for the transmitters
  int           rxFifoSizes[ MAX_NUM_OF_CHANNELS];              // List of Fifo Sizes for the transmitters
//      char    busSpeed[10];                                   // A text description of the bus speed
  char          busWidth[10];                                   // A text description of the bus width
  char          formFactor[10];                                 // A text description of the form factor
  char          subId[10];                                      // A text description of the sub ID
  char          rev[10];                                        // A text description of the board rev
  char          version[20];                                    // A text description of the version
  int           fpgaVersion;                                    // Binary version of FPGA version info
  char          fpgaVersionText[10];                            // Text version of FPGA version info
  char          boardDescription[25];                           // Text description of the board

  int           channelPacketFraming[ NUMBER_OF_CHANNELS];      // User specified packet framing mode
  int           channelMode[ NUMBER_OF_CHANNELS];               // User specified mode (async,HDLC, etc.)
  int           channelBitRate[ NUMBER_OF_CHANNELS];            // User specified bit rate
  int           channelEncoding[ NUMBER_OF_CHANNELS];           // User specified Encoding setting
  int           channelProtocol[ NUMBER_OF_CHANNELS];           // User specified Protocol setting
  int           channelTermination[ NUMBER_OF_CHANNELS];        // User specified termination setting
  int           channelGapSize[ NUMBER_OF_CHANNELS];            // User specified gap size
  int           channelParity[ NUMBER_OF_CHANNELS];             // User specified parity
  int           channelStopBits[ NUMBER_OF_CHANNELS];           // User specified stop bits
  int           channelDteDce[ NUMBER_OF_CHANNELS];             // User specified DTE/DCE setting
  int           channelLoopback[ NUMBER_OF_CHANNELS];           // User specified Loop back setting
  int           channelTxAlmostEmpty[ NUMBER_OF_CHANNELS];      // User specified Tx Almost Empty value
  int           channelTxAlmostFull[ NUMBER_OF_CHANNELS];       // User specified Tx Almost Full value
  int           channelRxAlmostEmpty[ NUMBER_OF_CHANNELS];      // User specified Rx Almost Empty value
  int           channelRxAlmostFull[ NUMBER_OF_CHANNELS];       // User specified Rx Almost Full value
  int           channelCrcMode[ NUMBER_OF_CHANNELS];            // User specified CRC mode
  int           channelCrcValue[ NUMBER_OF_CHANNELS];           // User specified Initial CRC value
  int           channelFrequency[ NUMBER_OF_CHANNELS];          // Currently set frequency on oscillator
  double        bytesPerMs[ NUMBER_OF_CHANNELS];                // Number of bytes transfered per mSec
  int           channelPinMode[NUMBER_OF_CHANNELS][GSC_PIN_TX_USCCLOCK-GSC_PIN_RX_CLOCK+1];
  char          lastError[ NUMBER_OF_CHANNELS][200];            // Last error for a channel
  int           intrQueueId;                                    // linux-only ipc message queue for dma threads
  int           dmaQueueId;                                     // linux-only ipc message queue for dma threads
} BOARD_INFORMATION, *PBOARD_INFORMATION;


//
// Structure that maintains the DMA transfer queue
//
typedef struct _DMA_TRANSFER_QUEUE
{
  int   size;                   // Size of Transfer
  int IsBufferLogicalAddr;
  char  *buffer;                // Start of buffer
  PLX_PHYSICAL_MEM      physBuffer;             // Physical memory buffer
  int   id;                     // Unique ID number
  int   tcbNeeded;              // Flag to send TCB
  int   tcbBytes[4];            // TCB bytes
  int   forceFlush;             // Flag to force a transfer flush
  int   originalSize;           // Copy of the intitial size of transfer
  GSC_CB_FUNCTION       *function;      // Callback function
} DMA_TRANSFER_QUEUE, *PDMA_TRANSFER_QUEUE;

//
// Structure for holding the data to log (Debug only)
//
#define MAXIMUM_DATALOG_ENTRIES         1000
typedef struct _DATA_LOG
{
  int           size;
  int           delta;
  int           time;
  char  *buffer;
  int           loopCounter;
  int           reason;
  int           timerDelay;
} DATA_LOG, *PDATA_LOG;

#define MAXIMUM_RX_PACKETS      1000


//
// Structure for passing thread messages
typedef struct _THREAD_MESSAGE
{
  long int type;
  int command;
} THREAD_MESSAGE, *PTHREAD_MESSAGE;

typedef struct
{
  int intrType;
  int   isrMask;
} INTERRUPT_INFO;

//
// Declare/Extern the global variables
//
#ifdef __MAIN__
#define EXTERN
#else
#define EXTERN extern
#endif

EXTERN HANDLE            sio4BoardHandlesLegacy[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN PLX_DEVICE_OBJECT6 sio4BoardHandles6[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN PLX_DEVICE_OBJECT sio4BoardHandles[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN int               processBoardRef[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN U8 drvMajorVersion;
EXTERN U8 drvMinorVersion;
//EXTERN INTERRUPT_INFO    analogIntrInfo[4];
EXTERN int gscBoardCountLegacy;
EXTERN U8 gscBoardCount;

EXTERN DATA_LOG        rxDataLog[ MAXIMUM_DATALOG_ENTRIES];
EXTERN DATA_LOG        txDataLog[ MAXIMUM_DATALOG_ENTRIES];
EXTERN int             rxDataLogPointer;
EXTERN int             txDataLogPointer;

EXTERN char    threadAbort[ MAXIMUM_NUMBER_OF_BOARDS][20];
EXTERN char    transferCompleteEvent[ MAXIMUM_NUMBER_OF_BOARDS][20];
EXTERN char    rxTransferCompleteEvent[ MAXIMUM_NUMBER_OF_BOARDS][20];

//
// Array that holds event names for starting Tx and Rx
//
EXTERN char     txStartEvent[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][20];
EXTERN char     rxStartEvent[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][20];
EXTERN char     rxCompleteChannels[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];

// Shared memory
//
#ifdef __linux__

EXTERN int *soRefCount;
EXTERN int *bdRefCount;

EXTERN BOARD_INFORMATION *boardInformation;
EXTERN DEVICE_LOCATION3 *sio4BoardsLegacy3;
EXTERN DEVICE_LOCATION *sio4BoardsLegacy;
EXTERN PLX_DEVICE_KEY6 sio4Boards6[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN PLX_DEVICE_KEY sio4Boards[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN BOARD_INFO *sio4BoardInfo;

//
// Queue to hold the Transmit Transfer Requests
//
EXTERN DMA_TRANSFER_QUEUE       *txDmaQueue[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int                      *txDmaQueueTo[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN int                      *txDmaQueueFrom[MAXIMUM_NUMBER_OF_BOARDS];

//
// Queue to hold the Receive Transfer Requests
//
EXTERN DMA_TRANSFER_QUEUE       *rxDmaQueue[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int                      *rxDmaQueueTo[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN int                      *rxDmaQueueFrom[MAXIMUM_NUMBER_OF_BOARDS];

//
// Variable to hold the next Transfer ID
//
EXTERN int *dmaRequestQueueId;

//
// Semaphore to implement mutual exlusion in tx/rx queue operations
//
EXTERN pthread_mutex_t   *PlxApiCalls;    // Mutex for Plx Api Calls
EXTERN pthread_mutex_t   *GscApiCalls;    // Mutex for GSC Api Calls
EXTERN pthread_mutex_t   *qmutex;         // Mutex for general dma queue operations
EXTERN int               *threadMessageId;

EXTERN int *rxPacketLengths[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int *rxPacketStatus[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int *rxPacketTo[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN int *rxPacketFrom[MAXIMUM_NUMBER_OF_BOARDS];

EXTERN pthread_t *intrThread[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN pthread_t *rxIntrThread[MAXIMUM_NUMBER_OF_BOARDS];

#else
#ifdef __MAIN__

#pragma data_seg(".GscApiShared")
int soRefCount = 0;
int bdRefCount[MAXIMUM_NUMBER_OF_BOARDS] = {0,0,0,0,0,0,0,0,0,0};

BOARD_INFORMATION boardInformation[MAXIMUM_NUMBER_OF_BOARDS] = {0};
PLX_DEVICE_KEY6 sio4Boards6[MAXIMUM_NUMBER_OF_BOARDS] = {0};
PLX_DEVICE_KEY sio4Boards[MAXIMUM_NUMBER_OF_BOARDS] = {0};
DEVICE_LOCATION3 sio4BoardsLegacy3[ MAXIMUM_NUMBER_OF_BOARDS] = {0};
DEVICE_LOCATION sio4BoardsLegacy[ MAXIMUM_NUMBER_OF_BOARDS] = {0};
BOARD_INFO sio4BoardInfo[MAXIMUM_NUMBER_OF_BOARDS] = {0};

//
// Queue to hold the Transmit Transfer Requests
//
DMA_TRANSFER_QUEUE      txDmaQueue[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_DMA_QUEUE_ITEMS] = {0};
int txDmaQueueTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS] = {0};
int txDmaQueueFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS] = {0};

//
// Queue to hold the Receive Transfer Requests
//
DMA_TRANSFER_QUEUE      rxDmaQueue[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_DMA_QUEUE_ITEMS] = {0};
int rxDmaQueueTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS] = {0};
int rxDmaQueueFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS] = {0};

//
// Variable to hold the next Transfer ID
//
int dmaRequestQueueId[MAXIMUM_NUMBER_OF_BOARDS] = {0};

int rxPacketLengths[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_RX_PACKETS] = {0};
int rxPacketStatus[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_RX_PACKETS] = {0};
int rxPacketTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS] = {0};
int rxPacketFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS] = {0};

#endif

EXTERN int bdRefCount[MAXIMUM_NUMBER_OF_BOARDS];

EXTERN BOARD_INFORMATION boardInformation[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN PLX_DEVICE_KEY6 sio4Boards6[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN PLX_DEVICE_KEY sio4Boards[MAXIMUM_NUMBER_OF_BOARDS];
EXTERN DEVICE_LOCATION3 sio4BoardsLegacy3[ MAXIMUM_NUMBER_OF_BOARDS];
EXTERN DEVICE_LOCATION sio4BoardsLegacy[ MAXIMUM_NUMBER_OF_BOARDS];
EXTERN BOARD_INFO sio4BoardInfo[MAXIMUM_NUMBER_OF_BOARDS];

//
// Queue to hold the Transmit Transfer Requests
//
EXTERN DMA_TRANSFER_QUEUE       txDmaQueue[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_DMA_QUEUE_ITEMS];
EXTERN int txDmaQueueTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int txDmaQueueFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];

//
// Queue to hold the Receive Transfer Requests
//
EXTERN DMA_TRANSFER_QUEUE       rxDmaQueue[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_DMA_QUEUE_ITEMS];
EXTERN int rxDmaQueueTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int rxDmaQueueFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];

//
// rx Packet structures
//
EXTERN int rxPacketLengths[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_RX_PACKETS];
EXTERN int rxPacketStatus[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS][MAXIMUM_RX_PACKETS];
EXTERN int rxPacketTo[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];
EXTERN int rxPacketFrom[MAXIMUM_NUMBER_OF_BOARDS][NUMBER_OF_CHANNELS];

//
// Variable to hold the next Transfer ID
//
EXTERN int dmaRequestQueueId[MAXIMUM_NUMBER_OF_BOARDS];

//
// Mutexes to implement mutual exlusion in tx/rx queue operations
//
EXTERN HANDLE   PlxApiCalls;    // Mutex for Plx Api Calls
EXTERN HANDLE   GscApiCalls;    // Mutex for GSC Api Calls
EXTERN HANDLE   qmutex;

#ifdef __MAIN__
#pragma data_seg()
#pragma comment(linker, "/SECTION:.GscApiShared,rws")
#endif

#endif

// Private routines
#ifdef PLX_LINUX
void __attribute__ ((constructor)) GscSio4InitializeApi();
#endif
int gscSio4OpenDma( int boardNumber, int dmaChannel);
int gscSio4CloseDma( int boardNumber, int dmaChannel);
int gscPerformDma(int boardNumber, int channel, char *buffer, int count, int direction, int dmaChannel);
int gscPerformPlxPhysDma( int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int direction, int dmaChannel);
int gscSio4PerformDma( int boardNumber, int channel, char *buffer, int count, int direction);
int gscAddDmaRequest( int boardNumber, int channel, char *buffer, int count, int direction, int *id);
int gscAddPlxPhysDmaRequest( int boardNumber, int channel, PLX_PHYSICAL_MEM *buffer, int count, int direction, int *id);
int gscSio4InitializeInterrupts( int boardNumber);
int validateBoardNumber( int boardNumber);
int validateChannel( int channel);
int validateBoardAndChannelNumber( int boardNumber, int channel);
int validatePinName( int pinName, int *offset, int *mask, int *shift);
int determineBoardType(int boardNumber);
int resetBoard( int boardNumber);

int gscSio4ChannelGetMsbLsbOrder(int boardNumber, int channel, int *txOrder, int *rxOrder);
int gscSio4ChannelSetMsbLsbOrder(int boardNumber, int channel, int txOrder, int rxOrder);
int gscSio4ChannelSetEncoding(int boardNumber, int channel, int encoding);
int gscSio4ChannelGetProtocol(int boardNumber, int channel, int *protocol, int *termination);
int gscSio4ChannelGetParity(int boardNumber, int channel, int *parity);
int gscSio4ChannelSetStopBits(int boardNumber, int channel, int stopBits);
int gscSio4ChannelSetDteDce(int boardNumber, int channel, int mode);
int gscSio4ChannelSetClockSource( int boardNumber, int channel, int txSource, int rxSource);
int getRxFifoCount( int boardNumber, int channel, int *rxFifoCount);
int resetRxFifo( int boardNumber, int channel);

int getCompletionPending( int boardNumber);
void setCompletionPending( int boardNumber, int channel, int txRx);
int  getRxCompletionPending( int boardNumber);
void insertRxCompletionPending( int boardNumber, int channel);
void setRxCompletionPending( int boardNumber);

// Protocol Specific Routines
int gscSio4HdlcInit(int boardNumber, int channel);
int gscSio4BiSyncInit(int boardNumber, int channel);
int gscSio4BiSync16Init(int boardNumber, int channel);
int syncBoardSetMode( int boardNumber, int channel, int mode, int bitRate);
int gscSio4AsyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue);
int gscSio4AsyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] );
int gscSio4HdlcSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue);
int gscSio4HdlcGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] );
int gscSio4BiSyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] );
int gscSio4BiSyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue);
int gscSio4SyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue);
int gscSio4SyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] );
int gscSio4BiSync16SetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue);
int gscSio4BiSync16GetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] );
int gscSio4ChannelGetBiSyncPattern( int boardNumber, int channel, int *TxSync, int *RxSync);
int gscSio4ChannelSetBiSyncPattern( int boardNumber, int channel, int TxSync, int RxSync);
int gscSio4ChannelGetBiSyncTxUnderrun( int boardNumber, int channel, int *txUnderrun);
int gscSio4ChannelGetBiSyncTxPreamble( int boardNumber, int channel, int *txPreamble);
int gscSio4ChannelGetBiSyncTxShortSync( int boardNumber, int channel, int *txShortSync);
int gscSio4ChannelGetBiSyncRxSyncStrip( int boardNumber, int channel, int *rxSyncStrip);
int gscSio4ChannelGetBiSyncRxShortSync( int boardNumber, int channel, int *rxShortSync);
int gscSio4ChannelGetBiSyncTxPreambleLength( int boardNumber, int channel, int *txPreambleLength);
int gscSio4ChannelGetBiSyncTxPreamblePattern( int boardNumber, int channel, int *txPreamblePattern);
int gscSio4ChannelSetFifoTimetagByte( int boardNumber, int channel, int fifoTimetagSetting);
int gscSio4ChannelSetFifoParityErrorByte( int boardNumber, int channel, int fifoParitySetting);
int gscSio4ChannelIsBxrSync( int boardNumber, int channel );

// Zilog common routines
int zilogRead(int boardNumber, int channel, int reg, int *value);
int zilogWrite(int boardNumber, int channel, int reg, int value);
int zilogGetDataLength(int boardNumber, int channel, int *txDataSize, int *rxDataSize);
int zilogSetDataLength(int boardNumber, int channel, int txDataSize, int rxDataSize);
int zilogSetTxUnderRun( int boardNumber, int channel, int mode);
int zilogGetTxPreamble( int boardNumber, int channel, int *mode, int *pattern);
int zilogSetTxPreamble( int boardNumber, int channel, int mode, int pattern);
int zilogSetTxSharedZero( int boardNumber, int channel, int mode);
int zilogGetRxAddrSearch( int boardNumber, int channel, int *mode, int *address);
int zilogSetRxAddrSearch( int boardNumber, int channel, int mode, int address);
int zilogSetCrcMode( int boardNumber, int channel, int crcMode, int initialValue);
int zilogGetCrcMode( int boardNumber, int channel, int *crcMode, int *initialValue);
int zilogGetTransmitterControl( int boardNumber, int channel, int *txMode);
int zilogTransmitterControl( int boardNumber, int channel, int txMode);
int zilogGetReceiverControl( int boardNumber, int channel, int *rxMode);
int zilogReceiverControl( int boardNumber, int channel, int rxMode);
int zilogSetEncoding(int boardNumber, int channel, int encoding, int mode, int bitRate);
int zilogGetIdleLineCondition(int boardNumber, int channel, int *mode);
int zilogSetIdleLineCondition(int boardNumber, int channel, int mode);

int fpgaRead( int boardNumber, int reg, int *result);
int fpgaWrite( int boardNumber, int reg, int value);
int fpgaRead8( int boardNumber, int reg, unsigned char *result);
int fpgaRead16( int boardNumber, int reg, unsigned short *result);
int fpgaWrite16( int boardNumber, int reg, unsigned short value);
int quickSetDteDce(int boardNumber, int channel, int mode);
int quickSetPinMode( int boardNumber, int channel, int pinName, int mode);
int fpgaSetEncoding(int boardNumber, int channel, int encoding, int bitRate);
int quickSetLoopBack( int boardNumber, int channel, int loopMode);
int quickNormalSetClockSource( int boardNumber, int channel, int txSource, int rxSource);
int quickSetProtocol(int boardNumber, int channel, int protocol, int termination);
int quickSetParity(int boardNumber, int channel, int parity);
int gscsio4ChannelGetClockSource( int boardNumber, int channel, int *txSource, int *rxSource);
int gscSio4RTDynLinkPlx();

// Analog Specific Routines
int gscAnaInitializeInterrupts( int boardNumber );
int gscAnaResetRxFifo(  int BoardNo );
int gscAnaGetRxFifoCount( int BoardNo, int *BuffSize  );
int gscAnaInvokeInterruptCallback( int BoardNo );
int gscAnaClearInterrupts( int BoardNo );
int gscAnaGetIDBRegOffset( int BoardNo );

#ifdef WIN32
void CALLBACK uscInterruptServiceRoutine( int boardNumber, int channel, int interrupt);

DWORD WINAPI gscSio4TxDmaHandler(LPVOID pParam);
DWORD WINAPI gscSio4RxDmaHandler(LPVOID pParam);
#endif

#ifdef __linux__
void uscInterruptServiceRoutine( int boardNumber, int channel, int interrupt);

void *gscSio4TxDmaHandler(void * pParam);
void *gscSio4RxDmaHandler(void * pParam);

int InitCriticalSection( pthread_mutex_t *mutex);
int DelCriticalSection( pthread_mutex_t *mutex);
int BeginCriticalSection( pthread_mutex_t *mutex);
int EndCriticalSection( pthread_mutex_t *mutex);
#else
int InitCriticalSection( HANDLE *ghMutex,  LPCTSTR sMutexName );
int DelCriticalSection( HANDLE ghMutex );
int BeginCriticalSection( HANDLE ghMutex );
int EndCriticalSection( HANDLE ghMutex );
#endif

#ifdef __cplusplus
}
#endif
#endif
