//////////////////////////////////////////////////////////////////////////////
//
// GscApiPins.c
//
//  This module contains all of the routines that control how the pins
// of the cable are configured.
//
#include "GscApiLocal.h"

//--------------------------------------------------- GscSio4ChannelGetPinMode
//
//  This routine returns the current mode for one of
// the programable pins.
//
int GscSio4ChannelGetPinMode( int boardNumber, int channel, int pinName, int *mode)
{
	int status = 0x00;
	int offset;

	// Validate the Board and Channel numbers
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Validate the Pin name
	status = validatePinName( pinName, &offset, NULL, NULL);
	if (status) return status;

	// Return the mode
	*mode = boardInformation[ boardNumber-1].channelPinMode[ channel-1][offset];

	return status;
}


//--------------------------------------------------- GscSio4ChannelSetPinMode
//
//  This routine sets the current mode for one of the
// programable pins.
//
int quickSetPinMode( int boardNumber, int channel, int pinName, int mode)
{
  int status = 0x00;
  int offset, oldValue;

  // Validate the Pin name
  status = validatePinName( pinName, &offset, NULL, NULL);
  if (status) return status;

  // Save Setting
  boardInformation[ boardNumber-1].channelPinMode[ channel-1][offset] = mode;

  // Handle Sync and BiSync16 cards separately
  if ((boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
      (gscSio4ChannelIsBxrSync(boardNumber, channel)) ||
      (boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_BISYNC16))
  {
    // Process AUTO mode
    if (mode == GSC_PIN_AUTO)
    {
      // Get the current Pin Source Register value
      status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &oldValue);
      if (status) return status;

      // Process the proper pin
      switch (pinName)
      {
        // These are output onlys
      case GSC_PIN_TX_CLOCK:	oldValue &= ~0x00000007; oldValue |= 0x00000000; break;
      case GSC_PIN_TX_DATA:	oldValue &= ~0x000001c0; oldValue |= 0x00000000; break;
      case GSC_PIN_RTS:		oldValue &= ~0x00000030; oldValue |= 0x00000000; break;
      }

      // Set the new Pin Source Register value
      status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, oldValue);
      if (status) return status;
    }

    // Process GPIO mode
    else
    {
      // Process the proper pin
      switch (pinName)
      {
        // These are output onlys
      case GSC_PIN_TX_CLOCK:
      case GSC_PIN_TX_DATA:
      case GSC_PIN_RTS:
        // Set the value to '1'. Forces GPIO mode
        GscSio4ChannelSetPinValue(boardNumber,channel, pinName, 1);
        break;
      }
    }
  }

  // Handle other boards
  else
  {
    // Handle the requested mode
    if (mode == GSC_PIN_AUTO)
    {
      // Get the current Pin Source Register value
      status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &oldValue);
      if (status) return status;

      // Process the proper pin
      switch (pinName)
      {
      case GSC_PIN_RX_CLOCK:	oldValue &= ~0x00000038; oldValue |= 0x00000020; break;
      case GSC_PIN_RX_DATA:
      case GSC_PIN_CTS:		oldValue &= ~0x00000600; oldValue |= 0x00000200; break;
        break;

        // These are output onlys
      case GSC_PIN_TX_CLOCK:	oldValue &= ~0x000001c0; oldValue |= 0x00000100; break;
      case GSC_PIN_TX_DATA:	oldValue &= ~0x00180000; oldValue |= 0x00000000; break;
      case GSC_PIN_RTS:		oldValue &= ~0x00006000; oldValue |= 0x00004000; break;

      case GSC_PIN_TX_USCCLOCK:
      case GSC_PIN_DCD:
      case GSC_PIN_AUXCLK:
        break;
      }

      // Set the new Pin Source Register value
      status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, oldValue);
      if (status) return status;
    }

    else if (mode == GSC_PIN_GPIO)
    {
      // Process the proper pin
      switch (pinName)
      {
      case GSC_PIN_RX_CLOCK:
      case GSC_PIN_RX_DATA:
      case GSC_PIN_CTS:
        break;

        // These are outputs only. Setting the value will also put them into GPIO mode
      case GSC_PIN_TX_CLOCK:	GscSio4ChannelSetPinValue(boardNumber,channel, pinName, 1); break;
      case GSC_PIN_TX_DATA:	GscSio4ChannelSetPinValue(boardNumber,channel, pinName, 1); break;
      case GSC_PIN_RTS:		GscSio4ChannelSetPinValue(boardNumber,channel, pinName, 1); break;
      case GSC_PIN_TX_USCCLOCK:

      case GSC_PIN_DCD:
      case GSC_PIN_AUXCLK:
        break;
      }
    }

    else if (mode == GSC_PIN_CLOCK_REVERSEPOLARITY)
    {
      // Get the current Pin Source Register value
      status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &oldValue);
      if (status) return status;

      // Process the proper pin
      switch (pinName)
      {
      case GSC_PIN_RX_CLOCK:
	oldValue &= ~0x00000038; oldValue |= 0x00000030;
        break;

        // These are outputs only. Setting the value will also put them into GPIO mode
      case GSC_PIN_TX_USCCLOCK:
	oldValue &= ~0x00000007;
        oldValue |= 0x00000006;
        break;
      }

      // Set the new Pin Source Register value
      status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, oldValue);
      if (status) return status;
    }

    else return GscApiInvalidParameter;
  }

  return status;
}

int GscSio4ChannelSetPinMode( int boardNumber, int channel, int pinName, int mode)
{
	int status = 0x00;

	// Validate the Board and Channel numbers
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure mode is valid
	if ((mode != GSC_PIN_AUTO) && (mode != GSC_PIN_GPIO)) return GscApiInvalidParameter;

	// Make the changes
	status = quickSetPinMode( boardNumber, channel, pinName, mode);

	return status;
}


//-------------------------------------------------- GscSio4ChannelGetPinValue
//
//  This routine returns the current value for one of
// the programable pins.
//
int GscSio4ChannelGetPinValue( int boardNumber, int channel, int pinName, int *value)
{
	int status = 0x00;
	int offset, mask, shift;

	// Validate the Board and Channel numbers
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Validate the Pin name
	status = validatePinName( pinName, &offset, &mask, &shift);
	if (status) return status;

	// Get the current pin states
	status = fpgaRead( boardNumber, PIN_STATUS_BASE_REG+(channel-1)*4, value);
	if (status) return status;

	// Handle DCD and AuxClk separately
	if ((pinName==GSC_PIN_DCD) && (boardInformation[boardNumber-1].channelDteDce[channel-1]==GSC_PIN_DCE))
	{
		mask = 0x80;
		shift = 0x07;
	}
	else if ((pinName==GSC_PIN_AUXCLK) && (boardInformation[boardNumber-1].channelDteDce[channel-1]==GSC_PIN_DTE))
	{
		mask = 0x200;
		shift = 0x09;
	}

	// Keep only the requested bit
	*value = (*value & mask) >> shift;
	return status;
}


//-------------------------------------------------- GscSio4ChannelSetPinValue
//
//  This routine sets the current value for one of
// the programable pins if the pin is programmed to
// be GPIO.
//
int GscSio4ChannelSetPinValue( int boardNumber, int channel, int pinName, int value)
{
  int status = 0x00;
  int offset, oldValue;

  // Validate the Board and Channel numbers
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Validate the Pin name
  status = validatePinName( pinName, &offset, NULL, NULL);
  if (status) return status;

  // Return if the pin is not GPIO
  if (boardInformation[ boardNumber-1].channelPinMode[ channel-1][offset] != GSC_PIN_GPIO) return GscApiInvalidCombination;

  // Get the current Pin Source Register value
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, &oldValue);
  if (status) return status;

  // Handle -Sync board separately
  if ((boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_SYNC) ||
      (gscSio4ChannelIsBxrSync(boardNumber, channel)) ||
      (boardInformation[ boardNumber-1].boardType == GSC_BOARD_TYPE_BISYNC16))
  {
    // Process the proper pin
    switch (pinName)
    {
      // These are input only, can't set their values
    case GSC_PIN_RX_CLOCK:	return GscApiInvalidCombination;
    case GSC_PIN_RX_DATA:	return GscApiInvalidCombination;
    case GSC_PIN_CTS:		return GscApiInvalidCombination;

      // These are always outputs, so just update them
    case GSC_PIN_TX_CLOCK:	oldValue &= ~0x00000007; oldValue |= (value==0)?0x00000006:0x00000007; break;
    case GSC_PIN_TX_DATA:	oldValue &= ~0x000001c0; oldValue |= (value==0)?0x00000180:0x000001c0; break;
    case GSC_PIN_RTS:		oldValue &= ~0x00000030; oldValue |= (value==0)?0x00000020:0x00000030; break;
    }
  }

  // Otherwise, process other boards
  else
  {
    // Process the proper pin
    switch (pinName)
    {
      // These are input only, can't set their values
    case GSC_PIN_RX_CLOCK:	return GscApiInvalidCombination;
    case GSC_PIN_RX_DATA:	return GscApiInvalidCombination;
    case GSC_PIN_CTS:		return GscApiInvalidCombination;

      // These are always outputs, so just update them
    case GSC_PIN_TX_CLOCK:	oldValue &= ~0x000001c0; oldValue |= (value==0)?0x00000080:0x000000c0; break;
    case GSC_PIN_TX_DATA:	oldValue &= ~0x00180000; oldValue |= (value==0)?0x00100000:0x00180000; break;
    case GSC_PIN_RTS:		oldValue &= ~0x00006000; oldValue |= (value==0)?0x00004000:0x00006000; break;

      // These are input/output based on the DCE/DTE setting
    case GSC_PIN_DCD:
      if (boardInformation[ boardNumber-1].channelDteDce[ channel-1] == GSC_PIN_DTE) return GscApiInvalidCombination;
      oldValue &= ~0x00018000;
      oldValue |= (value==0)?0x00010000:0x00018000;
      break;

    case GSC_PIN_AUXCLK:
      if (boardInformation[ boardNumber-1].channelDteDce[ channel-1] == GSC_PIN_DCE) return GscApiInvalidCombination;
      oldValue &= ~0x00e00000;
      oldValue |= (value==0)?0x00400000:0x00600000;
      break;
    }
  }

  // Set the new Pin Source Register value
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+(channel-1)*4, oldValue);
  return status;
}
