//////////////////////////////////////////////////////////////////////////////
//
//  GscApiPlxLegacy.h
//
//  This header file supplements the PLX SDK v.5 PlxLegacy.h header file with data structures that
// differ from the PLX SDK v. 4.4 definitions, or were not present in v.4.4. These data structures
// are necessary for calling Plx API functions in older versions of the PLX SDK. (prior to v. 4.4)
//
#ifndef GSCAPIPLXLEGACY_H
#define GSCAPIPLXLEGACY_H

#ifdef __cplusplus
extern "C" {
#endif


// DMA Transfer Element Union Structure for Plx v. 3
#ifdef PCI_CODE
#undef PCI_CODE
#endif
#define PCI_CODE

#ifndef PLX_LINUX
#define LITTLE_ENDIAN
#endif

//==============================================================================================
// Plx Sdk 6.0 - compatible structures.

// PCI Device Key Identifier
typedef struct _PLX_DEVICE_KEY6
{
    U32          IsValidTag;         // Magic number to determine validity
    U8           bus;                // Physical device location
    U8           slot;
    U8           function;
    U16          VendorId;           // Device Identifier
    U16          DeviceId;
    U16          SubVendorId;
    U16          SubDeviceId;
    U8           Revision;
    U8           ApiIndex;           // Used internally by the API
    U8           DeviceNumber;       // Used internally by device drivers
    PLX_API_MODE ApiMode;            // Mode API uses to access device
    U8           PlxPort;            // PLX port number of device
    U8           NTPortType;         // Stores if NT port & whether link or virtual
    U32          Reserved[8];        // Reserved for future use
} PLX_DEVICE_KEY6;


// PLX Device Object Structure
typedef struct _PLX_DEVICE_OBJECT6
{
    U32                IsValidTag;   // Magic number to determine validity
    PLX_DEVICE_KEY6    Key;          // Device location key identifier
    PLX_DRIVER_HANDLE  hDevice;      // Handle to driver
    PLX_PCI_BAR_PROP   PciBar[6];    // PCI BAR properties
    U64                PciBarVa[6];  // For PCI BAR user-mode BAR mappings
    U8                 BarMapRef[6]; // BAR map count used by API
    U32                PlxChipType;  // PLX chip type
    U8                 PlxRevision;  // PLX chip revision
    PLX_PHYSICAL_MEM   CommonBuffer; // Used to store common buffer information
    U32                Reserved[8];  // Reserved for future use
} PLX_DEVICE_OBJECT6;

// PLX Interrupt Structure
typedef struct _PLX_INTERRUPT6
{
    U32 Doorbell;
    U8  PciMain               :1;
    U8  PciAbort              :1;
    U8  LocalToPci_1          :1;
    U8  LocalToPci_2          :1;
    U8  DmaChannel_0          :1;
    U8  DmaChannel_1          :1;
    U8  MuInboundPost         :1;
    U8  MuOutboundPost        :1;
    U8  MuOutboundOverflow    :1;
    U8  TargetRetryAbort      :1;
    U8  Message_0             :1;
    U8  Message_1             :1;
    U8  Message_2             :1;
    U8  Message_3             :1;
    U8  SwInterrupt           :1;
    U8  ResetDeassert         :1;
    U8  PmeDeassert           :1;
    U8  GPIO_4_5              :1;
    U8  GPIO_14_15            :1;
    U8  HotPlugAttention      :1;
    U8  HotPlugPowerFault     :1;
    U8  HotPlugMrlSensor      :1;
    U8  HotPlugChangeDetect   :1;
    U8  HotPlugCmdCompleted   :1;
    U32 Reserved              :32;      // Reserved space for future
} PLX_INTERRUPT6;

// DMA Channel Properties Structure
typedef struct _PLX_DMA_PROP6
{
    U8 ReadyInput           :1;
    U8 Burst                :1;
    U8 BurstInfinite        :1;
    U8 WriteInvalidMode     :1;
    U8 LocalAddrConst       :1;
    U8 DualAddressCycles    :1;
    U8 ValidMode            :1;
    U8 TransferCountClear   :1;
    U8 ValidStopControl     :1;
    U8 DemandMode           :1;
    U8 EOTPin               :1;
    U8 EOTEndLink           :1;
    U8 StopTransferMode     :1;
    U8 LocalBusWidth        :2;
    U8 WaitStates           :4;
} PLX_DMA_PROP6;


// DMA Transfer Parameters
typedef struct _PLX_DMA_PARAMS6
{
    union
    {
        U64 UserVa;                // User space virtual address
        U32 PciAddrLow;            // Lower 32-bits of PCI address
    } u;
    U32     PciAddrHigh;           // Upper 32-bits of PCI address
    U32     LocalAddr;             // Local bus address
    U32     ByteCount;             // Number of bytes to transfer
    U32     TerminalCountIntr :1;
    U32     LocalToPciDma     :1;
} PLX_DMA_PARAMS6;


//==============================================================================================

// mrg -- from PlxLegcy.h in 5.1

/***********************************************
 * For portability to previous SDK versions.
 **********************************************/

// Type names that have changed
typedef PLX_PHYSICAL_MEM            PCI_MEMORY;
typedef PLX_ACCESS_TYPE             ACCESS_TYPE;
//typedef PLX_EEPROM_TYPE             EEPROM_TYPE;
typedef PLX_DMA_COMMAND             DMA_COMMAND;
typedef PLX_DMA_PARAMS6             DMA_TRANSFER_ELEMENT;


// Local Space Types
typedef enum _IOP_SPACE
{
    IopSpace0,
    IopSpace1,
    IopSpace2,
    IopSpace3
} IOP_SPACE;

// DMA Channel Definitions
typedef enum _DMA_CHANNEL
{
    IopChannel0,
    IopChannel1,
    IopChannel2,
    PrimaryPciChannel0,
    PrimaryPciChannel1,
    PrimaryPciChannel2,
    PrimaryPciChannel3
} DMA_CHANNEL;

// DMA Channel Priority Definitions
typedef enum _DMA_CHANNEL_PRIORITY
{
    Channel0Highest,
    Channel1Highest,
    Channel2Highest,
    Channel3Highest,
    Rotational
} DMA_CHANNEL_PRIORITY;

// Device Location Structure
typedef struct _DEVICE_LOCATION
{
    U8  BusNumber;
    U8  SlotNumber;
    U16 DeviceId;
    U16 VendorId;
    U8  SerialNumber[20];
} DEVICE_LOCATION;

// PLX Interrupt Structure
typedef struct _PLX_INTR
{
    unsigned int InboundPost      :1;
    unsigned int OutboundPost     :1;
    unsigned int OutboundOverflow :1;
    unsigned int OutboundOption   :1;
    unsigned int IopDmaChannel0   :1;
    unsigned int PciDmaChannel0   :1;
    unsigned int IopDmaChannel1   :1;
    unsigned int PciDmaChannel1   :1;
    unsigned int IopDmaChannel2   :1;
    unsigned int PciDmaChannel2   :1;
    unsigned int Mailbox0         :1;
    unsigned int Mailbox1         :1;
    unsigned int Mailbox2         :1;
    unsigned int Mailbox3         :1;
    unsigned int Mailbox4         :1;
    unsigned int Mailbox5         :1;
    unsigned int Mailbox6         :1;
    unsigned int Mailbox7         :1;
    unsigned int IopDoorbell      :1;
    unsigned int PciDoorbell      :1;
    unsigned int BIST             :1;
    unsigned int PowerManagement  :1;
    unsigned int PciMainInt       :1;
    unsigned int IopToPciInt      :1;
    unsigned int IopMainInt       :1;
    unsigned int PciAbort         :1;
    unsigned int PciReset         :1;
    unsigned int PciPME           :1;
    unsigned int Enum             :1;
    unsigned int AbortLSERR       :1;
    unsigned int ParityLSERR      :1;
    unsigned int RetryAbort       :1;
    unsigned int LocalParityLSERR :1;
    unsigned int PciSERR          :1;
    unsigned int IopToPciInt_2    :1;
    unsigned int Message          :1;
    unsigned int SwInterrupt      :1;
    unsigned int SecResetDeassert :1;
    unsigned int SecPmeDeassert   :1;
    unsigned int GPIO14           :1;
    unsigned int GPIO4            :1;
} PLX_INTR;

// DMA Channel Descriptor Structure
typedef struct _DMA_CHANNEL_DESC
{
    unsigned int EnableReadyInput         :1;
    unsigned int EnableBTERMInput         :1;
    unsigned int EnableIopBurst           :1;
    unsigned int EnableWriteInvalidMode   :1;
    unsigned int EnableDmaEOTPin          :1;
    unsigned int DmaStopTransferMode      :1;
    unsigned int HoldIopAddrConst         :1;
    unsigned int DemandMode               :1;
    unsigned int EnableTransferCountClear :1;
    unsigned int WaitStates               :4;
    unsigned int IopBusWidth              :2;
    unsigned int EOTEndLink               :1;
    unsigned int ValidStopControl         :1;
    unsigned int ValidModeEnable          :1;
    unsigned int EnableDualAddressCycles  :1;
    unsigned int TholdForIopWrites        :4;
    unsigned int TholdForIopReads         :4;
    unsigned int TholdForPciWrites        :4;
    unsigned int TholdForPciReads         :4;
    DMA_CHANNEL_PRIORITY DmaChannelPriority;
} DMA_CHANNEL_DESC;

//==============================================================================================
//==============================================================================================

typedef union _DMA_TRANSFER_ELEMENT3
{
    struct
    {
    #if defined(PCI_CODE)
        union
        {
            U32 PciAddrLow;
            U32 UserAddr;
        };
    #else
        U32 PciAddrLow;
    #endif
        U32 PciAddrHigh;
        U32 LocalAddr;
        U32 TransferCount;
        U32 NextSglPtr;
        U8  PciSglLoc;
        U8  LastSglElement;
        U8  TerminalCountIntr;
        U8  LocalToPciDma;
    } PlxDma;

    struct
    {
    #if defined(PCI_CODE)
        union
        {
            U32 LowPciAddr;
            U32 UserAddr;
        };
    #else
        U32 LowPciAddr;
    #endif
        U32 IopAddr;
        U32 TransferCount;
    #if defined(LITTLE_ENDIAN)
        U32 PciSglLoc         :1;
        U32 LastSglElement    :1;
        U32 TerminalCountIntr :1;
        U32 IopToPciDma       :1;
        U32 NextSglPtr        :28;
    #elif defined(BIG_ENDIAN)
        U32 NextSglPtr        :28;
        U32 IopToPciDma       :1;
        U32 TerminalCountIntr :1;
        U32 LastSglElement    :1;
        U32 PciSglLoc         :1;
    #endif
    } Pci9080Dma;

    struct
    {
    #if defined(PCI_CODE)
        union
        {
            U32 LowPciAddr;
            U32 UserAddr;
        };
    #else
        U32 LowPciAddr;
    #endif
        U32 IopAddr;
        U32 TransferCount;
    #if defined(LITTLE_ENDIAN)
        U32 PciSglLoc         :1;
        U32 LastSglElement    :1;
        U32 TerminalCountIntr :1;
        U32 IopToPciDma       :1;
        U32 NextSglPtr        :28;
    #elif defined(BIG_ENDIAN)
        U32 NextSglPtr        :28;
        U32 IopToPciDma       :1;
        U32 TerminalCountIntr :1;
        U32 LastSglElement    :1;
        U32 PciSglLoc         :1;
    #endif
        U32 HighPciAddr;
    } Pci9054Dma;

    struct
    {
    #if defined(PCI_CODE)
        union
        {
            U32 LowPciAddr;
            U32 UserAddr;
        };
    #else
        U32 LowPciAddr;
    #endif
        U32 IopAddr;
        U32 TransferCount;
    #if defined(LITTLE_ENDIAN)
        U32 PciSglLoc         :1;
        U32 LastSglElement    :1;
        U32 TerminalCountIntr :1;
        U32 IopToPciDma       :1;
        U32 NextSglPtr        :28;
    #elif defined(BIG_ENDIAN)
        U32 NextSglPtr        :28;
        U32 IopToPciDma       :1;
        U32 TerminalCountIntr :1;
        U32 LastSglElement    :1;
        U32 PciSglLoc         :1;
    #endif
        U32 HighPciAddr;
    } Pci9056Dma;

    struct
    {
    #if defined(PCI_CODE)
        union
        {
            U32 LowPciAddr;
            U32 UserAddr;
        };
    #else
        U32 LowPciAddr;
    #endif
        U32 IopAddr;
        U32 TransferCount;
    #if defined(LITTLE_ENDIAN)
        U32 PciSglLoc         :1;
        U32 LastSglElement    :1;
        U32 TerminalCountIntr :1;
        U32 IopToPciDma       :1;
        U32 NextSglPtr        :28;
    #elif defined(BIG_ENDIAN)
        U32 NextSglPtr        :28;
        U32 IopToPciDma       :1;
        U32 TerminalCountIntr :1;
        U32 LastSglElement    :1;
        U32 PciSglLoc         :1;
    #endif
        U32 HighPciAddr;
    } Pci9656Dma;

    struct
    {
        U32 TransferCount;
    #if defined(PCI_CODE)
        union
        {
            U32 LowPciAddr;
            U32 UserAddr;
            U32 SourceAddr;
        };
    #else
        union loc1
        {
            U32 LowPciAddr; // DMA channel 0, 1
            U32 SourceAddr; // DMA channel 2
        } Loc1;
    #endif

    #if defined(PCI_CODE)
        union
        {
            U32 IopAddr;    // DMA channel 0, 1
            U32 DestAddr;   // DMA channel 2
        };
    #else
        union loc2
        {
            U32 IopAddr;    // DMA channel 0, 1
            U32 DestAddr;   // DMA channel 2
        } Loc2;
    #endif

    #if defined(LITTLE_ENDIAN)
        U32 PciSglLoc         :1;
        U32 LastSglElement    :1;
        U32 TerminalCountIntr :1;
        U32 IopToPciDma       :1;
        U32 NextSglPtr        :28;
    #elif defined(BIG_ENDIAN)
        U32 NextSglPtr        :28;
        U32 IopToPciDma       :1;
        U32 TerminalCountIntr :1;
        U32 LastSglElement    :1;
        U32 PciSglLoc         :1;
    #endif
        U32 HighPciAddr;
    } Iop480Dma;

    /*
     * The DMA Transfer Element must always start on a 16 byte
     * boundary so the following reserve field ensures this. Total size = 0x30.
     */
    U32 Reserved[12];
} DMA_TRANSFER_ELEMENT3;

// DMA Channel Descriptor Structure for Plx v. 3
typedef struct _DMA_CHANNEL_DESC3
{
    U32 EnableReadyInput         :1;
    U32 EnableBTERMInput         :1;
    U32 EnableIopBurst           :1;
    U32 EnableWriteInvalidMode   :1;
    U32 EnableDmaEOTPin          :1;
    U32 DmaStopTransferMode      :1;
    U32 HoldIopAddrConst         :1;
    U32 HoldIopSourceAddrConst   :1;
    U32 HoldIopDestAddrConst     :1;
    U32 DemandMode               :1;
    U32 SrcDemandMode            :1;
    U32 DestDemandMode           :1;
    U32 EnableTransferCountClear :1;
    U32 WaitStates               :4;
    U32 IopBusWidth              :2;
    U32 EOTEndLink               :1;
    U32 ValidStopControl         :1;
    U32 ValidModeEnable          :1;
    U32 EnableDualAddressCycles  :1;
    U32 Reserved1                :9;
    U32 TholdForIopWrites        :4;
    U32 TholdForIopReads         :4;
    U32 TholdForPciWrites        :4;
    U32 TholdForPciReads         :4;
    U32 EnableFlybyMode          :1;
    U32 FlybyDirection           :1;
    U32 EnableDoneInt            :1;
    U32 Reserved2                :13;
    DMA_CHANNEL_PRIORITY DmaChannelPriority;
} DMA_CHANNEL_DESC3;

typedef struct _DEVICE_LOCATION3
{
    U32 DeviceId;
    U32 VendorId;
    U32  BusNumber;
    U32  SlotNumber;
    U8  SerialNumber[16];
} DEVICE_LOCATION3;

typedef struct _PLX_INTR3
{
    U32 InboundPost      :1;
    U32 OutboundPost     :1;
    U32 OutboundOverflow :1;
    U32 OutboundOption   :1;
    U32 IopDmaChannel0   :1;
    U32 PciDmaChannel0   :1;
    U32 IopDmaChannel1   :1;
    U32 PciDmaChannel1   :1;
    U32 IopDmaChannel2   :1;
    U32 PciDmaChannel2   :1;
    U32 Mailbox0         :1;
    U32 Mailbox1         :1;
    U32 Mailbox2         :1;
    U32 Mailbox3         :1;
    U32 Mailbox4         :1;
    U32 Mailbox5         :1;
    U32 Mailbox6         :1;
    U32 Mailbox7         :1;
    U32 IopDoorbell      :1;
    U32 PciDoorbell      :1;
    U32 SerialPort1      :1;
    U32 SerialPort2      :1;
    U32 BIST             :1;
    U32 PowerManagement  :1;
    U32 PciMainInt       :1;
    U32 IopToPciInt      :1;
    U32 IopMainInt       :1;
    U32 PciAbort         :1;
    U32 PciReset         :1;
    U32 PciPME           :1;
    U32 Enum             :1;
    U32 PciENUM          :1;
    U32 IopBusTimeout    :1;
    U32 AbortLSERR       :1;
    U32 ParityLSERR      :1;
    U32 RetryAbort       :1;
    U32 LocalParityLSERR :1;
    U32 PciSERR          :1;
    U32 IopRefresh       :1;
    U32 PciINTApin       :1;
    U32 IopINTIpin       :1;
    U32 TargetAbort      :1;
    U32 Ch1Abort         :1;
    U32 Ch0Abort         :1;
    U32 DMAbort          :1;
    U32 IopToPciInt_2    :1;
    U32 SwInterrupt      :1;
    U32 DmaChannel3      :1;
    U32 Reserved         :16;
} PLX_INTR3;

/* API Return Code Values for Plx Legacy versions 4.4 and 3.4 */
typedef enum _RETURN_CODE_LEGACY
{
    ApiSuccessLegacy = API_RETURN_CODE_STARTS,
    ApiFailedLegacy,
    ApiAccessDeniedLegacy,
    ApiDmaChannelUnavailableLegacy,
    ApiDmaChannelInvalidLegacy,
    ApiDmaChannelTypeErrorLegacy,
    ApiDmaInProgressLegacy,
    ApiDmaDoneLegacy,
    ApiDmaPausedLegacy,
    ApiDmaNotPausedLegacy,
    ApiDmaCommandInvalidLegacy,
    ApiDmaManReadyLegacy,
    ApiDmaManNotReadyLegacy,
    ApiDmaInvalidChannelPriorityLegacy,
    ApiDmaManCorruptedLegacy,
    ApiDmaInvalidElementIndexLegacy,
    ApiDmaNoMoreElementsLegacy,
    ApiDmaSglInvalidLegacy,
    ApiDmaSglQueueFullLegacy,
    ApiNullParamLegacy,
    ApiInvalidBusIndexLegacy,
    ApiUnsupportedFunctionLegacy,
    ApiInvalidPciSpaceLegacy,
    ApiInvalidIopSpaceLegacy,
    ApiInvalidSizeLegacy,
    ApiInvalidAddressLegacy,
    ApiInvalidAccessTypeLegacy,
    ApiInvalidIndexLegacy,
    ApiMuNotReadyLegacy,
    ApiMuFifoEmptyLegacy,
    ApiMuFifoFullLegacy,
    ApiInvalidRegisterLegacy,
    ApiDoorbellClearFailedLegacy,
    ApiInvalidUserPinLegacy,
    ApiInvalidUserStateLegacy,
    ApiEepromNotPresentLegacy,
    ApiEepromTypeNotSupportedLegacy,
    ApiEepromBlankLegacy,
    ApiConfigAccessFailedLegacy,
    ApiInvalidDeviceInfoLegacy,
    ApiNoActiveDriverLegacy,
    ApiInsufficientResourcesLegacy,
    ApiObjectAlreadyAllocatedLegacy,
    ApiAlreadyInitializedLegacy,
    ApiNotInitializedLegacy,
    ApiBadConfigRegEndianModeLegacy,
    ApiInvalidPowerStateLegacy,
    ApiPowerDownLegacy,
    ApiFlybyNotSupportedLegacy,
    ApiNotSupportThisChannelLegacy,
    ApiNoActionLegacy,
    ApiHSNotSupportedLegacy,
    ApiVPDNotSupportedLegacy,
    ApiVpdNotEnabledLegacy,
    ApiNoMoreCapLegacy,
    ApiInvalidOffsetLegacy,
    ApiBadPinDirectionLegacy,
    ApiPciTimeoutLegacy,
    ApiDmaChannelClosedLegacy,
    ApiDmaChannelErrorLegacy,
    ApiInvalidHandleLegacy,
    ApiBufferNotReadyLegacy,
    ApiInvalidDataLegacy,
    ApiDoNothingLegacy,
    ApiDmaSglBuildFailedLegacy,
    ApiPMNotSupportedLegacy,
    ApiInvalidDriverVersionLegacy,
    ApiWaitTimeoutLegacy,   // only defined for Plx 4.4
    ApiWaitCanceledLegacy,  // only defined for Plx 4.4
    ApiLastErrorLegacy      // Do not add API errors below this line
} RETURN_CODE_LEGACY;

#ifdef __cplusplus
}
#endif

#endif
