//////////////////////////////////////////////////////////////////////////////
//
// GscApiRAsync.c
//
//  This module contains all of the routines that handle the RASYNC modes of
// operation. These modes are only valid on the -RASYNC boards.
//
#include "GscApiLocal.h"

// Forward declarations of -SYNC mode helper functions defined in this file
// These functions are not part of the GscApi. They are used only in this .c file.

//--------------------------------------------------- GscSio4RAsyncGetDefaults
//
//  This routine fills in the provided Configuration
// structure with their default values.
//
int GscSio4RAsyncGetDefaults( PGSC_RASYNC_CONFIG config)
{
  int status = 0x00;

  // Setup the Channel defaults
  config->bitRate = 1000000;
  config->encoding = GSC_ENCODING_NRZ;
  config->protocol = GSC_PROTOCOL_RS422_RS485;
  config->termination = GSC_TERMINATION_ENABLED;
  config->parity = GSC_PARITY_NONE;
  config->stopBits = GSC_STOP_BITS_1;
  config->activeCircuit = GSC_CIRCUIT_AUTO;

  // Setup the Transmitter defaults
  config->txStatus = GSC_ENABLED;
  config->txGapLength = 0;

  // Setup the Receiver defaults
  config->rxStatus = GSC_ENABLED;
  config->rxSaveDataOnError = GSC_ENABLED;

  // Setup the Pin Configuration Variables
  config->interfaceMode = GSC_PIN_DTE;
  config->loopbackMode = GSC_LOOP_NONE;

  // Setup the Misc defaults
  config->packetFraming = GSC_DISABLED;

  return status;
}


//----------------------------------------------------- GscSio4RAsyncGetConfig
//
//  This routine returns the current configuration
// structure for a single channel.
//
int GscSio4RAsyncGetConfig(int boardNumber, int channel, PGSC_RASYNC_CONFIG config)
{
	int status = 0;
	int value = 0;
	int bn = boardNumber - 1;
	int ch = channel - 1;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected
	if (boardInformation[bn].boardType != GSC_BOARD_TYPE_RASYNC)
	  return GscApiInvalidBoardType;

	// If the channel is not configured, get the defaults
	if (boardInformation[ bn].channelMode[ ch] == GSC_MODE_UNINITIALIZED)
	{
		// Return the default values
		return GscSio4RAsyncGetDefaults( config);
	}

	// Get the tx gap length
	status = fpgaRead( boardNumber, TX_COUNT_BASE_REG + (ch)*4, &value);
	if (status) return status;
	config->txGapLength = value & 0x0000ffff;

	// Determine the encoding and bit rate
	config->encoding = boardInformation[ bn ].channelEncoding[ ch ];
	config->bitRate = boardInformation[ bn ].channelBitRate[ ch ];

	// Determine the protocol and termination settings
	status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol, &config->termination);
	if (status) return( status );

	// Determine the various pin modes
	config->interfaceMode = boardInformation[ bn].channelDteDce[ ch];

	// Determine the loopback mode
	config->loopbackMode = boardInformation[ bn].channelLoopback[ ch];

	// Read the Pin Source register
	status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+4*ch, &value);
	if (status) return status;

	// Get the "Active Circuit" setting
	if ((value & 0x00001000)==0) config->activeCircuit = GSC_CIRCUIT_AUTO;
	else
	{
		switch (value & 0x00000c00)
		{
		case 0x00000000: config->activeCircuit = GSC_CIRCUIT_A; break;
		case 0x00000400: config->activeCircuit = GSC_CIRCUIT_B; break;
		case 0x00000800: config->activeCircuit = GSC_CIRCUIT_C; break;
		case 0x00000c00: config->activeCircuit = GSC_CIRCUIT_D; break;
		}
	}

	// Read the Control/Status register
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*ch, &value);
	if (status) return status;

	// Determine the "Save Data On Rx Error" setting
	if (value & 0x40000000) config->rxSaveDataOnError = GSC_ENABLED;
	else config->rxSaveDataOnError = GSC_DISABLED;

	// Determine the parity
	if ((value & 0x08000000)==0x00) config->parity = GSC_PARITY_NONE;
	else
	{
		switch (value & 0x3000000)
		{
		case 0x00000000: config->parity = GSC_PARITY_EVEN; break;
		case 0x10000000: config->parity = GSC_PARITY_ODD; break;
		case 0x20000000: config->parity = GSC_PARITY_SPACE; break;
		case 0x30000000: config->parity = GSC_PARITY_MARK; break;
		}
	}

	// Determine the stop bits
	if (value & 0x04000000) config->stopBits = GSC_STOP_BITS_2;
	else  config->stopBits = GSC_STOP_BITS_1;

	// Determine the Rx mode
	if (value & 0x02000000)	config->rxStatus = GSC_ENABLED;
	else config->rxStatus = GSC_DISABLED;

	// Determine the Tx mode
	if (value & 0x01000000) config->txStatus = GSC_ENABLED;
	else config->rxStatus = GSC_DISABLED;

	// Get the packet framing mode
	config->packetFraming = boardInformation[ boardNumber-1].channelPacketFraming[ channel-1];
	return status;
}


//----------------------------------------------------- GscSio4RAsyncSetConfig
//
//  This routine sets the current configuration
// structure for a single channel.
//
int GscSio4RAsyncSetConfig(int boardNumber, int channel, GSC_RASYNC_CONFIG config)
{
	int status = 0x00;
	int value;
	int bn = boardNumber - 1;
	int ch = channel - 1;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected
	if (boardInformation[bn].boardType != GSC_BOARD_TYPE_RASYNC)
	  return GscApiInvalidBoardType;

	// Save some global info
	boardInformation[ bn].channelMode[ ch] = GSC_MODE_RASYNC;
	boardInformation[ bn].bytesPerMs[ ch] = (double)config.bitRate/1000/10;

	// Set the Tx Gap Length
	status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG + ch*4, config.txGapLength);
	if (status) return status;

	// Setup the encoding and bitRate
	status = fpgaSetEncoding( boardNumber, channel, config.encoding, config.bitRate);
	if (status) return status;

	// Setup the protocol and termination
	status = quickSetProtocol( boardNumber, channel, config.protocol, config.termination);
	if (status) return status;

	// Setup the Pin Modes
	status = quickSetDteDce(boardNumber, channel, config.interfaceMode);
	if (status) return status;

	// Setup the loopback mode
	status = quickSetLoopBack( boardNumber, channel, config.loopbackMode);
	if (status) return status;

	// Read the Pin Source register
	status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+4*ch, &value);
	if (status) return status;

	// Process the "Active Circuit Setting
	switch (config.activeCircuit)
	{
	case GSC_CIRCUIT_AUTO: value &= ~0x00001c00; break;
	case GSC_CIRCUIT_A:	   value &= ~0x00001c00; value |=  0x00001000; break;
	case GSC_CIRCUIT_B:	   value &= ~0x00001c00; value |=  0x00001400; break;
	case GSC_CIRCUIT_C:	   value &= ~0x00001c00; value |=  0x00001800; break;
	case GSC_CIRCUIT_D:	   value &= ~0x00001c00; value |=  0x00001c00; break;
	default: return GscApiInvalidParameter;
	}

	// Read the current Control/Status register
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(ch), &value);
	if (status) return status;

	// Process the desired "Save Data On Rx Error" setting
	switch (config.rxSaveDataOnError)
	{
	case GSC_ENABLED:  value |=  0x40000000; break;
	case GSC_DISABLED: value &= ~0x40000000; break;
	default:			return GscApiInvalidParameter;
	}

	// Process the desired parity
	switch (config.parity)
	{
	case GSC_PARITY_NONE:     value &= ~0x38000000; break;
	case GSC_PARITY_EVEN:	  value &= ~0x38000000; value |=  0x08000000; break;
	case GSC_PARITY_ODD:	  value &= ~0x38000000; value |=  0x18000000; break;
	case GSC_PARITY_SPACE:	  value &= ~0x38000000; value |=  0x28000000; break;
	case GSC_PARITY_MARK:	  value &= ~0x38000000; value |=  0x38000000; break;
	default: return GscApiInvalidParameter;
	}

	switch (config.stopBits)
	{
	case GSC_STOP_BITS_1: value &= ~0x04000000; break;
	case GSC_STOP_BITS_2: value |=  0x04000000; break;
	default:			return GscApiInvalidParameter;
	}

	// Process the desired Rx mode
	switch (config.rxStatus)
	{
	case GSC_ENABLED:	value |=  0x02000000; break;
	case GSC_DISABLED:	value &= ~0x02000000; break;
	default:			return GscApiInvalidParameter;
	}

	// Process the desired Tx mode
	switch (config.txStatus)
	{
	case GSC_ENABLED:	value |=  0x01000000; break;
	case GSC_DISABLED:	value &= ~0x01000000; break;
	default:			return GscApiInvalidParameter;
	}

	// Write the new value back out
	status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(ch), value);
	if (status) return status;

	// Update the board info structures copy of the framing mode
	boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = config.packetFraming;

	return status;
}

//----------------------------------------------------- gscSio4RAsyncGetOption
//
//  This routine gets the value of the identified option.
//
int gscSio4RAsyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
	int status   = 0;
	int oldValue = 0;

	switch ( optionName )
	{
	case GSC_SIO_GAPSIZE:
		{
		int *gapSize = &optionValue[0];

		// Return the data size
		*gapSize = boardInformation[ boardNumber-1].channelGapSize[ channel-1];
		}

		break;


	case GSC_SIO_RECEIVER:
		// Read the current Control/Status register
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldValue);
		if (status) return status;

		// Get the Rx mode
		if (oldValue & 0x02000000)
			optionValue[0] = GSC_ENABLED;
		else
			optionValue[0] = GSC_DISABLED;
		break;

	case GSC_SIO_TRANSMITTER:
		// Read the current Control/Status register
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldValue);
		if (status) return status;

		// Get the Tx mode
		if (oldValue & 0x01000000)
			optionValue[0] = GSC_ENABLED;
		else
			optionValue[0] = GSC_DISABLED;

	default:
		return GscApiInvalidParameter;
	}

	return status;
}

//----------------------------------------------------- gscSio4RAsyncSetOption
//
//  This routine sets the identified option to the supplied value.
//
int gscSio4RAsyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
	int status = 0;

	switch ( optionName )
	{
	case GSC_SIO_GAPSIZE:
		{
		int gapSize = optionValue;

		// Range check the data size
		if((gapSize < 0) || (gapSize > 0xffff)) return GscApiInvalidParameter;

		// Save the new data size
		boardInformation[ boardNumber-1].channelGapSize[ channel-1] = gapSize;

		// Write it  out
		status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), gapSize);
		}

		break;

	case GSC_SIO_ENCODING:
		{
		int encoding = optionValue;

		// Convert board and channel into indexes
		int bn = boardNumber - 1;
		int ch = channel - 1;

		return fpgaSetEncoding( boardNumber,
								channel,
								encoding,
								boardInformation[ bn].channelBitRate[ ch]);
		}

		break;

	case GSC_SIO_RECEIVER:
		{
		int rxMode = optionValue;
		int oldData = 0;

		// Read the current Control/Status register
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldData);
		if (status) return status;

		// Process the desired Rx mode
		switch (rxMode)
		{
		case GSC_ENABLED:	oldData |=  0x02000000; break;
		case GSC_DISABLED:	oldData &= ~0x02000000; break;
		default:			return GscApiInvalidParameter;
		}

		// Write the new value back out
		status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), oldData);
		if (status) return status;
		}
		break;

	case GSC_SIO_TRANSMITTER:
		{
		int txMode  = optionValue;
		int oldData = 0;

		// Read the current Control/Status register
		status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldData);
		if (status) return status;

		// Process the desired Tx mode
		switch (txMode)
		{
		case GSC_ENABLED:	oldData |=  0x01000000; break;
		case GSC_DISABLED:	oldData &= ~0x01000000; break;
		default:			return GscApiInvalidParameter;
		}

		// Write the new value back out
		status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), oldData);
		if (status) return status;
		}
		break;

	default:
		return GscApiInvalidParameter;
	}

	return status;
}
