//////////////////////////////////////////////////////////////////////////////
//
// GscApiSync.c
//
//  This module contains all of the routines that handle the SYNC and SYNC_ENV
// modes of operation. These modes are only valid on teh -SYNC boards.
//
#include "GscApiLocal.h"

// Forward declarations of -SYNC mode helper functions defined in this file
// These functions are not part of the GscApi. They are used only in this .c file.
int gscSio4ChannelGetClockEdge( int boardNumber, int channel, int *txClockEdge, int *rxClockEdge );
int gscSio4ChannelSetClockEdge( int boardNumber, int channel, int txClockEdge, int rxClockEdge );
int gscSio4ChannelGetEnvPolarity( int boardNumber, int channel, int *txEnvPolarity, int *rxEnvPolarity );
int gscSio4ChannelSetEnvPolarity( int boardNumber, int channel, int txEnvPolarity, int rxEnvPolarity );
int gscSio4ChannelGetTxClockIdle( int boardNumber, int channel, int *txClockIdleCondition );
int gscSio4ChannelSetTxClockIdle( int boardNumber, int channel, int txClockIdleCondition );
int gscSio4ChannelGetTxDataIdle( int boardNumber, int channel, int *txDataIdleCondition );
int gscSio4ChannelSetTxDataIdle( int boardNumber, int channel, int txDataIdleCondition );

//---------------------------------------------- GscSio4ChannelGetSyncDefaults
//
//  This routine fills in the provided SYNC Configuration
// structure with their default values.
//
int GscSio4SyncGetDefaults( PGSC_SYNC_CONFIG config)
{
  int status = 0x00;

  // Setup the Channel defaults
  config->bitRate = 1000000;
  config->encoding = GSC_ENCODING_NRZ;
  config->protocol = GSC_PROTOCOL_RS422_RS485;
  config->termination = GSC_TERMINATION_ENABLED;

  // Setup the Transmitter defaults
  config->txStatus = GSC_ENABLED;
  config->txCharacterLength = 8;
  config->txGapLength = 0;
  config->txClockSource = GSC_CLOCK_EXTERNAL;
  config->txClockEdge = GSC_RISING_EDGE;
  config->txEnvPolarity = GSC_HIGH_TRUE;
  config->txIdleCondition = GSC_IDLE_ALL_0;
  config->txClockIdleCondition = GSC_IDLE_ACTIVE;
  config->txMsbLsb = GSC_MSB_FIRST;
// Stop Tx on fifo empty

  // Setup the Receiver defaults
  config->rxStatus = GSC_ENABLED;
  config->rxClockSource = GSC_CLOCK_EXTERNAL;
  config->rxClockEdge = GSC_FALLING_EDGE;
  config->rxEnvPolarity = GSC_HIGH_TRUE;
  config->rxMsbLsb = GSC_MSB_FIRST;

  // Setup the Pin Configuration Variables
  config->interfaceMode = GSC_PIN_DTE;
  config->txDataPinMode = GSC_PIN_AUTO;
  config->rxDataPinMode = GSC_PIN_AUTO;
  config->txClockPinMode = GSC_PIN_AUTO;
  config->rxClockPinMode = GSC_PIN_AUTO;
  config->txEnvPinMode = GSC_PIN_GPIO;
  config->rxEnvPinMode = GSC_PIN_GPIO;
  config->loopbackMode = GSC_LOOP_NONE;

  // Setup the Misc defaults
  config->packetFraming = GSC_DISABLED;

  return status;
}


//------------------------------------------------ GscSio4ChannelSyncGetConfig
//
//  This routine returns the current configuration
// structure for a single channel that is set to SYNC
// mode.
//
int GscSio4SyncGetConfig(int boardNumber, int channel, PGSC_SYNC_CONFIG config)
{
	int status = 0;
	int value = 0;

	// Make sure the board and channel number are valid
	status = validateBoardAndChannelNumber( boardNumber, channel);
	if (status) return status;

	// Make sure the board type is as expected (include BXR board, which can also do SYNC mode)
	if ((boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_SYNC) &&
      (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_QPSK) &&
      (!gscSio4ChannelIsBxrSync(boardNumber, channel)))
	  return GscApiInvalidBoardType;

	// If the channel is not configured, get the defaults
	if (boardInformation[ boardNumber-1].channelMode[ channel-1] == GSC_MODE_UNINITIALIZED)
	{
		// Load the default values
		status = GscSio4SyncGetDefaults( config);
		return status;
	}

	// Get the data character lengths
	status = fpgaRead( boardNumber, TX_COUNT_BASE_REG + (channel-1)*4, &value);
	if (status) return status;

	config->txGapLength = (value & 0xffff0000) >> 16;
	config->txCharacterLength = value & 0xffff;

	// Determine the encoding and bit rate
	config->encoding = boardInformation[ boardNumber-1 ].channelEncoding[ channel-1 ];
	config->bitRate = boardInformation[ boardNumber-1 ].channelBitRate[ channel-1 ];

	// Determine the protocol and termination settings
	status = gscSio4ChannelGetProtocol( boardNumber, channel, &config->protocol,
		&config->termination);
	if (status) return( status );

	// Get the Tx and Rx byte order (MSB or LSB)
	status = gscSio4ChannelGetMsbLsbOrder( boardNumber, channel, &config->txMsbLsb,
		&config->rxMsbLsb );
	if (status) return status;

	// Get the Tx/Rx Clock Edge setting
	status = gscSio4ChannelGetClockEdge( boardNumber, channel, &config->txClockEdge,
		&config->rxClockEdge );
	if (status) return status;

	// Get the Tx/Rx Envelope polarity
	status = gscSio4ChannelGetEnvPolarity( boardNumber, channel, &config->txEnvPolarity,
		&config->rxEnvPolarity );
	if (status) return status;

	// Get the Tx data idle condition
	status = gscSio4ChannelGetTxDataIdle( boardNumber, channel, &config->txIdleCondition );
	if (status) return status;

	// Get the Tx clock idle condition
	status = gscSio4ChannelGetTxClockIdle( boardNumber, channel, &config->txClockIdleCondition );
	if (status) return status;

	// Determine the various pin modes
	config->interfaceMode = boardInformation[ boardNumber-1].channelDteDce[ channel-1];
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_DATA,
		&config->txDataPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_DATA,
		&config->rxDataPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK,
		&config->txClockPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK,
		&config->rxClockPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_TX_ENV,
		&config->txEnvPinMode);
	if (status) return( status );
	status = GscSio4ChannelGetPinMode( boardNumber, channel, GSC_PIN_RX_ENV,
		&config->rxEnvPinMode);
	if (status) return( status );

	// Determine the loopback mode
	config->loopbackMode = boardInformation[ boardNumber-1].channelLoopback[ channel-1];

	// Determine the clock sources
	status = gscsio4ChannelGetClockSource( boardNumber, channel, &config->txClockSource,
		&config->rxClockSource);
	if (status) return status;

	// Determine whether the receiver is enabled
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
	if (status) return status;

	// Determine the Rx mode
	if (value & 0x01000000)
		config->rxStatus = GSC_ENABLED;
	else
		config->rxStatus = GSC_DISABLED;

	// Determine whether the transmitter is enabled
	status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
	if (status) return status;

	// Determine the Tx mode
	if  (value & 0x0200000)
		config->txStatus = GSC_ENABLED;
	else
		config->rxStatus = GSC_DISABLED;

	// Get the packet framing mode
	config->packetFraming = boardInformation[ boardNumber-1].channelPacketFraming[ channel-1];
	return status;
}


//------------------------------------------------ GscSio4ChannelSyncSetConfig
//
//  This routine sets the current configuration
// structure for a single channel that is set to SYNC
// mode.
//
int GscSio4SyncSetConfig(int boardNumber, int channel, GSC_SYNC_CONFIG config)
{
  int status = 0x00;
  int value;

  // Make sure the board and channel number are valid
  status = validateBoardAndChannelNumber( boardNumber, channel);
  if (status) return status;

  // Make sure the board type is as expected
	if ((boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_SYNC) &&
      (boardInformation[boardNumber-1].boardType != GSC_BOARD_TYPE_QPSK) &&
      (!((boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_NORMAL) &&
      (!strcmp(boardInformation[boardNumber-1].subId, "-SIO4BXR")))))
      return GscApiInvalidBoardType;

  // Save some global info
  boardInformation[ boardNumber-1].channelMode[ channel-1] = GSC_MODE_SYNC;
  boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)config.bitRate/1000/8;

  // If the board is a BXR, change the channel to SYNC mode
  if (boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_QPSK)
  {
    int chMode = 0;

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    if (status) return status;

    chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
    chMode |= (0x0c << ((channel - 1) * 8)); // set channel type to sync

    status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
    if (status) return status;
  }
  else if (boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_NORMAL)
  {
    int chMode = 0;

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    if (status) return status;

    chMode = chMode & ~(0xff << ((channel - 1) * 8)); // clear channel type
    chMode |= (0x04 << ((channel - 1) * 8)); // set channel type to sync

    status = fpgaWrite(boardNumber, FW_TYPE_REG, chMode);
    if (status) return status;
  }

  // Set the Tx Data and Gap Lengths
  value = (config.txGapLength << 16) + config.txCharacterLength;
  status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG + (channel-1)*4, value);
  if (status) return status;

  // Setup the encoding and bitRate
  fpgaSetEncoding( boardNumber, channel, config.encoding, config.bitRate);
  if (status) return status;

  // Setup the protocol and termination
  status = quickSetProtocol( boardNumber, channel, config.protocol, config.termination);
  if (status) return status;

  // Setup Tx/Rx Msb/Lsb
  status = gscSio4ChannelSetMsbLsbOrder( boardNumber, channel, config.txMsbLsb, config.rxMsbLsb );
  if (status) return status;

  // Setup the Tx data idle condition
  status = gscSio4ChannelSetTxDataIdle( boardNumber, channel, config.txIdleCondition );
  if (status) return status;

  // Setup the Tx clock idle condition
  status = gscSio4ChannelSetTxClockIdle( boardNumber, channel, config.txClockIdleCondition );
  if (status) return status;

  // Setup the Pin Modes
  status = quickSetDteDce(boardNumber, channel, config.interfaceMode);

  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, config.txDataPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, config.rxDataPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, config.txClockPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, config.rxClockPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_ENV, config.txEnvPinMode);
  if (status) return status;
  status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_ENV, config.rxEnvPinMode);
  if (status) return status;

  // Setup the loopback mode
  status = quickSetLoopBack( boardNumber, channel, config.loopbackMode);
  if (status) return status;

  // Setup the clock sources
  status = gscSio4ChannelSetClockSource( boardNumber, channel, config.txClockSource, GSC_CLOCK_EXTERNAL);
  if (status) return status;

  // Setup Tx/Rx Clock Edge
  status = gscSio4ChannelSetClockEdge( boardNumber, channel, config.txClockEdge, config.rxClockEdge );
  if (status) return status;

  // Setup Tx/Rx Env Polarity
  status = gscSio4ChannelSetEnvPolarity( boardNumber, channel, config.txEnvPolarity, config.rxEnvPolarity );
  if (status) return status;

  // Enable/Disable the receiver and transmitter

  // Read the current Control/Status register
  status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &value);
  if (status) return status;
/*
  if (boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_NORMAL)
  {
    // Enable/Disable the receiver
	  status = zilogReceiverControl( boardNumber, channel, config.rxStatus);
	  if (status) return status;

	  // Enable/Disable the Transmitter (no auto enables)
	  status = zilogTransmitterControl( boardNumber, channel, config.txStatus);
	  if (status) return status;
  }
  else
  {*/
    // Process the desired Rx mode
    switch (config.rxStatus)
    {
    case GSC_ENABLED:	value |=  0x01000000; break;
    case GSC_DISABLED:	value &= ~0x01000000; break;
    default:			return GscApiInvalidParameter;
    }

    // Process the desired Tx mode
    switch (config.txStatus)
    {
    case GSC_ENABLED:	value |=  0x02000000; break;
    case GSC_DISABLED:	value &= ~0x02000000; break;
    default:			return GscApiInvalidParameter;
    }

    // Write the new value back out
    status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), value);
    if (status) return status;
  /*}*/

  // reset the FIFOs
  status = resetRxFifo( boardNumber, channel );
  if (status) return status;

  status = GscSio4ChannelResetTxFifo( boardNumber, channel );
  if (status) return( status );

  // Update the board info structures copy of the framing mode
  boardInformation[ boardNumber-1].channelPacketFraming[ channel-1] = config.packetFraming;

  return status;
}

//----------------------------------------------------- GscSio4SyncGetOption
//
//  This routine gets the value of the identified option.
//
int gscSio4SyncGetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue[] )
{
  int status   = 0;
  int oldValue = 0;
  int offset   = 0;

  switch ( optionName )
  {
  case GSC_SIO_DATASIZE: // Return the current data size for a channel
    {
      int *dataSize = &optionValue[0];
      // Get the current Tx Count Register
      status = fpgaRead( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), &oldValue);
      if (status) return status;

      // Adjust it to the new value
      *dataSize = oldValue & 0x0000ffff;
    }
    break;

  case GSC_SIO_GAPSIZE:
    {
      int *gapSize = &optionValue[0];

      // Return the data size
      *gapSize = boardInformation[ boardNumber-1].channelGapSize[ channel-1];
    }

    break;

  case GSC_SIO_MSBLSBORDER:
    {
      int *txOrder = &optionValue[0];
      int *rxOrder = &optionValue[1];

      // Get the current Channel Control/Status Register
      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &oldValue);
      if (status) return status;

      // Get the Tx Order
      if (oldValue & 0x10000000) *txOrder = GSC_LSB_FIRST;
      else *txOrder = GSC_MSB_FIRST;

      // Get the Rx Order
      if (oldValue & 0x08000000) *rxOrder = GSC_LSB_FIRST;
      else *rxOrder = GSC_MSB_FIRST;
    }
    break;

  case GSC_SIO_CLOCKEDGE:
    status = gscSio4ChannelGetClockEdge( boardNumber, channel, &optionValue[0],
                                         &optionValue[1] );
    break;

  case GSC_SIO_ENVELOPEPOLARITY:
    status = gscSio4ChannelGetEnvPolarity( boardNumber, channel, &optionValue[0],
                                           &optionValue[1] );
    break;

  case GSC_SIO_TXIDLECONDITION:
    status = gscSio4ChannelGetTxDataIdle( boardNumber, channel, &optionValue[0] );
    break;

  case GSC_SIO_TXCLOCKIDLECONDITION:
    status = gscSio4ChannelGetTxClockIdle( boardNumber, channel, &optionValue[0] );
    break;

  case GSC_SIO_TXDATAPINMODE:
  case GSC_SIO_RXDATAPINMODE:
  case GSC_SIO_TXCLOCKPINMODE:
  case GSC_SIO_RXCLOCKPINMODE:
  case GSC_SIO_CTSPINMODE:
  case GSC_SIO_RTSPINMODE:
    // Validate the Pin name
    status = validatePinName( optionName, &offset, NULL, NULL);
    if (status == 0)
      optionValue[0] = boardInformation[ boardNumber-1].channelPinMode[ channel-1][offset];
    break;

  case GSC_SIO_RECEIVER:
    // Read the current Control/Status register
    status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldValue);
    if (status) return status;

    // Get the Rx mode
    if (oldValue & 0x01000000)
      optionValue[0] = GSC_ENABLED;
    else
      optionValue[0] = GSC_DISABLED;
    break;

  case GSC_SIO_TRANSMITTER:
    // Read the current Control/Status register
    status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldValue);
    if (status) return status;

    // Get the Tx mode
    if (oldValue & 0x02000000)
      optionValue[0] = GSC_ENABLED;
    else
      optionValue[0] = GSC_DISABLED;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

//----------------------------------------------------- GscSio4SyncSetOption
//
//  This routine sets the identified option to the supplied value.
//
int gscSio4SyncSetOption( int boardNumber, int channel, enum GSC_OPTION_NAME optionName, int optionValue)
{
  int status = 0;
  int oldValue = 0;

  switch ( optionName )
  {
  case GSC_SIO_DATASIZE:
    {
      int dataSize = optionValue;
      // Range check the data size
      if((dataSize < 1) || (dataSize > 0xffff)) return GscApiInvalidParameter;

      // Get the current Tx Count Register
      status = fpgaRead( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), &oldValue);
      if (status) return 43; //status;

      // Adjust it to the new value
      oldValue &= 0xffff0000;
      oldValue |= dataSize;

      // Write it back out
      status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), oldValue);
      if (status) return status;
    }
    break;

  case GSC_SIO_GAPSIZE:
    {
      int gapSize = optionValue;

      // Range check the data size
      if((gapSize < 0) || (gapSize > 0xffff)) return GscApiInvalidParameter;

      // Save the new data size
      boardInformation[ boardNumber-1].channelGapSize[ channel-1] = gapSize;

      // Get the current Tx Count Register
      status = fpgaRead( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), &oldValue);
      if (status) return status;

      // Adjust it to the new value
      oldValue &= 0x0000ffff;
      oldValue |= (gapSize << 16);

      // Write it back out
      status = fpgaWrite( boardNumber, TX_COUNT_BASE_REG+4*(channel-1), oldValue);
    }

    break;

  case GSC_SIO_ENCODING:
    {
      int encoding = optionValue;

      // Convert board and channel into indexes
      int bn = boardNumber - 1;
      int ch = channel - 1;

      return fpgaSetEncoding( boardNumber,
                              channel,
                              encoding,
                              boardInformation[ bn].channelBitRate[ ch]);
    }

    break;

  case GSC_SIO_MSBLSBORDER:
    {
      int txOrder = (optionValue & 0xffff0000) >> 16;
      int rxOrder = optionValue & 0x0000ffff;

      // Get the current Channel Control/Status Register
      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), &oldValue);
      if (status) return status;

      // Process the Tx Order
      switch(txOrder)
      {
      case GSC_MSB_FIRST: oldValue &= ~0x10000000; break;
      case GSC_LSB_FIRST: oldValue |=  0x10000000; break;
      default:			return GscApiInvalidParameter;
      }

      // Process the Rx Source
      switch(rxOrder)
      {
      case GSC_MSB_FIRST: oldValue &= ~0x08000000; break;
      case GSC_LSB_FIRST: oldValue |=  0x08000000; break;
      default:			return GscApiInvalidParameter;
      }

      // Write it back out
      status = fpgaWrite( boardNumber, CONTROL_STATUS_BASE_REG+((channel-1)<<4), oldValue);
    }

    break;

  case GSC_SIO_CLOCKEDGE:
    {
      int txClockEdge = (optionValue & 0xffff0000) >> 16;
      int rxClockEdge = (optionValue & 0xffff);

      status = gscSio4ChannelSetClockEdge( boardNumber, channel, txClockEdge, rxClockEdge );
    }
    break;

  case GSC_SIO_ENVELOPEPOLARITY:
    {
      int txEnvPolarity = (optionValue & 0xffff0000) >> 16;
      int rxEnvPolarity = optionValue & 0xffff;

      status = gscSio4ChannelSetEnvPolarity( boardNumber, channel, txEnvPolarity, rxEnvPolarity );
    }
    break;

  case GSC_SIO_TXIDLECONDITION:
    status = gscSio4ChannelSetTxDataIdle( boardNumber, channel, optionValue );
    break;

  case GSC_SIO_TXCLOCKIDLECONDITION:
    status = gscSio4ChannelSetTxClockIdle( boardNumber, channel, optionValue );
    break;

  case GSC_SIO_TXDATAPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_DATA, optionValue);
    break;

  case GSC_SIO_RXDATAPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_DATA, optionValue);
    break;

  case GSC_SIO_TXCLOCKPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_CLOCK, optionValue);
    break;

  case GSC_SIO_RXCLOCKPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_CLOCK, optionValue);
    break;

  case GSC_SIO_CTSPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_RX_ENV, optionValue);
    break;

  case GSC_SIO_RTSPINMODE:
    // Setup the pin mode
    status = quickSetPinMode( boardNumber, channel, GSC_PIN_TX_ENV, optionValue);
    break;

  case GSC_SIO_RECEIVER:
    {
      int rxMode = optionValue;
      int oldData = 0;

      // Read the current Control/Status register
      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldData);
      if (status) return status;

      // Process the desired Rx mode
      switch (rxMode)
      {
      case GSC_ENABLED:	oldData |=  0x01000000; break;
      case GSC_DISABLED:	oldData &= ~0x01000000; break;
      default:			return GscApiInvalidParameter;
      }

      // Write the new value back out
      status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), oldData);
      if (status) return status;
    }
    break;

  case GSC_SIO_TRANSMITTER:
    {
      int txMode  = optionValue;
      int oldData = 0;

      // Read the current Control/Status register
      status = fpgaRead( boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), &oldData);
      if (status) return status;

      // Process the desired Tx mode
      switch (txMode)
      {
      case GSC_ENABLED:	oldData |=  0x02000000; break;
      case GSC_DISABLED:	oldData &= ~0x02000000; break;
      default:			return GscApiInvalidParameter;
      }

      // Write the new value back out
      status = fpgaWrite(boardNumber, CONTROL_STATUS_BASE_REG+16*(channel-1), oldData);
      if (status) return status;
    }
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

//----------------------------------------------------------- syncBoardSetMode
//
//  This routine is called from the main SetMode function. It
// is responsible for handling the modes available on the
// -Sync boards.
//
int syncBoardSetMode( int boardNumber, int channel, int mode, int bitRate)
{
  int status = 0x00;
  GSC_SYNC_CONFIG syncConfig;

  // Get the default parameters
  status = GscSio4SyncGetDefaults( &syncConfig );
  if (status) return status;

  // Set the desired bit rate and save the transfer rate
  syncConfig.bitRate = bitRate;

  // Process the requested mode
  switch (mode)
  {
  case GSC_MODE_SYNC:
    // Disable the Rx envelope input
    //syncConfig.rxEnvPolarity = GSC_DISABLED;

    // Stop TxC when there is no data
    //syncConfig.txClockIdleCondition = GSC_IDLE_NOT_ACTIVE;
    break;

  case GSC_MODE_SYNC_ENV:
    // Set ENV pins to AUTO since they are used
    syncConfig.rxEnvPinMode = GSC_PIN_AUTO;
    syncConfig.txEnvPinMode = GSC_PIN_AUTO;
    break;

  default:
    return GscApiInvalidParameter;
  }


  // Save the new board info
  boardInformation[ boardNumber-1].channelMode[channel-1] = mode;
  boardInformation[ boardNumber-1].channelBitRate[channel-1] = bitRate;
  boardInformation[ boardNumber-1].bytesPerMs[ channel-1] = (double)bitRate/1000/8;

  // Program the new configuration
  status = GscSio4SyncSetConfig( boardNumber, channel, syncConfig);
  return status;
}

int gscSio4ChannelGetClockEdge( int boardNumber, int channel, int *txClockEdge, int *rxClockEdge )
{
  // Get Tx and Rx clock edge settings

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Get Tx clock edge setting
  if ((oldValue & 0x0001) == 0)
    *txClockEdge = GSC_RISING_EDGE;
  else
    *txClockEdge = GSC_FALLING_EDGE;

  // Get Rx clock edge setting
  if ((oldValue & 0x2000) == 0)
    *rxClockEdge = GSC_FALLING_EDGE;
  else
    *rxClockEdge = GSC_RISING_EDGE;

  return status;
}

int gscSio4ChannelSetClockEdge( int boardNumber, int channel, int txClockEdge, int rxClockEdge )
{
  // Set TxC to clock data on rising or falling edge.
  // Set RxC to sample data on rising or falling edge.

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Set Tx clock edge bit
  if (txClockEdge == GSC_RISING_EDGE)
    oldValue &= ~0x0001;
  else if (txClockEdge == GSC_FALLING_EDGE)
    oldValue |= 0x0001;
  else
    return GscApiInvalidParameter;

  // Set Rx clock edge bit
  if (rxClockEdge == GSC_RISING_EDGE)
    oldValue |= 0x2000;
  else if (rxClockEdge == GSC_FALLING_EDGE)
    oldValue &= ~0x2000;
  else
    return GscApiInvalidParameter;

  // Write out the new value to the pin source register
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), oldValue);

  return status;
}

int gscSio4ChannelGetEnvPolarity( int boardNumber, int channel, int *txEnvPolarity, int *rxEnvPolarity )
{
  // Get Tx and Rx envelope polarity

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Get TxE polarity
  switch( oldValue & 0x0030 )
  {
  case 0x0000:
    *txEnvPolarity = GSC_HIGH_TRUE;
    break;

  case 0x0010:
    *txEnvPolarity = GSC_LOW_TRUE;
    break;

  case 0x0020:
  case 0x0030:
    *txEnvPolarity = GSC_DISABLED;
    break;

  default:
    return GscApiInvalidParameter;
  }

  // Get RxE polarity
  switch( oldValue & 0xc000 )
  {
  case 0x0000:
    *rxEnvPolarity = GSC_HIGH_TRUE;
    break;

  case 0x4000:
    *rxEnvPolarity = GSC_LOW_TRUE;
    break;

  case 0x8000:
  case 0xc000:
    *rxEnvPolarity = GSC_DISABLED;
    break;

  default:
    return GscApiInvalidParameter;
  }

  return status;
}

int gscSio4ChannelSetEnvPolarity( int boardNumber, int channel, int txEnvPolarity, int rxEnvPolarity )
{
  // Set TxE to active high or active low; and
  // set RxE to active high or active low

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Set TxE to desired value
  switch( txEnvPolarity )
  {
  case GSC_HIGH_TRUE:
    oldValue &= ~0x0010;
    break;

  case GSC_LOW_TRUE:
    oldValue |= 0x0010;
    break;

  case GSC_DISABLED:
    oldValue |= 0x0020;
    break;

  default:
    return GscApiInvalidParameter;
  }

  // Set RxE to desired value
  switch( rxEnvPolarity )
  {
  case GSC_HIGH_TRUE:
    oldValue &= ~0x4000;
    break;

  case GSC_LOW_TRUE:
    oldValue |= 0x4000;
    break;

  case GSC_DISABLED:
    oldValue |= 0x8000;
    break;

  default:
    return GscApiInvalidParameter;
  }

  // Write new value to Pin Source register
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), oldValue);
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);

  return status;
}

int gscSio4ChannelGetTxClockIdle( int boardNumber, int channel, int *txClockIdleCondition )
{
  // Get Tx clock to continuous or off while data invalid.

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Set the idle condition
  if ((oldValue & 0x0008) == 0)
    *txClockIdleCondition = GSC_IDLE_ACTIVE;
  else
    *txClockIdleCondition = GSC_IDLE_NOT_ACTIVE;

  return status;
}

int gscSio4ChannelSetTxClockIdle( int boardNumber, int channel, int txClockIdleCondition )
{
  // Set Tx clock to continuous or off while data invalid.

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Set the idle condition
  if (txClockIdleCondition == GSC_IDLE_ACTIVE)
    oldValue &= ~0x0008;
  else if (txClockIdleCondition == GSC_IDLE_NOT_ACTIVE)
    oldValue |= 0x0008;
  else
    return GscApiInvalidParameter;

  // Write new value to Pin Source register
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), oldValue);

  return status;
}

int gscSio4ChannelGetTxDataIdle( int boardNumber, int channel, int *txDataIdleCondition )
{
  // Retrieve the Tx data idle condition value - all zeros or all ones.

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Set TxD Idl bit to transmit all zeros or all ones.
  if ((oldValue & 0x00080000) == 0)
    *txDataIdleCondition = GSC_IDLE_ALL_0;
  else
    *txDataIdleCondition = GSC_IDLE_ALL_1;

  return status;
}

int gscSio4ChannelSetTxDataIdle( int boardNumber, int channel, int txDataIdleCondition )
{
  // This function configures what will be transmitted when the channel is idle.

  int status		= 0;
  int oldValue	= 0;

  // Get the current Channel Pin Source Register
  status = fpgaRead( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), &oldValue);
  if (status) return status;

  // Set TxD Idl bit to transmit all zeros or all ones.
  if (txDataIdleCondition == GSC_IDLE_ALL_0)
    oldValue &= ~0x00080000;
  else if (txDataIdleCondition == GSC_IDLE_ALL_1)
    oldValue |= 0x00080000;
  else
    return GscApiInvalidParameter;

  // Write new value to Pin Source register
  status = fpgaWrite( boardNumber, PIN_SOURCE_BASE_REG+((channel-1) * 4), oldValue);

  return status;
}

// Determine whether this is a BXR channel configured for SYNC mode operation.
int gscSio4ChannelIsBxrSync( int boardNumber, int channel )
{
  int chMode = 0;
  int status = 0;

  if ((boardInformation[boardNumber-1].boardType == GSC_BOARD_TYPE_NORMAL) &&
      (!strcmp(boardInformation[boardNumber-1].subId, "-SIO4BXR")))
  {

    status = fpgaRead(boardNumber, FW_TYPE_REG, &chMode);
    chMode = chMode & (0xff << ((channel - 1) * 8));
    if (((chMode >> ((channel - 1) * 8)) & 0xff) == 0x04)
      return 1;
  }
  return 0;
}