//////////////////////////////////////////////////////////////////////////////
//
// GscApiUtils.c
//
//  This module contains all of the generic utility routines that are used
// by the API routines.
//
#ifdef PLX_LINUX
#include <unistd.h>
#include <stdio.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#else
#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <sddl.h>
#include <stdio.h>
#endif

#include "GscApiLocal.h"

//
// External References
//


//-------------------------------------------------------- validateBoardNumber
//
//  This routine verifies that a board number is valid.
// Board numbers must be in the range of 1 through
// gscBoardCount.
//
int validateBoardNumber( int boardNumber)
{
  if ((boardNumber < 1) || (boardNumber > gscBoardCount)) return GscApiInvalidBoardNumber;

  return 0x00;
}


//-------------------------------------------------------- validateChannel
//
//  This routine verifies that the channel number is valid.
// Channel numbers must be in the range of 1 through
// NUMBER_OF_CHANNELS.
//
int validateChannel( int channel)
{
  if((channel < 1) || (channel > NUMBER_OF_CHANNELS)) return GscApiInvalidChannel;
  return 0x00;
}


//---------------------------------------------- validateBoardAndChannelNumber
//
//  This routine verifies that both the board number
// and the Channel number are valid.
//
int validateBoardAndChannelNumber( int boardNumber, int channel)
{
  int status = 0x00;

  // Validate the Board Number and return if bad
  status = validateBoardNumber( boardNumber);
  if (status) return status;

  // Validate the Channel and return the result
  status = validateChannel( channel);
  return status;
}


//------------------------------------------------------------ validatePinName
//
//  This routine validates the Pin name. It must be one of the
// Values below. The routine also returns the corresponding
// information for the pin.
//
int validatePinName( int pinName, int *offset, int *mask, int *shift)
{
  int tempMask, tempShift;

  // Process the pin name
  switch (pinName)
  {
  case GSC_PIN_RX_CLOCK:        tempMask = 0x0001;	tempShift = 0x0000; break;
  case GSC_PIN_RX_DATA:         tempMask = 0x0002;	tempShift = 0x0001; break;
  case GSC_PIN_CTS:             tempMask = 0x0004;	tempShift = 0x0002; break;
  case GSC_PIN_DCD:             tempMask = 0x0008;	tempShift = 0x0003; break;
  case GSC_PIN_TX_CLOCK:        tempMask = 0x0010;	tempShift = 0x0004; break;
  case GSC_PIN_TX_DATA:         tempMask = 0x0020;	tempShift = 0x0005; break;
  case GSC_PIN_RTS:             tempMask = 0x0040;	tempShift = 0x0006; break;
  case GSC_PIN_AUXCLK:          tempMask = 0x0100;	tempShift = 0x0008; break;
  case GSC_PIN_TX_USCCLOCK:     tempMask = 0x0200;	tempShift = 0x0009; break;
  default:				return GscApiInvalidParameter;
  }

  // Calculate the offset
  *offset = pinName - GSC_PIN_RX_CLOCK;

  // If the mask pointer is not null, update it
  if (mask != NULL) *mask = tempMask;

  // If the shift pointer is not null, update it
  if (shift != NULL) *shift = tempShift;

  // return
  return 0x00;
}

//
// The following routines are Linux implementations of the Win32 critical sections
//
#ifdef __linux__
int InitCriticalSection( pthread_mutex_t *mutex)
{
  pthread_mutexattr_t a;

  // Initialize the mutex
  pthread_mutexattr_init( &a );
  pthread_mutexattr_setpshared( &a, PTHREAD_PROCESS_SHARED ); // not supported on linux 2.4
                                                              // error can be ignored
  if (pthread_mutex_init(mutex, &a))
  {
    printf("Init Critical Section Error\n");
    return ApiFailed;
  }
  return 0;
}

int DelCriticalSection( pthread_mutex_t *mutex)
{
  // Destroy the mutex
  if (pthread_mutex_destroy(mutex))
  {
    printf("Delete Critical Section Error\n");
    return ApiFailed;
  }
  return 0;
}

int BeginCriticalSection( pthread_mutex_t *mutex)
{
  // Lock the mutex
  if (pthread_mutex_lock(mutex))
  {
    printf("Enter Critical Section Error\n");
    return ApiFailed;
  }
  return 0;
}

int EndCriticalSection( pthread_mutex_t *mutex)
{
  // Unlock the mutex
  if (pthread_mutex_unlock(mutex))
  {
    printf("Leave Critical Section Error\n");
    return ApiFailed;
  }
  return 0;
}

#else

int InitCriticalSection( HANDLE *ghMutex, LPCTSTR lpName)
{

  // Create/open the mutex
  SECURITY_ATTRIBUTES sa;
  BOOL status;
  TCHAR * szSD = TEXT("D:(A;ID;0x001f0001;;;WD)");
    //TEXT("D:")    // Discretionary ACL
    //TEXT("(D;OICI;GA;;;BA)")     // Deny all for built-in Administrators group :o)
    //TEXT("(D;OICI;GRGWGX;;;AU)") // Allow read/write/execute to Authenticated users
    //TEXT("(A;OICI;GA;;;AN)")     // Allow all to anonymous logon
    //TEXT("(A;OICI;GA;;;BG)");    // Allow all to built-in guests
  PULONG nSize = 0;

  // Convert the string to the security descriptor binary and return
  status = ConvertStringSecurityDescriptorToSecurityDescriptor(
        szSD,                       // The ACE strings
        SDDL_REVISION_1,            // Standard revision level
        &(sa.lpSecurityDescriptor), // Pointer to the converted security descriptor
        nSize);                     // The size in byte the converted security descriptor
  sa.nLength = sizeof( SECURITY_ATTRIBUTES );
  sa.bInheritHandle = FALSE;

  if ((*ghMutex = CreateMutex( &sa, FALSE, lpName )) == NULL )
  {
    DWORD dw = GetLastError();
    printf("Init Critical Section Error - 0x%x\n", dw);
    return ApiFailed;
  }
  ReleaseMutex( ghMutex );
  return 0;
}

int DelCriticalSection( HANDLE ghMutex )
{
  // Close the mutex
  if (CloseHandle(ghMutex) == 0)
  {
    printf("Delete Critical Section Error\n");
    return ApiFailed;
  }
  return 0;
}

int BeginCriticalSection( HANDLE ghMutex )
{
  DWORD dwWaitResult = 0;

  // Acquire the mutex
  if (((dwWaitResult = WaitForSingleObject( ghMutex, INFINITE )) != 0) &&
       (dwWaitResult != WAIT_ABANDONED))
  {
    printf("Enter Critical Section Error - 0x%x\n", dwWaitResult);
    return ApiFailed;
  }
  return 0;
}

int EndCriticalSection( HANDLE ghMutex )
{
  // Release ownership of the mutex
  if (ReleaseMutex(ghMutex) == 0)
  {
    DWORD dw = GetLastError();
    printf("Leave Critical Section Error - 0x%x\n", dw);
    return ApiFailed;
  }
  return 0;
}
#endif
